(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[   1005112,      32420]
NotebookOptionsPosition[    919983,      29838]
NotebookOutlinePosition[    942242,      30578]
CellTagsIndexPosition[    936648,      30394]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["TARCER 1.995", "Title",
 TextAlignment->Center,
 CellTags->"T1.1"],

Cell[TextData[{
 "A ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " program for the reduction of two-loop propagator integrals "
}], "Text",
 TextAlignment->Center],

Cell[TextData[{
 "Rolf Mertig\n",
 ButtonBox["Mertig Research & Consulting\n",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.mertig.com"], None}],
 "and\nRainer Scharf\n",
 ButtonBox["Universit\[ADoubleDot]t Leipzig",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.physik.uni-leipzig.de/~scharf/"], None}]
}], "Text",
 Editable->False,
 TextAlignment->Center,
 CellTags->"T1.2"],

Cell["14th March, 2004", "Text",
 TextAlignment->Center],

Cell[TextData[{
 "COMMENTS (RM 20.7.2011) evaluated this notebook with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 8 under Windows 64bit."
}], "Text"],

Cell["\<\
COMMENTS (RM 14.3..2004) uncommented a global variable assignement.changed \
default setting of $RankLimit to {2,5}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 31.10.2003) Change default settings of TarcerExpand, modify \
$BasisIntegras for massless integrals,
changed default setting of $RankLimit to {1,6}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 28.10.2003) Fixed a bug in TarcerExpand (only introduced \
recently),
changed default setting of $RankLimit to {1,5}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 24.9.2003) Relaxed a condition in equation (61) and (6153)\
\>", "Text"],

Cell["\<\
COMMENTS (RM 18.9.2003): Fixed things for 5.0, eliminated a TVI symmetry \
(causing infinite recursion) and added
TFI decomposition into products of TAI's (was somehow missing but shows up \
in, e.g., the 2-loop Higss selfenergy).
Putting TFI back into the Tarcer context, because the fcloops context created \
trouble ...
Changed default of $Comment to False.\
\>", "Text"],

Cell["\<\
COMMENTS (FO 8.9.2002.): Changed context of TFI to \
HighEnergyPhysics`fcloops`TFI` and updated other HighEnergyPhysics`* contexts.\
\>", "Text"],

Cell["\<\
COMMENTS (FO 15.3.2001.): Changed PreFactor into Prefactor1 to avoid conflict \
with FeynArts and FeynCalc.\
\>", "Text"],

Cell["\<\
COMMENTS (FO 15.3.2001.): Changed context of Zeta2 and made definition \
dependent on not having loaded FeynCalc.\
\>", "Text"],

Cell[TextData[{
 "COMMENTS (RM 25.4.): ",
 ButtonBox["TFISimplify",
  BaseStyle->"Hyperlink",
  ButtonData:>"T1.7.1.1"],
 " (like TFIRecurse, but without trule[9])."
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 23.4.): bug fixed in ",
 ButtonBox["tlrule[17]",
  BaseStyle->"Hyperlink",
  ButtonData:>"tlrule17"],
 "."
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 6.4.):  ",
 ButtonBox["special TVR relations",
  BaseStyle->"Hyperlink",
  ButtonData:>"specialTVR"],
 " added to TVR."
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 22.3.):  ",
 ButtonBox["TVI Symmetry",
  BaseStyle->"Hyperlink",
  ButtonData:>"Tsymmetry"],
 " added to TVR."
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 18.3.): More rules added to ",
 ButtonBox["$TTable",
  BaseStyle->"Hyperlink",
  ButtonData:>"$TTable"],
 ". "
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 18.2.): More rules added to ",
 ButtonBox["$TTable",
  BaseStyle->"Hyperlink",
  ButtonData:>"$TTable"],
 ". "
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 21.2.): Included ",
 ButtonBox["$TTable",
  BaseStyle->"Hyperlink",
  ButtonData:>"$TTable"],
 ". added tlrule[25]"
}], "Text"],

Cell["\<\
COMMENTS (RM 20.2.): $RankLimit = {0,6} ;  Changed TarcerRecurse such that \
special TJR and TVR (reducing D -> D-2) are used first.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 15.2.): $RankLimit = {0,8} ;  
                                              The following test now works:
                                              
                                                 Print[\"test 1 :  TFI[D, pp, \
{0, 0, 0, 1, 3}, {{1, MW}, {1, MZ}, {0, 0}, {1, MZ}, {1,MW}}] = \",
                                                            Timing[t1 \
=TarcerRecurse[TFI[D, pp, {0, 0, 0, 1, 3}, {{1, MW}, {1, MZ}, {0, 0}, {1, \
MZ}, {1,MW}}]]
                                                                       ] // \
InputForm
                                                         ]

                                                  Print[\"test 2 :  TFI[d, \
pp, 0, {0, 0, 0, 0, 4},{{1, m1}, {1, m2}, 0, 0, 0}] = \",
                                                           Timing[t2 \
=TarcerRecurse[TFI[D, pp,  0, {0, 0, 0, 0, 4},{{1, m1}, {1, m2}, 0, 0, 0}]];
                                                                       ] // \
InputForm
                                                          ]
\
\>", "Text"],

Cell["\<\
COMMENTS (RM 13.2.): tlrule[11] and trule[12] made more restrictive: \
inhibiting thus a recursion if {n5,m5} equals {0,0}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 12.2.): tlrule[15] made safer (in order to not get   0 :> 0 );
                                             TFIRecurse made safer (to not \
get into a recursion);
                                                  There is still something \
wrong with the tlrule's ; since 
                                         TFIRecurse[TFI[D, pp, {0, 0, 0, 1, \
3}, {{1, MW}, {1, MZ}, {0, 0}, {1, MZ}, {1,   MW}}]]
                                         doesn't work ..\
\>", "Text"],

Cell["\<\
COMMENTS (RM 5.2.): tlrule[10] slightly changed to cover more cases.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 2.2.):  FunctionExpand added in tlrule[9]. Option TimeConstraint \
added to TarcerRecurse (because TimeConstrained is buggy under Linux).\
\>", "Text"],

Cell[TextData[{
 "COMMENTS (RM 1.2.):  Changed TarcerRecurse. New TLR ",
 ButtonBox["definition",
  BaseStyle->"Hyperlink",
  ButtonData:>"TLRdef"],
 ". \n                          default $RankLimit={0,6};"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Remember", "=", "True"}], ";"}]], "Input"],

Cell["\<\
COMMENTS (RM 19.1.):   changes are:  tlrule[22] and tlrule[23], minor \
efficiency changes (Expand3 = Identity) and other small programming \
optimizations.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 22.1.):   changes are: bug in tlrule[23] fixed, new tlrule[24], \
new option Factor to TarcerReduce (doesn't change anything by default, but \
allows for improved Factor-ing).\
\>", "Text"],

Cell[TextData[{
 "See also: ",
 ButtonBox["hep-ph/9801383",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9801383"], None}],
 ", published in Comput. Phys. Commun. 111 (1998) 265-273"
}], "Text",
 Editable->False],

Cell[CellGroupData[{

Cell["Table of contents", "Section",
 CellDingbat->"\[FilledSmallCircle]"],

Cell[TextData[ButtonBox["Preliminaries",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.1.1"]], "Text"],

Cell[TextData[ButtonBox["Usages",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.2.1"]], "Text"],

Cell[TextData[ButtonBox["Creating the tarcerij.mx file",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.2.1.1"]], "Text"],

Cell[TextData[ButtonBox["Prologue (setting of $RankLimit)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.1"]], "Text"],

Cell[TextData[ButtonBox["Typesetting rules",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.1.1"]], "Text"],

Cell[TextData[ButtonBox["The Palettes",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.2.1"]], "Text"],

Cell[TextData[ButtonBox["OperatorApply",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.3.1"]], "Text"],

Cell[TextData[ButtonBox["MassDerivative",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.4.1"]], "Text"],

Cell[TextData[ButtonBox["TarasovT",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.5.1"]], "Text"],

Cell[TextData[ButtonBox["ApplyTarasovT",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.6.1"]], "Text"],

Cell[TextData[ButtonBox["Misc",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.1"]], "Text"],

Cell[TextData[ButtonBox["Colors",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.1.1"]], "SmallText"],

Cell[TextData[ButtonBox["PQ",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.2.1"]], "SmallText"],

Cell[TextData[ButtonBox["PNQ",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.3.1"]], "SmallText"],

Cell[TextData[ButtonBox["FactorC",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.4.1"]], "SmallText"],

Cell[TextData[ButtonBox["TFI",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.5.1"]], "SmallText"],

Cell[TextData[ButtonBox["TJI",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.6.1"]], "SmallText"],

Cell[TextData[ButtonBox["TBI",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.7.1"]], "SmallText"],

Cell[TextData[ButtonBox["TAI",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.8.1"]], "SmallText"],

Cell[TextData[ButtonBox["Definitions and notation",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.4.1"]], "Text"],

Cell[TextData[StyleBox[ButtonBox["ToArgs",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.4.1.1"],
 FontColor->RGBColor[0.2, 0.6, 1]]], "Text"],

Cell[TextData[StyleBox[ButtonBox["Cayley",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.4.2.1"],
 FontColor->RGBColor[1, 0, 0.4]]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{
      StyleBox["Cayley",
       FontColor->RGBColor[1, 0, 0.4]], "[", "i", "]"}], "=", 
     SubscriptBox["\[CapitalDelta]", "i"]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.4.3.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TextForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{
      StyleBox["CayleyD",
       FontColor->RGBColor[1, 0.2, 1]], "[", "i", "]"}], "=", 
     RowBox[{
      SubscriptBox["\[CapitalDelta]", "i"], "/", "\[CapitalDelta]"}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.4.4.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TextForm]]]], "Text"],

Cell[TextData[ButtonBox["Utility Functions",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun and friends",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.1.1"]], "Text"],

Cell[TextData[ButtonBox["Interchange",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.1.1.1"]], "SmallText"],

Cell[TextData[{
 ButtonBox["\[Nu]",
  BaseStyle->"Hyperlink",
  ButtonData:>"T1.5.2.1"],
 ButtonBox["explicit",
  BaseStyle->"Hyperlink",
  ButtonData:>"T1.5.2.1"]
}], "Text"],

Cell[TextData[ButtonBox["TA",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.2.1.1"]], "SmallText"],

Cell[TextData[ButtonBox["CheckRecursion",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.3.1"]], "Text"],

Cell[TextData[ButtonBox["CheckTVIRecursion",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.3.1.1"]], "SmallText"],

Cell[TextData[ButtonBox["CheckTJIRecursion",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.3.2.1"]], "SmallText"],

Cell[TextData[ButtonBox["Recurrence Relations Input and Recursion Function \
Generation",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.1"]], "Text"],

Cell[TextData[ButtonBox["Input (35)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.1.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (35)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.2.1"]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     FormBox[
      SuperscriptBox["1", 
       RowBox[{"+", " "}]],
      TraditionalForm], 
     RowBox[{" ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"35", "_"}], "1"}], ")"}]}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.2.1.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     FormBox[
      SuperscriptBox["2", 
       RowBox[{"+", " "}]],
      TraditionalForm], 
     RowBox[{" ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"35", "_"}], "2"}], ")"}]}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.2.2.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     FormBox[
      SuperscriptBox["3", 
       RowBox[{"+", " "}]],
      TraditionalForm], 
     RowBox[{" ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"35", "_"}], "3"}], ")"}]}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.2.3.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     FormBox[
      SuperscriptBox["4", 
       RowBox[{"+", " "}]],
      TraditionalForm], 
     RowBox[{" ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"35", "_"}], "4"}], ")"}]}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.2.4.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[ButtonBox["Input (3501)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.3.1"]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{
      RowBox[{"MakeFun", " ", 
       RowBox[{"(", "3501", ")"}], " ", "\[CapitalDelta]"}], "=", "0", " "}], 
     
     FormBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["p", "2"], "=", 
        SuperscriptBox["m", "2"]}], ")"}],
      TraditionalForm]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.4.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[ButtonBox["Input (3502)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.5.1"]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{
      RowBox[{"MakeFun", " ", 
       RowBox[{"(", "3502", ")"}], " ", "\[CapitalDelta]"}], "=", "0", " "}], 
     
     FormBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["p", "2"], "=", 
        SuperscriptBox["m", "2"]}], ")"}],
      TraditionalForm]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.6.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{
      RowBox[{"MakeFun", " ", 
       RowBox[{"(", "3503", ")"}], " ", "\[CapitalDelta]"}], "=", "0", " "}], 
     
     FormBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["p", "2"], "=", 
        SuperscriptBox["m", "2"]}], ")"}],
      TraditionalForm]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.7.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{
      RowBox[{"MakeFun", " ", 
       RowBox[{"(", "3504", ")"}], " ", "\[CapitalDelta]"}], "=", "0", " "}], 
     
     FormBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["p", "2"], "=", 
        SuperscriptBox["m", "2"]}], ")"}],
      TraditionalForm]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.8.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[ButtonBox["Input (41)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.9.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (41)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.10.1"]], "Text"],

Cell[TextData[ButtonBox["Input (43)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.11.1"]], "Text"],

Cell[TextData[ButtonBox["Input (44)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.12.1"]], "Text"],

Cell[TextData[ButtonBox["Input (45)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.13.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (43) and (44)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.14.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (50)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.15.1"]], "Text"],

Cell[TextData[ButtonBox["Input (51)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.16.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (51)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.17.1"]], "Text"],

Cell[TextData[ButtonBox["Input (52)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.18.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (52)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.19.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (53)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.20.1"]], "Text"],

Cell[TextData[ButtonBox["Input (54)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.21.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (54)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.22.1"]], "Text"],

Cell[TextData[ButtonBox["Input (55)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.23.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (55)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.24.1"]], "Text"],

Cell[TextData[ButtonBox["Input (56)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.25.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (56)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.26.1"]], "Text"],

Cell[TextData[ButtonBox["Input (59)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.27.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (59)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.28.1"]], "Text"],

Cell[TextData[ButtonBox["Input (60)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.29.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (60)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.30.1"]], "Text"],

Cell[TextData[ButtonBox["Input (61)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.31.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (61)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.32.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6153)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.33.1"]], "Text"],

Cell[TextData[ButtonBox["Input (62)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.34.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (62)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.35.1"]], "Text"],

Cell[TextData[ButtonBox["Input (63)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.36.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (63)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.37.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6552)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.38.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6552)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.39.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6553)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.40.1"]], "Text"],

Cell[TextData[ButtonBox["Input (66)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.41.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (66)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.42.1"]], "Text"],

Cell[TextData[ButtonBox["Input (alternative67)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.43.1"]], "Text"],

Cell[TextData[ButtonBox["Input (67)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.44.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (67)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.45.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6713)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.46.1"]], "Text"],

Cell[TextData[ButtonBox["Input (68)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.47.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (68)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.48.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6813)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.49.1"]], "Text"],

Cell[TextData[ButtonBox["Input (70)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.50.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (70)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.51.1"]], "Text"],

Cell[TextData[ButtonBox["Input (71)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.52.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (71)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.53.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (7123)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.54.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (7113)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.55.1"]], "Text"],

Cell[TextData[ButtonBox["Input (78)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.56.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (78)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.57.1"]], "Text"],

Cell[TextData[ButtonBox["Input (80)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.58.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (80)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.59.1"]], "Text"],

Cell[TextData[ButtonBox["Input (81)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.60.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (81)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.61.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8123)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.62.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8113)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.63.1"]], "Text"],

Cell[TextData[ButtonBox["Input (82)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.64.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (82)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.65.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8213)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.66.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8212)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.67.1"]], "Text"],

Cell[TextData[ButtonBox["Input (83)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.68.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (83)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.69.1"]], "Text"],

Cell[TextData[ButtonBox["Input (87)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.70.1"]], "Text"],

Cell[TextData[ButtonBox["(88)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.70.1.1"]], "SmallText"],

Cell[TextData[ButtonBox["(89)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.70.2.1"]], "SmallText"],

Cell[TextData[ButtonBox["MakeFun (87)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.71.1"]], "Text"],

Cell[TextData[ButtonBox["Input (90)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.72.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (90)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.73.1"]], "Text"],

Cell[TextData[ButtonBox["Input (91)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.74.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (911)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.75.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (912)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.76.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (913)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.77.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (914)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.78.1"]], "Text"],

Cell[TextData[ButtonBox["Input (92)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.79.1"]], "Text"],

Cell[TextData[ButtonBox["Input (93)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.80.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (93)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.81.1"]], "Text"],

Cell[TextData[ButtonBox["Input (94)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.82.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (94)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.83.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9401)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.84.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9402)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.85.1"]], "Text"],

Cell[TextData[ButtonBox["Input (95)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.86.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (95)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.87.1"]], "Text"],

Cell[TextData[ButtonBox["Input (96)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.88.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (96)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.89.1"]], "Text"],

Cell[TextData[ButtonBox["Input (97)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.90.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (97)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.91.1"]], "Text"],

Cell[TextData[ButtonBox["Input (98)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.92.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (98)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.93.1"]], "Text"],

Cell[TextData[ButtonBox["Input (99)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.94.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (99)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.95.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9912)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.96.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9913)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.97.1"]], "Text"],

Cell[TextData[ButtonBox["Input (100)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.98.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (100)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.99.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1002)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.100.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1003)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.101.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1004)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.102.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1005)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.103.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1006)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.104.1"]], "Text"],

Cell[TextData[ButtonBox["Input (150)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.105.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (150)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.106.1"]], "Text"],

Cell[TextData[ButtonBox["Input (151)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.107.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (151)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.108.1"]], "Text"],

Cell[TextData[ButtonBox["Input (152)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.109.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (152)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.110.1"]], "Text"],

Cell[TextData[ButtonBox["Input (153)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.111.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (153)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.112.1"]], "Text"],

Cell[TextData[ButtonBox["Input (200)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.113.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (200)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.114.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (201)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.115.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (202)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.116.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (203)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.117.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (204)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.118.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (205)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.119.1"]], "Text"],

Cell[TextData[ButtonBox["Input (1xxxx)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.120.1"]], "Text"],

Cell[TextData[ButtonBox["five -> four",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.121.1"]], "Text"],

Cell[TextData[ButtonBox["four -> three",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.122.1"]], "Text"],

Cell[TextData[ButtonBox["three -> two",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.123.1"]], "Text"],

Cell[TextData[ButtonBox["two -> one",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.124.1"]], "Text"],

Cell[TextData[ButtonBox["TarcerRecurse and TFIRecurse",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.1"]], "Text"],

Cell[TextData[ButtonBox["TFIRecurse",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.1.1"]], "Text"],

Cell[TextData[ButtonBox["T*IS",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.2.1"]], "Text"],

Cell[TextData[ButtonBox["TComment",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.3.1"]], "Text"],

Cell[TextData[ButtonBox["TarcerRecurse",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.4.1"]], "Text"],

Cell[TextData[ButtonBox["TarcerExpand",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.8.1"]], "Text"],

Cell[TextData[ButtonBox["Construct T's",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.9.1"]], "Text"],

Cell[TextData[ButtonBox["$BasisIntegrals",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.10.1"]], "Text"],

Cell[TextData[ButtonBox["Save",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.9.1"]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Preliminaries", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.1.1"],

Cell[TextData[{
 "This is free software.\nYou may use it, distribute it, change it, improve \
it, etc..\nIf you have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " questions, ask them to support@wolfram.com or to your local ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " reseller."
}], "SmallText",
 Editable->False],

Cell["\<\
 We consider the following type of integrals, where a,b,u,v,r,s,t are \
non-negative integers.\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", 
     SuperscriptBox["p", "2"], ",", "\[CapitalDelta]p", ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", "v", ",", "r", ",", "s", ",", "t"}], "}"}]}]}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               SubscriptBox["\[Nu]", "1"], ",", 
               SubscriptBox["m", "1"]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               SubscriptBox["\[Nu]", "2"], ","}]}]}]}],
           TraditionalForm], 
          SubscriptBox["m", "2"]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], ",", 
          SubscriptBox["m", "3"]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["\[Nu]", "4"], ",", 
          SubscriptBox["m", "4"]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["\[Nu]", "5"], ",", 
          SubscriptBox["m", "5"]}], "}"}]}], "}"}], "]"}], " ", "="}], " "}], 
   TraditionalForm]]],
 "\n\n",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    SuperscriptBox["\[Pi]", "d"]], TraditionalForm]]],
 "\[Integral] \[Integral] ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{
     FormBox[
      RowBox[{
       SuperscriptBox["d", "d"], 
       SubscriptBox["k", "1"], " ", 
       SuperscriptBox["d", "d"], 
       SubscriptBox["k", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[CapitalDelta]", "\[CenterDot]", 
          SubscriptBox["k", "1"]}], ")"}], "a"], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[CapitalDelta]", "\[CenterDot]", 
          SubscriptBox["k", "1"]}], ")"}], "b"]}],
      TraditionalForm], 
     SuperscriptBox[
      RowBox[{"(", 
       SubsuperscriptBox["k", "1", "2"], ")"}], "u"], 
     SuperscriptBox[
      RowBox[{"(", 
       SubsuperscriptBox["k", "2", "2"], ")"}], "v"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "\[CenterDot]", 
        SubscriptBox["k", "1"]}], ")"}], "r"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "\[CenterDot]", 
        SubscriptBox["k", "2"]}], ")"}], "s"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["k", "1"], "\[CenterDot]", 
        SubscriptBox["k", "2"]}], ")"}], "t"]}], 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"[", 
              RowBox[{
               SubsuperscriptBox["k", "1", "2"], "-", 
               SubsuperscriptBox["m", "1", "2"]}], "]"}], 
             SubscriptBox["\[Nu]", "1"]], "[", 
            RowBox[{
             SubsuperscriptBox["k", "2", "2"], "-", 
             SubsuperscriptBox["m", "2", "2"]}], "]"}], 
           SubscriptBox["\[Nu]", "2"]], "[", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["k", "1"], "-", "p"}], ")"}], "2"], "-", 
           SubsuperscriptBox["m", "3", "2"]}], "]"}], 
         SubscriptBox["\[Nu]", "3"]], "[", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["k", "2"], "-", "p"}], ")"}], "2"], "-", 
         SubsuperscriptBox["m", "4", "2"]}], "]"}], 
       SubscriptBox["\[Nu]", "4"]], "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["k", "1"], "-", 
          SubscriptBox["k", "2"]}], ")"}], "2"], "-", 
       SubsuperscriptBox["m", "5", "2"]}], "]"}], 
     SubscriptBox["\[Nu]", "5"]]], TraditionalForm]]]
}], "Text"],

Cell["\<\
Putting the        $Post = (#;)&;       instruction after   
Unset[$Post]   suppresses all output cells.
Putting Unset[$Post] behind  $Post=(#:)&;
will produce a lot of typsetted reduction formulae.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unset", "[", "$Post", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Post", " ", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"#", ";"}], ")"}], "&"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$Notebooks", "=!=", "True"}], ",", 
    RowBox[{"$Post", " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"#", ";"}], ")"}], "&"}]}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$HistoryLength", "=", "0"}], ";"}]], "Input"],

Cell["\<\
This determines the directory into which the resulting file tarcer.mx is \
written.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{"$Notebooks", ",", 
   RowBox[{"SetDirectory", "[", 
    RowBox[{"ToFileName", "[", 
     RowBox[{"First", "[", 
      RowBox[{"\"\<FileName\>\"", "/.", 
       RowBox[{"NotebookInformation", "[", 
        RowBox[{"InputNotebook", "[", "]"}], "]"}]}], "]"}], "]"}], "]"}]}], 
  "]"}]], "Input"],

Cell[TextData[{
 "Somehow ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " crashes on my (R.M.) good old NeXTStation if I don't open all cells first \
...; this opens all cells automatically:"
}], "SmallText"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"$System", "===", "\"\<NeXT\>\""}], ",", 
   RowBox[{
    RowBox[{"SelectionMove", "[", 
     RowBox[{
      RowBox[{"EvaluationNotebook", "[", "]"}], ",", "All", ",", "Notebook"}],
      "]"}], ";", 
    RowBox[{"FrontEndExecute", "[", 
     RowBox[{"{", 
      RowBox[{"FrontEnd`FrontEndToken", "[", 
       RowBox[{
        RowBox[{"EvaluationNotebook", "[", "]"}], ",", 
        "\"\<SelectionOpenAllGroups\>\""}], "]"}], "}"}], "]"}]}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "spell"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "spell1"}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Creating the tarcerij.mx file", "Subsubsection",
 CellTags->"T1.2.1.1"],

Cell[TextData[{
 "Running this notebook will produce a tarcerij.mx file where ij will be \
replaced by \nthe two number set by the variable $RankLimit (default 33). \n\
In the same directory as this notebook which can be loaded  into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by    <<tarcer33.mx,\nor, when running in Terminal mode, on Unix systems, \
already when starting ",
 StyleBox["Mathematica, ",
  FontSlant->"Italic"],
 "as \nmyshellprompt>  math -initfile tarcer33.mx"
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Usages", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.2.1"],

Cell[BoxData[""], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"BeginPackage", "[", "\"\<HighEnergyPhysics`Tarcer`\>\"", "]"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerRecurse", "::", "usage"}], "=", 
   "\"\<TarcerRecurse[expr] applies recurrence relations for all TFI's (and \
TVI, TJI, TBI, TAI) in expr. For evaluation and Laurent expansion of the \
resulting basis integrals use TarcerExpand.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerExpand", "::", "usage"}], "=", 
   "\"\<TarcerExpand[expr, d -> (4+e), order] or TarcerExpand[epxr, d -> \
(4-e), order] or TarcerExpand[expr, d -> (4 - 2 om)substitutes integrals as \
specified by the option TarcerRules, and does a Laurent expansion in the \
variable around 0 at the right hand side of the second argument (i.e., you \
can use whatever convention you like ...). The same effect as \
TarcerExpand[TarcerRecurse[expr], ...] can be achieved by setting the option \
TarcerRecurse of TarcerExpand to True. TarcerExpand[expr, d -> (4+e)] is \
equivalent to TarcerExpand[expr, d -> (4+e), 0].\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerReduce", "::", "usage"}], "=", 
   "\"\<TarcerReduce is an option to TarcerExpand. It should be set to a list \
of basic integrals.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Cayley", "::", "usage"}], "=", 
   "\"\<Cayley[m1,m2,m3,m4,m5,Sqrt[pp]] is the Cayley determinant.\>\""}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Cayleyu", "::", "usage"}], "=", 
   "\"\<Cayleyu are the sub-Cayley-determinants as described in Tarasov's \
paper.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CayleyD", "::", "usage"}], "=", 
   "\"\<CayleyD are the sub-Cayley-determinants as described in Tarasov's \
paper.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Prefactor1", "::", "usage"}], "=", 
   "\"\<Prefactor1[expr] may be used to prohibit TarcerExpand from \
Taylor-expanding expr.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SEpsilon", "::", "usage"}], "=", 
   "\"\<Sepsilon[d] is an abbreviation for Exp[(d-4)/2 EulerGamma].\>\""}], 
  ";"}]], "Input"],

Cell["\<\
TFI::\"usage\" =
\"TFI[d, pp, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] is
the 2-loop d-dimensional integral
1/( (q1^2 - m1^2)^n1  (q2^2 - m2^2)^n2 ((q1-p)^2 - m3^2)^n3 *
    ((q2-p)^2 - m4^2)^n4  ((q1-q2)^2 - m5^2)^n5 ) .

 TFI[d, pp, {x,y,z,v,w}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}]
has as additional factors in the numerator (q1^2)^x*(q2^2)^y*(q1.p)^z*
(q2.p)^v*(q1.q2)^w .

 TFI[d, pp, dp, {a,b}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}]
has as additional factors in the numerator  (OPEDelta.q1)^a * \
(OPEDelta.q2)^b;
dp is (OPEDelta.p).\";\
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFIRecurse", "::", "usage"}], "=", 
   "\"\<TFIRecurse[expr] applies the recursion algorithm implemented in \
TARCER.nb (following Tarasov) to TFI's (and TVI's, TJI's, TBI's, TAI's in \
expr.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFISimplify", "::", "usage"}], "=", 
   "\"\<TFISimplify[expr] applies the recursion algorithm implemented in \
TARCER.nb (following Tarasov) to TFI's (and TVI's, TJI's, TBI's, TAI's in \
expr, except k1.k2.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TVI", "::", "usage"}], "=", 
   "\"\<TVI[d,pp,{ {n1,m1},{n2,m2},{n3,m3},{n4,m4}}] is the massive 2-loop \
integral with four propagators in d dimensions. \>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TJI", "::", "usage"}], "=", 
   "\"\<TJI[d, pp, { {n1,m1},{n2,m2},{n3,m3} }] is the massive 2-loop \
integral with three propagators in d dimensions. \>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TKI", "::", "usage"}], "=", 
   "\"\<TKI[d, 0, { {n1,m1},{n2,m2},{n3,m3} }] is the bubble massive 2-loop \
selfenergy type integral with three propagators in d dimensions. \>\""}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TBI", "::", "usage"}], "=", 
   "\"\<TBI[d, pp, { {n1,m1},{n2,m2} }] is the massive 1-loop selfenergy type \
integral with two propagators in d dimensions.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAI", "::", "usage"}], "=", 
   "\"\<TAI[d, pp, { {n1,m1} }] is the massive 1-loop selfenergy type \
integral with one propagator in d dimensions.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFR", "::", "usage"}], "=", 
   "\"\<TFR is similar in syntax to TFI but applies once the recurrence \
relations derived for TFI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TVR", "::", "usage"}], "=", 
   "\"\<TVR is similar in syntax to TVI but applies once the recurrence \
relations derived for TVI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TJR", "::", "usage"}], "=", 
   "\"\<TJR is similar in syntax to TJI but applies once the recurrence \
relations derived for TJI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TBR", "::", "usage"}], "=", 
   "\"\<TBR is similar in syntax to TBI but applies once the recurrence \
relations derived for TBI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAR", "::", "usage"}], "=", 
   "\"\<TAR is similar in syntax to TAI but applies once the recurrence \
relations derived for TAI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$RankLimit", "::", "usage"}], "=", 
   "\"\<$RankLimit is the list of integers specifying the highest possible \
sums of exponents of scalar products; i.e., {a+b, r+s} means that at most \
(Delta.k1)^a * (Delta.k2)^2 * (p.k1)^r * (p.k2)^s in the numerator will be \
calculated (a,b,r,s have to be non-negative integares). $RankLimit has to be \
set in the TARCER.nb notebook. In the tarcer.mx file this is not possible \
anymore.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$TarcerRecursed", "::", "usage"}], "=", 
   "\"\<If $Comment is set to True all integrals\\n\\t\\twhich are reduced \
are collected in the global variable $TarcerRecured.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$BasisIntegrals", "::", "usage"}], "=", 
   "\"\<$BasisIntegrals is a list of some basis integrals. It is the default \
setting of TarcerReduce of TarcerExpand.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$Comment", "::", "usage"}], "=", 
   "\"\<$Comment can be set to True or False. If set to False no comments are \
generated.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$CommentNotebook", "::", "usage"}], "=", 
   "\"\<$Comment can be set to True or False. Setting it to True will \
generate a separate notebook (if and only if $Comment is True) where the used \
recursion relations are displayed.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$RecursionStop", "::", "usage"}], "=", 
   "\"\<$RecursionStop sets an upper limit on the total number of recursion \
levels. Its purpose is to inhibit erroneous infinite recursions.\>\""}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$TTable", "::", "usage"}], "=", 
   "\"\<$TTable is a list of precomputed reductions. $TTable is the default \
setting of the option Table of TarcerRecurse. \>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Expand3", "=", "Identity"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Prologue (setting of $RankLimit)", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.3.1"],

Cell[TextData[{
 "This sets the limit up to which the ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 "-operators are constructed automatically, i.e., $RankLimit = {a+b, r+s} , \
concerning: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"\[CapitalDelta]", "\[CenterDot]", 
       SubscriptBox["k", "1"]}], ")"}], "a"], " "}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"\[CapitalDelta]", "\[CenterDot]", 
      SubscriptBox["k", "2"]}], ")"}], "b"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "\[CenterDot]", 
       SubscriptBox["k", "1"]}],
      TraditionalForm], ")"}], "r"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "\[CenterDot]", 
       SubscriptBox["k", "2"]}],
      TraditionalForm], ")"}], "s"], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$RankLimit", "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "6"}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"$RankLimit", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "6"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"$RankLimit", "=", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5"}], "}"}]}], ";"}]}], "Input"],

Cell["Check recursion relations by changing False to True here:", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$CheckRecursion", "=", "False"}], ";"}]], "Input"],

Cell["\<\
Change False to True for getting the input palettes for entering new \
recursion relations.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$PutPalettes", "=", "False"}], ";"}]], "Input"],

Cell[TextData[{
 "Notice:  Instead of using shift operators",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"  ", 
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"]}],
     TraditionalForm], ",", 
    StyleBox[
     SuperscriptBox["2", "+"],
     FontWeight->"Bold"]}], TraditionalForm]]],
 " etc. the \[Nu]-indices are colored blue; instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"  ", 
      StyleBox[
       SuperscriptBox["1", "-"],
       FontWeight->"Bold"]}],
     TraditionalForm], ",", 
    StyleBox[
     SuperscriptBox["2", "-"],
     FontWeight->"Bold"]}], TraditionalForm]]],
 " etc. the \[Nu]-indices are colored red."
}], "Text"],

Cell[CellGroupData[{

Cell["Typesetting rules", "Subsection",
 CellTags->"T1.3.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"atime", "=", 
   RowBox[{"AbsoluteTime", "[", "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"SEpsilon", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"SEpsilon", "[", "d_", "]"}], ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"InterpretationBox", "@@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"StyleBox", "[", 
      RowBox[{
       RowBox[{"SubscriptBox", "[", 
        RowBox[{"\"\<S\>\"", ",", 
         RowBox[{"First", "[", 
          RowBox[{"Variables", "[", "d", "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"FontWeight", "\[Rule]", "\"\<Bold\>\""}]}], "]"}], ",", 
     RowBox[{"SEpsilon", "[", "d", "]"}], ",", 
     RowBox[{"Editable", "\[Rule]", "False"}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"SEpsilon", "[", "d_", "]"}], "n_"], ",", "fmt_"}], "]"}], ":=", 
  
  RowBox[{"InterpretationBox", "@@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"StyleBox", "[", 
      RowBox[{
       RowBox[{"SubsuperscriptBox", "[", 
        RowBox[{"\"\<S\>\"", ",", 
         RowBox[{"First", "[", 
          RowBox[{"Variables", "[", "d", "]"}], "]"}], ",", "n"}], "]"}], ",", 
       RowBox[{"FontWeight", "\[Rule]", "\"\<Bold\>\""}]}], "]"}], ",", 
     RowBox[{"SEpsilon", "[", "d", "]"}], ",", 
     RowBox[{"Editable", "\[Rule]", "False"}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"Prefactor1", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"Prefactor1", "[", "d_", "]"}], ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"InterpretationBox", "@@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"MakeBoxes", "[", 
      RowBox[{"d", ",", "fmt"}], "]"}], ",", 
     RowBox[{"Prefactor1", "[", "d", "]"}], ",", 
     RowBox[{"Editable", "->", "False"}]}], "}"}]}]}]], "Input"],

Cell["\<\
Don't mess with the InterpretationBox's !! (important for the palette and \
OperatorApply).\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"Schiebe", "[", 
     RowBox[{"i_", ",", "pm_String"}], "]"}], " ", ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"InterpretationBox", "[", 
   RowBox[{
    RowBox[{"StyleBox", "[", 
     RowBox[{
      RowBox[{"SuperscriptBox", "[", 
       RowBox[{"i", ",", "pm"}], "]"}], ",", 
      RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
    RowBox[{"Schiebe", "[", 
     RowBox[{"i", ",", "pm"}], "]"}], ",", 
    RowBox[{"Editable", "->", "True"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CayleyD", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CayleyD", "[", "i_Integer", "]"}], "[", "args_", "]"}], ",", 
     "fmt_"}], "]"}], ":=", 
   RowBox[{"FractionBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<\[CapitalDelta]\>\"", ",", " ", "i"}], "]"}], ",", 
     "\"\<\[CapitalDelta]\>\""}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CayleyD", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CayleyD", "[", 
       RowBox[{"i_Integer", ",", "j_Integer"}], "]"}], "[", "args_", "]"}], 
     ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"FractionBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<\[CapitalDelta]\>\"", ",", 
       RowBox[{
        RowBox[{"10", " ", "i"}], "+", "j"}]}], "]"}], ",", 
     "\"\<\[CapitalDelta]\>\""}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CayleyD", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CayleyD", "[", 
       RowBox[{"i_Integer", ",", "j_Integer", ",", "k_Integer"}], "]"}], "[", 
      "args_", "]"}], ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"FractionBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<\[CapitalDelta]\>\"", ",", 
       RowBox[{
        RowBox[{"100", "  ", "i"}], "+", 
        RowBox[{"10", "j"}], "+", "k"}]}], "]"}], ",", 
     "\"\<\[CapitalDelta]\>\""}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cayley", "[", "i_Integer", "]"}], "[", "args_", "]"}], ",", 
     "fmt_"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[CapitalDelta]\>\"", ",", " ", "i"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cayley", "[", 
       RowBox[{"i_Integer", ",", "j_Integer"}], "]"}], "[", "args_", "]"}], 
     ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[CapitalDelta]\>\"", ",", 
     RowBox[{
      RowBox[{"10", " ", "i"}], "+", "j"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cayley", "[", 
       RowBox[{"i_Integer", ",", "j_Integer", ",", "k_Integer"}], "]"}], "[", 
      "args_", "]"}], ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[CapitalDelta]\>\"", ",", 
     RowBox[{
      RowBox[{"100", "  ", "i"}], "+", 
      RowBox[{"10", "j"}], "+", "k"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayleyu", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cayleyu", "[", 
       RowBox[{"i_Integer", ",", "j_Integer", ",", "k_Integer"}], "]"}], "[", 
      "args_", "]"}], ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<u\>\"", ",", 
     RowBox[{
      RowBox[{"100", "  ", "i"}], "+", 
      RowBox[{"10", "j"}], "+", "k"}]}], "]"}]}], ";"}]], "Input"],

Cell["\<\
MakeBoxes[PP, fm_] := InterpretationBox[SuperscriptBox[p, 2], PP]; 
MakeBoxes[m1^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 1, p], m1^p]; \

MakeBoxes[m2^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 2, p], m2^p]; \

MakeBoxes[m3^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 3, p], m3^p]; \

MakeBoxes[m4^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 4, p], m4^p]; \

MakeBoxes[m5^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 5, p], m5^p]; \

m6 = Sqrt[PP]; \
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m1", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "1"}], "]"}], ",", "m1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m2", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "2"}], "]"}], ",", "m2"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m3", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "3"}], "]"}], ",", "m3"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m4", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "4"}], "]"}], ",", "m4"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m5", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "5"}], "]"}], ",", "m5"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"mbt", "[", "z_", "]"}], ":=", 
   RowBox[{"ToBoxes", "[", 
    RowBox[{"z", ",", "TraditionalForm"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "z", "]"}], "=!=", "Plus"}]}], "]"}], ":=", 
   RowBox[{"mbt", "[", "z", "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "-", "1"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"mbt", "[", "z", "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_Subscript", "-", "2"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{
       RowBox[{"OverscriptBox", "[", 
        RowBox[{
         RowBox[{"z", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         ",", "\"\<_\>\""}], "]"}], ",", 
       RowBox[{"z", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
      "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "-", "2"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"OverscriptBox", "[", 
      RowBox[{
       RowBox[{"mbt", "[", "z", "]"}], ",", "\"\<_\>\""}], "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "-", "3"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"UnderscriptBox", "[", 
      RowBox[{
       RowBox[{"OverscriptBox", "[", 
        RowBox[{
         RowBox[{"mbt", "[", "z", "]"}], ",", "\"\<_\>\""}], "]"}], ",", 
       "\"\<_\>\""}], "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "+", "1"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"mbt", "[", "z", "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", "0", ",", "1"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "+", "2"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"OverscriptBox", "[", 
      RowBox[{
       RowBox[{"mbt", "[", "z", "]"}], ",", "\"\<_\>\""}], "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", "0", ",", "1"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_Subscript", "+", "2"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{
       RowBox[{"OverscriptBox", "[", 
        RowBox[{
         RowBox[{"z", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         ",", "\"\<_\>\""}], "]"}], ",", 
       RowBox[{"z", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
      "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", "0", ",", "1"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFI", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", "ur__", "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "ur"}], "}"}], "]"}]}], "]"}], ",", "\n", 
        "\t\t", 
        RowBox[{"TFI", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", "ur", "}"}], ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFI", "[", 
      RowBox[{"d_", ",", "pp_", ",", "dp_", ",", "any__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<F\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"",
               ",", "\"\< \>\"", ",", 
              RowBox[{"Apply", "[", 
               RowBox[{"Sequence", ",", 
                RowBox[{"Flatten", "[", 
                 RowBox[{"{", "any", "}"}], "]"}]}], "]"}]}], "}"}], "]"}]}], 
          "]"}], ",", "\n", "\t\t", 
         RowBox[{"TFI", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", "any", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}]}], "/;", 
    RowBox[{
     RowBox[{"Head", "[", "dp", "]"}], "=!=", "List"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFI", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{
               RowBox[{"First", "[", 
                RowBox[{"{", "dpp", "}"}], "]"}], ",", "TraditionalForm"}], 
              "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
        "\t\t", 
        RowBox[{"TFI", "[", 
         RowBox[{"dpp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TVI", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n", 
   RowBox[{
    RowBox[{"(", "\t", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"InterpretationBox", "@@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"SubsuperscriptBox", "[", 
           RowBox[{
            RowBox[{"StyleBox", "[", 
             RowBox[{"\"\<V\>\"", ",", 
              RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
              RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
            RowBox[{"RowBox", "@@", 
             RowBox[{"{", 
              RowBox[{"(", 
               RowBox[{"redblue", "/@", 
                RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
            RowBox[{"RowBox", "[", 
             RowBox[{"{", 
              RowBox[{"\"\<(\>\"", ",", 
               RowBox[{"ToBoxes", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"{", "dpp", "}"}], "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
                "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
          "\t", 
          RowBox[{
           RowBox[{"Hold", "[", "TVI", "]"}], "[", 
           RowBox[{"dpp", ",", 
            RowBox[{"{", "den", "}"}]}], "]"}], ",", 
          RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
      RowBox[{
       RowBox[{"Hold", "[", "TVI", "]"}], "->", "TVI"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
      "}"}]}], "]"}], ",", 
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "+", "1"}], ",", "\[Beta]", ",", "\[Gamma]", ",", 
       "\[Delta]"}], "}"}]}], "]"}], ",", 
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "-", "1"}], ",", "\[Beta]", ",", "\[Gamma]", ",", 
       "\[Delta]"}], "}"}]}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"TJI", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJI", "[", 
     RowBox[{"d_", ",", "0", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", "\n", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"InterpretationBox", "@@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"SubsuperscriptBox", "[", 
            RowBox[{
             RowBox[{"StyleBox", "[", 
              RowBox[{"\"\<K\>\"", ",", 
               RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
               RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
             RowBox[{"RowBox", "@@", 
              RowBox[{"{", 
               RowBox[{"(", 
                RowBox[{"redblue", "/@", 
                 RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
             RowBox[{"RowBox", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<(\>\"", ",", 
                RowBox[{"ToBoxes", "[", 
                 RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
                "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
           RowBox[{
            RowBox[{"Hold", "[", "TJI", "]"}], "[", 
            RowBox[{"d", ",", "0", ",", 
             RowBox[{"{", "den", "}"}]}], "]"}], ",", 
           RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
       RowBox[{
        RowBox[{"Hold", "[", "TJI", "]"}], "->", "TJI"}]}], ")"}], "/;", 
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}]}], "\n", "\t", ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"TJI", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJI", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<J\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", "dpp", "}"}], "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
               "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
         "\t\t", 
         RowBox[{
          RowBox[{"Hold", "[", "TJI", "]"}], "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
     RowBox[{
      RowBox[{"Hold", "[", "TJI", "]"}], "->", "TJI"}]}], ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TKI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TKI", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"SubsuperscriptBox", "[", 
      RowBox[{
       RowBox[{"StyleBox", "[", 
        RowBox[{"\"\<K\>\"", ",", 
         RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
         RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
       RowBox[{"RowBox", "@@", 
        RowBox[{"{", 
         RowBox[{"(", 
          RowBox[{"redblue", "/@", 
           RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
       RowBox[{"RowBox", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<(\>\"", ",", 
          RowBox[{"ToBoxes", "[", 
           RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\""}], 
         "}"}], "]"}]}], "]"}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"TKI", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TKI", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<K\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", "dpp", "}"}], "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
               "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
         "\t\t", 
         RowBox[{
          RowBox[{"Hold", "[", "TKI", "]"}], "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
     RowBox[{
      RowBox[{"Hold", "[", "TKI", "]"}], "->", "TKI"}]}], ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"TKI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]"}], "}"}]}], "]"}], 
   ",", 
   RowBox[{"TKI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "+", "1"}], ",", "\[Beta]", ",", "\[Gamma]"}], 
      "}"}]}], "]"}], ",", 
   RowBox[{"TKI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "-", "1"}], ",", "\[Beta]", ",", "\[Gamma]"}], 
      "}"}]}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBI", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"SubsuperscriptBox", "[", 
      RowBox[{
       RowBox[{"StyleBox", "[", 
        RowBox[{"\"\<B\>\"", ",", 
         RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
         RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
       RowBox[{"RowBox", "@@", 
        RowBox[{"{", 
         RowBox[{"(", 
          RowBox[{"redblue", "/@", 
           RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
       RowBox[{"RowBox", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<(\>\"", ",", 
          RowBox[{"ToBoxes", "[", 
           RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\""}], 
         "}"}], "]"}]}], "]"}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBI", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<B\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TBI", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "}"}]}], "]"}], ",", 
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "+", "1"}], ",", "\[Beta]"}], "}"}]}], "]"}], ",", 
   
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "-", "1"}], ",", "\[Beta]"}], "}"}]}], "]"}]}], 
  "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TAI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TAI", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<A\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TAI", "[", 
         RowBox[{"d", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", "\[Alpha]", "}"}]}], "]"}], ",", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", "+", "1"}], "}"}]}], "]"}], ",", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", "-", "1"}], "}"}]}], "]"}]}], "}"}]], "Input"],

Cell["\<\
T*IC are just like T*I , but used only for commentary in the 
recursion (TComment).\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFIC", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "i_", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "i", ",", "\"\<0\>\""}], "}"}], "]"}]}], 
         "]"}], ",", "\n", "\t\t", 
        RowBox[{"TFIC", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "i", ",", "0", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFIC", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "i_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "\"\<0\>\"", ",", "i"}], "}"}], "]"}]}], 
         "]"}], ",", "\n", "\t\t", 
        RowBox[{"TFIC", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "i", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFIC", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{
               RowBox[{"First", "[", 
                RowBox[{"{", "dpp", "}"}], "]"}], ",", "TraditionalForm"}], 
              "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
        "\t\t", 
        RowBox[{"TFIC", "[", 
         RowBox[{"dpp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TVIC", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n", 
   RowBox[{
    RowBox[{"(", "\t", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"InterpretationBox", "@@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"SubsuperscriptBox", "[", 
           RowBox[{
            RowBox[{"StyleBox", "[", 
             RowBox[{"\"\<V\>\"", ",", 
              RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
              RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
            RowBox[{"RowBox", "@@", 
             RowBox[{"{", 
              RowBox[{"(", 
               RowBox[{"redblue", "/@", 
                RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
            RowBox[{"RowBox", "[", 
             RowBox[{"{", 
              RowBox[{"\"\<(\>\"", ",", 
               RowBox[{"ToBoxes", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"{", "dpp", "}"}], "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
                "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
          "\t", 
          RowBox[{
           RowBox[{"Hold", "[", "TVIC", "]"}], "[", 
           RowBox[{"dpp", ",", 
            RowBox[{"{", "den", "}"}]}], "]"}], ",", 
          RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
      RowBox[{
       RowBox[{"Hold", "[", "TVIC", "]"}], "->", "TVIC"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"TJIC", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJIC", "[", 
     RowBox[{"d_", ",", "0", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", "\n", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"InterpretationBox", "@@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"SubsuperscriptBox", "[", 
            RowBox[{
             RowBox[{"StyleBox", "[", 
              RowBox[{"\"\<K\>\"", ",", 
               RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
               RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
             RowBox[{"RowBox", "@@", 
              RowBox[{"{", 
               RowBox[{"(", 
                RowBox[{"redblue", "/@", 
                 RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
             RowBox[{"RowBox", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<(\>\"", ",", 
                RowBox[{"ToBoxes", "[", 
                 RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
                "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
           RowBox[{
            RowBox[{"Hold", "[", "TJIC", "]"}], "[", 
            RowBox[{"d", ",", "0", ",", 
             RowBox[{"{", "den", "}"}]}], "]"}], ",", 
           RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
       RowBox[{
        RowBox[{"Hold", "[", "TJIC", "]"}], "->", "TJIC"}]}], ")"}], "/;", 
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}]}], "\n", "\t", ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"TJIC", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJIC", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<J\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", "dpp", "}"}], "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
               "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
         "\t\t", 
         RowBox[{
          RowBox[{"Hold", "[", "TJIC", "]"}], "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
     RowBox[{
      RowBox[{"Hold", "[", "TJIC", "]"}], "->", "TJIC"}]}], ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBIC", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"SubsuperscriptBox", "[", 
      RowBox[{
       RowBox[{"StyleBox", "[", 
        RowBox[{"\"\<B\>\"", ",", 
         RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
         RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
       RowBox[{"RowBox", "@@", 
        RowBox[{"{", 
         RowBox[{"(", 
          RowBox[{"redblue", "/@", 
           RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
       RowBox[{"RowBox", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<(\>\"", ",", 
          RowBox[{"ToBoxes", "[", 
           RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\""}], 
         "}"}], "]"}]}], "]"}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBIC", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<B\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TBIC", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TAIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TAIC", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<A\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TAIC", "[", 
         RowBox[{"d", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFR", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "i_", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "i", ",", "\"\<0\>\""}], "}"}], "]"}]}], 
         "]"}], ",", "\n", "\t\t", 
        RowBox[{"TFR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "i", ",", "0", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFR", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "i_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "\"\<0\>\"", ",", "i"}], "}"}], "]"}]}], 
         "]"}], ",", "\n", "\t\t", 
        RowBox[{"TFR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "i", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFR", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{
               RowBox[{"First", "[", 
                RowBox[{"{", "dpp", "}"}], "]"}], ",", "TraditionalForm"}], 
              "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
        "\t\t", 
        RowBox[{"TFR", "[", 
         RowBox[{"dpp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TVR", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n", 
   RowBox[{
    RowBox[{"(", "\t", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"InterpretationBox", "@@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"SubsuperscriptBox", "[", 
           RowBox[{
            RowBox[{"StyleBox", "[", 
             RowBox[{"\"\<V\>\"", ",", 
              RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
              RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
            RowBox[{"RowBox", "@@", 
             RowBox[{"{", 
              RowBox[{"(", 
               RowBox[{"redblue", "/@", 
                RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
            RowBox[{"RowBox", "[", 
             RowBox[{"{", 
              RowBox[{"\"\<(\>\"", ",", 
               RowBox[{"ToBoxes", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"{", "dpp", "}"}], "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
                "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
          "\t", 
          RowBox[{
           RowBox[{"Hold", "[", "TVR", "]"}], "[", 
           RowBox[{"dpp", ",", 
            RowBox[{"{", "den", "}"}]}], "]"}], ",", 
          RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
      RowBox[{
       RowBox[{"Hold", "[", "TVR", "]"}], "->", "TVR"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"TJR", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", "0", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", "\n", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"InterpretationBox", "@@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"SubsuperscriptBox", "[", 
            RowBox[{
             RowBox[{"StyleBox", "[", 
              RowBox[{"\"\<K\>\"", ",", 
               RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
               RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
             RowBox[{"RowBox", "@@", 
              RowBox[{"{", 
               RowBox[{"(", 
                RowBox[{"redblue", "/@", 
                 RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
             RowBox[{"RowBox", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<(\>\"", ",", 
                RowBox[{"ToBoxes", "[", 
                 RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
                "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
           RowBox[{
            RowBox[{"Hold", "[", "TJR", "]"}], "[", 
            RowBox[{"d", ",", "0", ",", 
             RowBox[{"{", "den", "}"}]}], "]"}], ",", 
           RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
       RowBox[{
        RowBox[{"Hold", "[", "TJR", "]"}], "->", "TJR"}]}], ")"}], "/;", 
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}]}], "\n", "\t", ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"TJR", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<J\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", "dpp", "}"}], "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
               "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
         "\t\t", 
         RowBox[{
          RowBox[{"Hold", "[", "TJR", "]"}], "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
     RowBox[{
      RowBox[{"Hold", "[", "TJR", "]"}], "->", "TJR"}]}], ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBR", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"SubsuperscriptBox", "[", 
      RowBox[{
       RowBox[{"StyleBox", "[", 
        RowBox[{"\"\<B\>\"", ",", 
         RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
         RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
       RowBox[{"RowBox", "@@", 
        RowBox[{"{", 
         RowBox[{"(", 
          RowBox[{"redblue", "/@", 
           RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
       RowBox[{"RowBox", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<(\>\"", ",", 
          RowBox[{"ToBoxes", "[", 
           RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\""}], 
         "}"}], "]"}]}], "]"}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBR", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<B\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TBR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TAR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TAR", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<A\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TAR", "[", 
         RowBox[{"d", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n1", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n1", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n2", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n2", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "2"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n3", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n3", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "3"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n4", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n4", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "4"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n5", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n5", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "5"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"ParD", "[", "i_", "]"}], "j_"], ",", "TraditionalForm"}], 
    "]"}], ":=", 
   RowBox[{"SubsuperscriptBox", "[", 
    RowBox[{"\"\<\[PartialD]\>\"", ",", "i", ",", "j"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"ParD", "[", "i_", "]"}], ",", "TraditionalForm"}], "]"}], "^=", 
   
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[PartialD]\>\"", ",", "i"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"DPlus", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"SuperscriptBox", "[", 
      RowBox[{"\"\<d\>\"", ",", "\"\<+\>\""}], "]"}], ",", 
     RowBox[{"FontWeight", "\[Rule]", "\"\<Bold\>\""}], ",", 
     RowBox[{"SingleLetterItalics", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TpD", "[", 
     RowBox[{"_", ",", "b_"}], "]"}], ",", "TraditionalForm"}], "]"}], ":=", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{"b", ",", "TraditionalForm"}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Palettes", "Subsection",
 CellTags->"T1.3.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$PutPalettes", "===", "True"}], ",", "\n", "\t", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"tobut", "[", "a_", "]"}], " ", ":=", " ", 
                 RowBox[{"ButtonBox", "[", 
                  RowBox[{
                   RowBox[{"ToBoxes", "[", 
                    RowBox[{"a", ",", "StandardForm"}], "]"}], ",", " ", 
                   RowBox[{"ButtonStyle", "->", "\"\<Paste\>\""}]}], "]"}]}], 
                ";"}], "\n", " ", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Head", "[", "pal1", "]"}], "===", 
                  "NotebookObject"}], ",", "\n", "\t", 
                 RowBox[{"NotebookClose", "[", "pal1", "]"}]}], "]"}]}], 
              ";"}], "\n", "\t", "pal1"}], "=", 
            RowBox[{"NotebookPut", "[", 
             RowBox[{"Notebook", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"Cell", "[", 
                 RowBox[{
                  RowBox[{"BoxData", "[", 
                   RowBox[{"GridBox", "[", "\n", "\t\t\t\t\t\t\t", 
                    RowBox[{
                    RowBox[{"Join", "[", 
                    RowBox[{
                    RowBox[{"Transpose", "@", 
                    RowBox[{"{", "\n", "\t\t\t\t\t\t\t\t\t", 
                    RowBox[{
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"tobut", "@", 
                    SuperscriptBox[
                    SubscriptBox["m", "j"], "2"]}], ",", 
                    RowBox[{"{", 
                    RowBox[{"j", ",", "5"}], "}"}]}], "]"}], ",", "\n", 
                    "\t\t\t\t\t\t\t\t\t", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"tobut", "@", 
                    SubscriptBox["\[Nu]", "j"]}], ",", 
                    RowBox[{"{", 
                    RowBox[{"j", ",", "5"}], "}"}]}], "]"}], ",", "\n", 
                    "\t\t\t\t\t\t\t\t\t", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"tobut", "@", 
                    RowBox[{"Schiebe", "[", 
                    RowBox[{"i", ",", "\"\<+\>\""}], "]"}]}], ",", 
                    RowBox[{"{", 
                    RowBox[{"i", ",", "5"}], "}"}]}], "]"}], ",", "\n", 
                    "\t                ", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"tobut", "@", 
                    RowBox[{"Schiebe", "[", 
                    RowBox[{"i", ",", "\"\<-\>\""}], "]"}]}], ",", 
                    RowBox[{"{", 
                    RowBox[{"i", ",", "5"}], "}"}]}], "]"}]}], "}"}]}], ",", 
                    "\n", "\t\t\t\t\t\t\t", 
                    RowBox[{"{", "\t", 
                    RowBox[{"tobut", "/@", 
                    RowBox[{"{", 
                    RowBox[{"PP", ",", 
                    SubscriptBox["\[CapitalDelta]", "\[Placeholder]"], ",", 
                    SubscriptBox["u", "\[Placeholder]"], ",", 
                    "\[ScriptCapitalC]"}], " ", "}"}]}], "}"}]}], "\n", 
                    "\t\t\t\t\t\t\t", "]"}], ",", 
                    RowBox[{"RowSpacings", "->", "0"}], ",", "\n", "      ", 
                    RowBox[{"ColumnSpacings", "->", "0"}]}], "]"}], "]"}], 
                  ",", "NotebookDefault"}], "]"}], "}"}], ",", "\n", "\t", 
               RowBox[{"Editable", "->", "True"}], ",", "\n", 
               RowBox[{"WindowToolbars", "->", 
                RowBox[{"{", "}"}]}], ",", "\n", 
               RowBox[{"WindowSize", "->", 
                RowBox[{"{", 
                 RowBox[{"Fit", ",", " ", "Fit"}], "}"}]}], ",", "\n", 
               RowBox[{"WindowMargins", "->", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", " ", "Automatic"}], "}"}], ",", " ", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
               "\n", 
               RowBox[{"WindowFrame", "->", "\"\<Palette\>\""}], ",", "\n", 
               RowBox[{"WindowElements", "->", 
                RowBox[{"{", "}"}]}], ",", "\n", 
               RowBox[{"WindowFrameElements", "->", "\"\<CloseBox\>\""}], ",",
                "\n", 
               RowBox[{"WindowClickSelect", "->", "False"}], ",", "\n", 
               RowBox[{"ScrollingOptions", "->", 
                RowBox[{"{", 
                 RowBox[{"\"\<PagewiseScrolling\>\"", "->", "True"}], "}"}]}],
                ",", "\n", 
               RowBox[{"ShowCellBracket", "->", "False"}], ",", "\n", 
               RowBox[{"CellMargins", "->", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", " ", "0"}], "}"}], ",", " ", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", " ", "0"}], "}"}]}], "}"}]}], ",", "\n", 
               
               RowBox[{"Active", "->", "True"}], ",", "\n", 
               RowBox[{"CellOpen", "->", "True"}], ",", "\n", 
               RowBox[{"ShowCellLabel", "->", "False"}], ",", "\n", 
               RowBox[{"ShowCellTags", "->", "False"}], ",", "\n", 
               RowBox[{"ImageMargins", "->", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", " ", "0"}], "}"}], ",", " ", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", " ", "0"}], "}"}]}], "}"}]}], ",", "\n", 
               
               RowBox[{"Magnification", "->", "2"}], ",", 
               RowBox[{"Visible", "->", "True"}]}], "]"}], "]"}]}], ";", "\n", 
           RowBox[{"SetOptions", "[", 
            RowBox[{"pal1", ",", 
             RowBox[{"WindowMargins", "->", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "Automatic"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"Automatic", ",", "0"}], "}"}]}], "}"}]}]}], "]"}], 
           ";", "\n", 
           RowBox[{"SetOptions", "[", 
            RowBox[{"pal1", ",", 
             RowBox[{"Visible", "->", "True"}], ",", 
             RowBox[{"WindowSize", "->", 
              RowBox[{"{", 
               RowBox[{"Fit", ",", "Fit"}], "}"}]}]}], "]"}], ";"}], "\n", 
          " ", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Head", "[", "pal2", "]"}], "===", "NotebookObject"}], 
            ",", "\n", "\t", 
            RowBox[{"NotebookClose", "[", "pal2", "]"}]}], "]"}]}], ";"}], 
        "\n", "\t", "pal2"}], "=", 
       RowBox[{"NotebookPut", "[", 
        RowBox[{"Notebook", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Cell", "[", 
            RowBox[{
             RowBox[{"BoxData", "[", 
              RowBox[{"GridBox", "[", "\n", "\t\t\t\t\t\t\t", 
               RowBox[{
                RowBox[{"JJJ", "=", 
                 RowBox[{"Join", "[", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"tobut", "/@", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"TFI", "[", 
                    RowBox[{
                    RowBox[{"d", "+", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"], ",", 
                    SubscriptBox["\[Nu]", "4"], ",", 
                    SubscriptBox["\[Nu]", "5"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TVI", "[", 
                    RowBox[{
                    RowBox[{"d", "+", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"], ",", 
                    SubscriptBox["\[Nu]", "4"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TJI", "[", 
                    RowBox[{
                    RowBox[{"d", "+", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TKI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TBI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TAI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    SubscriptBox["\[Nu]", "1"], "}"}]}], "]"}]}], "}"}]}], 
                    "}"}], ",", "\n", "\t\t\t\t\t\t\t\t", 
                   RowBox[{"{", 
                    RowBox[{"tobut", "/@", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"TFI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"], ",", 
                    SubscriptBox["\[Nu]", "4"], ",", 
                    SubscriptBox["\[Nu]", "5"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TVI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"], ",", 
                    SubscriptBox["\[Nu]", "4"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TJI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TKI", "[", 
                    RowBox[{
                    RowBox[{"d", "-", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TBI", "[", 
                    RowBox[{
                    RowBox[{"d", "-", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TAI", "[", 
                    RowBox[{
                    RowBox[{"d", "-", "2"}], ",", 
                    RowBox[{"{", 
                    SubscriptBox["\[Nu]", "1"], "}"}]}], "]"}]}], "}"}]}], 
                    "}"}]}], "\n", "\t\t\t\t\t\t\t", "]"}]}], ",", 
                RowBox[{"RowSpacings", "->", "0"}], ",", "\n", "      ", 
                RowBox[{"ColumnSpacings", "->", "0"}]}], "]"}], "]"}], ",", 
             "NotebookDefault"}], "]"}], "}"}], ",", "\n", "\t", 
          RowBox[{"Editable", "->", "True"}], ",", "\n", 
          RowBox[{"WindowToolbars", "->", 
           RowBox[{"{", "}"}]}], ",", "\n", 
          RowBox[{"WindowSize", "->", 
           RowBox[{"{", 
            RowBox[{"FitAll", ",", " ", "FitAll"}], "}"}]}], ",", "\n", 
          RowBox[{"WindowMargins", "->", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "Automatic"}], "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"0", ",", "Automatic"}], "}"}]}], "}"}]}], ",", "\n", 
          RowBox[{"WindowFrame", "->", "\"\<Palette\>\""}], ",", "\n", 
          RowBox[{"WindowElements", "->", 
           RowBox[{"{", "}"}]}], ",", "\n", 
          RowBox[{"WindowFrameElements", "->", "\"\<CloseBox\>\""}], ",", 
          "\n", 
          RowBox[{"WindowClickSelect", "->", "False"}], ",", "\n", 
          RowBox[{"ScrollingOptions", "->", 
           RowBox[{"{", 
            RowBox[{"\"\<PagewiseScrolling\>\"", "->", "True"}], "}"}]}], ",",
           "\n", 
          RowBox[{"ShowCellBracket", "->", "False"}], ",", "\n", 
          RowBox[{"CellMargins", "->", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ",", "\n", 
          RowBox[{"Active", "->", "True"}], ",", "\n", 
          RowBox[{"CellOpen", "->", "True"}], ",", "\n", 
          RowBox[{"ShowCellLabel", "->", "False"}], ",", "\n", 
          RowBox[{"ShowCellTags", "->", "False"}], ",", "\n", 
          RowBox[{"ImageMargins", "->", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"0", ",", " ", "0"}], "}"}]}], "}"}]}], ",", "\n", 
          RowBox[{"Magnification", "->", "2"}], ",", 
          RowBox[{"Visible", "->", "True"}]}], "]"}], "]"}]}], ";", "\n", 
      RowBox[{"SetOptions", "[", 
       RowBox[{"pal2", ",", 
        RowBox[{"WindowMargins", "->", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"Automatic", ",", "15"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"Automatic", ",", "4"}], "}"}]}], "}"}]}]}], "]"}], ";", 
      "\n", 
      RowBox[{"SetOptions", "[", 
       RowBox[{"pal2", ",", 
        RowBox[{"Visible", "->", "True"}], ",", 
        RowBox[{"WindowSize", "->", 
         RowBox[{"{", 
          RowBox[{"Fit", ",", "Fit"}], "}"}]}]}], "]"}], ";"}], "\n", "\t", 
     RowBox[{"SetOptions", "[", 
      RowBox[{"pal2", ",", 
       RowBox[{"WindowSize", "->", 
        RowBox[{"{", 
         RowBox[{"Fit", ",", "Fit"}], "}"}]}]}], "]"}]}]}], "\n", "\t", "]"}],
   ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["OperatorApply", "Subsection",
 CellTags->"T1.3.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyF", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TFI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TFI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TFI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TFI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TFI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyV", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TVI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TVI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TVI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TVI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TVI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyJ", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TJI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TJI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TJI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TJI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TJI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyK", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TKI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TKI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TKI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TKI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TKI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyK", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TKI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TKI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TKI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TKI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TKI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyB", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TBI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TBI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TBI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TBI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TBI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyA", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TAI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TAI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TAI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TAI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TAI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MassDerivative", "Subsection",
 CellTags->"T1.3.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MassDerivative", "[", 
    RowBox[{
     RowBox[{"STLI", "[", 
      RowBox[{"any__", ",", 
       RowBox[{"{", "\n", "       ", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"al_", ",", "m1_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"be_", ",", "m2_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"ga_", ",", "m3_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"de_", ",", "m4_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"ep_", ",", "m5_"}], "}"}]}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "d1_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "d2_"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"0", ",", "d3_"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"0", ",", "d4_"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"0", ",", "d5_"}], "}"}]}], "}"}]}], "]"}], 
   StyleBox[" ",
    FontWeight->"Plain"], 
   StyleBox[":=",
    FontWeight->"Plain"], "\n", 
   RowBox[{
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"Pochhammer", "[", 
        RowBox[{"al", ",", "d1"}], "]"}], " ", 
       RowBox[{"Pochhammer", "[", 
        RowBox[{"be", ",", "d2"}], "]"}], " ", 
       RowBox[{"Pochhammer", "[", 
        RowBox[{"ga", ",", "d3"}], "]"}]}], "*", "\n", "  ", 
      RowBox[{"Pochhammer", "[", 
       RowBox[{"de", ",", "d4"}], "]"}], " ", 
      RowBox[{"Pochhammer", "[", 
       RowBox[{"ep", ",", "d5"}], "]"}]}], " ", ")"}], " ", "*", "\n", " ", 
    RowBox[{"STLI", "[", 
     RowBox[{"any", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"al", "+", "d1"}], ",", "m1"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"be", "+", "d2"}], ",", "m2"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"ga", "+", "d3"}], ",", "m3"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"de", "+", "d4"}], ",", "m4"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"ep", "+", "d5"}], ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}],
   ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarasovT", "Subsection",
 CellTags->"T1.3.5.1"],

Cell["\<\
TarasovT[abrs__, qq_ /; Head[qq] =!= List] := 
  TarasovT[abrs, qq, {\"g\", \"g\", \"g\", \"g\", \"g\"}]\
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarasovT", "[", 
    RowBox[{"r_", ",", " ", "s_", ",", " ", "qq_", ",", " ", 
     RowBox[{"{", 
      RowBox[{
      "alp1_", ",", "alp2_", ",", " ", "alp3_", ",", "alp4_", ",", "alp5_"}], 
      "}"}]}], "]"}], " ", ":=", "\n", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", " ", 
      RowBox[{
      "al1", ",", "al2", ",", "al3", ",", "al4", ",", "al5", ",", "alrul", 
       ",", "new", ",", "Q1", ",", "Q11", ",", "Q12", ",", "Q2", ",", "Q22", 
       ",", "dum1", ",", "dum2"}], "}"}], ",", "\n", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp1", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al1", " ", "=", " ", "0"}]}], "]"}], ";", " ", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp2", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al2", " ", "=", " ", "0"}]}], "]"}], ";"}], "\n",
                     " ", 
                    RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp3", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al3", " ", "=", " ", "0"}]}], "]"}], ";"}], "\n", 
                    RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp4", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al4", " ", "=", " ", "0"}]}], "]"}], ";", " ", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp5", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al5", " ", "=", " ", "0"}]}], "]"}], ";"}], "\n",
                     " ", "alrul"}], " ", "=", " ", 
                    RowBox[{"Select", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"al1", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "1", "]"}]}]}], ",", " ", 
                    RowBox[{"al2", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "2", "]"}]}]}], ",", " ", 
                    RowBox[{"al3", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "3", "]"}]}]}], ",", "\n", 
                    "                 ", 
                    RowBox[{"al4", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "4", "]"}]}]}], ",", " ", 
                    RowBox[{"al5", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "5", "]"}]}]}]}], "}"}], ",", " ", 
                    RowBox[{
                    RowBox[{"!", 
                    RowBox[{"MatchQ", "[", 
                    RowBox[{"#", ",", " ", 
                    RowBox[{"0", ":>", "_"}]}], "]"}]}], "&"}]}], "]"}]}], 
                    ";"}], "\n", " ", "Q1"}], " ", "=", " ", 
                    RowBox[{
                    RowBox[{"al3", " ", "al5"}], " ", "+", " ", 
                    RowBox[{"al4", " ", "al5"}], " ", "+", " ", 
                    RowBox[{"al2", " ", "al3"}], " ", "+", " ", 
                    RowBox[{"al3", " ", "al4"}]}]}], ";"}], "\n", " ", "Q2"}],
                    " ", "=", " ", 
                   RowBox[{
                    RowBox[{"al4", " ", "al5"}], " ", "+", " ", 
                    RowBox[{"al3", " ", "al5"}], " ", "+", " ", 
                    RowBox[{"al1", " ", "al4"}], " ", "+", " ", 
                    RowBox[{"al3", " ", "al4"}]}]}], ";"}], "\n", " ", 
                 "Q11"}], " ", "=", 
                RowBox[{
                 RowBox[{"1", "/", 
                  RowBox[{"(", 
                   RowBox[{"-", "4"}], ")"}]}], " ", 
                 RowBox[{"(", " ", 
                  RowBox[{"al2", " ", "+", " ", "al4", " ", "+", " ", "al5"}],
                   " ", ")"}]}]}], ";"}], "\n", " ", "Q22"}], " ", "=", 
             RowBox[{
              RowBox[{"1", "/", 
               RowBox[{"(", 
                RowBox[{"-", "4"}], ")"}]}], " ", 
              RowBox[{"(", " ", 
               RowBox[{"al1", " ", "+", " ", "al3", " ", "+", " ", "al5"}], 
               " ", ")"}]}]}], ";"}], "\n", " ", "Q12"}], " ", "=", 
          RowBox[{
           RowBox[{"1", "/", 
            RowBox[{"(", 
             RowBox[{"-", "2"}], ")"}]}], " ", "al5"}]}], ";"}], "\n", "\t\t",
         "new"}], "=", "\n", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"I", "^", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "r"}], "-", "s"}], ")"}]}], 
         RowBox[{"(", " ", 
          RowBox[{"D", "[", " ", 
           RowBox[{
            RowBox[{"Exp", "[", " ", 
             RowBox[{"I", " ", "qq", " ", 
              RowBox[{"(", "\n", "    ", 
               RowBox[{
                RowBox[{"be1", " ", "Q1"}], " ", "+", " ", 
                RowBox[{"be2", " ", "Q2"}], " ", "+", 
                RowBox[{
                 RowBox[{"be1", "^", "2"}], " ", "Q11"}], " ", "+", 
                RowBox[{
                 RowBox[{"be2", "^", "2"}], " ", "Q22"}], " ", "+", " ", 
                RowBox[{"be1", " ", "be2", " ", "Q12"}]}], "\n", 
               "                                 ", ")"}], " ", "rho"}], " ", 
             "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"be1", ",", "r"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"be2", ",", "s"}], "}"}]}], "\n", "\t\t\t\t", "]"}], " ",
           ")"}]}], "/.", "\n", " ", 
        RowBox[{"(", " ", 
         RowBox[{"Join", "[", 
          RowBox[{"alrul", ",", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"be1", " ", ":>", " ", "0"}], ",", 
             RowBox[{"be2", " ", ":>", " ", "0"}], ",", " ", 
             RowBox[{"rho", " ", ":>", " ", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"Pi", "^", "2"}]}], "/", 
                RowBox[{"Pi", "^", "2"}]}], " ", "DPlus"}]}]}], " ", "}"}]}], 
          "]"}], ")"}]}]}], ";", 
      RowBox[{
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{"#", ",", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"FreeQ", "[", 
                  RowBox[{"#", ",", "ParD"}], "]"}], "&&", 
                 RowBox[{"FreeQ", "[", 
                  RowBox[{"#", ",", "DPlus"}], "]"}]}], ")"}], "&"}]}], "]"}],
             " ", 
            RowBox[{"TpD", "[", 
             RowBox[{"qq", ",", 
              RowBox[{"Select", "[", 
               RowBox[{"#", ",", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"!", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "ParD"}], "]"}], "&&", 
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "DPlus"}], "]"}]}], ")"}]}], ")"}], 
                 "&"}]}], "]"}]}], "]"}]}], ")"}], "&"}], ",", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Expand", "[", 
            RowBox[{"new", ",", "ParD"}], "]"}], "+", " ", 
           RowBox[{"dum1", " ", "dum2"}]}], ")"}]}], "\n", "\t\t", "]"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"dum1", ":>", "0"}], ",", 
         RowBox[{"dum2", ":>", "0"}]}], "}"}]}]}]}], "\n", "\t\t", "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{"TarasovT", "[", 
    RowBox[{"r_", ",", " ", "s_", ",", " ", "pp_", ",", " ", 
     RowBox[{"dp_", "/;", 
      RowBox[{
       RowBox[{"Head", "[", "dp", "]"}], "=!=", "List"}]}]}], "]"}], " ", ":=",
    " ", 
   RowBox[{"TarasovT", "[", 
    RowBox[{"r", ",", "s", ",", "pp", ",", "dp", ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<g\>\"", ",", "\"\<g\>\"", ",", "\"\<g\>\"", ",", "\"\<g\>\"", ",", 
       "\"\<g\>\""}], "}"}]}], "]"}]}]}]], "Input"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"\[CapitalDelta]", " ", 
      SubscriptBox["q", "1"]}], ")"}], "a"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"\[CapitalDelta]", " ", 
      SubscriptBox["q", "2"]}], ")"}], "b"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"p", " ", 
      SubscriptBox["q", "1"]}], ")"}], "r"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"p", " ", 
      SubscriptBox["q", "2"]}], ")"}], "s"], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{
    RowBox[{"TarasovT", "[", 
     RowBox[{
     "a_", ",", "b_", ",", "r_", ",", "s_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{
       "alp1_", ",", "alp2_", ",", "alp3_", ",", "alp4_", ",", "alp5_"}], 
       "}"}]}], "]"}], ":=", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "al1", ",", "al2", ",", "al3", ",", "al4", ",", "al5", ",", "alrul", 
        ",", "new", ",", "Q1", ",", "Q11", ",", "Q12", ",", "Q2", ",", "Q22", 
        ",", "dum1", ",", "dum2"}], "}"}], ",", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp1", "===", "0"}], ",", 
         RowBox[{"al1", "=", "0"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp2", "===", "0"}], ",", 
         RowBox[{"al2", "=", "0"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp3", "===", "0"}], ",", 
         RowBox[{"al3", "=", "0"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp4", "===", "0"}], ",", 
         RowBox[{"al4", "=", "0"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp5", "===", "0"}], ",", 
         RowBox[{"al5", "=", "0"}]}], "]"}], ";", 
       RowBox[{"alrul", "=", 
        RowBox[{"Select", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"al1", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "1", "]"}]}]}], ",", 
            RowBox[{"al2", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "2", "]"}]}]}], ",", 
            RowBox[{"al3", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "3", "]"}]}]}], ",", 
            RowBox[{"al4", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "4", "]"}]}]}], ",", 
            RowBox[{"al5", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "5", "]"}]}]}]}], "}"}], ",", 
          "\[InvisibleSpace]", 
          RowBox[{
           RowBox[{"!", 
            RowBox[{"(", 
             RowBox[{"MatchQ", "[", 
              RowBox[{"#1", ",", 
               RowBox[{"0", "\[RuleDelayed]", "_"}]}], "]"}], ")"}]}], 
           "&"}]}], "]"}]}], ";", 
       RowBox[{"Q1", "=", 
        RowBox[{
         RowBox[{"al3", " ", "al5"}], "+", 
         RowBox[{"al4", " ", "al5"}], "+", 
         RowBox[{"al2", " ", "al3"}], "+", 
         RowBox[{"al3", " ", "al4"}]}]}], ";", 
       RowBox[{"Q2", "=", 
        RowBox[{
         RowBox[{"al4", " ", "al5"}], "+", 
         RowBox[{"al3", " ", "al5"}], "+", 
         RowBox[{"al1", " ", "al4"}], "+", 
         RowBox[{"al3", " ", "al4"}]}]}], ";", 
       RowBox[{"Q11", "=", 
        RowBox[{
         RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
         RowBox[{"(", 
          RowBox[{"al2", "+", "al4", "+", "al5"}], ")"}]}]}], ";", 
       RowBox[{"Q22", "=", 
        RowBox[{
         RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
         RowBox[{"(", 
          RowBox[{"al1", "+", "al3", "+", "al5"}], ")"}]}]}], ";", 
       RowBox[{"Q12", "=", 
        RowBox[{"-", 
         FractionBox["al5", "2"]}]}], ";", 
       RowBox[{"new", "=", 
        RowBox[{
         RowBox[{
          SuperscriptBox["I", 
           RowBox[{
            RowBox[{"-", "r"}], "-", "s", "-", "a", "-", "b"}]], " ", 
          RowBox[{
           SubscriptBox["\[PartialD]", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"be1", ",", "r"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"be2", ",", "s"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ga1", ",", "a"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ga2", ",", "b"}], "}"}]}]], 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be1", " ", "pp"}], "+", 
                  RowBox[{"ga1", " ", "dp"}]}], ")"}], " ", "Q1"}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be2", " ", "pp"}], "+", 
                  RowBox[{"ga2", " ", "dp"}]}], ")"}], " ", "Q2"}], "+", 
               RowBox[{"be1", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be1", " ", "pp"}], "+", 
                  RowBox[{"2", " ", "ga1", " ", "dp"}]}], ")"}], " ", "Q11"}],
                "+", 
               RowBox[{"be2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be2", " ", "pp"}], "+", 
                  RowBox[{"2", " ", "ga2", " ", "dp"}]}], ")"}], " ", "Q22"}],
                "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be1", " ", "be2", " ", "pp"}], "+", 
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"be1", " ", "ga2"}], "+", 
                    RowBox[{"be2", " ", "ga1"}]}], ")"}], " ", "dp"}]}], 
                 ")"}], " ", "Q12"}]}], ")"}], " ", "rho"}], "]"}]}]}], "/.", 
         
         RowBox[{"Join", "[", 
          RowBox[{"alrul", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"be1", "\[RuleDelayed]", "0"}], ",", 
             RowBox[{"be2", "\[RuleDelayed]", "0"}], ",", 
             RowBox[{"ga1", ":>", "0"}], ",", 
             RowBox[{"ga2", ":>", "0"}], ",", 
             RowBox[{"rho", "\[RuleDelayed]", 
              RowBox[{"-", 
               FractionBox[
                RowBox[{
                 SuperscriptBox["\[Pi]", "2"], " ", "DPlus"}], 
                SuperscriptBox["\[Pi]", "2"]]}]}]}], "}"}]}], "]"}]}]}], ";", 
       
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{"#1", ",", 
              RowBox[{
               RowBox[{
                RowBox[{"FreeQ", "[", 
                 RowBox[{"#1", ",", "ParD"}], "]"}], "&&", 
                RowBox[{"FreeQ", "[", 
                 RowBox[{"#1", ",", "DPlus"}], "]"}]}], "&"}]}], "]"}], " ", 
            RowBox[{"TpD", "[", 
             RowBox[{"pp", ",", 
              RowBox[{"Select", "[", 
               RowBox[{"#1", ",", "\[InvisibleSpace]", 
                RowBox[{
                 RowBox[{"!", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#1", ",", "ParD"}], "]"}], "&&", 
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#1", ",", "DPlus"}], "]"}]}], ")"}]}], "&"}]}], 
               "]"}]}], "]"}]}], "&"}], ")"}], "/@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Expand", "[", "new", "]"}], "+", 
           RowBox[{"dum1", " ", "dum2"}]}], ")"}]}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"dum1", "\[RuleDelayed]", "0"}], ",", 
          RowBox[{"dum2", "\[RuleDelayed]", "0"}]}], "}"}]}]}]}], "]"}]}], 
   ";"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["ApplyTarasovT", "Subsection",
 CellTags->"T1.3.6.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ApplyTarasovT", "[", 
   RowBox[{"tij_", ",", " ", "exp_"}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", "dump", "}"}], ",", "\n", "\t\t", 
    RowBox[{
     RowBox[{"dump", "=", 
      RowBox[{"Product", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"ParD", "[", "j", "]"}], "^", "dummy"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "5"}], "}"}]}], "]"}]}], ";", "\n", "\t\t", 
     RowBox[{
      RowBox[{"Expand", "[", 
       RowBox[{"tij", " ", "exp"}], "]"}], "/.", "\n", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"STLI", "[", 
          RowBox[{"de_", ",", "pp_", ",", "pe__List"}], "]"}], " ", 
         RowBox[{"TpD", "[", 
          RowBox[{"tpp_", ",", 
           RowBox[{
            RowBox[{"DPlus", "^", "i_."}], " ", "t_"}]}], "]"}]}], ")"}], ":>",
        "\n", "\t", 
       RowBox[{"MassDerivative", "[", 
        RowBox[{
         RowBox[{"STLI", "[", " ", 
          RowBox[{
           RowBox[{"de", "/.", 
            RowBox[{"de", ":>", 
             RowBox[{"(", 
              RowBox[{"de", " ", "+", " ", 
               RowBox[{"2", " ", "i"}]}], ")"}]}]}], ",", "pp", ",", "pe"}], 
          "]"}], ",", "\n", "\t\t\t\t\t\t\t", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"t", " ", "dump"}], " ", "/.", 
            RowBox[{"Times", "->", "List"}]}], "/.", 
           RowBox[{
            RowBox[{"Power", "[", 
             RowBox[{"_", ",", "n_"}], "]"}], ":>", 
            RowBox[{"{", 
             RowBox[{"0", ",", "n"}], "}"}]}]}], "/.", 
          RowBox[{"dummy", "->", "0"}]}]}], "]"}]}]}]}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Misc", "Subsection",
 CellTags->"T1.3.7.1"],

Cell[BoxData[
 RowBox[{"Prefactor1", "/:", "\t\t", 
  RowBox[{
   RowBox[{"Prefactor1", "[", "a_", "]"}], "^", "n_"}], ":=", 
  RowBox[{"Prefactor1", "[", 
   RowBox[{"a", "^", "n"}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell["Colors", "Subsubsection",
 CellTags->"T1.3.7.1.1"],

Cell[TextData[{
 StyleBox["TFI",
  FontColor->RGBColor[1, 0.6, 0]],
 " ",
 StyleBox["TVI ",
  FontColor->RGBColor[0.2, 0.4, 0.8]],
 StyleBox["TFR ",
  FontColor->RGBColor[0, 0.6, 0]],
 StyleBox["TVR ",
  FontColor->RGBColor[0.2, 0.8, 1]],
 StyleBox["MakeFun ",
  FontColor->RGBColor[1, 0.6, 0.6]],
 StyleBox["Cayley ",
  FontColor->RGBColor[1, 0, 0.4]],
 StyleBox["CayleyD ",
  FontColor->RGBColor[1, 0.2, 1]],
 StyleBox["Cayleyu",
  FontColor->RGBColor[0.8, 0.8, 0.4]],
 StyleBox[" ",
  FontColor->RGBColor[1, 0.2, 1]],
 StyleBox["IFF ",
  FontColor->RGBColor[0.6, 0.4, 1]],
 StyleBox["ToArgs ",
  FontColor->RGBColor[0.2, 0.6, 1]],
 StyleBox["Block",
  FontSize->16,
  FontWeight->"Bold",
  FontColor->RGBColor[1, 0.2, 0.2]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"Block", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<Block\>\"", ",", 
     RowBox[{"FontSize", "\[Rule]", "16"}], ",", 
     RowBox[{"FontWeight", "\[Rule]", "\"\<Bold\>\""}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0.2", ",", "0.2"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"Cayley", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<Cayley\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0.4"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"CayleyD", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<CayleyD\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0.2", ",", "1"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"Cayleyu", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<Cayleyu\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.8", ",", "0.8", ",", "0.4"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"IFF", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<IFF\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.6", ",", "0.4", ",", "1"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"MakeFun", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<MakeFun\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0.6", ",", "0.6"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"TFI", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<TFI\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0.6", ",", "0"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"TFR", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<TFR\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", "0.6", ",", "0"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"ToArgs", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<ToArgs\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.2", ",", "0.6", ",", "1"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"TVI", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<TVI\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.2", ",", "0.4", ",", "0.8"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{"TVR", ",", "StandardForm"}], "]"}], ":=", 
  RowBox[{"StyleBox", "[", 
   RowBox[{"\"\<TVR\>\"", ",", 
    RowBox[{"FontColor", "\[Rule]", 
     RowBox[{"RGBColor", "[", 
      RowBox[{"0.2", ",", "0.8", ",", "1"}], "]"}]}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["PQ", "Subsubsection",
 CellTags->"T1.3.7.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PQ", "[", 
    RowBox[{"_Integer", "?", "Positive"}], "]"}], " ", ":=", " ", "True"}], 
  ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["PNQ", "Subsubsection",
 CellTags->"T1.3.7.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PNQ", "[", "0", "]"}], "=", "True"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PNQ", "[", 
    RowBox[{"_Integer", "?", "Positive"}], "]"}], " ", ":=", " ", "True"}], 
  ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["FactorC", "Subsubsection",
 CellTags->"T1.3.7.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FactorC", "[", "z_", "]"}], ":=", 
  RowBox[{
   RowBox[{"FactorC", "[", "z", "]"}], "=", 
   RowBox[{"Factor", "[", "z", "]"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TFI", "Subsubsection",
 CellTags->"T1.3.7.5.1"],

Cell["\<\

TFI[depp__,{a___,b_Integer,c___}]:=TFI[depp,{a,{b,0},c}];
TFI[depp__,{a___,{0,m_/;m=!=0},b___}]:=TFI[depp,{a,{0,0},b}];
TFI[a__,{0,0},b__List]:=TFI[a,b];
TFI[d_,pp_,dp_,{0,0},{x1_,x2_,x3_,x4_,x5_},list_List]:=
    TFI[d,pp,{x1,x2,x3,x4,x5},list];
TFI[d_,pp_,dp_,{a_,b_},{0,0,0,0,0},list_List]:=TFI[d,pp,dp,{a,b},list];
TFI[d_,pp_,dp_/;Head[dp]=!=List,{0,0,0,0,0},list_List]:=TFI[d,pp,list];
TFI[d_,pp_,dp_/;Head[dp]=!=List,list_List]:=TFI[d,pp,list];
TFI[d_,pp_,dp_/;Head[dp]=!=List,{x1_,x2_,x3_,x4_,x5_},list_List]:=
    TFI[d,pp,{x1,x2,x3,x4,x5},list];
TFI[__,{{_,_},{0,0},{_,_},{0,0},{_,0}}]:=0;
TFI[__,{{0,0},{_,_},{0,0},{_,_},{_,0}}]:=0;
TFI[__,{{0,0},{_,_},{_,0},{_,_},{0,0}}]:=0;
TFI[__,{{_,0},{_,_},{0,0},{_,_},{0,0}}]:=0;
TFI[__,{{_,_},{_,0},{_,_},{0,0},{0,0}}]:=0;
TFI[__,{{_,_},{0,0},{_,_},{_,0},{0,0}}]:=0;
TFI[__,{{0,0},{0,0},{0,0},{_,0},{_,_}}]:=0;
TFI[__,{{0,0},{0,0},{_,0},{0,0},{_,_}}]:=0;
TFI[__,{{0,0},{_,0},{0,0},{0,0},{_,_}}]:=0;
TFI[__,{{_,0},{0,0},{0,0},{0,0},{_,_}}]:=0;
TFI[__,{{_,_},{_,0},{0,0},{0,0},{0,0}}]:=0;
TFI[__,{{_,0},{_,_},{0,0},{0,0},{0,0}}]:=0;
TFI[__,{{_,_},{0,0},{0,0},{_,0},{0,0}}]:=0;
TFI[__,{{_,0},{0,0},{0,0},{_,_},{0,0}}]:=0;
TFI[__,{{0,0},{_,0},{_,_},{0,0},{0,0}}]:=0;
TFI[__,{{0,0},{_,_},{_,0},{0,0},{0,0}}]:=0;
TFI[__,{{0,0},{0,0},{_,_},{_,0},{0,0}}]:=0;
TFI[__,{{0,0},{0,0},{_,0},{_,_},{0,0}}]:=0;

If[$Notebooks,

mbt[z_] := ToBoxes[z, TraditionalForm];
  redblue[z_ /; Head[z] =!= Plus] := mbt[z];
  redblue[(z_) - 1] := StyleBox[mbt[z], FontColor -> RGBColor[1, 0, 0]];
  redblue[(z_Subscript) - 2] :=
   StyleBox[SubscriptBox[OverscriptBox[z[[1]], \"_\"], z[[2]]],
    FontColor -> RGBColor[1, 0, 0]];
  redblue[(z_) - 2] :=
   StyleBox[OverscriptBox[mbt[z], \"_\"], FontColor -> RGBColor[1, 0, 0]];
  redblue[(z_) - 3] :=
   StyleBox[UnderscriptBox[OverscriptBox[mbt[z], \"_\"], \"_\"],
    FontColor -> RGBColor[1, 0, 0]];
  redblue[(z_) + 1] := StyleBox[mbt[z], FontColor -> RGBColor[0, 0, 1]];
  redblue[(z_) + 2] :=
   StyleBox[OverscriptBox[mbt[z], \"_\"], FontColor -> RGBColor[0, 0, 1]];
  redblue[(z_Subscript) + 2] :=
   StyleBox[SubscriptBox[OverscriptBox[z[[1]], \"_\"], z[[2]]],
    FontColor -> RGBColor[0, 0, 1]];

  TFI/: MakeBoxes[TFI[d_, pp_, {ur__}, {den__}], TraditionalForm] :=
    InterpretationBox @@
      {SubsuperscriptBox[StyleBox[\"F\", Rule[SingleLetterItalics, False],
         Rule[FontWeight, \"Bold\"]], RowBox @@ {redblue /@ {den}},
        RowBox[{\"(\", ToBoxes[d, TraditionalForm], \")\", \" \", ur}]],
       TFI[d, pp, {ur}, {den}], Editable -> True} /;
     MemberQ[{StandardForm, TraditionalForm}, TraditionalForm];

  TFI/: MakeBoxes[TFI[d_, pp_, dp_, any__, {den__}], TraditionalForm] :=
    (InterpretationBox @@
       {SubsuperscriptBox[StyleBox[\"F\",
          Rule[SingleLetterItalics, False], Rule[FontWeight, \"Bold\"]],
         RowBox @@ {redblue /@ {den}},
         RowBox[{\"(\", ToBoxes[d, TraditionalForm], \")\", \" \",
           Sequence @@ Flatten[{any}]}]], TFI[d, pp, dp, any, {den}],
        Editable -> True}) /; Head[dp] =!= List;

TFI/:
   MakeBoxes[TFI[dpp__, {den__}], TraditionalForm] :=
    InterpretationBox @@
      {SubsuperscriptBox[StyleBox[\"F\", Rule[SingleLetterItalics, False],
         Rule[FontWeight, \"Bold\"]], RowBox @@ {redblue /@ {den}},
        RowBox[{\"(\", ToBoxes[First[{dpp}], TraditionalForm], \")\"}]],
       TFI[dpp, {den}], Editable -> True} /;
     MemberQ[{StandardForm, TraditionalForm}, TraditionalForm] &&
      Head[Last[{dpp}]] =!= List;

];\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TJI", "Subsubsection",
 CellTags->"T1.3.7.6.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"de_", ",", "pp_", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_Integer", ",", "m1_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n2_Integer", ",", "m2_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n3_Integer", ",", "m3_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"TJI", "[", 
      RowBox[{"de", ",", "pp", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}]}], "}"}]}], "]"}], "=", 
     RowBox[{"TJI", "[", 
      RowBox[{"de", ",", "pp", ",", 
       RowBox[{"Reverse", "[", 
        RowBox[{"Sort", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"n1", ",", "m1"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"n2", ",", "m2"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"n3", ",", "m3"}], "}"}]}], "}"}], "]"}], "]"}]}], 
      "]"}]}], ")"}], "/;", "\[InvisibleSpace]", 
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{"OrderedQ", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2", ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}]}], "}"}], "]"}], 
     ")"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"TJI", "[", 
  RowBox[{"d", ",", "pp", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "m1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m3"}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"d_", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TBI", "Subsubsection",
 CellTags->"T1.3.7.7.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["TBI",
    FontColor->RGBColor[1, 0.6, 0]], "[", 
   RowBox[{"de_", ",", "pp_", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_Integer", ",", "m1_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n2_Integer", ",", "m2_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"TJI", "[", 
      RowBox[{"de", ",", "pp", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}]}], "}"}]}], "]"}], "=", 
     RowBox[{
      StyleBox["TBI",
       FontColor->RGBColor[1, 0.6, 0]], "[", 
      RowBox[{"de", ",", "pp", ",", 
       RowBox[{"Reverse", "@", 
        RowBox[{"Sort", "@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"n1", ",", "m1"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"n2", ",", "m2"}], "}"}]}], "}"}]}]}]}], "]"}]}], ")"}], "/;",
    "\[InvisibleSpace]", 
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{"OrderedQ", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n2", ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}]}], "}"}], "]"}], 
     ")"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "[", 
   RowBox[{"d_", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TAI", "Subsubsection",
 CellTags->"T1.3.7.8.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["TAI",
     FontColor->RGBColor[1, 0.6, 0]], "[", 
    RowBox[{"de_", ",", "pp_", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1_Integer", ",", "0"}], "}"}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Definitions and notation", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalDelta]\[ScriptCapitalC]", "=", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], 
    RowBox[{"Det", "[", " ", 
     RowBox[{"(", GridBox[{
        {"0", "1", "1", "1", "1"},
        {"1", "0", 
         SubscriptBox["\[Mu]", "6"], 
         SubscriptBox["\[Mu]", "4"], 
         SubscriptBox["\[Mu]", "3"]},
        {"1", 
         SubscriptBox["\[Mu]", "6"], "0", 
         SubscriptBox["\[Mu]", "2"], 
         SubscriptBox["\[Mu]", "1"]},
        {"1", 
         SubscriptBox["\[Mu]", "4"], 
         SubscriptBox["\[Mu]", "2"], "0", 
         SubscriptBox["\[Mu]", "5"]},
        {"1", 
         SubscriptBox["\[Mu]", "3"], 
         SubscriptBox["\[Mu]", "1"], 
         SubscriptBox["\[Mu]", "5"], "0"}
       }], ")"}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"checkli", " ", "=", " ", 
   RowBox[{"{", "\n", "\t\t", 
    RowBox[{
     RowBox[{"\[ScriptCapitalC]", "->", "\[CapitalDelta]\[ScriptCapitalC]"}], 
     ",", "\n", 
     RowBox[{
      SubscriptBox["\[CapitalDelta]", "i_"], ":>", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["\[Mu]", "i"]], "\[CapitalDelta]\[ScriptCapitalC]"}]}], 
     ",", "\n", "\t\t", 
     RowBox[{
      SubscriptBox["\[Mu]", "6"], ":>", "PP"}], ",", 
     RowBox[{
      SubscriptBox["\[Mu]", "i_"], ":>", 
      RowBox[{
       RowBox[{"ToExpression", "[", 
        RowBox[{"\"\<m\>\"", "<>", 
         RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "^", "2"}]}], ",", 
     "\n", "\t\t", 
     RowBox[{
      SubscriptBox["m", "1"], ":>", 
      SqrtBox[
       SuperscriptBox["m1", "2"]]}], ",", 
     RowBox[{
      SubscriptBox["m", "2"], ":>", 
      SqrtBox[
       SuperscriptBox["m2", "2"]]}], ",", 
     RowBox[{
      SubscriptBox["m", "3"], ":>", 
      SqrtBox[
       SuperscriptBox["m3", "2"]]}], ",", 
     RowBox[{
      SubscriptBox["m", "4"], ":>", 
      SqrtBox[
       SuperscriptBox["m4", "2"]]}], ",", 
     RowBox[{
      SubscriptBox["m", "5"], ":>", 
      SqrtBox[
       SuperscriptBox["m5", "2"]]}], ",", "\n", 
     RowBox[{
      SubscriptBox["\[CapitalDelta]", 
       RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
      RowBox[{
       SubsuperscriptBox["\[Mu]", "i", "2"], "+", 
       SubsuperscriptBox["\[Mu]", "j", "2"], "+", 
       SubsuperscriptBox["\[Mu]", "k", "2"], "-", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Mu]", "i"], 
           SubscriptBox["\[Mu]", "j"]}], "+", 
          RowBox[{
           SubscriptBox["\[Mu]", "j"], 
           SubscriptBox["\[Mu]", "k"]}], "+", 
          RowBox[{
           SubscriptBox["\[Mu]", "k"], 
           SubscriptBox["\[Mu]", "i"]}]}], ")"}]}]}]}], ",", "\n", 
     RowBox[{
      SubscriptBox["u", 
       RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
      RowBox[{
       SubscriptBox["\[Mu]", "i"], "-", 
       SubscriptBox["\[Mu]", "j"], "-", 
       SubscriptBox["\[Mu]", "k"]}]}], ",", "\n", 
     RowBox[{
      SubscriptBox["\[CapitalSigma]", "i_"], ":>", 
      RowBox[{
       RowBox[{"3", " ", "d"}], " ", "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["\[Sum]", 
           RowBox[{"j", "=", "1"}], "i"], 
          SubscriptBox["\[Nu]", "j"]}], ")"}]}], "-", "2"}]}], ",", "\n", 
     "\t", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}]}], "}"}]}], 
  ";"}]], "Input"],

Cell[TextData[{
 "Define symmetry property for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"i", ",", "j", ",", "k"}]], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[CapitalDelta]", 
   RowBox[{"i_", ",", "j_", ",", "k_"}]], ":=", 
  RowBox[{
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"Sequence", "@@", 
     RowBox[{"Sort", "[", 
      RowBox[{"{", 
       RowBox[{"i", ",", "j", ",", "k"}], "}"}], "]"}]}]], "/;", 
   RowBox[{"!", 
    RowBox[{"OrderedQ", "[", 
     RowBox[{"{", 
      RowBox[{"i", ",", "j", ",", "k"}], "}"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 SubscriptBox["\[CapitalDelta]", 
  RowBox[{"3", ",", "1", ",", "2"}]]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["ToArgs",
  FontColor->RGBColor[0.2, 0.6, 1]]], "Subsection",
 CellTags->"T1.4.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["ToArgs",
     FontColor->RGBColor[0.2, 0.6, 1]], "[", "z_", "]"}], ":=", 
   RowBox[{"Expand", "[", 
    RowBox[{"z", "//.", "checkli"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ToArgs", "[", 
   SubscriptBox["\[CapitalSigma]", "4"], "]"}], "//", "InputForm"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["Cayley",
  FontColor->RGBColor[1, 0, 0.4]]], "Subsection",
 CellTags->"T1.4.2.1"],

Cell[TextData[{
 "This is (40).  ( p = ",
 Cell[BoxData[
  FormBox[
   SqrtBox[
    SuperscriptBox["p", "2"]], TraditionalForm]]],
 ")"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setd", "[", 
   RowBox[{
    RowBox[{"hCayley", "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], ":", "_"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "2"],
        m2], ":", "_"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], ":", "_"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4], ":", "_"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "5"],
        m5], ":", "_"}], ",", "p_"}], "]"}], ",", 
    RowBox[{"set", "[", 
     RowBox[{
      RowBox[{"hCayley", "[", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "1"],
         m1], ",", 
        InterpretationBox[
         SubscriptBox["m", "2"],
         m2], ",", 
        InterpretationBox[
         SubscriptBox["m", "3"],
         m3], ",", 
        InterpretationBox[
         SubscriptBox["m", "4"],
         m4], ",", 
        InterpretationBox[
         SubscriptBox["m", "5"],
         m5], ",", "p"}], "]"}], ",", 
      RowBox[{"factor", "[", 
       RowBox[{
        RowBox[{
         StyleBox["ToArgs",
          FontColor->RGBColor[0.2, 0.6, 1]], "[", "\[ScriptCapitalC]", "]"}], 
        "/.", 
        RowBox[{
         InterpretationBox[
          SuperscriptBox["p", "2"],
          PP], "\[Rule]", 
         SuperscriptBox["p", "2"]}]}], "]"}]}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hCayley", "\[RuleDelayed]", 
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]}], ",", 
    RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
    RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
    RowBox[{"factor", "\[RuleDelayed]", "Factor"}]}], "}"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["Cayley",
    FontColor->RGBColor[1, 0, 0.4]], "[", "i", "]"}], "=", 
  SubscriptBox["\[CapitalDelta]", "i"]}]], "Subsection",
 CellTags->"T1.4.3.1"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"setd", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayley", "[", "i", "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
      RowBox[{"set", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"hCayley", "[", "i", "]"}], "[", 
         RowBox[{"{", 
          RowBox[{
           InterpretationBox[
            SubscriptBox["m", "1"],
            m1], ",", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            m2], ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            m3], ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            m4], ",", 
           InterpretationBox[
            SubscriptBox["m", "5"],
            m5], ",", "p"}], "}"}], "]"}], ",", 
        RowBox[{"factor", "[", 
         RowBox[{
          RowBox[{
           StyleBox["ToArgs",
            FontColor->RGBColor[0.2, 0.6, 1]], "[", 
           SubscriptBox["\[CapitalDelta]", "i"], "]"}], "/.", 
          RowBox[{
           InterpretationBox[
            SuperscriptBox["p", "2"],
            PP], "\[Rule]", 
           SuperscriptBox["p", "2"]}]}], "]"}]}], "]"}]}], "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"hCayley", "\[RuleDelayed]", 
       StyleBox["Cayley",
        FontColor->RGBColor[1, 0, 0.4]]}], ",", 
      RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
      RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
      RowBox[{"factor", "\[RuleDelayed]", "Factor"}]}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "6"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["ToArgs",
   FontColor->RGBColor[0.2, 0.6, 1]], "[", 
  SubscriptBox["\[CapitalDelta]", 
   RowBox[{"1", ",", "2", ",", "6"}]], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]], "[", 
     RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], "[", "m_List", "]"}], ":=", 
   
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox["Cayley",
       FontColor->RGBColor[1, 0, 0.4]], "[", 
      RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "m", "]"}], "=", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"mm", "=", "m"}], "}"}], ",", "\n", "\t\t\t", 
      RowBox[{
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "mm", "]"}], "<", "7"}], ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"mm", ",", 
           RowBox[{"Last", "[", "mm", "]"}]}], "]"}]}], "]"}], ";", "\n", 
       "\t\t\t", 
       RowBox[{"FactorC", "[", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], "4"], 
         "-", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], "2"],
           " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], 
           "2"]}], "-", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], "2"],
           " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], 
           "2"]}], "+", 
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], "4"], 
         "+", 
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], "4"], 
         "-", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], "2"],
           " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
           "2"]}]}], "]"}]}]}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     StyleBox["Cayleyu",
      FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
     RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], "[", "m_List", "]"}], ":=", 
   
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox["Cayleyu",
       FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
      RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "mm", "]"}], "=", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"mm", "=", "m"}], "}"}], ",", "\n", "\t\t\t\t\t", 
      RowBox[{
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "mm", "]"}], "<", "7"}], ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"mm", ",", 
           RowBox[{"Last", "[", "mm", "]"}]}], "]"}]}], "]"}], ";", 
       RowBox[{"FactorC", "[", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], "2"], 
         "-", 
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], "2"], 
         "-", 
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
          "2"]}], "]"}]}]}], "]"}]}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["CayleyD",
    FontColor->RGBColor[1, 0.2, 1]], "[", "i", "]"}], "=", 
  FractionBox[
   SubscriptBox["\[CapitalDelta]", "i"], "\[CapitalDelta]"]}]], "Subsection",
 CellTags->"T1.4.4.1"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"setd", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayleyD", "[", "i", "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
      RowBox[{"set", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"hCayleyD", "[", "i", "]"}], "[", 
         RowBox[{"{", 
          RowBox[{
           InterpretationBox[
            SubscriptBox["m", "1"],
            m1], ",", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            m2], ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            m3], ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            m4], ",", 
           InterpretationBox[
            SubscriptBox["m", "5"],
            m5], ",", "p"}], "}"}], "]"}], ",", 
        RowBox[{"factor", "[", 
         FractionBox[
          RowBox[{
           RowBox[{
            StyleBox["ToArgs",
             FontColor->RGBColor[0.2, 0.6, 1]], "[", 
            SubscriptBox["\[CapitalDelta]", "i"], "]"}], "/.", 
           RowBox[{
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], "\[Rule]", 
            SuperscriptBox["p", "2"]}]}], 
          RowBox[{"\[ScriptCapitalC]", "[", 
           RowBox[{
            InterpretationBox[
             SubscriptBox["m", "1"],
             m1], ",", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             m2], ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             m3], ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             m4], ",", 
            InterpretationBox[
             SubscriptBox["m", "5"],
             m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"hCayleyD", "\[RuleDelayed]", 
       StyleBox["CayleyD",
        FontColor->RGBColor[1, 0.2, 1]]}], ",", 
      RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
      RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
      RowBox[{"factor", "\[RuleDelayed]", "Factor"}], ",", 
      RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
       StyleBox["Cayley",
        FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "6"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setd", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hCayleyD", "[", 
      RowBox[{"1", ",", "2", ",", "5"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "1"],
         m1], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "2"],
         m2], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "3"],
         m3], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "4"],
         m4], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "5"],
         m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
    RowBox[{"set", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayleyD", "[", 
        RowBox[{"1", ",", "2", ",", "5"}], "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", "p"}], "}"}], "]"}], ",", 
      RowBox[{"factor", "[", 
       FractionBox[
        RowBox[{
         RowBox[{
          StyleBox["ToArgs",
           FontColor->RGBColor[0.2, 0.6, 1]], "[", 
          SubscriptBox["\[CapitalDelta]", 
           RowBox[{"1", ",", "2", ",", "5"}]], "]"}], "/.", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "\[Rule]", 
          SuperscriptBox["p", "2"]}]}], 
        RowBox[{"\[ScriptCapitalC]", "[", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ",", 
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ",", 
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ",", 
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ",", 
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hCayleyD", "\[RuleDelayed]", 
     StyleBox["CayleyD",
      FontColor->RGBColor[1, 0.2, 1]]}], ",", 
    RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
    RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
    RowBox[{"factor", "\[RuleDelayed]", "Factor"}], ",", 
    RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setd", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hCayleyD", "[", 
      RowBox[{"1", ",", "3", ",", "6"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "1"],
         m1], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "2"],
         m2], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "3"],
         m3], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "4"],
         m4], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "5"],
         m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
    RowBox[{"set", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayleyD", "[", 
        RowBox[{"1", ",", "3", ",", "6"}], "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", "p"}], "}"}], "]"}], ",", 
      RowBox[{"factor", "[", 
       FractionBox[
        RowBox[{
         RowBox[{
          StyleBox["ToArgs",
           FontColor->RGBColor[0.2, 0.6, 1]], "[", 
          SubscriptBox["\[CapitalDelta]", 
           RowBox[{"1", ",", "3", ",", "6"}]], "]"}], "/.", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "\[Rule]", 
          SuperscriptBox["p", "2"]}]}], 
        RowBox[{"\[ScriptCapitalC]", "[", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ",", 
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ",", 
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ",", 
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ",", 
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hCayleyD", "\[RuleDelayed]", 
     StyleBox["CayleyD",
      FontColor->RGBColor[1, 0.2, 1]]}], ",", 
    RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
    RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
    RowBox[{"factor", "\[RuleDelayed]", "Factor"}], ",", 
    RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setd", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hCayleyD", "[", 
      RowBox[{"3", ",", "4", ",", "5"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "1"],
         m1], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "2"],
         m2], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "3"],
         m3], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "4"],
         m4], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "5"],
         m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
    RowBox[{"set", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayleyD", "[", 
        RowBox[{"3", ",", "4", ",", "5"}], "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", "p"}], "}"}], "]"}], ",", 
      RowBox[{"factor", "[", 
       FractionBox[
        RowBox[{
         RowBox[{
          StyleBox["ToArgs",
           FontColor->RGBColor[0.2, 0.6, 1]], "[", 
          SubscriptBox["\[CapitalDelta]", 
           RowBox[{"3", ",", "4", ",", "5"}]], "]"}], "/.", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "\[Rule]", 
          SuperscriptBox["p", "2"]}]}], 
        RowBox[{"\[ScriptCapitalC]", "[", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ",", 
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ",", 
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ",", 
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ",", 
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hCayleyD", "\[RuleDelayed]", 
     StyleBox["CayleyD",
      FontColor->RGBColor[1, 0.2, 1]]}], ",", 
    RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
    RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
    RowBox[{"factor", "\[RuleDelayed]", "Factor"}], ",", 
    RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["ToArgs",
   FontColor->RGBColor[0.2, 0.6, 1]], "[", 
  SubscriptBox["\[CapitalDelta]", 
   RowBox[{"i", ",", "j", ",", "k"}]], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["ToArgs",
   FontColor->RGBColor[0.2, 0.6, 1]], "[", 
  SubscriptBox["\[CapitalDelta]", 
   RowBox[{"1", ",", "2", ",", "6"}]], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["ToArgs",
   FontColor->RGBColor[0.2, 0.6, 1]], "[", 
  SubscriptBox["u", 
   RowBox[{"i", ",", "j", ",", "k"}]], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Utility Functions", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.5.1"],

Cell[CellGroupData[{

Cell["MakeFun and friends", "Subsection",
 CellTags->"T1.5.1.1"],

Cell["\<\
This function generates the recursion function definitions for 
TARCER starting from the entered relations.\
\>", "Text"],

Cell["The idea is to recurse TFR into TFI's step by step.", "Text"],

Cell[TextData[{
 "Define: S cD[ijk__] := ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[ScriptCapitalC]", 
     RowBox[{"-", "1"}]], 
    SubscriptBox["\[CapitalDelta]", "ijk"]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nutonand\[CapitalDelta]u", "[", 
    RowBox[{"w_", ",", "5"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"w", "/.", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}], ",", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"\[ScriptCapitalC]", " ", 
         RowBox[{"cD", "[", "ijk", "]"}]}]}], ",", 
       RowBox[{
        SubscriptBox["u", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"cu", "[", "ijk", "]"}]}], ",", "\n", "\t\t", 
       RowBox[{
        SubscriptBox["D", "ijk__"], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"D", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Rho]", "ijk__"], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Rho]", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Phi]", 
         RowBox[{"ijk__", " "}]], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Phi]", ",", "ijk"}], "]"}]}], ",", "\n", "\t\t\t", 
       RowBox[{
        SubscriptBox["\[Sigma]", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Sigma]", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["h", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"h", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"S", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", "\n", "\t\t\t", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "i_"], ":>", 
        RowBox[{"ToArgs", "[", 
         SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "\n", "\t\t", 
      "}"}]}], "/.", 
    RowBox[{"{", "\t", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}], ",", 
      RowBox[{
       SubscriptBox["m", "1"], ":>", "m1"}], ",", 
      RowBox[{
       SubscriptBox["m", "2"], ":>", "m2"}], ",", 
      RowBox[{
       SubscriptBox["m", "3"], ":>", "m3"}], ",", 
      RowBox[{
       SubscriptBox["m", "4"], ":>", "m4"}], ",", 
      RowBox[{
       SubscriptBox["m", "5"], ":>", "m5"}]}], "}"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nutonand\[CapitalDelta]u", "[", "w_", "]"}], ":=", 
   RowBox[{
    RowBox[{"w", "/.", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}], ",", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"cD", "[", "ijk", "]"}]}], ",", 
       RowBox[{
        SubscriptBox["u", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"cu", "[", "ijk", "]"}]}], ",", 
       RowBox[{
        SubscriptBox["D", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"D", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Rho]", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Rho]", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Phi]", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Phi]", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Sigma]", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Sigma]", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["h", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"h", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"S", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "i_"], "\[RuleDelayed]", 
        RowBox[{"ToArgs", "[", 
         SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "}"}]}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}], ",", 
      RowBox[{
       SubscriptBox["m", "1"], "\[RuleDelayed]", "m1"}], ",", 
      RowBox[{
       SubscriptBox["m", "2"], "\[RuleDelayed]", "m2"}], ",", 
      RowBox[{
       SubscriptBox["m", "3"], "\[RuleDelayed]", "m3"}], ",", 
      RowBox[{
       SubscriptBox["m", "4"], "\[RuleDelayed]", "m4"}], ",", 
      RowBox[{
       SubscriptBox["m", "5"], "\[RuleDelayed]", "m5"}]}], "}"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "li1_", ",", "rest_"}], "]"}], "]"}], ":=", 
   RowBox[{"TFR", "[", 
    RowBox[{"d", ",", "pp", ",", "li1", ",", "rest"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFI", "[", 
     RowBox[{"d_", ",", "pp_", ",", "li1_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", "pp", ",", "li1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_Symbol", ",", "li1_", ",", "rest_"}], 
     "]"}], "]"}], ":=", 
   RowBox[{"TFR", "[", 
    RowBox[{"d", ",", "pp", ",", "dp", ",", "li1", ",", "rest"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFI", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_Symbol", ",", "li1_List"}], "]"}], 
    "]"}], ":=", 
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", "pp", ",", "dp", ",", "li1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TVI", "[", 
     RowBox[{"d_", ",", "pp_", ",", "li1_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", "pp", ",", "li1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nnnnnaddPeP", "[", 
   RowBox[{"any_", "[", 
    RowBox[{"d_", ",", "pp_", ",", "li1_List", ",", "rest__List"}], "]"}], 
   "]"}], ":=", 
  RowBox[{
   RowBox[{"any", "[", 
    RowBox[{"d", ",", "pp", ",", "li1", ",", "rest"}], "]"}], "/;", 
   RowBox[{"any", "=!=", "List"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", "z_List", "]"}], ":=", 
   RowBox[{"Append", "[", 
    RowBox[{"z", ",", 
     SqrtBox["PP"]}], "]"}]}], ";"}]], "Input"],

Cell["addPeP[TFI[d_, z_List]] := TFI[d, PP, z]; ", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TVI", "[", 
     RowBox[{"d_", ",", "z_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", "PP", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TJI", "[", 
     RowBox[{"d_", ",", "z_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TJI", "[", 
    RowBox[{"d", ",", "PP", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFR", "[", 
     RowBox[{"d_", ",", "z_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TFR", "[", 
    RowBox[{"d", ",", "PP_", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFR", "[", 
     RowBox[{"d_", ",", "pl_List", ",", "z_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TFR", "[", 
    RowBox[{"d", ",", "PP_", ",", "pl", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TVR", "[", 
     RowBox[{"d_", ",", "z__List"}], "]"}], "]"}], ":=", 
   RowBox[{"TVR", "[", 
    RowBox[{"d", ",", "PP_", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", "z__List"}], "]"}], "]"}], ":=", 
   RowBox[{"TJR", "[", 
    RowBox[{"d", ",", "PP_", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nutomass", "[", "w_", "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"w", "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "1"], "+", "i1_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n1", "+", "i1"}], ",", "m1"}], "}"}]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "2"], "+", "i2_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n2", "+", "i2"}], ",", "m2"}], "}"}]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "3"], "+", "i3_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n3", "+", "i3"}], ",", "m3"}], "}"}]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "4"], "+", "i4_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n4", "+", "i4"}], ",", "m4"}], "}"}]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "5"], "+", "i5_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n5", "+", "i5"}], ",", "m5"}], "}"}]}]}], "}"}]}], "/.", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "in1_Integer", ",", "in2_Integer", ",", "in3_Integer", ",", 
        "in4_Integer"}], "}"}], "\[RuleDelayed]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"in1", ",", "m1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"in2", ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"in3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"in4", ",", "m4"}], "}"}]}], "}"}]}]}], "/.", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"in1_Integer", ",", "in2_Integer", ",", "in3_Integer"}], "}"}], 
     "\[RuleDelayed]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"in1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"in2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"in3", ",", "m3"}], "}"}]}], "}"}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "maybeF", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"maybeF", "[", "z_Times", "]"}], ":=", 
   RowBox[{
    RowBox[{"FactorC", "[", "z", "]"}], "/;", 
    RowBox[{
     RowBox[{"FreeQ", "[", 
      RowBox[{"z", ",", "TFI"}], "]"}], "&&", 
     RowBox[{"FreeQ", "[", 
      RowBox[{"z", ",", "TVI"}], "]"}], "&&", 
     RowBox[{"FreeQ", "[", 
      RowBox[{"z", ",", "TJI"}], "]"}], "&&", 
     RowBox[{
      RowBox[{"Length", "[", 
       RowBox[{"Cases", "[", 
        RowBox[{"z", ",", 
         RowBox[{"cD", "[", "__", "]"}], ",", 
         RowBox[{"-", "1"}]}], "]"}], "]"}], ">", "1"}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"maybeF", "[", "z_", "]"}], ":=", "z"}], ";", 
  RowBox[{"SetAttributes", "[", 
   RowBox[{"top", ",", "Listable"}], "]"}], ";"}]], "Input"],

Cell["This is the function for the information of side conditions.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{
    StyleBox["IFF",
     FontColor->RGBColor[0.6, 0.4, 1]], ",", "HoldAll"}], "]"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "[", "t__", "]"}]}], "==", "rhs_"}],
      ",", "eqn_String", ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    StyleBox["Block",
     CellFrame->True,
     FontSize->16,
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0.2, 0.2]], "[", 
    RowBox[{
     RowBox[{"{", "nm", "}"}], ",", 
     RowBox[{
      RowBox[{"nmrel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Flatten", "[", 
          RowBox[{"{", "mrel", "}"}], "]"}], ",", 
         RowBox[{
          RowBox[{"FreeQ", "[", 
           RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"HoldForm", "@@", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    StyleBox["TFR",
                    FontColor->RGBColor[0, 0.6, 0]], "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", "t", "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    "\t", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TFR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TFR", "->", "TFIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "te__", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "te", "]"}], "]"}], 
                    "]"}]}]}], ")"}]}], ",", "5"}], "]"}], "]"}]}], ",", 
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "__", "]"}], ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cD", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cD", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", "nmrel"}]}], "]"}], ",", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds", "]"}], 
                    ",", "5"}], "]"}]}], "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", "DP", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "5"],
                    m5]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", "\n", 
                    "\t\t\t\t\t", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", 
                 RowBox[{
                  RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                  RowBox[{
                   RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["CayleyD",
                    FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}], "]"}],
                    "[", 
                   RowBox[{"addPeP", "[", 
                    RowBox[{"BLA", "=", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}]}], "]"}], "]"}]}]}], "/.", 
                RowBox[{"addPeP", "->", "Identity"}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]", 
                "CompoundExpression"}]}], "/.", 
              RowBox[{
               RowBox[{
                RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                RowBox[{"a_", ",", 
                 RowBox[{
                  StyleBox["IFF",
                   FontColor->RGBColor[0.6, 0.4, 1]], "[", "b_", "]"}]}], 
                "]"}], "\[Rule]", 
               RowBox[{"a", "/;", "b"}]}]}], "/.", 
             RowBox[{
              RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
            RowBox[{
             RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", 
             RowBox[{
              StyleBox["CayleyD",
               FontColor->RGBColor[1, 0.2, 1]], "[", "ijk__", "]"}], "]"}], 
            "\[Rule]", 
            RowBox[{
             StyleBox["CayleyD",
              FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}]}]}], "/.", 
          RowBox[{"{", 
           RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}], ")"}],
         "/.", 
        RowBox[{"Identity", "->", "identit"}]}], "/.", 
       RowBox[{
        RowBox[{"identit", "[", "a_", "]"}], "->", "a"}]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "[", "t__", "]"}]}], "==", "rhs_"}],
      ",", "eqn_String", ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     StyleBox["Block",
      CellFrame->True,
      FontSize->16,
      FontWeight->"Bold",
      FontColor->RGBColor[1, 0.2, 0.2]], "[", 
     RowBox[{
      RowBox[{"{", "nm", "}"}], ",", 
      RowBox[{
       RowBox[{"nmrel", "=", 
        RowBox[{"Select", "[", 
         RowBox[{
          RowBox[{"Flatten", "[", 
           RowBox[{"{", "mrel", "}"}], "]"}], ",", 
          RowBox[{
           RowBox[{"FreeQ", "[", 
            RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"HoldForm", "@@", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    StyleBox["TFR",
                    FontColor->RGBColor[0, 0.6, 0]], "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", "t", "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TFR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TFR", "->", "TFIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "te__", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "te", "]"}], "]"}], 
                    "]"}]}]}], ")"}]}], "]"}], "]"}]}], ",", 
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "__", "]"}], ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cD", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cD", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", "nmrel"}]}], "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", "DP", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "5"],
                    m5]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", "\t", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", 
                  RowBox[{
                   RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                   RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["CayleyD",
                    FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}], "]"}],
                     "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", 
                    RowBox[{"addPeP", "->", "Identity"}]}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]",
                   "CompoundExpression"}]}], "/.", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                  RowBox[{"a_", ",", 
                   RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "b_", "]"}]}], 
                  "]"}], "\[Rule]", 
                 RowBox[{"a", "/;", "b"}]}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
              RowBox[{
               RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
             RowBox[{
              RowBox[{"Hold", "[", 
               RowBox[{
                StyleBox["CayleyD",
                 FontColor->RGBColor[1, 0.2, 1]], "[", "ijk__", "]"}], "]"}], 
              "\[Rule]", 
              RowBox[{
               StyleBox["CayleyD",
                FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}]}]}], "/.", 
            
            RowBox[{"{", 
             RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}], 
           ")"}], "/.", 
          RowBox[{"addPeP", "->", "Identity"}]}], "/.", 
         RowBox[{"Identity", "->", "identit"}]}], "/.", 
        RowBox[{
         RowBox[{"identit", "[", "a_", "]"}], "->", "a"}]}]}]}], "]"}], "/;", 
    
    RowBox[{
     RowBox[{"Head", "[", "mrel", "]"}], "=!=", 
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TVI",
         FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
      "rhs_"}], ",", "eqn_String", ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    StyleBox["Block",
     FontSize->16,
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0.2, 0.2]], "[", 
    RowBox[{
     RowBox[{"{", "nm", "}"}], ",", 
     RowBox[{
      RowBox[{"nmrel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Flatten", "[", 
          RowBox[{"{", "mrel", "}"}], "]"}], ",", 
         RowBox[{
          RowBox[{"FreeQ", "[", 
           RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
      RowBox[{"nmrel", "=", 
       RowBox[{"Join", "[", 
        RowBox[{"nmrel", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            SubscriptBox["m", "1"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "1"],
             m1]}], ",", 
           RowBox[{
            SubscriptBox["m", "2"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             m2]}], ",", 
           RowBox[{
            SubscriptBox["m", "3"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             m3]}], ",", 
           RowBox[{
            SubscriptBox["m", "4"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             m4]}], ",", 
           RowBox[{
            SubscriptBox["\[CapitalSigma]", "i_"], ":>", 
            RowBox[{"ToArgs", "[", 
             SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "}"}]}], 
        "]"}]}], ";", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"HoldForm", "@@", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    StyleBox["TVR",
                    FontColor->RGBColor[0.2, 0.8, 1]], "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", "t", "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TVR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TVR", "->", "TVIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "te__", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "te", "]"}], 
                    "]"}], "]"}]}]}], ")"}]}], "]"}], "]"}]}], ",", 
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "__", "]"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", 
                    RowBox[{"\[ScriptCapitalC]", "->", "1"}]}], "/.", 
                    "nmrel"}]}], "]"}], ",", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds", "]"}], 
                    "]"}]}], "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", "\t", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", "\n", "\t\t\t", 
                    RowBox[{
                    RowBox[{"HoldPattern", "[", 
                    RowBox[{"TA", "[", "aijk__", "]"}], "]"}], "\[Rule]", 
                    RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{"TA", "[", "aijk", "]"}], "]"}], "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", "\n", "\t\t\t", 
                   RowBox[{
                    RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                    RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{"Cayley", "[", "ijk", "]"}], "]"}], "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", 
                  RowBox[{
                   RowBox[{"cu", "[", "ijk__", "]"}], "\[Rule]", 
                   RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["Cayleyu",
                    FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}], 
                    "]"}], "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", 
                    RowBox[{"addPeP", "->", "Identity"}]}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]",
                   "CompoundExpression"}]}], "/.", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                  RowBox[{"a_", ",", 
                   RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "b_", "]"}]}], 
                  "]"}], "\[Rule]", 
                 RowBox[{"a", "/;", "b"}]}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
              RowBox[{
               RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
             RowBox[{"addPeP", "->", "Identity"}]}], "/.", 
            RowBox[{
             RowBox[{"Hold", "[", 
              RowBox[{
               StyleBox["Cayleyu",
                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk__", "]"}], 
              "]"}], "\[Rule]", 
             RowBox[{
              StyleBox["Cayleyu",
               FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", 
             RowBox[{
              StyleBox["Cayley",
               FontColor->RGBColor[1, 0.2, 1]], "[", "ijk__", "]"}], "]"}], 
            "\[Rule]", 
            RowBox[{
             StyleBox["Cayley",
              FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}]}]}], "/.", 
          "\n", "\t\t\t", 
          RowBox[{
           RowBox[{"Hold", "[", "TA", "]"}], "->", "TA"}]}], "/.", 
         RowBox[{"{", 
          RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}], "/.", 
        RowBox[{"Identity", "->", "identit"}]}], "/.", 
       RowBox[{
        RowBox[{"identit", "[", "a_", "]"}], "->", "a"}]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TVI",
         FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
      "rhs_"}], ",", "eqn_String", ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     StyleBox["Block",
      FontSize->16,
      FontWeight->"Bold",
      FontColor->RGBColor[1, 0.2, 0.2]], "[", 
     RowBox[{
      RowBox[{"{", "nm", "}"}], ",", 
      RowBox[{
       RowBox[{"nmrel", "=", 
        RowBox[{"Select", "[", 
         RowBox[{
          RowBox[{"Flatten", "[", 
           RowBox[{"{", "mrel", "}"}], "]"}], ",", 
          RowBox[{
           RowBox[{"FreeQ", "[", 
            RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
       RowBox[{"nmrel", "=", 
        RowBox[{"Join", "[", 
         RowBox[{"nmrel", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             SubscriptBox["m", "1"], "\[RuleDelayed]", 
             InterpretationBox[
              SubscriptBox["m", "1"],
              m1]}], ",", 
            RowBox[{
             SubscriptBox["m", "2"], "\[RuleDelayed]", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              m2]}], ",", 
            RowBox[{
             SubscriptBox["m", "3"], "\[RuleDelayed]", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              m3]}], ",", 
            RowBox[{
             SubscriptBox["m", "4"], "\[RuleDelayed]", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              m4]}], ",", 
            RowBox[{
             SubscriptBox["\[CapitalSigma]", "i_"], "\[RuleDelayed]", 
             RowBox[{
              StyleBox["ToArgs",
               FontColor->RGBColor[0.2, 0.6, 1]], "[", 
              SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "}"}]}], 
         "]"}]}], ";", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"HoldForm", "@@", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    StyleBox["TVR",
                    FontColor->RGBColor[0.2, 0.8, 1]], "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", "t", "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TVR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TVR", "->", "TVIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], "\n", "\t\t\t\t\t\t", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "te__", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "te", "]"}], 
                    "]"}], "]"}]}]}], ")"}]}], "]"}], "]"}]}], ",", 
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "__", "]"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{"\[ScriptCapitalC]", "\[Rule]", "1"}]}]}], 
                    "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", 
                  RowBox[{
                   RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                   RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", "]"}], "]"}],
                     "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_Symbol"}], "}"}], 
                    "\[RuleDelayed]", "ma"}]}], "]"}], "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"cu", "[", "ijk__", "]"}], "\[Rule]", 
                  RowBox[{
                   RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["Cayleyu",
                    FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}], 
                    "]"}], "[", 
                   RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_Symbol"}], "}"}], 
                    "\[RuleDelayed]", "ma"}]}], "]"}], "]"}]}]}], "/.", 
                RowBox[{
                 RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]", 
                 "CompoundExpression"}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
              RowBox[{
               RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
             RowBox[{"addPeP", "->", "Identity"}]}], "/.", 
            RowBox[{
             RowBox[{"Hold", "[", 
              RowBox[{
               StyleBox["Cayleyu",
                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk__", "]"}], 
              "]"}], "\[Rule]", 
             RowBox[{
              StyleBox["Cayleyu",
               FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", 
             RowBox[{
              StyleBox["Cayley",
               FontColor->RGBColor[1, 0, 0.4]], "[", "ijk__", "]"}], "]"}], 
            "\[Rule]", 
            RowBox[{
             StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
          RowBox[{"{", 
           RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}], "/.", 
         
         RowBox[{"Identity", "->", "identit"}]}], "/.", 
        RowBox[{
         RowBox[{"identit", "[", "a_", "]"}], "->", "a"}]}]}]}], "]"}], "/;", 
    
    RowBox[{
     RowBox[{"Head", "[", "mrel", "]"}], "=!=", 
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{"TJI", "[", "t__", "]"}]}], "==", "rhs_"}], ",", "eqn_String", 
     ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    StyleBox["Block",
     FontSize->16,
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0.2, 0.2]], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"nm", ",", "nmrel", ",", " ", "result"}], "}"}], ",", 
     RowBox[{
      RowBox[{"\[ScriptCapitalC]", "=", "1"}], ";", 
      RowBox[{"nmrel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Flatten", "[", 
          RowBox[{"{", "mrel", "}"}], "]"}], ",", 
         RowBox[{
          RowBox[{"FreeQ", "[", 
           RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
      RowBox[{"nmrel", "=", 
       RowBox[{"Join", "[", 
        RowBox[{"nmrel", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            SubscriptBox["m", "1"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "1"],
             m1]}], ",", 
           RowBox[{
            SubscriptBox["m", "2"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             m2]}], ",", 
           RowBox[{
            SubscriptBox["m", "3"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             m3]}], ",", 
           RowBox[{
            SubscriptBox["m", "4"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             m4]}], ",", 
           RowBox[{
            SubscriptBox["\[CapitalSigma]", "i_"], "\[RuleDelayed]", 
            RowBox[{
             StyleBox["ToArgs",
              FontColor->RGBColor[0.2, 0.6, 1]], "[", 
             SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "}"}]}], 
        "]"}]}], ";", "\n", "\t\t", 
      RowBox[{"result", " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"HoldForm", "@@", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"TJR", "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", 
                    RowBox[{"tT", "=", "t"}], "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TJR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TJR", "->", "TJIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], "\t", ",", 
                    RowBox[{"TJ", "=", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{"TJI", "[", "te__", "]"}], "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TJI", "[", "te", "]"}], "]"}], "]"}]}]}], 
                    ")"}]}], "]"}], "]"}]}], ",", 
                    RowBox[{"TJI", "[", "__", "]"}], ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", 
                    RowBox[{"\[ScriptCapitalC]", "\[Rule]", "1"}]}], "/.", 
                    "nmrel"}]}]}], "]"}], ",", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds", "]"}], 
                    "]"}]}], "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", 
                  RowBox[{
                   RowBox[{"HoldPattern", "[", 
                    RowBox[{"TA", "[", "aijk__", "]"}], "]"}], "\[Rule]", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TA", "]"}], "[", "aijk", "]"}], "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                  RowBox[{
                   RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", "]"}], "]"}],
                    "[", 
                   RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_Symbol"}], "}"}], 
                    "\[RuleDelayed]", "ma"}]}], "]"}], "]"}]}]}], "/.", 
                RowBox[{
                 RowBox[{"cu", "[", "ijk__", "]"}], "\[Rule]", 
                 RowBox[{
                  RowBox[{"Hold", "[", 
                   RowBox[{
                    StyleBox["Cayleyu",
                    FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}], 
                   "]"}], "[", 
                  RowBox[{"addPeP", "[", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_Symbol"}], "}"}], 
                    "\[RuleDelayed]", "ma"}]}], "]"}], "]"}]}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]", 
                "CompoundExpression"}]}], "/.", 
              RowBox[{
               RowBox[{
                RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                RowBox[{"a_", ",", 
                 RowBox[{
                  StyleBox["IFF",
                   FontColor->RGBColor[0.6, 0.4, 1]], "[", "b_", "]"}]}], 
                "]"}], "\[Rule]", 
               RowBox[{"a", "/;", "b"}]}]}], "/.", 
             RowBox[{
              RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
            RowBox[{
             RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", 
             RowBox[{
              StyleBox["Cayleyu",
               FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk__", "]"}], 
             "]"}], "\[Rule]", 
            RowBox[{
             StyleBox["Cayleyu",
              FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
          RowBox[{
           RowBox[{"Hold", "[", 
            RowBox[{
             StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]], "[", "ijk__", "]"}], "]"}], 
           "\[Rule]", 
           RowBox[{
            StyleBox["Cayley",
             FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
         RowBox[{
          RowBox[{"Hold", "[", "TA", "]"}], "\[Rule]", "TA"}]}], "/.", 
        RowBox[{"{", 
         RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}]}], ";", 
      RowBox[{"\[ScriptCapitalC]", "=", "."}], ";", " ", "result"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"paTT", ",", "HoldFirst"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"c_.", " ", 
      RowBox[{
       StyleBox["TKI",
        FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
     "rhs_"}], ",", "eqn_String", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
    RowBox[{"mrel_", ":", 
     RowBox[{"{", "}"}]}]}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"MakeFun", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"c", " ", 
           RowBox[{"TJI", "[", "t", "]"}]}], "==", "rhs"}], "/.", 
         RowBox[{"TKI", "->", "TJI"}]}], ",", "eqn", ",", 
        RowBox[{"IFF", "[", "conds", "]"}], ",", "mrel"}], "]"}], "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"PP", ":>", "0"}], ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "PP", "]"}], ":>", "0"}], ",", 
        RowBox[{"Pattern", ":>", "paTT"}]}], "}"}]}], "/.", 
     RowBox[{
      RowBox[{"paTT", "[", 
       RowBox[{"0", ",", "_"}], "]"}], "->", "0"}]}], "/.", 
    RowBox[{"paTT", "->", "Pattern"}]}], "/.", 
   RowBox[{
    RowBox[{"TJIC", "[", 
     RowBox[{"d_", ",", "li_List"}], "]"}], ":>", 
    RowBox[{"TJIC", "[", 
     RowBox[{"d", ",", "0", ",", "li"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TBI",
         FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
      "rhs_"}], ",", "eqn_String", ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"MakeFun", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"c", " ", 
         RowBox[{"TJI", "[", "t", "]"}]}], "==", "rhs"}], "/.", 
       RowBox[{"TBI", "->", "TJI"}]}], ",", "eqn", ",", 
      RowBox[{"IFF", "[", "conds", "]"}], ",", "mrel"}], "]"}], "/.", 
    RowBox[{"{", "\n", "\t\t\t", 
     RowBox[{
      RowBox[{"TJI", ":>", "TBI"}], ",", 
      RowBox[{"TJR", ":>", "TBR"}], ",", 
      RowBox[{"TJIC", ":>", "TBIC"}]}], "}"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TAI",
         FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
      "rhs_"}], ",", "eqn_String", ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"MakeFun", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"c", " ", 
         RowBox[{"TJI", "[", "t", "]"}]}], "==", "rhs"}], "/.", 
       RowBox[{"TAI", "->", "TJI"}]}], ",", "eqn", ",", 
      RowBox[{"IFF", "[", "conds", "]"}], ",", "mrel"}], "]"}], "/.", 
    RowBox[{"{", "\n", "\t\t\t", 
     RowBox[{
      RowBox[{"TJI", ":>", "TAI"}], ",", 
      RowBox[{"TJR", ":>", "TAR"}], ",", 
      RowBox[{"TJIC", ":>", "TAIC"}]}], "}"}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Interchange", "Subsubsection",
 CellTags->"T1.5.1.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Interchange", "[", 
   RowBox[{"z_", ",", 
    RowBox[{"i_", "\[LeftRightArrow]", "j_"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Nu]explicit", "[", "z", "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "i"], ":>", 
       SubscriptBox["\[Nu]", "j"]}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "j"], ":>", 
       SubscriptBox["\[Nu]", "i"]}], ",", 
      RowBox[{
       SubscriptBox["m", "i"], ":>", 
       SubscriptBox["m", "j"]}], ",", 
      RowBox[{
       SubscriptBox["m", "j"], ":>", 
       SubscriptBox["m", "i"]}], ",", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[CapitalDelta]", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["u", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["u", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", "\n", "\t\t", 
      RowBox[{
       SubscriptBox["h", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["\[Phi]", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Phi]", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["\[Rho]", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Rho]", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["\[Sigma]", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Sigma]", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["D", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["D", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}]}], "}"}]}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       StyleBox["TFI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"depp__", ",", "args_List"}], "]"}], ":>", 
      RowBox[{
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "[", 
        RowBox[{"depp", ",", 
         RowBox[{"Sort", "[", "args", "]"}]}], "]"}], "/;", 
       "\[InvisibleSpace]", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"OrderedQ", "[", "args", "]"}], ")"}]}]}]}], ",", "\n", "\t", 
     RowBox[{
      RowBox[{
       StyleBox["TVI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"depp__", ",", "args_List"}], "]"}], ":>", 
      RowBox[{
       RowBox[{
        StyleBox["TVI",
         FontColor->RGBColor[1, 0.6, 0]], "[", 
        RowBox[{"depp", ",", 
         RowBox[{"Sort", "[", "args", "]"}]}], "]"}], "/;", 
       "\[InvisibleSpace]", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"OrderedQ", "[", "args", "]"}], ")"}]}]}]}], ",", "\n", "\t", 
     RowBox[{
      RowBox[{
       StyleBox["TJI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"depp__", ",", "args_List"}], "]"}], ":>", 
      RowBox[{
       RowBox[{
        StyleBox["TJI",
         FontColor->RGBColor[1, 0.6, 0]], "[", 
        RowBox[{"depp", ",", 
         RowBox[{"Sort", "[", "args", "]"}]}], "]"}], "/;", 
       "\[InvisibleSpace]", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"OrderedQ", "[", "args", "]"}], ")"}]}]}]}], ",", "\n", 
     "\t\t\t", 
     RowBox[{
      RowBox[{
       StyleBox["TBI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"depp__", ",", "args_List"}], "]"}], ":>", 
      RowBox[{
       RowBox[{
        StyleBox["TBI",
         FontColor->RGBColor[1, 0.6, 0]], "[", 
        RowBox[{"depp", ",", 
         RowBox[{"Sort", "[", "args", "]"}]}], "]"}], "/;", 
       "\[InvisibleSpace]", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"OrderedQ", "[", "args", "]"}], ")"}]}]}]}]}], 
    "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"i_", "\[LeftRightArrow]", "j_"}], ",", "vw__"}], "]"}], ":=", 
   RowBox[{"Interchange", "[", 
    RowBox[{
     RowBox[{"Interchange", "[", 
      RowBox[{"z", ",", 
       RowBox[{"i", "\[LeftRightArrow]", "j"}]}], "]"}], ",", "vw"}], "]"}]}],
   ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"{", 
      RowBox[{"i_", "\[LeftRightArrow]", "j_"}], "}"}]}], "]"}], ":=", 
   RowBox[{"Interchange", "[", 
    RowBox[{"Interchange", "[", 
     RowBox[{"z", ",", 
      RowBox[{"i", "\[LeftRightArrow]", "j"}]}], "]"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"i_", "\[LeftRightArrow]", "j_"}], ",", "vw__"}], "}"}]}], 
    "]"}], ":=", 
   RowBox[{"Interchange", "[", 
    RowBox[{
     RowBox[{"Interchange", "[", 
      RowBox[{"z", ",", 
       RowBox[{"i", "\[LeftRightArrow]", "j"}]}], "]"}], ",", "vw"}], "]"}]}],
   ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"Interchange", "[", 
  RowBox[{
   RowBox[{"test", "[", 
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"x", ",", "b", ",", "y"}]], ",", 
     SubscriptBox["\[Nu]", "x"], ",", 
     SubscriptBox["\[Nu]", "y"], ",", 
     SubscriptBox["\[Nu]", "a"], ",", 
     SubscriptBox["\[Nu]", "b"], ",", 
     SubscriptBox["m", "x"]}], "]"}], ",", " ", 
   RowBox[{"x", "\[LeftRightArrow]", "y"}], ",", 
   RowBox[{"a", "\[LeftRightArrow]", "b"}]}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\[Nu]explicit", "Subsection",
 CellTags->"T1.5.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Nu]explicit", "[", "z_", "]"}], ":=", 
  RowBox[{
   RowBox[{"z", "/.", 
    RowBox[{
     SubscriptBox["\[CapitalSigma]", "i_"], ":>", 
     RowBox[{"(", 
      RowBox[{"Expand", "[", 
       RowBox[{
        RowBox[{"3", " ", "d"}], " ", "-", "2", " ", "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SubsuperscriptBox["\[Sum]", 
           RowBox[{"j", "=", "1"}], "i"], 
          SubscriptBox["\[Nu]", "j"]}]}]}], "]"}], ")"}]}]}], "/.", "\n", 
   "\t", 
   RowBox[{"{", "\t", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Sigma]", 
       RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "4"]}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "i"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}]}], ")"}], 
        SubscriptBox["\[Phi]", 
         RowBox[{"i", ",", "j", ",", "k"}]]}], "-", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "i"]}], "-", 
          SubscriptBox["\[Nu]", "j"]}], ")"}], 
        SubscriptBox["\[Phi]", 
         RowBox[{"j", ",", "i", ",", "k"}]]}], "-", "\n", "\t\t\t\t\t", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "i"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}], "-", 
          SubscriptBox["\[Nu]", "k"], "-", "1"}], ")"}], 
        SubscriptBox["\[Phi]", 
         RowBox[{"k", ",", "i", ",", "j"}]]}]}]}], ",", "\n", "\t\t\t", 
     RowBox[{
      SubscriptBox["h", 
       RowBox[{"i_", ",", "j_", ",", "k_", " "}]], ":>", "\n", "\t\t\t\t", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}], "-", 
          SubscriptBox["\[Nu]", "k"]}], ")"}], 
        SuperscriptBox[
         SubscriptBox["m", "k"], "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"i", ",", "j", ",", "k"}]]}], "-", 
       RowBox[{
        FractionBox["1", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", 
          SubscriptBox["\[Nu]", "i"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "k"]}], "-", "1"}], ")"}], 
        SuperscriptBox[
         SubscriptBox["m", "i"], "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"k", ",", "i", ",", "j"}]]}], "+", "\n", "\t\t\t\t\t", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "j"], "-", 
          RowBox[{"2", " ", 
           SubscriptBox["\[Nu]", "k"]}]}], ")"}], 
        SubscriptBox["\[Rho]", 
         RowBox[{"i", ",", "j", ",", "k"}]]}]}]}], ",", "\n", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"i_", ",", "j_", ",", "k_", " "}]], ":>", "\n", "\t\t\t\t", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{"d", "-", 
           RowBox[{"2", 
            SubscriptBox["\[Nu]", "j"]}], "-", 
           SubscriptBox["\[Nu]", "k"]}], ")"}]}], 
        SuperscriptBox[
         SubscriptBox["m", "k"], "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"j", ",", "i", ",", "k"}]]}], " ", "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "j"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "k"]}]}], ")"}], 
        SuperscriptBox[
         SubscriptBox["m", "j"], "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"k", ",", "i", ",", "j"}]]}], "+", "\n", "\t\t\t\t\t", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", 
          SubscriptBox["\[Nu]", "i"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "k"]}], "-", "1"}], ")"}], 
        SubscriptBox["\[Rho]", 
         RowBox[{"i", ",", "j", ",", "k"}]]}]}]}]}], "}"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell["TA", "Subsubsection",
 CellTags->"T1.5.2.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"TA", "[", 
      RowBox[{"a_", ",", "d___Symbol", ",", "ijk___Integer"}], "]"}], "[", 
     "__", "]"}], ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"SubscriptBox", "[", 
   RowBox[{"a", ",", 
    RowBox[{"RowBox", "[", 
     RowBox[{"{", "ijk", "}"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"TA", "[", 
      RowBox[{
      "a_", ",", "d___Symbol", ",", "ijk___Integer", ",", "___Symbol"}], 
      "]"}], "[", "__", "]"}], ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"SubscriptBox", "[", 
   RowBox[{"a", ",", 
    RowBox[{"RowBox", "[", 
     RowBox[{"{", "ijk", "}"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"TA", "[", "args__", "]"}], "[", "mar_List", "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"TA", "[", "args", "]"}], "[", "mar", "]"}], "=", 
     RowBox[{
      RowBox[{"ta", "[", "args", "]"}], "[", "mar", "]"}]}], ")"}]}], 
  ";"}]], "Input"],

Cell[TextData[{
 "Assume that the last argument of {em__} is always p =",
 Cell[BoxData[
  FormBox[
   SqrtBox[
    RowBox[{" ", 
     FormBox[
      SuperscriptBox["p", "2"],
      TraditionalForm]}]], TraditionalForm]]],
 "."
}], "Text"],

Cell["(72)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"D", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer"}], 
     "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"Factor", "[", 
      RowBox[{
       SuperscriptBox["q", "8"], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox["q", "6"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["mi", "2"], "+", 
          SuperscriptBox["mj", "2"], "+", 
          SuperscriptBox["mk", "2"]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["q", "4"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"6", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mi", "4"], "+", 
             SuperscriptBox["mj", "4"], "+", 
             SuperscriptBox["mk", "4"]}], ")"}]}], "+", 
          RowBox[{"4", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox["mi", "2"], " ", 
              SuperscriptBox["mj", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["mk", "2"], " ", 
              SuperscriptBox["mi", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["mk", "2"], " ", 
              SuperscriptBox["mj", "2"]}]}], ")"}]}]}], ")"}]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox["q", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["mi", "6"], "+", 
          SuperscriptBox["mj", "6"], "+", 
          SuperscriptBox["mk", "6"], "-", 
          RowBox[{
           SuperscriptBox["mi", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mj", "4"], "+", 
             SuperscriptBox["mk", "4"]}], ")"}]}], "-", 
          RowBox[{
           SuperscriptBox["mj", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mi", "4"], "+", 
             SuperscriptBox["mk", "4"]}], ")"}]}], "-", 
          RowBox[{
           SuperscriptBox["mk", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mi", "4"], "+", 
             SuperscriptBox["mj", "4"]}], ")"}]}], "+", 
          RowBox[{"10", " ", 
           SuperscriptBox["mi", "2"], " ", 
           SuperscriptBox["mj", "2"], " ", 
           SuperscriptBox["mk", "2"]}]}], ")"}]}], "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          StyleBox["Cayley",
           FontColor->RGBColor[1, 0, 0.4]], "[", 
          RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}], "2"]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell["(73)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"\[Rho]", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer"}],
      "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"Factor", "[", 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["q", "6"]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox["q", "4"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["mi", "2"], "+", 
          SuperscriptBox["mj", "2"], "+", 
          SuperscriptBox["mk", "2"]}], ")"}]}], "-", 
       RowBox[{
        SuperscriptBox["q", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mi", "4"], "+", 
             SuperscriptBox["mj", "4"], "+", 
             SuperscriptBox["mk", "4"]}], ")"}]}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox["mi", "2"], " ", 
              SuperscriptBox["mj", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["mk", "2"], " ", 
              SuperscriptBox["mi", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["mk", "2"], " ", 
              SuperscriptBox["mj", "2"]}]}], ")"}]}]}], ")"}]}], "+", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["mi", "6"], "+", 
         SuperscriptBox["mj", "6"], "+", 
         SuperscriptBox["mk", "6"], "-", 
         RowBox[{
          SuperscriptBox["mi", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["mj", "4"], "+", 
            SuperscriptBox["mk", "4"]}], ")"}]}], "-", 
         RowBox[{
          SuperscriptBox["mj", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["mi", "4"], "+", 
            SuperscriptBox["mk", "4"]}], ")"}]}], "-", 
         RowBox[{
          SuperscriptBox["mk", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["mi", "4"], "+", 
            SuperscriptBox["mj", "4"]}], ")"}]}], "+", 
         RowBox[{"10", " ", 
          SuperscriptBox["mi", "2"], " ", 
          SuperscriptBox["mj", "2"], " ", 
          SuperscriptBox["mk", "2"]}]}], ")"}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell["(74)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"\[Phi]", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer"}],
      "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"4", 
      RowBox[{"Factor", "[", 
       RowBox[{
        SuperscriptBox["q", "4"], " ", "+", 
        RowBox[{"2", 
         SuperscriptBox["q", "2"], 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["mi", "2"], "-", 
           SuperscriptBox["mj", "2"], "-", 
           SuperscriptBox["mk", "2"]}], ")"}]}], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["mj", "2"], "-", 
           SuperscriptBox["mk", "2"]}], ")"}], "2"], "+", 
        RowBox[{
         SuperscriptBox["mi", "2"], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", 
            SuperscriptBox["mj", "2"]}], "+", 
           RowBox[{"2", 
            SuperscriptBox["mk", "2"]}], "-", 
           RowBox[{"3", 
            SuperscriptBox["mi", "2"]}]}], ")"}]}]}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell["(75)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{
     "\[Sigma]", ",", "d_", ",", "i_Integer", ",", "j_Integer", ",", 
      "k_Integer", ",", "\[Nu]i_Integer", ",", "\[Nu]j_Integer", ",", 
      "\[Nu]k_Integer"}], "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", 
   RowBox[{"Factor", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "4"]}], 
       RowBox[{"(", 
        RowBox[{"d", "-", "\[Nu]i", "-", 
         RowBox[{"2", "\[Nu]j"}]}], ")"}], 
       RowBox[{
        RowBox[{"TA", "[", 
         RowBox[{"\[Phi]", ",", "i", ",", "j", ",", "k"}], "]"}], "[", 
        RowBox[{"{", "em", "}"}], "]"}]}], " ", 
      RowBox[{"-", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", "\[Nu]i"}], "-", "\[Nu]j"}], ")"}], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "j", ",", "i", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}]}]}], "-", 
     RowBox[{
      FractionBox["1", "4"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", "d"}], "-", 
        RowBox[{"2", "\[Nu]i"}], "-", 
        RowBox[{"2", "\[Nu]j"}], "-", "\[Nu]k", "-", "1"}], ")"}], 
      RowBox[{
       RowBox[{"TA", "[", 
        RowBox[{"\[Phi]", ",", "k", ",", "i", ",", "j"}], "]"}], "[", 
       RowBox[{"{", "em", "}"}], "]"}]}]}], " ", "  ", "]"}]}], 
  ";"}]], "Input"],

Cell["(76)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{
     "h", ",", "d_", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer", 
      ",", "\[Nu]i_Integer", ",", "\[Nu]j_Integer", ",", "\[Nu]k_Integer"}], 
     "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", "\n", "\t", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"Factor", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", "\[Nu]j"}], "-", "\[Nu]k"}], ")"}], 
        SuperscriptBox["mk", "2"], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "i", ",", "j", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", "\[Nu]i", "-", 
          RowBox[{"2", "\[Nu]j"}], "-", 
          RowBox[{"2", "\[Nu]k"}], "-", "1"}], ")"}], 
        SuperscriptBox["mi", "2"], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "k", ",", "i", ",", "j"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", "\[Nu]j", "-", 
          RowBox[{"2", " ", "\[Nu]k"}]}], ")"}], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Rho]", ",", "i", ",", "j", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}]}], " ", "  ", "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell["(79)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{
     "S", ",", "d_", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer", 
      ",", "\[Nu]i_Integer", ",", "\[Nu]j_Integer", ",", "\[Nu]k_Integer"}], 
     "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", "\n", "\t\t", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"Factor", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{"d", "-", 
           RowBox[{"2", "\[Nu]j"}], "-", "\[Nu]k"}], ")"}]}], 
        SuperscriptBox["mk", "2"], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "j", ",", "i", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], " ", "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", "\[Nu]j", "-", 
          RowBox[{"2", "\[Nu]k"}]}], ")"}], 
        SuperscriptBox["mj", "2"], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "k", ",", "i", ",", "j"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}], "test", "\n", 
        FormBox["test",
         TraditionalForm], "2", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", "\[Nu]i", "-", 
          RowBox[{"2", "\[Nu]j"}], "-", 
          RowBox[{"2", "\[Nu]k"}], "-", "1"}], ")"}], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Rho]", ",", "i", ",", "j", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}]}], "   ", "]"}]}], "]"}]}], 
  ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CheckRecursion", "Subsection",
 CellTags->"T1.5.3.1"],

Cell["\<\
The following functions check the recurrence relations for J and V (using the \
final form as generated by MakeFun) via their Mellin-Integral Kernels.\
\>", "Text"],

Cell["\<\
This test is applicable in the general case  and if some of the 
masses vanish, but not for other special cases !!!\
\>", "Text"],

Cell[CellGroupData[{

Cell["CheckTVIRecursion", "Subsubsection",
 CellTags->"T1.5.3.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TVIKernel", "[", 
    RowBox[{"d_", ",", "PP", ",", " ", "lis_"}], "]"}], ":=", "\n", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "PP"}], ")"}], 
     RowBox[{"d", "-", "n1", "-", "n2", "-", "n3", "-", "n4"}]], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m1", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s1"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m2", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s2"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m3", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s3"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m4", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s4"], "*", 
    RowBox[{"TVIpsi", " ", "[", 
     RowBox[{"d", ",", " ", 
      RowBox[{"(", 
       RowBox[{"lis", "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"m1", "->", "s1"}], ",", 
          RowBox[{"m2", "->", "s2"}], ",", 
          RowBox[{"m3", "->", "s3"}], ",", 
          RowBox[{"m4", "->", "s4"}]}], "}"}]}], ")"}]}], "]"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TVIPsi", "[", 
    RowBox[{"d_", ",", "\n", "\t", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1_", ",", " ", "s1_"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"n2_", ",", "s2_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3_", ",", "s3_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n4_", ",", "s4_"}], "}"}]}], "}"}]}], "]"}], ":=", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], 
     RowBox[{"n1", "+", "n2", "+", "n3", "+", "n4", "+", "1"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"gamma", "[", 
       RowBox[{"-", "s1"}], "]"}], 
      RowBox[{"gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "n1", "-", "s1"}], "]"}]}], 
     RowBox[{"gamma", "[", "n1", "]"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"gamma", "[", 
       RowBox[{"-", "s2"}], "]"}], 
      RowBox[{"gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "n2", "-", "s2"}], "]"}]}], 
     RowBox[{"gamma", "[", "n2", "]"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"gamma", "[", 
       RowBox[{"-", "s3"}], "]"}], 
      RowBox[{"gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "n3", "-", "s3"}], "]"}]}], 
     RowBox[{"gamma", "[", "n3", "]"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"gamma", "[", 
       RowBox[{"-", "s4"}], "]"}], 
      RowBox[{"gamma", "[", 
       RowBox[{"n4", "+", "s4"}], "]"}]}], 
     RowBox[{"gamma", "[", "n4", "]"}]], 
    RowBox[{
     RowBox[{"gamma", "[", 
      RowBox[{
      "n1", "+", "n2", "+", "n3", "+", "n4", "+", "s1", "+", "s2", "+", "s3", 
       "+", "s4", "-", "d"}], "]"}], "/", 
     RowBox[{"gamma", "[", 
      RowBox[{
       RowBox[{"3", 
        RowBox[{"d", "/", "2"}]}], "-", "n1", "-", "n2", "-", "n3", "-", "n4",
        "-", "s1", "-", "s2", "-", "s3", "-", "s4"}], "]"}]}], 
    FractionBox[
     RowBox[{"gamma", "[", 
      RowBox[{"n1", "+", "n3", "+", "s1", "+", "s3", "-", 
       RowBox[{"d", "/", "2"}]}], "]"}], 
     RowBox[{"gamma", "[", 
      RowBox[{
      "n1", "+", "n3", "+", "n4", "+", "s1", "+", "s3", "+", "s4", "-", 
       RowBox[{"d", "/", "2"}]}], "]"}]], 
    FractionBox[
     RowBox[{"gamma", "[", 
      RowBox[{
      "d", "-", "n1", "-", "n3", "-", "n4", "-", "s1", "-", "s3", "-", "s4"}],
       "]"}], 
     RowBox[{"gamma", "[", 
      RowBox[{"d", "-", "n1", "-", "n3", "-", "s1", "-", "s3"}], "]"}]]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CheckTVIRecursion", "[", "expr_", "]"}], ":=", " ", 
  RowBox[{"If", "[", " ", 
   RowBox[{
    RowBox[{"$CheckRecursion", "===", "True"}], ",", "\n", "\t", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"rel", ",", "kern", ",", " ", "nkern"}], "}"}], ",", "\n", 
      RowBox[{
       RowBox[{"rel", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Extract", "[", 
            RowBox[{
             RowBox[{"expr", "/.", 
              RowBox[{"TVIC", "->", "TVIkernel"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", "2", ",", "1", ",", "1", ",", "2"}], "}"}]}], 
            "]"}], ")"}], "-", 
          RowBox[{"Extract", "[", 
           RowBox[{
            RowBox[{"expr", "/.", 
             RowBox[{"TVI", "->", "TVIkernel"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "}"}]}], "]"}]}], 
         ")"}]}], ";", "\n", 
       RowBox[{"If", "[", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"{", "TVIkernel", "}"}], "===", 
          RowBox[{"Union", "[", 
           RowBox[{"Head", "/@", 
            RowBox[{"Cases", "[", 
             RowBox[{"rel", ",", 
              RowBox[{
               RowBox[{"ff_", "[", "__", "]"}], "/;", 
               RowBox[{
                RowBox[{"Context", "[", "ff", "]"}], "=!=", 
                "\"\<System`\>\""}]}], ",", "4"}], "]"}]}], "]"}]}], ",", 
         "\t", "\n", 
         RowBox[{
          RowBox[{"rel", "=", 
           RowBox[{"Collect", "[", 
            RowBox[{
             RowBox[{"Numerator", "[", 
              RowBox[{"Together", "[", "rel", "]"}], "]"}], ",", " ", 
             RowBox[{"TVI", "[", "__", "]"}], ",", "Factor"}], "]"}]}], ";", 
          "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"rel", "/.", 
            RowBox[{"TVIkernel", "->", "TVIKernel"}]}]}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"PP", "->", 
               RowBox[{"-", "1"}]}], ",", 
              RowBox[{"m1", "->", 
               RowBox[{"Sqrt", "[", "z1", "]"}]}], ",", 
              RowBox[{"m2", "->", 
               RowBox[{"Sqrt", "[", "z2", "]"}]}], ",", 
              RowBox[{"m3", "->", 
               RowBox[{"Sqrt", "[", "z3", "]"}]}], ",", 
              RowBox[{"m4", "->", 
               RowBox[{"Sqrt", "[", "z4", "]"}]}]}], "}"}]}]}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"PowerExpand", "[", "kern", "]"}]}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"Expand", "[", "kern", "]"}]}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               SuperscriptBox["z1", 
                RowBox[{"s1", "+", "w1_."}]], 
               SuperscriptBox["z2", 
                RowBox[{"s2", "+", "w2_."}]], 
               SuperscriptBox["z3", 
                RowBox[{"s3", "+", "w3_."}]], 
               SuperscriptBox["z4", 
                RowBox[{"s4", "+", "w4_."}]], "fac_"}], ":>", 
              RowBox[{"(", 
               RowBox[{"fac", "/.", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"s1", "->", 
                   RowBox[{"s1", "-", "w1"}]}], ",", 
                  RowBox[{"s2", "->", 
                   RowBox[{"s2", "-", "w2"}]}], ",", 
                  RowBox[{"s3", "->", 
                   RowBox[{"s3", "-", "w3"}]}], ",", 
                  RowBox[{"s4", "->", 
                   RowBox[{"s4", "-", "w4"}]}]}], "}"}]}], ")"}]}], "}"}]}]}],
           " ", ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{
            RowBox[{
             RowBox[{"kern", "/.", 
              RowBox[{"TVIpsi", "->", "TVIPsi"}]}], "/.", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"gamma", "[", "zz_", "]"}], ":>", 
               RowBox[{"gamma", "[", 
                RowBox[{"Expand", "[", "zz", "]"}], "]"}]}], "}"}]}], "/.", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"gamma", "[", 
               RowBox[{"zz_", "+", "nn_Integer"}], "]"}], ":>", 
              RowBox[{
               RowBox[{"gamma", "[", "zz", "]"}], 
               RowBox[{"Pochhammer", "[", 
                RowBox[{"zz", ",", "nn"}], "]"}]}]}], "}"}]}]}], ";", "\n", 
          "\t\t\t\t", 
          RowBox[{"KERN", "=", "kern"}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"gamma", "[", "zz_", "]"}], ":>", "1"}], "}"}]}]}], ";",
           "\n", 
          RowBox[{"kern", "=", 
           RowBox[{
            RowBox[{"kern", "/.", 
             RowBox[{"{", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], 
                RowBox[{"nn_", "+", "ww_Integer"}]], ":>", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], "ww"]}], "}"}]}], "/.", 
            RowBox[{"{", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "nn_"], ":>", "1"}], "}"}]}]}], 
          ";", "\n", 
          RowBox[{"nkern", "=", 
           RowBox[{"Table", "[", "\n", "\t\t\t", 
            RowBox[{
             RowBox[{"kern", "/.", 
              RowBox[{"Thread", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{
                 "d", ",", " ", "n1", ",", " ", "n2", ",", " ", "n3", ",", 
                  "n4", ",", "s1", ",", "s2", ",", " ", "s3", ",", "s4"}], 
                 "}"}], "->", 
                RowBox[{"Table", "[", 
                 RowBox[{
                  RowBox[{"Random", "[", "]"}], ",", 
                  RowBox[{"{", "9", "}"}]}], "]"}]}], "]"}]}], ",", 
             RowBox[{"{", "3", "}"}]}], "]"}]}], ";", "\n", "nkern"}], "  ", 
         ",", "\n", "\t\t", 
         RowBox[{"Print", "[", "\"\<Matching Error\>\"", "]"}]}], " ", 
        "]"}]}]}], " ", "]"}], ",", "\n", "\t\t", 
    RowBox[{"If", "[", 
     RowBox[{"$Notebooks", ",", 
      RowBox[{"Print", "[", "\"\<Off\>\"", "]"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["CheckTJIRecursion", "Subsubsection",
 CellTags->"T1.5.3.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TJIKernel", "[", 
    RowBox[{"d_", ",", "PP", ",", " ", "lis_"}], "]"}], ":=", "\n", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "PP"}], ")"}], 
     RowBox[{"d", "-", "n1", "-", "n2", "-", "n3"}]], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m1", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s1"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m2", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s2"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m3", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s3"], "*", 
    RowBox[{"TJIpsi", " ", "[", 
     RowBox[{"d", ",", " ", 
      RowBox[{"(", 
       RowBox[{"lis", "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"m1", "->", "s1"}], ",", 
          RowBox[{"m2", "->", "s2"}], ",", 
          RowBox[{"m3", "->", "s3"}]}], "}"}]}], ")"}]}], "]"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIPsi", " ", "[", 
   RowBox[{"d_", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_", ",", " ", "s1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", "s2_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n3_", ",", "s3_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{
    RowBox[{"TVIPsi", "[", 
     RowBox[{"d", ",", "\n", "\t", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1", ",", " ", "s1"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{"n2", ",", "s2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "s3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "/", 
    RowBox[{"gamma", "[", "0", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "CheckTJIRecursion", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CheckTJIRecursion", "[", "expr_", "]"}], ":=", " ", 
  RowBox[{"If", "[", " ", 
   RowBox[{
    RowBox[{"$CheckRecursion", "===", "True"}], ",", "\n", "\t", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"rel", ",", "kern", ",", "nkern"}], "}"}], ",", "\n", 
      RowBox[{
       RowBox[{"rel", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Extract", "[", 
            RowBox[{
             RowBox[{"expr", "/.", 
              RowBox[{"TJIC", "->", "TJIkernel"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", "2", ",", "1", ",", "1", ",", "2"}], "}"}]}], 
            "]"}], ")"}], "-", 
          RowBox[{"Extract", "[", 
           RowBox[{
            RowBox[{"expr", "/.", 
             RowBox[{"TJI", "->", "TJIkernel"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "}"}]}], "]"}]}], 
         ")"}]}], ";", "\n", "\t\t\t", 
       RowBox[{"REL", "=", "rel"}], ";", "\t\t\t", "\n", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", "TJIkernel", "}"}], "===", 
          RowBox[{"Union", "[", 
           RowBox[{"Head", "/@", 
            RowBox[{"Cases", "[", 
             RowBox[{"rel", ",", 
              RowBox[{
               RowBox[{"ff_", "[", "__", "]"}], "/;", 
               RowBox[{
                RowBox[{"Context", "[", "ff", "]"}], "=!=", 
                "\"\<System`\>\""}]}], ",", "4"}], "]"}]}], "]"}]}], " ", ",",
          "\t\t\t", "\n", 
         RowBox[{
          RowBox[{"rel", "=", 
           RowBox[{"Collect", "[", 
            RowBox[{
             RowBox[{"Numerator", "[", 
              RowBox[{"Together", "[", "rel", "]"}], "]"}], ",", " ", 
             RowBox[{"TJI", "[", "__", "]"}], ",", "Factor"}], "]"}]}], ";", 
          "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"rel", "/.", 
            RowBox[{"TJIkernel", "->", "TJIKernel"}]}]}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"PP", "->", 
               RowBox[{"-", "1"}]}], ",", 
              RowBox[{"m1", "->", 
               RowBox[{"Sqrt", "[", "z1", "]"}]}], ",", 
              RowBox[{"m2", "->", 
               RowBox[{"Sqrt", "[", "z2", "]"}]}], ",", 
              RowBox[{"m3", "->", 
               RowBox[{"Sqrt", "[", "z3", "]"}]}]}], "}"}]}]}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"PowerExpand", "[", "kern", "]"}]}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"Expand", "[", "kern", "]"}]}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               SuperscriptBox["z1", 
                RowBox[{"s1", "+", "w1_."}]], 
               SuperscriptBox["z2", 
                RowBox[{"s2", "+", "w2_."}]], 
               SuperscriptBox["z3", 
                RowBox[{"s3", "+", "w3_."}]], "fac_"}], ":>", 
              RowBox[{"(", 
               RowBox[{"fac", "/.", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"s1", "->", 
                   RowBox[{"s1", "-", "w1"}]}], ",", 
                  RowBox[{"s2", "->", 
                   RowBox[{"s2", "-", "w2"}]}], ",", 
                  RowBox[{"s3", "->", 
                   RowBox[{"s3", "-", "w3"}]}]}], "}"}]}], ")"}]}], "}"}]}]}],
           ";", " ", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{
            RowBox[{
             RowBox[{"kern", "/.", 
              RowBox[{"TJIpsi", "->", "TJIPsi"}]}], "/.", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"gamma", "[", "zz_", "]"}], ":>", 
               RowBox[{"gamma", "[", 
                RowBox[{"Expand", "[", "zz", "]"}], "]"}]}], "}"}]}], "/.", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"gamma", "[", 
               RowBox[{"zz_", "+", "nn_Integer"}], "]"}], ":>", 
              RowBox[{
               RowBox[{"gamma", "[", "zz", "]"}], 
               RowBox[{"Pochhammer", "[", 
                RowBox[{"zz", ",", "nn"}], "]"}]}]}], "}"}]}]}], ";", "\n", 
          "      ", 
          RowBox[{"KERN", "=", "kern"}], ";", "\n", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"gamma", "[", "zz_", "]"}], ":>", "1"}], "}"}]}]}], ";",
           "\n", 
          RowBox[{"kern", "=", 
           RowBox[{
            RowBox[{"kern", "/.", 
             RowBox[{"{", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], 
                RowBox[{"nn_", "+", "ww_Integer"}]], ":>", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], "ww"]}], "}"}]}], "/.", 
            RowBox[{"{", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "nn_"], ":>", "1"}], "}"}]}]}], 
          ";", "\n", 
          RowBox[{"nkern", "=", 
           RowBox[{"Table", "[", "\n", "\t\t\t", 
            RowBox[{
             RowBox[{"kern", "/.", 
              RowBox[{"Thread", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{
                 "d", ",", " ", "n1", ",", " ", "n2", ",", " ", "n3", ",", 
                  "s1", ",", "s2", ",", " ", "s3"}], "}"}], "->", 
                RowBox[{"Table", "[", 
                 RowBox[{
                  RowBox[{"Random", "[", "]"}], ",", 
                  RowBox[{"{", "7", "}"}]}], "]"}]}], "]"}]}], ",", 
             RowBox[{"{", "3", "}"}]}], "]"}]}], ";", "\n", "nkern"}], "  ", 
         ",", "\n", "\t\t", 
         RowBox[{"Print", "[", "\"\<Matching Error\>\"", "]"}]}], " ", 
        "]"}]}]}], " ", "]"}], ",", "\n", "\t\t", 
    RowBox[{"If", "[", 
     RowBox[{"$Notebooks", ",", 
      RowBox[{"Print", "[", "\"\<Off\>\"", "]"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Recurrence Relations Input and Recursion Function Generation", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.6.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", "pp_", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_", ",", "m1_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", "m2_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "pp", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], " ", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], "}"}]}], "]"}]}]}]], "Input"],

Cell[TextData[{
 "Most ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", 
    RowBox[{"...", " "}]], TraditionalForm]]],
 "equations below were entered by the palettes which show up when \
$PutPalettes is set to True.\nExamples of alternative (hand) input are  (50) \
, (201) -- (205)."
}], "Text"],

Cell[CellGroupData[{

Cell["Input (35)", "Subsection",
 CellTags->"T1.6.1.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "35"], "=", "\n", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["\[Nu]", "1"], "\[ScriptCapitalC]", " ", 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "\[Equal]", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "3"], "-", 
          SubscriptBox["\[Nu]", "5"]}], ")"}], 
        SubscriptBox["\[CapitalDelta]", "1"]}], "+", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", 
         RowBox[{"3", ",", "4", ",", "5"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True]}], ")"}]}], "-", 
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True]}]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True]}], ")"}]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}], "+", 
          SubscriptBox["\[Nu]", "5"], "-", 
          SubscriptBox["\[Nu]", "1"]}], ")"}]}], "+", "\n", "\t\t\t\t", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "6"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}], "+", 
          SubscriptBox["\[Nu]", "3"], "-", 
          SubscriptBox["\[Nu]", "1"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "35"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "35"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (35)", "Subsection",
 CellTags->"T1.6.2.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "351"], "=", 
  RowBox[{
   SubscriptBox["s", "35"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "1"}], ")"}]}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["1", 
    RowBox[{"+", " "}]], TraditionalForm]]],
 " (35_1)"
}], "Subsubsection",
 CellTags->"T1.6.2.1.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "351"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "351"], ",", "\"\<(35_1)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    RowBox[{"+", " "}]], TraditionalForm]]],
 " (35_2)"
}], "Subsubsection",
 CellTags->"T1.6.2.2.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "352"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["h", "351"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "2"}], ",", 
      RowBox[{"3", "\[LeftRightArrow]", "4"}]}], "]"}], ",", "\"\<(35_2)\>\"",
     ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["3", 
    RowBox[{"+", " "}]], TraditionalForm]]],
 " (35_3)"
}], "Subsubsection",
 CellTags->"T1.6.2.3.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "353"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["h", "351"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "3"}], ",", 
      RowBox[{"2", "\[LeftRightArrow]", "4"}]}], "]"}], ",", "\"\<(35_3)\>\"",
     ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["4", 
    RowBox[{"+", " "}]], TraditionalForm]]],
 " (35_4)"
}], "Subsubsection",
 CellTags->"T1.6.2.4.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "354"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["h", "351"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "4"}], ",", 
      RowBox[{"2", "\[LeftRightArrow]", "3"}]}], "]"}], ",", "\"\<(35_4)\>\"",
     ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "4"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (3501)", "Subsection",
 CellTags->"T1.6.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "[", 
   RowBox[{"m1", ",", "m5", ",", "m5", ",", "m4", ",", "m5", ",", "m5"}], 
   "]"}], "//", "Factor"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["m", "1"], "=", "0"}], "  ", ";", "  ", 
   RowBox[{
    SubscriptBox["m", "2"], "=", 
    RowBox[{
     SubscriptBox["m", "3"], "=", 
     SubscriptBox["m", "5"]}]}], ";", "   ", 
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SubsuperscriptBox["m", "5", "2"]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "3501"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "3"], "-", 
       SubscriptBox["\[Nu]", "5"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "5"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["5", "+"],
          FontWeight->"Bold"],
         Schiebe[5, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True], 
        InterpretationBox[
         RowBox[{
          StyleBox[
           SuperscriptBox["3", "+"],
           FontWeight->"Bold"], " "}],
         Schiebe[3, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "3501"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "3501"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "MakeFun (3501) \[CapitalDelta]=0 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["p", "2"], "=", 
     SuperscriptBox["m", "2"]}], ")"}], TraditionalForm]]]
}], "Subsection",
 CellTags->"T1.6.4.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "3501"], "=", 
  SubscriptBox["s", "3501"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "3501"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["s", "3501"], ",", "\"\<(3501)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{"PP", "===", 
      RowBox[{"m5", "^", "2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", ":>", "0"}], ",", 
      RowBox[{"m2", ":>", "m5"}], ",", 
      RowBox[{"m3", ":>", "m5"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (3502)", "Subsection",
 CellTags->"T1.6.5.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "[", 
   RowBox[{"m1", ",", "m5", ",", "m5", ",", "m4", ",", "m5", ",", "m5"}], 
   "]"}], "//", "Factor"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["m", "4"], "=", "0"}], "  ", ";", "  ", 
   RowBox[{
    SubscriptBox["m", "2"], "=", 
    RowBox[{
     SubscriptBox["m", "3"], "=", 
     SubscriptBox["m", "5"]}]}], ";", "   ", 
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SubsuperscriptBox["m", "5", "2"]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "3502"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "3"], "-", 
       SubscriptBox["\[Nu]", "5"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "5"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["5", "+"],
          FontWeight->"Bold"],
         Schiebe[5, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True], 
        InterpretationBox[
         RowBox[{
          StyleBox[
           SuperscriptBox["3", "+"],
           FontWeight->"Bold"], " "}],
         Schiebe[3, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "3502"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "3502"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "MakeFun (3502) \[CapitalDelta]=0 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["p", "2"], "=", 
     SuperscriptBox["m", "2"]}], ")"}], TraditionalForm]]]
}], "Subsection",
 CellTags->"T1.6.6.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "3502"], "=", 
  SubscriptBox["s", "3502"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "3502"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "3502"], ",", "\"\<(3502)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{"PP", "===", 
      RowBox[{"m5", "^", "2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m4", ":>", "0"}], ",", 
      RowBox[{"m2", ":>", "m5"}], ",", 
      RowBox[{"m3", ":>", "m5"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "MakeFun (3503) \[CapitalDelta]=0 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["p", "2"], "=", 
     SuperscriptBox["m", "2"]}], ")"}], TraditionalForm]]]
}], "Subsection",
 CellTags->"T1.6.7.1"],

Cell[BoxData[
 RowBox[{"Cayley", "[", 
  RowBox[{"m5", ",", "0", ",", "m3", ",", "m5", ",", "m5", ",", "m5"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "3503"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "3502"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}], ",", " ", 
    RowBox[{"3", "\[LeftRightArrow]", "4"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "3503"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "3503"], ",", "\"\<(3503)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{"PP", "===", 
      RowBox[{"m5", "^", "2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m2", ":>", "0"}], ",", 
      RowBox[{"m1", ":>", "m5"}], ",", 
      RowBox[{"m4", ":>", "m5"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "MakeFun (3504) \[CapitalDelta]=0 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["p", "2"], "=", 
     SuperscriptBox["m", "2"]}], ")"}], TraditionalForm]]]
}], "Subsection",
 CellTags->"T1.6.8.1"],

Cell[BoxData[
 RowBox[{"Cayley", "[", 
  RowBox[{"m5", ",", "m2", ",", "0", ",", "m5", ",", "m5", ",", "m5"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "3504"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "3502"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}], ",", " ", 
    RowBox[{"2", "\[LeftRightArrow]", "4"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "3504"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "3504"], ",", "\"\<(3504)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{"PP", "===", 
      RowBox[{"m5", "^", "2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m3", ":>", "0"}], ",", 
      RowBox[{"m1", ":>", "m5"}], ",", 
      RowBox[{"m4", ":>", "m5"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (41)", "Subsection",
 CellTags->"T1.6.9.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "41"], "=", "\n", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["\[Nu]", "5"], " ", "\[ScriptCapitalC]", " ", 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["5", "+"],
       FontWeight->"Bold"],
      Schiebe[5, "+"]], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}]]}], " ", "\[Equal]", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[CapitalDelta]", 
         RowBox[{"1", ",", "3", ",", "6"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"]], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"]]}], ")"}]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"]], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"]]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "3"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "5"]}]}], ")"}], 
        SubscriptBox["\[CapitalDelta]", "5"]}], "+", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"]], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"]]}], ")"}]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"]], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"]]}], "+", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "5"]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "4"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"]], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"]]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"]], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"]]}], ")"}]}], "+", 
          SubscriptBox["\[Nu]", "3"], "-", 
          SubscriptBox["\[Nu]", "5"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "41"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "41"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (41)", "Subsection",
 CellTags->"T1.6.10.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "41"], "=", 
  RowBox[{
   SubscriptBox["s", "41"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "5"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "5"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "41"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "41"], ",", "\"\<(41)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "5"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (43)", "Subsection",
 CellTags->"T1.6.11.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SuperscriptBox["p", "2"]]],
 " and ( ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["m", "5", "2"], "==", "0"}], TraditionalForm]]],
 "  &&  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "1", "2"], TraditionalForm]]],
 "==",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
 " && ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
 "== ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
 " ) \[DoubleLongRightArrow] \[CapitalDelta] == 0"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "43"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "3"], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "5"]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "\[Equal]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["5", "-"],
            FontWeight->"Bold"],
           Schiebe[5, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["5", "-"],
            FontWeight->"Bold"],
           Schiebe[5, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "43"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "43"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (44)", "Subsection",
 CellTags->"T1.6.12.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SuperscriptBox["p", "2"]]],
 " and ( ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["m", "5", "2"], "==", "0"}], TraditionalForm]]],
 "  &&  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "1", "2"], TraditionalForm]]],
 "==",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
 " && ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
 "== ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
 " ) \[DoubleLongRightArrow] \[CapitalDelta] == 0"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "44"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "4"], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "5"]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "\[Equal]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["5", "-"],
            FontWeight->"Bold"],
           Schiebe[5, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["5", "-"],
            FontWeight->"Bold"],
           Schiebe[5, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "44"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "44"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (45)", "Subsection",
 CellTags->"T1.6.13.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "45"], "=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "3", ",", "6"}]], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "\[Equal]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"6", ",", "1", ",", "3"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubscriptBox["u", 
         RowBox[{"1", ",", "3", ",", "6"}]], 
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True]}], ")"}]}], "+", " ", 
          RowBox[{
           SubscriptBox["\[Nu]", "4"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["4", "+"],
             FontWeight->"Bold"],
            Schiebe[4, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True]}], ")"}]}], "+", 
          RowBox[{"-", 
           SubscriptBox["\[CapitalSigma]", "5"]}], "-", 
          SubscriptBox["\[Nu]", "1"], "+", 
          RowBox[{"3", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True]}], ")"}]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True]}], "+", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "45"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "45"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (43) and (44)", "Subsection",
 CellTags->"T1.6.14.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "43"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["s", "43"], ",", "\"\<(43)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "+", 
       SubscriptBox["\[Nu]", "3"], "+", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "5"]}], "-", 
       RowBox[{"(", 
        RowBox[{"d", "/.", 
         RowBox[{"dd_Symbol", ":>", "4"}]}], ")"}]}], "=!=", "0"}], "]"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m5", ":>", "0"}], ",", 
      RowBox[{"m4", ":>", "m3"}], ",", 
      RowBox[{"m2", ":>", "m1"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "44"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["s", "44"], ",", "\"\<(44)\>\"", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m5", ":>", "0"}], ",", 
      RowBox[{"m4", ":>", "m3"}], ",", 
      RowBox[{"m2", ":>", "m1"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (50)", "Subsection",
 CellTags->"T1.6.15.1"],

Cell["\<\
Subscript[f, 50] = 
   HoldForm[TFR[d_, PP_, 
      {{(n1_)?PQ, m1_}, {(n2_)?PQ, m2_}, {(n3_)?PQ, m3_}, 
       {(n4_)?PQ, m4_}, {(n5_)?PQ, m5_}}] := 
     (TComment[\"(50)\", TFIC[d, PP, 
          {{n1, m1}, {n2, m2}, {n3, m3}, {n4, m4}, 
           {n5, m5}}]]; 
        Block[{r1 = 3 - d, r2 = 4 - d, 
          r3 = 
           -(m1^2*m2^2*m3^2) + m2^4*m3^2 + m2^2*m3^4 + 
            m1^4*m4^2 - m1^2*m2^2*m4^2 - 
            m1^2*m3^2*m4^2 - m2^2*m3^2*m4^2 + 
            m1^2*m4^4 + m1^2*m2^2*m5^2 - 
            m2^2*m3^2*m5^2 - m1^2*m4^2*m5^2 + 
            m3^2*m4^2*m5^2 + m1^2*m3^2*PP - 
            m2^2*m3^2*PP - m1^2*m4^2*PP + m2^2*m4^2*PP - 
            m1^2*m5^2*PP - m2^2*m5^2*PP - m3^2*m5^2*PP - 
            m4^2*m5^2*PP + m5^4*PP + m5^2*PP^2, 
          r4 = -4 + d, r5 = -3 + d, r6 = -7 + 2*d, 
          r7 = -(m2^2*r4) + m3^2*r5 - m5^2*r5 + m4^2*r6, 
          r8 = -10 + 3*d, r9 = -m3^2 + m4^2, 
          r10 = -11 + 3*d, 
          r11 = -(m4^2*r10) - 3*m5^2*r5 + m3^2*r6, 
          r12 = 
           m2^2*r11 + m5^4*r5 + m2^4*r8 + m4^2*r4*r9 + 
            m5^2*r5*r9, r13 = -6 + d, 
          r14 = -(m4^2*r13) + m5^2*r5, 
          r15 = -3*m3^2 - m4^2*r10 + m5^2*r5, 
          r16 = 
           m3^2*r14 + m2^2*r15 + m2^4*r4 - 2*m3^4*r5 - 
            3*m4^2*m5^2*r5 + m5^4*r5 + m4^4*r8, 
          r17 = m3^2 + 2*m4^2, r18 = -1 + d, 
          r19 = -(m3^2*r13) - 2*m4^2*r18 + 2*m5^2*r5, 
          r20 = 
           2*m2^4 - 3*m3^4 + d*m3^4 - 3*m3^2*m4^2 + 
            2*m4^4 + m2^2*r19 - 2*m5^4*r5 + m5^2*r17*r5, 
          r21 = 2 + d, r22 = 
           2*m5^2 - m4^2*r21 + m2^2*r4 - m3^2*r4, 
          r23 = -7 + d, r24 = 6*m4^2 - 4*m5^2, 
          r25 = 3*m3^2 + m4^2*r18 + m5^2*r23, 
          r26 = 
           -6*m4^4 - 2*m4^2*m5^2*r23 + m3^2*r24 + 
            2*m2^2*r25 - 2*m2^4*r4 - m3^4*r4, 
          r27 = m3^4 + m3^2*m4^2 - 2*m4^4, 
          r28 = m3^2 + m4^2*r23, 
          r29 = -2*m4^2*r18 + m3^2*r21 + 2*m5^2*r23, 
          r30 = -6*m2^4 + 2*m5^2*r28 - m2^2*r29 + r27*r4, 
          r31 = m1^4*r22 + m1^2*r26 + m3^2*r30 + m1^6*r4, 
          r32 = m3 - m4, r33 = m3 + m4, 
          r34 = 2*m3^2 + m4^2, r35 = -17 + 3*d, 
          r36 = m3^2*r23 + m4^2*r35, r37 = -9 + d, 
          r38 = 17 - 3*d, 
          r39 = 5*m3^2 + m4^2*r37 + m5^2*r38, 
          r40 = 1 + d, r41 = 
           5*m4^2 - m5^2*r23 + m2^2*r4 - m3^2*r40, 
          r42 = 
           m5^4*r35 - m5^2*r36 + m2^2*r39 - 2*m1^4*r4 + 
            m2^4*r4 - r32*r33*r34*r4 + m1^2*r41, 
          r43 = m1^2 - m2^2 + m3^2 - m4^2, 
          r44 = 2*m5^2*r13 + r4*r43, 
          r45 = 
           m3^4*r12 + m1^4*r16 + m1^2*m3^2*r20 + 
            PP*r31 + PP^2*r42 + PP^3*r44 + m1^6*r7, 
          r46 = m2^2 + m4^2, r47 = 13 - 3*d, 
          r48 = m4^2*r47 - m5^2*r5 + m3^2*r6, 
          r49 = m5^4*r5 + m5^2*r32*r33*r5 + m4^2*r4*r9, 
          r50 = -18 + 4*d, r51 = -(m4^2*r13) - 3*m5^2*r5, 
          r52 = 
           2*m3^4*r5 + m4^2*m5^2*r5 + m5^4*r5 + 
            m4^4*r50 + m3^2*r51, 
          r53 = -3*m4^2 + 2*m5^2*r5, 
          r54 = 
           m4^4*r47 + 2*m3^4*r5 + m4^2*m5^2*r5 - 
            2*m5^4*r5 + m3^2*r53, 
          r55 = m4^2*r13 + 6*m3^2*r5 - 2*m5^2*r5, 
          r56 = 3*m4^2 + m3^2*r5 - m5^2*r5, 
          r57 = 7 - 2*d, 
          r58 = m3^2*r5 + 3*m5^2*r5 + m4^2*r57, 
          r59 = 
           m2^6*r4 + m2^2*m4^2*r55 + m2^4*r56 + m4^4*r58, 
          r60 = m4^2 - m3^2*r5, r61 = 10 - 3*d, 
          r62 = 2*m5^2 + 3*m4^2*r4 + m3^2*r61, 
          r63 = m4^2 + m5^2*r5, 
          r64 = 
           -4*m4^2*m5^2 + 3*m4^4*r4 - 2*m3^4*r5 + 
            2*m3^2*r63, r65 = m4^2 + m3^2*r5 - m5^2*r5, 
          r66 = 2*m3^2*r5 + 2*m5^2*r5 + m4^2*r61, 
          r67 = 3*m2^4*r4 + 2*m2^2*r65 + m4^2*r66, 
          r68 = 
           -3*m2^6*r4 - 2*m1^4*m4^2*r5 + 
            2*m4^2*m5^2*r60 + m2^4*r62 + m2^2*r64 + 
            m1^2*r67 - 3*m4^4*r4*r9, r69 = -13 + 3*d, 
          r70 = m3^2*r5 + m4^2*r69, 
          r71 = m4^2 - 3*m2^2*r4 - m3^2*r5 + m5^2*r5, 
          r72 = -5 + d, r73 = m3^2 + m5^2*r69 + m4^2*r72, 
          r74 = 
           3*m2^4*r4 - m5^4*r5 + m5^2*r70 + m1^2*r71 + 
            m2^2*r73 + 3*m4^2*r4*r9, 
          r75 = m1^2 - m2^2 + m3^2 - m4^2 - 2*m5^2, 
          r76 = 
           m2^8*r4 + m2^6*r48 + m4^4*r49 + 
            2*m1^4*m4^2*r46*r5 + m2^4*r52 + 
            m2^2*m4^2*r54 - m1^2*r59 + PP*r68 + 
            PP^2*r74 + PP^3*r4*r75, 
          r77 = m3^2 - m4^2 + m5^2, 
          r78 = -2*m2^2 + m4^2 + m5^2, 
          r79 = -2*m1^2 - m3^2 - m4^2 + m5^2, 
          r80 = 
           m1^4 + PP^2 - m1^2*r77 + m3^2*r78 + PP*r79, 
          r81 = m3^2 + m4^2, r82 = m3^2 + m4^2 + m5^2, 
          r83 = m1^2 + m3^2 - m5^2, 
          r84 = 
           m1^4 + 2*m3^2*m4^2 - m4^4 + PP^2 - m2^2*r77 + 
            m5^2*r81 - m1^2*r82 - 2*PP*r83, 
          r85 = m1 - m3, r86 = -PP + r85^2, 
          r87 = m1 + m3, r88 = -PP + r87^2, 
          r89 = m2 - m4, r90 = m2 + m4, 
          r91 = -m1^2 - m2^2 - m3^2 - m4^2 + 2*m5^2, 
          r92 = PP^2 + r85*r87*r89*r90 + PP*r91, 
          r93 = m2^2 - 3*m4^2, r94 = m4 - m5, 
          r95 = m4 + m5, r96 = 2*m3^2 - m4^2 + m5^2, 
          r97 = -m1^2 - m2^2 - 2*m3^2 + 3*m5^2, 
          r98 = 
           PP^2 + m1^2*r93 + m2^2*r94*r95 + m4^2*r96 + 
            PP*r97, r99 = -2 + d, r100 = -m1^2 + m2^2, 
          r101 = m2^2 + m1^2*r72, 
          r102 = m3^2*r101 - 2*m3^4*r4 + m2^2*r100*r4, 
          r103 = 9 - 2*d, r104 = m2^2*r23 + m3^2*r72, 
          r105 = 
           2*m1^4 + m2^4*r103 + m1^2*r104 + m2^2*m3^2*r5, 
          r106 = -9 + 2*d, r107 = m1 - m2, 
          r108 = m1 + m2, 
          r109 = -(r106*r107*r108) + m3^2*r4, 
          r110 = 
           m2^2*r102 + m4^2*r105 + m4^4*r109 - m4^6*r4, 
          r111 = m1^2 + m2^2, 
          r112 = m2^2*r4 - 2*m1^2*r72, r113 = -19 + 5*d, 
          r114 = m3^2*r112 + m3^4*r113 + m2^2*r111*r4, 
          r115 = m3^2*r5 + m2^2*r6, 
          r116 = 
           2*m1^4 + 2*m1^2*r115 + m2^4*r13 + m3^4*r72 - 
            2*m2^2*m3^2*r99, 
          r117 = m2^2*r106 + m3^2*r4 - m1^2*r99, 
          r118 = 
           m2^2*r114 - m4^2*r116 + m4^4*r117 - m4^6*r6, 
          r119 = -(m2^2*r5) + 2*m3^2*r57 + m1^2*r72, 
          r120 = m1^2*r18 + 2*m3^2*r4 - m2^2*r5, 
          r121 = m2^2*r119 + m4^2*r120 + 2*m4^4*r5, 
          r122 = m3^2*r5 + m4^2*r72, 
          r123 = -2*m2^2*r4 - m4^2*r69 + m3^2*r72, 
          r124 = 
           m2^2*r122 + m1^2*r123 + 2*m2^4*r4 + 
            2*m4^2*r32*r33*r4, 
          r125 = -3*m4^2 + m2^2*r4 - m3^2*r72, 
          r126 = m3^2 - m4^2*r72, 
          r127 = 
           -2*m3^2*m4^2 - 2*m1^2*r125 - 2*m2^2*r126 + 
            m4^4*r18 - 4*m2^4*r4 + m3^4*r72, 
          r128 = 
           -(m2^2*r18) + 2*m4^2*r5 + m1^2*r72 + 
            2*m3^2*r72, 
          r129 = 
           m5^2*r127 - m5^4*r128 - r124*r32*r33 + 
            m5^6*r72, r130 = -m1^2 + m2^2 + m3^2 - m4^2, 
          r131 = -14 + 3*d, 
          r132 = 
           -(m3^2*r13) + m4^2*r131 + m1^2*r4 + 5*m2^2*r4, 
          r133 = -2*m5^4 + m5^2*r132 + r130*r32*r33*r4, 
          r134 = 
           m5^2*r118 + m5^4*r121 + PP*r129 + PP^2*r133 + 
            r110*r32*r33 - 2*m5^2*PP^3*r4 + 
            m5^6*r5*r89*r90, r135 = m3 - m4 - m5, 
          r136 = m3 + m4 - m5, r137 = m3 - m4 + m5, 
          r138 = m3 + m4 + m5, r139 = -10 + d, 
          r140 = 13 - 2*d, r141 = d*m4^2 + 2*m3^2*r5, 
          r142 = -8 + d, 
          r143 = m4^2*r142 - m3^2*r99 + m5^2*r99, 
          r144 = 
           m3^2*m4^2*r139 + m4^4*r140 - m5^2*r141 + 
            m2^2*r143 + m3^4*r5 + m5^4*r5, 
          r145 = 3*m3^4 - 12*m3^2*m4^2 + 7*m4^4, 
          r146 = 3*m3^2 + 4*m4^2, r147 = m3^2 - 11*m4^2, 
          r148 = 
           7*m3^4 + 10*m3^2*m4^2 - 17*m4^4 - 5*m5^4 - 
            2*m5^2*r147, r149 = m2^2 + 2*m3^2 + m4^2, 
          r150 = 
           -m2^4 + m2^2*m3^2 + m3^4 + 3*m2^2*m4^2 + 
            m3^2*m4^2 - m4^4 + m5^4 - m5^2*r149, 
          r151 = 
           3*m3^6 + 6*m3^4*m4^2 - 17*m3^2*m4^4 + 
            8*m4^6 + 3*m5^6 - m5^2*r145 - m5^4*r146 + 
            m2^2*r148 - d*r150*r77 + 6*m2^4*r94*r95, 
          r152 = -(m3^2*m4) + m4^3, r153 = 11 - 2*d, 
          r154 = -(d*m3^2*m4^2) + m4^4*r153 + m3^4*r5, 
          r155 = m4^2 + m3^2*r5, 
          r156 = r142*r32*r33 + m5^2*r99, 
          r157 = 11 - 3*d, r158 = -8 + 3*d, 
          r159 = m5^2*r158 + m4^2*r21, 
          r160 = 
           -3*m3^4 + 2*m4^2*m5^2*r106 + m5^4*r157 + 
            m3^2*r159 - m4^4*r18, 
          r161 = 
           -2*m2^6 + m5^2*r154 - 2*m5^4*r155 + 
            m2^4*r156 + m2^2*r160 + r152^2*r4 + m5^6*r5, 
          r162 = m4^2*r23 - 3*m3^2*r72, 
          r163 = -4*m2^2 + m4^2*r18 + m3^2*r23, 
          r164 = -m3^4 + m4^4, 
          r165 = 
           -(m2^4*r13) + m2^2*r162 + m1^2*r163 + 
            r164*r4 + m1^4*r99, r166 = -19 + 4*d, 
          r167 = m4^2 + m3^2*r72, r168 = m3^2 + m4^2*r69, 
          r169 = -2*r168 + 4*m2^2*r4, 
          r170 = 
           -m3^4 + m2^4*r13 - m4^4*r166 + 2*m2^2*r167 + 
            m1^2*r169 + 2*m3^2*m4^2*r72 + m1^4*r99, 
          r171 = m1^2 - m2^2 + 4*m4^2, 
          r172 = 
           -(m5^2*r170) + r165*r32*r33 + m5^6*r72 - 
            m5^4*r171*r72, r173 = m2^2 + m3^2, 
          r174 = -(m4^2*r13) + r131*r173 + m1^2*r8, 
          r175 = m5^2*r174 - 2*m5^4*r4 + r130*r32*r33*r4, 
          r176 = 
           2*m1^6*m4^2 + m1^4*r144 + m1^2*r151 + 
            m3^2*r161 + PP*r172 + PP^2*r175 - 
            2*m5^2*PP^3*r4, r177 = m3^2 + m4^2 - m5^2, 
          r178 = -2*m1^2 + m3^2 - m4^2 + m5^2, 
          r179 = -m3^2 + m4^2 + m5^2, 
          r180 = m2^2*r177 + m4^2*r178 + PP*r179, 
          r181 = -2*m2^2 - m3^2 + m4^2 + m5^2, 
          r182 = m1^2*r177 + m3^2*r181 + PP*r77, 
          r183 = -(m2^2*r13) - 2*m3^2*r5, 
          r184 = 
           m1^2*r183 - 2*m3^4*r4 + m2^4*r57 + m1^4*r72 - 
            m2^2*m3^2*r99, r185 = 3*m2^2*r4 + m3^2*r72, 
          r186 = 
           m2^4*r106 + 2*m1^2*r185 + m1^4*r40 - 
            2*m2^2*m3^2*r6, r187 = r111*r13 - m3^2*r4, 
          r188 = 
           m2^2*r184 + m4^2*r186 - m4^4*r187 + m4^6*r4, 
          r189 = 2*m1^2 + m3^2*r10 + 2*m2^2*r5, 
          r190 = m2^2*r5 + m3^2*r72 + 2*m1^2*r99, 
          r191 = m2^2*r189 - m4^2*r190 - m4^4*r5, 
          r192 = -(m3^2*r5) - m4^2*r72, 
          r193 = 2*m2^2*r4 + m4^2*r69 - m3^2*r72, 
          r194 = 
           m2^2*r192 + m1^2*r193 - 2*m2^4*r4 + 
            2*m4^2*r4*r9, r195 = m2^2 + m4^2 + m3^2*r72, 
          r196 = -(m3^2*r6) - m4^2*r72, 
          r197 = 
           m2^4*r157 + 2*m1^2*r195 + 2*m2^2*r196 + 
            2*m4^2*r17*r4 + m1^4*r72, 
          r198 = 
           m4^2*r10 - 2*m2^2*r5 + 2*m1^2*r72 + m3^2*r72, 
          r199 = 
           r107*r108*r194 + m5^2*r197 - m5^4*r198 + 
            m5^6*r72, r200 = m1^2 - m2^2 - m3^2 + m4^2, 
          r201 = m1^2 + m2^2 + m3^2 + 5*m4^2, 
          r202 = 2*m5^4 - r107*r108*r200 - m5^2*r201, 
          r203 = 
           r107*r108*r110 - m5^2*r188 - m5^4*r191 + 
            PP*r199 + 2*m5^2*PP^3*r4 + PP^2*r202*r4 + 
            m5^6*r5*r89*r90, r204 = m4^2 + m5^2, 
          r205 = -2*m5^2*r5 + m4^2*r99, r206 = 18 - 5*d, 
          r207 = m4^2*r206 + m5^2*r8 - 2*m3^2*r99, 
          r208 = 
           m2^2*r207 + 2*m2^4*r4 - m3^4*r5 - 
            m3^2*r204*r5 + r205*r94*r95, 
          r209 = 2*m3^2*m4^2 + m4^4*r142 + 2*m3^4*r5, 
          r210 = -(m4^2*r18) + m3^2*r5, r211 = 3 - 2*d, 
          r212 = m5^2 + m4^2*r157 + m3^2*r211, 
          r213 = m4^2*r106 + m3^2*r4, 
          r214 = 
           -(d*m3^4) + 10*m3^2*m4^2 - 2*d*m3^2*m4^2 - 
            2*m4^4 + d*m4^4 + 2*m5^2*r213 - 2*m5^4*r4, 
          r215 = 
           d*m3^2*m4^4 - 2*m4^6 - m5^2*r209 + 
            m5^4*r210 + m2^4*r212 + m2^2*r214 + 
            m2^6*r4 + m5^6*r5 - m3^4*m4^2*r99, 
          r216 = m4^2*r13 + m3^2*r99, 
          r217 = m3^2*r5 - m4^2*r72, r218 = 13 - 4*d, 
          r219 = 3*m3^2 + m5^2*r218 + m4^2*r23, 
          r220 = m4^2*r142 + m5^2*r61, 
          r221 = 
           2*m3^4 + 2*m4^2*m5^2 - m4^4*r13 + m3^2*r220 + 
            2*m5^4*r6, r222 = 
           m4^2*m5^2*r216 + m5^4*r217 + m2^4*r219 + 
            m2^2*r221 - m5^6*r5 + m2^6*r99, 
          r223 = -(m3^2*r10) + m4^2*r5, 
          r224 = 
           4*m3^2*m4^2 + m1^2*r122 + m4^4*r13 + 
            m2^2*r223 + m1^4*r4 - m2^4*r4 - m3^4*r99, 
          r225 = -m3^2 + m2^2*r72 - m4^2*r72, 
          r226 = -2*m4^2*r106 + 2*m3^2*r69, 
          r227 = 
           m1^4*r106 + m2^4*r106 - m4^4*r13 - 
            2*m1^2*r225 + m2^2*r226 - 4*m3^2*m4^2*r4 - 
            m3^4*r99, r228 = m3^2 + 3*m4^2, 
          r229 = 
           r107*r108*r224 - m5^2*r227 + m5^6*r72 - 
            m5^4*r228*r72, 
          r230 = m4^2*r131 + m1^2*r4 + m2^2*r4 + m3^2*r8, 
          r231 = 
           2*m5^4*r106 - m5^2*r230 - r107*r108*r200*r4, 
          r232 = 
           m1^4*r208 - m1^2*r215 - m3^2*r222 + PP*r229 + 
            PP^2*r231 + 2*m5^2*PP^3*r4 + m1^6*r7, 
          r233 = -2*m3^2 + m4^2, 
          r234 = -m2^2 + m4^2 - 2*m5^2, 
          r235 = -m1^2 + m2^2 + m5^2, 
          r236 = 
           m1^4 + m5^4 + m2^2*r233 + m1^2*r234 + 
            PP*r235 - m5^2*r46, r237 = -m2^2 + m5^2, 
          r238 = m2^2 + m3^2 - 2*m4^2 + m5^2, 
          r239 = -m1^2 + m2^2 - m5^2, 
          r240 = m1^4 + m3^2*r237 - m1^2*r238 + PP*r239, 
          r241 = m3^2 + 3*m4^2 - m5^2, 
          r242 = -3*m3^2 - m4^2 + m5^2, 
          r243 = 
           m5^4 - m5^2*r228 + m1^2*r241 + m2^2*r242 + 
            2*PP*r32*r33 + 2*m4^2*r9, 
          r244 = 2*m1^2 + m2^2 + m4^2, 
          r245 = -2*m1^2 - m3^2 - m4^2 + 3*m5^2, 
          r246 = 
           m1^4 - 3*m2^2*m3^2 + m5^4 + PP^2 + 
            m4^2*r173 - m5^2*r244 + PP*r245, 
          r247 = -2*m3^2 + m4^2 + m5^2, 
          r248 = 
           -m2^4 - m4^2*m5^2 + PP*r235 + m2^2*r247 + 
            m1^2*r46, r249 = m2^2*r153 + m1^2*r5, 
          r250 = 2*m1^2 + m2^2*r4, 
          r251 = m2^2*r13 + m1^2*r139, 
          r252 = r107*r108*r250 + m3^2*r251 - m3^4*r4, 
          r253 = m1^2*r140 + 2*m3^2*r4 + m2^2*r72, 
          r254 = 
           m3^4*r249 + m4^2*r252 + m4^4*r253 - m4^6*r4 + 
            m2^2*m3^2*r100*r99, r255 = 2*m3^2 + m4^2*r13, 
          r256 = -2*m3^2*r5 + m2^2*r99 - m4^2*r99, 
          r257 = -11 + 2*d, 
          r258 = 
           -(m4^4*r106) + 2*m3^2*m4^2*r23 - m3^4*r257, 
          r259 = 4*m4^2 + m3^2*r13, 
          r260 = 
           -2*m2^4 + 6*m3^2*m4^2 + 2*m2^2*r259 + 
            m4^4*r37 + m3^4*r5, 
          r261 = 
           -5*m3^4*m4^2 + m2^4*r255 + m1^4*r256 + 
            m2^2*r258 + m1^2*r260 - m2^6*r4 + m3^6*r5 + 
            m4^6*r6 - 3*m3^2*m4^4*r72, 
          r262 = m3^4 + m3^2*m4^2 + m4^4, 
          r263 = 
           m1^4 + m2^4 - 2*r262 + m1^2*r81 + m2^2*r81, 
          r264 = m1^2 + m2^2 - m3^2 - m4^2, 
          r265 = -(r13*r32*r33) + m2^2*r99, 
          r266 = 
           -2*m1^2*r265 - r32^2*r33^2*r4 + 
            2*m2^2*r13*r9 + m1^4*r99 + m2^4*r99, 
          r267 = m3^2*r13 + m4^2*r4, r268 = -16 + 3*d, 
          r269 = m3^2*r13 - m4^2*r268 + m2^2*r4, 
          r270 = 
           2*m2^2*r267 + 2*m1^2*r269 + 
            r13*r228*r32*r33 + m2^4*r61 + m1^4*r99, 
          r271 = 2*m2^2 + m4^2*r23 - m3^2*r72, 
          r272 = 
           -(m5^2*r270) - 2*m5^4*r271 + r266*r32*r33, 
          r273 = 2*m3^2 - 2*m4^2 - m5^2, 
          r274 = m1^2*r5 - m2^2*r5 + r273*r72, 
          r275 = 
           r200*r254 + m5^2*r261 + PP*r272 + 
            2*m5^2*PP^2*r274 + m5^4*r263*r5 - 
            m5^6*r264*r5, 
          r276 = 
           -2*m2^2*m3^2 - 2*m3^4 + 3*m3^2*m4^2 - m4^4 + 
            3*m3^2*m5^2 + 2*m4^2*m5^2 - m5^4 + 
            m1^2*r177 + PP*r77, 
          r277 = -(m4^2*r106) + m2^2*r4 - m3^2*r5, 
          r278 = 
           -2*m3^4 - m3^2*m4^2*r13 + m2^2*r196 + 
            m2^4*r4 + m4^4*r4, 
          r279 = m4^2*r131 + m3^2*r8, 
          r280 = 
           m2^2*r279 - 2*m2^4*r4 + m4^2*r32*r33*r99, 
          r281 = m1^4*r277 + m2^2*r278 + m1^2*r280, 
          r282 = 11*m3^2 - 3*d*m3^2 + 9*m4^2 - 2*d*m4^2, 
          r283 = 15 - 4*d, 
          r284 = m4^2*r106 + m2^2*r283 + m3^2*r5, 
          r285 = m4^4*r13 + 4*m3^2*m4^2*r5 - m3^4*r99, 
          r286 = -m4^2 + m3^2*r5, 
          r287 = -(m4^2*r5) + m3^2*r6, 
          r288 = 2*m2^2*r287 + m2^4*r72 - 2*r286*r81, 
          r289 = 
           -2*m3^2*m4^4 + m2^4*r282 + m1^4*r284 + 
            m2^2*r285 + m1^2*r288 - m4^6*r4 + m1^6*r5 + 
            m2^6*r6 + m3^4*m4^2*r99, 
          r290 = 2*m2^2 - m3^2 - m4^2, 
          r291 = 
           2*m1^4 + 2*m2^4 - m3^4 - m4^4 + m1^2*r290 - 
            m2^2*r81, r292 = m2^2 - m3^2 + m4^2, 
          r293 = 
           m1^4*r4 + m2^4*r4 - 2*m1^2*r292*r4 + 
            2*m2^2*r4*r9 - r32^2*r33^2*r99, 
          r294 = m3^2*r13 - m4^2*r131, 
          r295 = m2^2*r13 - r4*r81, 
          r296 = 
           m1^4*r131 + 2*m2^2*r294 - 2*m1^2*r295 - 
            2*m3^2*m4^2*r4 + m4^4*r8 - m2^4*r99 - 
            m3^4*r99, r297 = 2*m4^2 - m2^2*r5 + m1^2*r72, 
          r298 = 
           r107*r108*r293 - m5^2*r296 + 2*m5^4*r297, 
          r299 = -2*m5^2*r32*r33*r5 + 2*m5^4*r72, 
          r300 = 
           r130*r281 - m5^2*r289 + PP*r298 + PP^2*r299 - 
            m5^6*r264*r5 + m5^4*r291*r5, 
          r301 = 
           -2*m2^2*m3^2 - m3^2*m4^2 + m4^4 + m5^4 - 
            m5^2*r17 + m1^2*r177 + PP*r77, 
          r302 = m2^2 + 2*m3^2 - 3*m4^2, 
          r303 = -m1^2 + m2^2 - 3*m5^2, 
          r304 = 
           m1^4 - m2^2*m4^2 - m5^4 + m5^2*r149 - 
            m1^2*r302 + PP*r303, 
          r305 = 
           -((r45*TBI[d, PP, {{1, m4}, {1, m2}}]*
                 TBI[d, PP, {{2, m1}, {1, m3}}])/(4*r3)) 
+ (r76*TBI[d, PP, {{1, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m2}, {1, m4}}])/(4*r3) - 
            (r80*TBI[d, PP, {{2, m1}, {1, m3}}]*
               TBI[d, PP, {{2, m2}, {1, m4}}])/2 - 
            (r45*TBI[d, PP, {{1, m4}, {1, m2}}]*
               TBI[d, PP, {{2, m3}, {1, m1}}])/(4*r3) - 
            (r84*TBI[d, PP, {{2, m2}, {1, m4}}]*
               TBI[d, PP, {{2, m3}, {1, m1}}])/2 - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{2, m3}, {2, m1}}] + 
            (r76*TBI[d, PP, {{1, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/(4*r3) - 
            (r92*TBI[d, PP, {{2, m1}, {1, m3}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/2 - 
            (r98*TBI[d, PP, {{2, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/2 - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{3, m1}, {1, m3}}] - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{3, m3}, {1, m1}}] - 
            r3*TFI[r99, PP, 
              {{1, m1}, {1, m2}, {1, m3}, {1, m4}, 
               {1, m5}}] - 
            (r134*TVI[d, PP, 
                {{1, m5}, {1, m1}, {2, m4}, {1, m3}}])/
             (4*r3) - (r135*r136*r137*r138*
               TVI[d, PP, 
                {{1, m5}, {1, m1}, {2, m4}, {2, m3}}])/2 
- (r176*TVI[d, PP, {{1, m5}, {1, m2}, {2, m3}, 
                 {1, m4}}])/(4*r3) + 
            (r180*TVI[d, PP, 
                {{1, m5}, {1, m2}, {2, m3}, {2, m4}}])/2 
- r182*TVI[d, PP, {{1, m5}, {1, m2}, {3, m3}, 
               {1, m4}}] + 
            (r203*TVI[d, PP, 
                {{1, m5}, {1, m3}, {2, m2}, {1, m1}}])/
             (4*r3) + (r232*
               TVI[d, PP, 
                {{1, m5}, {1, m4}, {2, m1}, {1, m2}}])/
             (4*r3) + (r236*
               TVI[d, PP, 
                {{1, m5}, {1, m4}, {2, m1}, {2, m2}}])/2 
+ r240*TVI[d, PP, {{1, m5}, {1, m4}, {3, m1}, 
               {1, m2}}] + 
            (r45*TVI[d, PP, 
                {{1, m5}, {2, m1}, {1, m4}, {1, m3}}])/
             (4*r3) + (r86*r88*
               TVI[d, PP, 
                {{1, m5}, {2, m1}, {1, m4}, {2, m3}}])/2 
+ (r92*TVI[d, PP, {{1, m5}, {2, m1}, {2, m4}, 
                 {1, m3}}])/2 - 
            (r76*TVI[d, PP, 
                {{1, m5}, {2, m2}, {1, m3}, {1, m4}}])/
             (4*r3) - (r243*
               TVI[d, PP, 
                {{1, m5}, {2, m2}, {2, m3}, {1, m4}}])/2 
+ (r45*TVI[d, PP, {{1, m5}, {2, m3}, {1, m2}, 
                 {1, m1}}])/(4*r3) + 
            (r86*r88*TVI[d, PP, 
                {{1, m5}, {2, m3}, {1, m2}, {2, m1}}])/2 
+ (r246*TVI[d, PP, {{1, m5}, {2, m3}, {2, m2}, 
                 {1, m1}}])/2 - 
            (r76*TVI[d, PP, 
                {{1, m5}, {2, m4}, {1, m1}, {1, m2}}])/
             (4*r3) + (r248*
               TVI[d, PP, 
                {{1, m5}, {2, m4}, {2, m1}, {1, m2}}])/2 
+ r86*r88*TVI[d, PP, {{1, m5}, {3, m1}, {1, m4}, 
               {1, m3}}] + 
            r86*r88*TVI[d, PP, 
              {{1, m5}, {3, m3}, {1, m2}, {1, m1}}] + 
            (r275*TVI[d, PP, 
                {{2, m5}, {1, m1}, {1, m4}, {1, m3}}])/
             (4*r3) + (r276*
               TVI[d, PP, 
                {{2, m5}, {1, m1}, {1, m4}, {2, m3}}])/2 
- (r180*TVI[d, PP, {{2, m5}, {1, m1}, {2, m4}, 
                 {1, m3}}])/2 - 
            (r275*TVI[d, PP, 
                {{2, m5}, {1, m2}, {1, m3}, {1, m4}}])/
             (4*r3) + (r180*
               TVI[d, PP, 
                {{2, m5}, {1, m2}, {1, m3}, {2, m4}}])/2 
- (r276*TVI[d, PP, {{2, m5}, {1, m2}, {2, m3}, 
                 {1, m4}}])/2 - 
            (r300*TVI[d, PP, 
                {{2, m5}, {1, m3}, {1, m2}, {1, m1}}])/
             (4*r3) - (r240*
               TVI[d, PP, 
                {{2, m5}, {1, m3}, {1, m2}, {2, m1}}])/2 
- (r236*TVI[d, PP, {{2, m5}, {1, m3}, {2, m2}, 
                 {1, m1}}])/2 + 
            (r300*TVI[d, PP, 
                {{2, m5}, {1, m4}, {1, m1}, {1, m2}}])/
             (4*r3) + (r236*
               TVI[d, PP, 
                {{2, m5}, {1, m4}, {1, m1}, {2, m2}}])/2 
+ (r240*TVI[d, PP, {{2, m5}, {1, m4}, {2, m1}, 
                 {1, m2}}])/2 + 
            (r182*TVI[d, PP, 
                {{2, m5}, {2, m1}, {1, m4}, {1, m3}}])/2 
- (r301*TVI[d, PP, {{2, m5}, {2, m2}, {1, m3}, 
                 {1, m4}}])/2 - 
            (r304*TVI[d, PP, 
                {{2, m5}, {2, m3}, {1, m2}, {1, m1}}])/2 
+ (r248*TVI[d, PP, {{2, m5}, {2, m4}, {1, m1}, 
                 {1, m2}}])/2, r306}, 
         r306 = Factor /@ r305/(PP*r1*r2); r306]) /; 
         n1 === 1 && n2 === 1 && n3 === 1 && n4 === 1 && n5 === 1 &&  
       MatchQ[d, _Symbol + _Integer?Positive] && PP =!= 0 &&
       Cayley[m1, m2, m3, m4, m5, Sqrt[PP]] =!= 0]; \
\>", "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (51)", "Subsection",
 CellTags->"T1.6.16.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "51"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "3", ",", "4"}]], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"1", ",", "3", ",", "4"}]], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "51"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "51"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (51)", "Subsection",
 CellTags->"T1.6.17.1"],

Cell[BoxData[
 SubscriptBox["s", "51"]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "51"], "=", 
  RowBox[{
   SubscriptBox["s", "51"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "51"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "51"], ",", "\"\<(51)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "3", ",", "4"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "51"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (52)", "Subsection",
 CellTags->"T1.6.18.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "52"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"2", ",", "4", ",", "6"}]], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "4", "2"], 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", 
           SubsuperscriptBox["m", "2", "2"]}], "-", 
          SubscriptBox["u", 
           RowBox[{"4", ",", "2", ",", "6"}]]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[CapitalSigma]", "4"], "+", "2"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"2", ",", "4", ",", "6"}]], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"3", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "4", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "4"], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "52"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "52"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (52)", "Subsection",
 CellTags->"T1.6.19.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "52"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["s", "52"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "2"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "52"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "52"], ",", "\"\<(52)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"2", ",", "4", ",", "6"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "52"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (53)", "Subsection",
 CellTags->"T1.6.20.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "53"], "=", 
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{
     SubscriptBox["s", "51"], ",", 
     RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "53"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "53"], ",", "\"\<(53)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"3", ",", "1", ",", "4"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "53"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (54)", "Subsection",
 CellTags->"T1.6.21.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "54"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "3", "2"], 
     SubscriptBox["\[Nu]", "3"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["3", "+"],
       FontWeight->"Bold"],
      Schiebe[3, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         SubscriptBox["\[Nu]", "1"], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "54"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "54"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (54)", "Subsection",
 CellTags->"T1.6.22.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "54"], "=", 
  RowBox[{
   SubscriptBox["s", "54"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "54"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "54"], ",", "\"\<(54)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", " ", 
      RowBox[{"m3", "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "54"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (55)", "Subsection",
 CellTags->"T1.6.23.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "55"], "=", 
   RowBox[{
    RowBox[{"2", " ", 
     SubsuperscriptBox["m", "4", "2"], 
     SubscriptBox["\[Nu]", "4"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["4", "+"],
       FontWeight->"Bold"],
      Schiebe[4, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"3", ",", "1", ",", "4"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"6", ",", "2", ",", "4"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "+", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         SubscriptBox["\[Nu]", "1"], "-", 
         SubscriptBox["\[Nu]", "2"], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "4"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "55"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "55"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (55)", "Subsection",
 CellTags->"T1.6.24.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "55"], "=", 
  RowBox[{
   SubscriptBox["s", "55"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "4"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "4"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "55"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "55"], ",", "\"\<(55)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "4"], ">", "1"}], "&&", " ", 
      RowBox[{"m4", "=!=", "0"}], " ", "&&", " ", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}]}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "55"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (56)", "Subsection",
 CellTags->"T1.6.25.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "56"], "=", 
   RowBox[{
    RowBox[{"2", " ", 
     SubsuperscriptBox["m", "4", "2"], 
     SubscriptBox["\[Nu]", "4"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["4", "+"],
       FontWeight->"Bold"],
      Schiebe[4, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"6", ",", "2", ",", "4"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "-", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", "\n", "\t\t\t\t", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "+", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "d"}], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "1"]}], "-", 
         SubscriptBox["\[Nu]", "2"], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "3"]}], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "4"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "56"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "56"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (56)", "Subsection",
 CellTags->"T1.6.26.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "56"], "=", 
  RowBox[{
   SubscriptBox["s", "56"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "4"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "4"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "56"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "56"], ",", "\"\<(56)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "4"], ">", "1"}], "&&", " ", 
      RowBox[{"m4", "=!=", "0"}], " ", "&&", " ", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "===", "1"}]}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "56"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (59)", "Subsection",
 CellTags->"T1.6.27.1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 but all  ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " != 0"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "59"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["u", 
         RowBox[{"1", ",", "3", ",", "4"}]], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], 
        SubsuperscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}], "2"]}], "+", "\n", "\t", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True]}], ")"}]}], "-", "\n", "\t", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"4", 
           SubsuperscriptBox["m", "1", "2"], 
           SubsuperscriptBox["m", "3", "2"]}], "+", 
          SubsuperscriptBox["u", 
           RowBox[{"4", ",", "1", ",", "3"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "-", "\n", "\t\t", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "3"], "+", "1"}], ")"}], 
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}], "+", "1"}], ")"}], 
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "59"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "59"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (59)", "Subsection",
 CellTags->"T1.6.28.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "59"], "=", 
  SubscriptBox["s", "59"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "59"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "59"], ",", "\"\<(59)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "3", ",", "4"}]], "==", "0"}], "&&", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", 
          SubsuperscriptBox["m", "3", "2"], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "-", 
         RowBox[{"4", " ", 
          SubsuperscriptBox["m", "3", "2"], " ", 
          SubscriptBox["\[Nu]", "3"], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "+", 
         RowBox[{"2", " ", "d", " ", 
          SubscriptBox["u", 
           RowBox[{"1", ",", "3", ",", "4"}]], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "-", 
         RowBox[{"4", " ", 
          SubscriptBox["\[Nu]", "3"], " ", 
          SubscriptBox["u", 
           RowBox[{"1", ",", "3", ",", "4"}]], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "-", 
         RowBox[{"2", " ", 
          SubscriptBox["u", 
           RowBox[{"1", ",", "3", ",", "4"}]], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "+", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], " ", 
          SubsuperscriptBox["u", 
           RowBox[{"4", ",", "1", ",", "3"}], "2"]}], "-", 
         SubsuperscriptBox["u", 
          RowBox[{"4", ",", "1", ",", "3"}], "2"]}], ")"}], "  ", "=!=", 
       "0"}], "&&", 
      RowBox[{"n1", " ", "===", " ", "1"}], "&&", " ", 
      RowBox[{"n2", " ", "===", " ", "1"}], "&&", " ", 
      RowBox[{"n4", " ", "===", " ", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", "_Integer"}], "]"}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (60) ", "Subsection",
 CellTags->"T1.6.29.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "60"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "1", "2"], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True]}], ")"}]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         SubscriptBox["\[Nu]", "3"], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "1"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "60"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "60"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (60)", "Subsection",
 CellTags->"T1.6.30.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "60"], "=", 
  RowBox[{
   SubscriptBox["s", "60"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["notnecessaryf", "60"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "60"], ",", "\"\<(60)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", " ", 
      RowBox[{"m1", "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["notnecessaryf", "60"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (61) ", "Subsection",
 CellTags->"T1.6.31.1"],

Cell[TextData[{
 " ( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 " )  \[DoubleLongRightArrow] ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 )"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "61"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["3", "+"],
       FontWeight->"Bold"],
      Schiebe[3, "+"],
      Editable->True], 
     RowBox[{"(", 
      RowBox[{
       InterpretationBox[
        StyleBox[
         SuperscriptBox["1", "-"],
         FontWeight->"Bold"],
        Schiebe[1, "-"],
        Editable->True], "-", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["4", "-"],
         FontWeight->"Bold"],
        Schiebe[4, "-"],
        Editable->True]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "61"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "61"], "]"}]}]], "Input"],

Cell[TextData[{
 "For  ( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 " )  \[DoubleLongRightArrow] ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 )\nuse ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3], 
         Subscript[\[Nu], 4]}],
       Editable->True], "==", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3], 
         Subscript[\[Nu], 4]}],
       Editable->True]}], ")"}], " "}], TextForm]]],
 " and switch the masses accordingly."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (61)", "Subsection",
 CellTags->"T1.6.32.1"],

Cell[TextData[{
 "( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 " )  \[DoubleLongRightArrow] ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 )"
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "61"], "=", 
  SubscriptBox["s", "61"]}]], "Input"],

Cell["\<\
Don't need the restriction to n1=1 and n4=1 (I hope ...). Changed September \
24th 2003.\
\>", "SmallText"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "61"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "61"], ",", "\"\<(61)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], "===", "0"}], "&&", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "===", 
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4]}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n1", ":>", "1"}], ",", 
      RowBox[{"n4", ":>", "1"}]}], "}"}]}], "]"}]}]], "SmallText"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "61"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "61"], ",", "\"\<(61)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], "===", "0"}], "&&", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "===", 
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4]}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6153)", "Subsection",
 CellTags->"T1.6.33.1"],

Cell[TextData[{
 " ( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 ")  \[DoubleLongRightArrow] ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 )"
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "6153"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "61"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}]}]], "Input"],

Cell["\<\
Don't need the restriction to n1=1 and n4=1 (I hope ...). Changed September \
24th 2003.\
\>", "SmallText"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6153"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6153"], ",", "\"\<(6153)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "===", "0"}], "&&", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], "===", 
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4]}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n3", ":>", "1"}], ",", 
      RowBox[{"n4", ":>", "1"}]}], "}"}]}], "]"}]}]], "SmallText"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6153"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6153"], ",", "\"\<(6153)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "===", "0"}], "&&", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], "===", 
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4]}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (62) ", "Subsection",
 CellTags->"T1.6.34.1"],

Cell[TextData[{
 "( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 " ) \[DoubleLongRightArrow] (",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"1", ",", "3", ",", "4"}]]}], TraditionalForm]]],
 "== 0 )\nlower ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Nu]", "2"], TraditionalForm]]],
 " via (52) to one first"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "62"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], "-", 
       SubsuperscriptBox["m", "2", "2"]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "2"]}], "+", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "4"]}], "+", "2"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["4", "+"],
       FontWeight->"Bold"],
      Schiebe[4, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", "\n", "\t\t", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["m", "1", "2"], "-", 
          SubscriptBox["u", 
           RowBox[{"6", ",", "1", ",", "2"}]]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[CapitalSigma]", "4"], "-", 
          RowBox[{"2", 
           SubsuperscriptBox["m", "1", "2"], 
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True]}], "-", 
          RowBox[{"2", 
           SubsuperscriptBox["m", "1", "2"], 
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True]}]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "2", "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "-", "\n", "\t\t\t\t", 
       SubscriptBox["\[CapitalSigma]", "4"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "62"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "62"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (62)", "Subsection",
 CellTags->"T1.6.35.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "62"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["s", "62"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "4"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "4"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "62"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "62"], ",", "\"\<(62)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], "=!=", 
       InterpretationBox[
        SubsuperscriptBox["m", "2", "2"],
        m2^2]}], "&&", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], "-", 
        RowBox[{"2", " ", 
         SubscriptBox["\[Nu]", "2"]}], "+", 
        RowBox[{"2", " ", 
         SubscriptBox["\[Nu]", "4"]}]}], "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4], "\[RuleDelayed]", "0"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "\[RuleDelayed]", 
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1]}], ",", 
      RowBox[{"n2", "\[RuleDelayed]", "1"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (63) ", "Subsection",
 CellTags->"T1.6.36.1"],

Cell[TextData[{
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== 0  && ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"1", ",", "3", ",", "4"}]]}], TraditionalForm]]],
 " != 0\nreplace (55) by the following"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "63"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], "-", 
       SubsuperscriptBox["m", "2", "2"]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["m", "3", "2"], "-", 
          SubsuperscriptBox["m", "1", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}], "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "4"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], "-", 
          SubscriptBox["\[Nu]", "1"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", "\n", "\t\t", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["m", "1", "2"], "-", 
          SubsuperscriptBox["m", "3", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubsuperscriptBox["m", "1", "2"], 
              SubscriptBox["\[Nu]", "1"], 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["1", "+"],
                FontWeight->"Bold"],
               Schiebe[1, "+"],
               Editable->True]}], "+", 
             RowBox[{"2", 
              SubsuperscriptBox["m", "3", "2"], 
              SubscriptBox["\[Nu]", "3"], 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["3", "+"],
                FontWeight->"Bold"],
               Schiebe[3, "+"],
               Editable->True]}], "-", 
             SubscriptBox["\[CapitalSigma]", "4"], "-", "2"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}], "-", 
          RowBox[{"4", 
           SubsuperscriptBox["m", "2", "2"], 
           SubscriptBox["\[Nu]", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["4", "-"],
             FontWeight->"Bold"],
            Schiebe[4, "-"],
            Editable->True]}]}], ")"}]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "2", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubsuperscriptBox["m", "3", "2"], 
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "1", "2"], 
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True]}], ")"}]}]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "63"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "63"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (63)", "Subsection",
 CellTags->"T1.6.37.1"],

Cell[TextData[{
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== 0  && ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"1", ",", "3", ",", "4"}]]}], TraditionalForm]]],
 " != 0"
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "63"], "=", 
  RowBox[{"\[Nu]explicit", "[", 
   SubscriptBox["s", "63"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "63"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "63"], ",", "\"\<(63)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", " ", "=!=", " ", 
       RowBox[{"m2", "^", "2"}]}], "&&", 
      RowBox[{
       RowBox[{
        RowBox[{"Cayley", "[", 
         RowBox[{"1", ",", "3", ",", "4"}], "]"}], "[", 
        RowBox[{"{", 
         RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}], "]"}], "=!=",
        "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      InterpretationBox[
       SubscriptBox["m", "4"],
       m4], "\[RuleDelayed]", "0"}], "}"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "63"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "63"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "63"], ",", "\"\<(63)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", " ", "=!=", " ", 
       RowBox[{"m2", "^", "2"}]}], "&&", 
      RowBox[{
       RowBox[{
        RowBox[{"Cayley", "[", 
         RowBox[{"1", ",", "3", ",", "4"}], "]"}], "[", 
        RowBox[{"{", 
         RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}], "]"}], "=!=",
        "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4], "\[RuleDelayed]", "0"}], ",", 
      RowBox[{"n1", ":>", "1"}], ",", 
      RowBox[{"n3", ":>", "1"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\

CheckTVIRecursion not applicable for special indices since A's and 
B's appear .
\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6552)", "Subsection",
 CellTags->"T1.6.38.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "6552"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["e", "52"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"2", ",", "4", ",", "6"}]], "->", "0"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"formal", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["e", "55"], "/.", 
       RowBox[{
        RowBox[{"TVI", "[", "__", "]"}], "->", "1"}]}], ",", 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["4", "+"],
        FontWeight->"Bold"],
       Schiebe[4, "+"],
       Editable->True]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "2"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "6552"], "/.", "formal"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "6552"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   RowBox[{
    SubscriptBox["r", "6552"], "/.", "formal"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "6552"], "[", 
  RowBox[{"[", 
   RowBox[{"2", ",", "4"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "6552"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["n", "6552"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "4"}], "]"}], "]"}]}], "==", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["n", "6552"], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "-", 
     RowBox[{
      SubscriptBox["n", "6552"], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "4"}], "]"}], "]"}]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6552"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["s", "6552"], ",", "\"\<(6552)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"2", ",", "4", ",", "6"}]], "===", "0"}], "&&", 
      RowBox[{
       RowBox[{"Expand", "[", 
        RowBox[{
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["\[Nu]", "2"], "-", 
            SubscriptBox["\[Nu]", "4"]}], ")"}], " ", 
          InterpretationBox[
           SubsuperscriptBox["m", "4", "2"],
           m4^2]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             SubscriptBox["\[Nu]", "2"]}], "-", "d"}], ")"}], " ", 
          SubscriptBox["u", 
           RowBox[{"2", ",", "4", ",", "6"}]]}], "+", 
         RowBox[{
          SubscriptBox["\[Nu]", "2"], " ", 
          SubscriptBox["u", 
           RowBox[{"6", ",", "2", ",", "4"}]]}]}], "]"}], "=!=", "0"}]}], 
     "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6552)", "Subsection",
 CellTags->"T1.6.39.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "6552"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["e", "52"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"2", ",", "4", ",", "6"}]], "->", "0"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "6552"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["r", "6552"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "6552"], "[", 
  RowBox[{"[", 
   RowBox[{"2", ",", "5"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "6552"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["n", "6552"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "5"}], "]"}], "]"}]}], "==", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["n", "6552"], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "-", 
     RowBox[{
      SubscriptBox["n", "6552"], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "5"}], "]"}], "]"}]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6552"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["s", "6552"], ",", "\"\<(6552)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"2", ",", "4", ",", "6"}]], "===", "0"}], "&&", 
      RowBox[{
       RowBox[{"Expand", "[", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "2"}], " ", 
           SubscriptBox["\[Nu]", "2"], " ", 
           SubsuperscriptBox["m", "4", "2"]}], "+", 
          RowBox[{"2", " ", 
           SubscriptBox["\[Nu]", "4"], " ", 
           SubsuperscriptBox["m", "4", "2"]}], "+", 
          RowBox[{"d", " ", 
           SubscriptBox["u", 
            RowBox[{"2", ",", "4", ",", "6"}]]}], "-", 
          RowBox[{"3", " ", 
           SubscriptBox["\[Nu]", "2"], " ", 
           SubscriptBox["u", 
            RowBox[{"2", ",", "4", ",", "6"}]]}]}], ")"}], "]"}], "=!=", 
       "0"}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6553)", "Subsection",
 CellTags->"T1.6.40.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "6553"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Nu]explicit", "[", 
      SubscriptBox["s", "55"], "]"}], "/.", 
     RowBox[{
      SubscriptBox["u", 
       RowBox[{"3", ",", "1", ",", "4"}]], "->", "0"}]}], "/.", 
    RowBox[{
     SubscriptBox["u", 
      RowBox[{"6", ",", "2", ",", "4"}]], "->", "0"}]}], "/.", 
   RowBox[{
    SubscriptBox["m", "4"], "->", "0"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "6553"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["r", "6553"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "==", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["r", "6553"], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "-", 
     RowBox[{
      SubscriptBox["r", "6553"], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6553"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6553"], ",", "\"\<(6553)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"6", ",", "2", ",", "4"}]], "===", "0"}], "&&", 
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"1", ",", "3", ",", "4"}]], "===", "0"}]}], "]"}], ",", 
    RowBox[{"m4", ":>", "0"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (66) ", "Subsection",
 CellTags->"T1.6.41.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "66"], "=", 
   RowBox[{
    RowBox[{"4", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "3"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["3", "+"],
       FontWeight->"Bold"],
      Schiebe[3, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TVI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"6", "d"}], "-", 
          RowBox[{"6", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"5", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"6", 
           SubscriptBox["\[Nu]", "3"]}], "-", 
          RowBox[{"4", 
           SubscriptBox["\[Nu]", "4"]}], "-", 
          RowBox[{"6", 
           SubsuperscriptBox["m", "1", "2"], 
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True]}]}], ")"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["u", 
            RowBox[{"6", ",", "2", ",", "4"}]], "-", 
           RowBox[{"4", 
            SubsuperscriptBox["m", "2", "2"]}]}], ")"}], 
         SubscriptBox["\[Nu]", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "+"],
           FontWeight->"Bold"],
          Schiebe[2, "+"],
          Editable->True]}], "-", 
        RowBox[{"6", 
         SubsuperscriptBox["m", "3", "2"], 
         SubscriptBox["\[Nu]", "3"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["3", "+"],
           FontWeight->"Bold"],
          Schiebe[3, "+"],
          Editable->True]}], "+", 
        RowBox[{"2", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["u", 
            RowBox[{"6", ",", "2", ",", "4"}]], "-", 
           SubsuperscriptBox["m", "4", "2"]}], ")"}], 
         SubscriptBox["\[Nu]", "4"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "+"],
           FontWeight->"Bold"],
          Schiebe[4, "+"],
          Editable->True]}], "-", 
        RowBox[{
         SubscriptBox["\[Nu]", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "+"],
           FontWeight->"Bold"],
          Schiebe[2, "+"],
          Editable->True], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "-"],
           FontWeight->"Bold"],
          Schiebe[4, "-"],
          Editable->True]}], "-", 
        RowBox[{"2", 
         SubscriptBox["\[Nu]", "4"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "+"],
           FontWeight->"Bold"],
          Schiebe[4, "+"],
          Editable->True], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "-"],
           FontWeight->"Bold"],
          Schiebe[2, "-"],
          Editable->True]}]}], ")"}]}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "66"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "66"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (66)", "Subsection",
 CellTags->"T1.6.42.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "66"], "=", 
  RowBox[{
   SubscriptBox["s", "66"], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "1"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "1"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "66"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "66"], ",", "\"\<(66)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "66"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (alternative67)", "Subsection",
 CellTags->"T1.6.43.1"],

Cell["\<\
we dereived an alternative formula for Tarasov's (67), but do not save it by \
default.\
\>", "Text"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "67"], "=", 
   RowBox[{
    RowBox[{"4", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubscriptBox["\[Nu]", "1"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TVI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], " ", "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"6", "d"}], "-", 
         RowBox[{"6", 
          SubscriptBox["\[Nu]", "1"]}], "-", 
         RowBox[{"5", 
          SubscriptBox["\[Nu]", "2"]}], "-", 
         RowBox[{"6", 
          SubscriptBox["\[Nu]", "3"]}], "-", 
         RowBox[{"4", 
          SubscriptBox["\[Nu]", "4"]}]}], ")"}], "-", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["u", 
           RowBox[{"2", ",", "4", ",", "6"}]], "+", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["u", 
           RowBox[{"6", ",", "2", ",", "4"}]], "-", 
          RowBox[{"4", 
           SubsuperscriptBox["m", "2", "2"]}]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "-", 
       RowBox[{"6", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["u", 
           RowBox[{"6", ",", "2", ",", "4"}]], "-", 
          SubsuperscriptBox["m", "4", "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "67"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "67"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (67)", "Subsection",
 CellTags->"T1.6.44.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "67"], "=", 
   RowBox[{
    RowBox[{"4", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubscriptBox["\[Nu]", "1"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TVI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], " ", "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}], "-", 
          SubscriptBox["\[Nu]", "4"]}], ")"}]}], "-", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["u", 
           RowBox[{"2", ",", "4", ",", "6"}]], "+", 
          RowBox[{"2", 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"-", "4"}], 
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "-", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubscriptBox["u", 
         RowBox[{"6", ",", "2", ",", "4"}]], 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "4"}], " ", 
    SubscriptBox["\[Nu]", "2"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True], " ", 
    SubsuperscriptBox["m", "2", "2"]}], "-", 
   RowBox[{"2", " ", 
    SubscriptBox["\[Nu]", "4"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[1, 0, 0]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], -1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 1 + Subscript[\[Nu], 4]}],
     Editable->True]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "d"}], "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "1"]}], "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "2"]}], "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "3"]}], "-", 
      SubscriptBox["\[Nu]", "4"]}], ")"}], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["m", "2", "2"], "+", 
      SubsuperscriptBox["m", "4", "2"], "-", 
      SuperscriptBox["p", "2"]}], ")"}], " ", 
    SubscriptBox["\[Nu]", "4"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 1 + Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"4", " ", 
    SubsuperscriptBox["m", "3", "2"], " ", 
    SubscriptBox["\[Nu]", "3"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 1 + Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"2", " ", 
    SubscriptBox["\[Nu]", "1"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[1, 0, 0]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {1 + Subscript[\[Nu], 1], -1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}], "+", 
   RowBox[{"2", " ", 
    SubscriptBox["\[Nu]", "1"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm],
        FontColor->RGBColor[1, 0, 0]]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {1 + Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], -1 + Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", 
       SubsuperscriptBox["m", "1", "2"]}], "+", 
      SubsuperscriptBox["m", "2", "2"], "-", 
      SubsuperscriptBox["m", "4", "2"], "-", 
      SuperscriptBox["p", "2"]}], ")"}], " ", 
    SubscriptBox["\[Nu]", "1"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {1 + Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"4", " ", 
    SuperscriptBox["p", "2"], " ", 
    SubscriptBox["\[Nu]", "1"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], " ", 
    SubscriptBox["\[Nu]", "2"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontWeight->"Bold",
        FontColor->RGBColor[0, 0, 1]], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox[
        RowBox[{"d", "+", "2"}],
        TraditionalForm], ")"}]],
     TVI[2 + d, {2 + Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "67"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "67"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (67)", "Subsection",
 CellTags->"T1.6.45.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "67"], "=", 
  RowBox[{
   SubscriptBox["s", "67"], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "1"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "2"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "67"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "67"], ",", "\"\<(67)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "67"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6713)", "Subsection",
 CellTags->"T1.6.46.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "6713"], "=", 
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{
     SubscriptBox["s", "67"], ",", 
     RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "3"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "2"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6713"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6713"], ",", "\"\<(6713)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "2"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "6713"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (68)", "Subsection",
 CellTags->"T1.6.47.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "68"], "=", 
   RowBox[{
    RowBox[{"4", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubsuperscriptBox["m", "4", "2"], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "+", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TVI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubsuperscriptBox["m", "1", "2"], "-", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             SubscriptBox["\[Nu]", "2"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "3"]}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "4"]}]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "4", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "+", 
             SubscriptBox["\[Nu]", "2"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}]}], ")"}], "-", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"1", ",", "3", ",", "4"}]], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", 
            SubsuperscriptBox["m", "1", "2"], 
            SubscriptBox["\[Nu]", "1"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["1", "+"],
              FontWeight->"Bold"],
             Schiebe[1, "+"],
             Editable->True]}], "+", 
           RowBox[{
            SubscriptBox["u", 
             RowBox[{"2", ",", "4", ",", "6"}]], 
            SubscriptBox["\[Nu]", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}]}], ")"}]}], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"3", ",", "1", ",", "4"}]], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", 
            SubsuperscriptBox["m", "3", "2"], 
            SubscriptBox["\[Nu]", "3"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["3", "+"],
              FontWeight->"Bold"],
             Schiebe[3, "+"],
             Editable->True]}], "+", 
           RowBox[{
            SubscriptBox["\[Nu]", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["4", "-"],
              FontWeight->"Bold"],
             Schiebe[4, "-"],
             Editable->True]}]}], ")"}]}], "-", 
        RowBox[{"2", 
         SubsuperscriptBox["m", "4", "2"], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["\[Nu]", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}], "+", 
           RowBox[{
            SubscriptBox["\[Nu]", "4"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["4", "+"],
              FontWeight->"Bold"],
             Schiebe[4, "+"],
             Editable->True]}]}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True], "-", 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True]}], ")"}]}]}], ")"}]}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "68"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "68"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (68)", "Subsection",
 CellTags->"T1.6.48.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "68"], "=", 
  RowBox[{
   SubscriptBox["s", "68"], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "1"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "2"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "68"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "68"], ",", "\"\<(68)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{"m4", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "68"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6813)", "Subsection",
 CellTags->"T1.6.49.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "6813"], "=", 
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{
     SubscriptBox["s", "68"], ",", 
     RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "2"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "1"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6813"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6813"], ",", "\"\<(6813)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{"m4", "=!=", "0"}]}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "6813"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (70)", "Subsection",
 CellTags->"T1.6.50.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "70"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "d"}], "-", "10"}], ")"}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"d", "-", "3"}], ")"}], "2"], 
     SubsuperscriptBox["m", "4", "2"], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {1, 1, 1, 1}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{
      FractionBox["1", 
       RowBox[{"4", 
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP]}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", "d"}], "-", "10"}], ")"}], 
         SubscriptBox["\[CapitalDelta]", 
          RowBox[{"1", ",", "3", ",", "4"}]], 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["\[CapitalDelta]", 
            RowBox[{"2", ",", "4", ",", "6"}]], "+", 
           RowBox[{
            SubscriptBox["u", 
             RowBox[{"2", ",", "4", ",", "6"}]], 
            RowBox[{"(", 
             RowBox[{
              InterpretationBox[
               StyleBox[
                SuperscriptBox["2", "-"],
                FontWeight->"Bold"],
               Schiebe[2, "-"],
               Editable->True], "-", 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["4", "-"],
                FontWeight->"Bold"],
               Schiebe[4, "-"],
               Editable->True]}], ")"}]}]}], ")"}]}], "-", 
        RowBox[{"8", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], "2"], 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          PP], 
         SubsuperscriptBox["m", "3", "2"], 
         SubsuperscriptBox["m", "4", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "-"],
           FontWeight->"Bold"],
          Schiebe[4, "-"],
          Editable->True]}]}], ")"}], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox[
          RowBox[{"d", "-", "2"}],
          TraditionalForm], ")"}]],
       TVI[-2 + d, {1, 1, 1, 1}],
       Editable->True]}], "+", "\n", "\t\t\t", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", "-", "3"}], ")"}], 
         SubsuperscriptBox["m", "4", "2"], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"4", 
            SubsuperscriptBox["m", "1", "2"], 
            RowBox[{"(", 
             RowBox[{"d", "-", "4"}], ")"}], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["1", "+"],
              FontWeight->"Bold"],
             Schiebe[1, "+"],
             Editable->True], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["1", "+"],
              FontWeight->"Bold"],
             Schiebe[1, "+"],
             Editable->True]}], "-", 
           RowBox[{"8", 
            RowBox[{"(", 
             RowBox[{"d", "-", "3"}], ")"}], 
            SubsuperscriptBox["m", "2", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", "d"}], "-", "10"}], ")"}], 
            RowBox[{"(", 
             RowBox[{"d", "-", "3"}], ")"}], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}]}], ")"}]}], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"6", ",", "2", ",", "4"}]], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FractionBox["1", "2"], 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", "d"}], "-", "10"}], ")"}], 
            SubscriptBox["u", 
             RowBox[{"1", ",", "3", ",", "4"}]], 
            RowBox[{"(", 
             RowBox[{
              InterpretationBox[
               StyleBox[
                SuperscriptBox["1", "+"],
                FontWeight->"Bold"],
               Schiebe[1, "+"],
               Editable->True], "-", 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["3", "+"],
                FontWeight->"Bold"],
               Schiebe[3, "+"],
               Editable->True]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"d", "-", "2"}], ")"}], 
            RowBox[{"(", 
             RowBox[{"d", "-", "3"}], ")"}], 
            SubsuperscriptBox["m", "4", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["1", "+"],
              FontWeight->"Bold"],
             Schiebe[1, "+"],
             Editable->True]}]}], ")"}], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "+"],
           FontWeight->"Bold"],
          Schiebe[2, "+"],
          Editable->True]}], "-", 
        RowBox[{
         SubsuperscriptBox["m", "4", "2"], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"d", "-", "4"}], ")"}], "2"], 
            SubscriptBox["u", 
             RowBox[{"1", ",", "3", ",", "4"}]], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", "d"}], "-", "10"}], ")"}], 
            SubscriptBox["u", 
             RowBox[{"4", ",", "1", ",", "3"}]], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["3", "+"],
              FontWeight->"Bold"],
             Schiebe[3, "+"],
             Editable->True]}]}], ")"}], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "+"],
           FontWeight->"Bold"],
          Schiebe[1, "+"],
          Editable->True]}]}], ")"}], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["4", "-"],
        FontWeight->"Bold"],
       Schiebe[4, "-"],
       Editable->True], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {1, 1, 1, 1}],
       Editable->True]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"d", "-", "3"}], ")"}], 
      SubsuperscriptBox["m", "4", "2"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", 
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], 
         SubsuperscriptBox["m", "1", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "+"],
           FontWeight->"Bold"],
          Schiebe[1, "+"],
          Editable->True]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", "-", "2"}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["u", 
             RowBox[{"6", ",", "2", ",", "4"}]], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}], "-", 
           RowBox[{"2", 
            SubsuperscriptBox["m", "4", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["4", "+"],
              FontWeight->"Bold"],
             Schiebe[4, "+"],
             Editable->True]}]}], ")"}]}]}], ")"}], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["1", "+"],
        FontWeight->"Bold"],
       Schiebe[1, "+"],
       Editable->True], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["3", "-"],
        FontWeight->"Bold"],
       Schiebe[3, "-"],
       Editable->True], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {1, 1, 1, 1}],
       Editable->True]}], "+", 
     RowBox[{
      FractionBox["1", "2"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"3", "d"}], "-", "10"}], ")"}], 
      SubscriptBox["u", 
       RowBox[{"1", ",", "3", ",", "4"}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"6", ",", "2", ",", "4"}]], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "+"],
           FontWeight->"Bold"],
          Schiebe[2, "+"],
          Editable->True]}], "-", 
        RowBox[{"2", 
         SubsuperscriptBox["m", "4", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "+"],
           FontWeight->"Bold"],
          Schiebe[4, "+"],
          Editable->True]}]}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         InterpretationBox[
          StyleBox[
           SuperscriptBox["3", "+"],
           FontWeight->"Bold"],
          Schiebe[3, "+"],
          Editable->True], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "-"],
           FontWeight->"Bold"],
          Schiebe[1, "-"],
          Editable->True]}], "-", 
        RowBox[{
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "+"],
           FontWeight->"Bold"],
          Schiebe[1, "+"],
          Editable->True], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["3", "-"],
           FontWeight->"Bold"],
          Schiebe[3, "-"],
          Editable->True]}]}], ")"}], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {1, 1, 1, 1}],
       Editable->True]}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "70"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "70"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (70)", "Subsection",
 CellTags->"T1.6.51.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "70"], "=", 
  SubscriptBox["s", "70"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "70"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "70"], ",", "\"\<(70)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m4", "=!=", "0"}], " ", "&&", " ", 
       RowBox[{"PP", "=!=", "0"}]}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell["\<\
CheckTVIRecursion not applicable for special indices since A's and 
B's appear .\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (71)", "Subsection",
 CellTags->"T1.6.52.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ",", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], ">", "0"}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "71"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["D", 
      RowBox[{"1", ",", "2", ",", "3"}]], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubscriptBox["h", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubscriptBox["h", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Sigma]", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Rho]", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "+", 
       RowBox[{
        FractionBox["1", "2"], 
        SubscriptBox["\[CapitalSigma]", "3"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}], 
        SubscriptBox["\[Phi]", 
         RowBox[{"3", ",", "2", ",", "1"}]]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "71"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "71"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (71)", "Subsection",
 CellTags->"T1.6.53.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "71"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Nu]explicit", "[", 
     SubscriptBox["s", "71"], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "2"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "71"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "71"], ",", "\"\<(71)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["D", 
        RowBox[{"1", ",", "2", ",", "3"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "71"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (7123)", "Subsection",
 CellTags->"T1.6.54.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "7123"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["s", "71"], ",", 
      RowBox[{"2", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "7123"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "7123"], ",", "\"\<(7123)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["D", 
        RowBox[{"1", ",", "3", ",", "2"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "7123"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (7113)", "Subsection",
 CellTags->"T1.6.55.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "7113"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["s", "71"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "7113"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "7113"], ",", "\"\<(7113)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["D", 
        RowBox[{"3", ",", "2", ",", "1"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "7113"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (78)", "Subsection",
 CellTags->"T1.6.56.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ">", "0"}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "78"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "1", "2"], " ", 
     SubscriptBox["D", 
      RowBox[{"1", ",", "2", ",", "3"}]], 
     SubscriptBox["\[Nu]", "1"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SubscriptBox["\[CapitalSigma]", "3"]}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}], 
        SubscriptBox["\[Rho]", 
         RowBox[{"1", ",", "2", ",", "3"}]]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"3", ",", "1", ",", "2"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", "2", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}]}], ")"}], 
        SubscriptBox["D", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["S", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["S", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["S", 
         RowBox[{"3", ",", "1", ",", "2"}]], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "78"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "78"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (78)", "Subsection",
 CellTags->"T1.6.57.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "78"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["s", "78"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "78"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "78"], ",", "\"\<(78)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
       RowBox[{"m1", "=!=", "0"}]}], "&&", 
      RowBox[{
       SubscriptBox["D", 
        RowBox[{"1", ",", "2", ",", "3"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "78"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (80)", "Subsection",
 CellTags->"T1.6.58.1"],

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["m", "1", "2"], "==", "0"}]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ">", "0"}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "80"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[Nu]", "1"], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "1"]}], "-", "2"}], ")"}], 
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"2", ",", "3", ",", "6"}]], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "4"}], 
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "2", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             SubscriptBox["\[Nu]", "2"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "3"]}], "-", "1"}], ")"}]}], "-", 
          RowBox[{
           SubsuperscriptBox["m", "3", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             SubscriptBox["\[Nu]", "2"], "-", "1"}], ")"}]}]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", "\n", "\t\t\t  ", 
       RowBox[{"2", " ", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}]}], "-", 
          RowBox[{
           SubsuperscriptBox["m", "2", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}]}]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", "\n", "\t\t\t\t", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "3"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}], 
        SubscriptBox["u", 
         RowBox[{"6", ",", "2", ",", "3"}]]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "80"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "80"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (80)", "Subsection",
 CellTags->"T1.6.59.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "80"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["s", "80"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "80"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "80"], ",", "\"\<(80)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"2", ",", "3", ",", "6"}]], "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m1", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "80"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (81)", "Subsection",
 CellTags->"T1.6.60.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ",", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], ">", "0"}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "81"], "=", 
   RowBox[{
    RowBox[{
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TJI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "3", "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "1", "2"], "-", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "2", "2"]}], "+", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "3", "2"]}], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "+", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "1", "2"]}], "-", 
          SubsuperscriptBox["m", "2", "2"], "+", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "3", "2"]}], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "+", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "81"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "81"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (81)", "Subsection",
 CellTags->"T1.6.61.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "81"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Nu]explicit", "[", 
      SubscriptBox["s", "81"], "]"}], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "81"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "81"], ",", "\"\<(81)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "81"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8123)", "Subsection",
 CellTags->"T1.6.62.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "8123"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Interchange", "[", 
      RowBox[{
       SubscriptBox["s", "81"], ",", 
       RowBox[{"2", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "8123"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "8123"], ",", "\"\<(8123)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "8123"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8113)", "Subsection",
 CellTags->"T1.6.63.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "8113"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Interchange", "[", 
      RowBox[{
       SubscriptBox["s", "81"], ",", 
       RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "\[Rule]", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "\[Rule]", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}], "/.", 
   RowBox[{"d", "\[Rule]", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "8113"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "8113"], ",", "\"\<(8113)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "8113"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (82)", "Subsection",
 CellTags->"T1.6.64.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ">", "0"}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "82"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP]}], 
      SubsuperscriptBox["m", "1", "2"], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
      SubscriptBox["\[Nu]", "1"], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["1", "+"],
        FontWeight->"Bold"],
       Schiebe[1, "+"],
       Editable->True], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["1", "+"],
        FontWeight->"Bold"],
       Schiebe[1, "+"],
       Editable->True], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox[
          RowBox[{"d", "+", "2"}],
          TraditionalForm], ")"}]],
       TJI[2 + d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3]}],
       Editable->True]}], "+", 
     RowBox[{
      FractionBox["1", "2"], 
      InterpretationBox[
       SuperscriptBox["p", "2"],
       PP], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "3"], "+", "4"}], ")"}], 
      SubscriptBox["\[Nu]", "1"], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["1", "+"],
        FontWeight->"Bold"],
       Schiebe[1, "+"],
       Editable->True], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox[
          RowBox[{"d", "+", "2"}],
          TraditionalForm], ")"}]],
       TJI[2 + d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3]}],
       Editable->True]}]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{
        FractionBox["1", "2"], 
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubsuperscriptBox["m", "1", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"6", 
              SubscriptBox["\[Nu]", "1"]}], "+", 
             RowBox[{"6", 
              SubscriptBox["\[Nu]", "2"]}], "+", 
             RowBox[{"6", 
              SubscriptBox["\[Nu]", "3"]}], "-", 
             RowBox[{"7", "d"}]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "2", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"4", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             RowBox[{"10", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "3", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"10", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             RowBox[{"4", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox["m", "2", "2"], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox["m", "3", "2"], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "-", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "-", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], ")"}], "2"]}], "-", 
          RowBox[{
           InterpretationBox[
            SuperscriptBox["p", "2"],
            PP], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubsuperscriptBox["m", "1", "2"]}], "+", 
             SubsuperscriptBox["m", "2", "2"], "+", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "1", "2"], 
           RowBox[{"(", 
            RowBox[{
             SubsuperscriptBox["m", "2", "2"], "+", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}]}], "+", 
          RowBox[{"3", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              SubsuperscriptBox["m", "2", "2"], "-", 
              SubsuperscriptBox["m", "3", "2"]}], ")"}], "2"]}]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], ")"}], "2"], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "2", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", 
              SubsuperscriptBox["m", "1", "2"]}], "-", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "3", "2"]}], "+", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True]}], ")"}]}], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox["m", "2", "2"], ")"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], ")"}], "2"], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", 
              SubsuperscriptBox["m", "1", "2"]}], "-", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "2", "2"]}], "+", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True]}], ")"}]}], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox["m", "3", "2"], ")"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", 
       RowBox[{
        FractionBox["1", "2"], 
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["m", "2", "2"], "-", 
          SubsuperscriptBox["m", "3", "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "82"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "82"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "82"], "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "1"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["s", "82"], "=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["r", "82"], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}], "==", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["r", "82"], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "+", 
      RowBox[{
       SubscriptBox["r", "82"], "[", 
       RowBox[{"[", "2", "]"}], "]"}]}]}], ")"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (82)", "Subsection",
 CellTags->"T1.6.65.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "82"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Nu]explicit", "[", 
     SubscriptBox["s", "82"], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "2"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "82"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "82"], ",", "\"\<(82)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
       RowBox[{"m1", "=!=", "0"}]}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "82"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8213)", "Subsection",
 CellTags->"T1.6.66.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "8213"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["s", "82"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "-", "2"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "8213"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "8213"], ",", "\"\<(8213)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{"m3", "=!=", "0"}]}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "2"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "8213"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8212)", "Subsection",
 CellTags->"T1.6.67.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "8212"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["s", "82"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "-", "2"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "8212"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "8212"], ",", "\"\<(8212)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
       RowBox[{"m2", "=!=", "0"}]}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "8212"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (83)", "Subsection",
 CellTags->"T1.6.68.1"],

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["m", "1", "2"], "==", "0"}]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ">", "0"}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "83"], "=", 
   RowBox[{
    RowBox[{
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "3"], "+", "4"}], ")"}], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TJI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            PP], ")"}], "2"]}], 
        RowBox[{"(", 
         RowBox[{"d", "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubsuperscriptBox["m", "2", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"4", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             RowBox[{"10", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "3", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"10", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             RowBox[{"4", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox["m", "2", "2"], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}], "+", 
       RowBox[{"2", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox["m", "3", "2"], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "-", 
       RowBox[{"8", 
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], ")"}], "2"], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "2", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", 
              SubsuperscriptBox["m", "3", "2"]}], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "+", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True]}], ")"}]}], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox["m", "2", "2"], ")"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], ")"}], "2"], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", 
              SubsuperscriptBox["m", "2", "2"]}], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "+", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True]}], ")"}]}], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox["m", "3", "2"], ")"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", 
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "83"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "83"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (83)", "Subsection",
 CellTags->"T1.6.69.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "83"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Nu]explicit", "[", 
     SubscriptBox["s", "83"], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "83"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "83"], ",", "\"\<(83)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}], ",", 
    RowBox[{"m1", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "83"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (87)", "Subsection",
 CellTags->"T1.6.70.1"],

Cell[CellGroupData[{

Cell["(88)", "Subsubsection",
 CellTags->"T1.6.70.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["t", "1"], "=", 
   RowBox[{
    SubsuperscriptBox["m", "1", "2"], "+", 
    SubsuperscriptBox["m", "2", "2"], "+", 
    SubsuperscriptBox["m", "3", "2"]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["t", "2"], "=", 
   RowBox[{
    RowBox[{"3", 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "1", "4"], "+", 
       SubsuperscriptBox["m", "2", "4"], "+", 
       SubsuperscriptBox["m", "3", "4"]}], ")"}]}], "+", 
    RowBox[{"2", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "2", "2"]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"]}]}], ")"}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["t", "3"], "=", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "1", "4"], "-", 
       SubsuperscriptBox["m", "2", "4"], "-", 
       SubsuperscriptBox["m", "3", "4"]}], ")"}]}], "+", 
    RowBox[{
     SubsuperscriptBox["m", "2", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "2", "4"], "-", 
       SubsuperscriptBox["m", "1", "4"], "-", 
       SubsuperscriptBox["m", "3", "4"]}], ")"}]}], "+", 
    RowBox[{
     SubsuperscriptBox["m", "3", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "3", "4"], "-", 
       SubsuperscriptBox["m", "1", "4"], "-", 
       SubsuperscriptBox["m", "2", "4"]}], ")"}]}], "+", 
    RowBox[{"10", 
     SubsuperscriptBox["m", "1", "2"], 
     SubsuperscriptBox["m", "2", "2"], 
     SubsuperscriptBox["m", "3", "2"]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"\[Mu]1_", ",", "\[Mu]2_", ",", " ", "\[Mu]3_"}], "]"}], ":=", 
   RowBox[{"\[Mu]1", 
    RowBox[{"(", 
     RowBox[{
      InterpretationBox[
       SuperscriptBox["p", "2"],
       PP], "-", "\[Mu]1"}], ")"}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], 
       RowBox[{"(", 
        RowBox[{"d", "-", "4"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          PP], ")"}], "2"]}], "+", 
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", 
          SubscriptBox["t", "1"], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"5", "d"}], "-", "18"}], ")"}]}], "-", 
         RowBox[{"24", "\[Mu]1", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", "d"}], "-", "7"}], ")"}]}]}], ")"}]}], "-", 
      RowBox[{"2", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", "d"}], "-", "13"}], ")"}], 
       SuperscriptBox[
        SubscriptBox["t", "1"], "2"]}], "+", 
      RowBox[{"2", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"9", "d"}], "-", "31"}], ")"}], 
       SubscriptBox["t", "2"]}], "-", 
      RowBox[{"24", "\[Mu]2", " ", "\[Mu]3", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", "d"}], "-", "13"}], ")"}]}], "-", 
      RowBox[{"24", " ", 
       SuperscriptBox["\[Mu]1", "2"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "d"}], "-", "7"}], ")"}]}]}], ")"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"1", ",", "2", ",", "3"}]], "=", 
   RowBox[{"f", "[", 
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"2", ",", "1", ",", "3"}]], "=", 
   RowBox[{"f", "[", 
    RowBox[{
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "1", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"3", ",", "2", ",", "1"}]], "=", 
   RowBox[{"f", "[", 
    RowBox[{
     SubsuperscriptBox["m", "3", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "1", "2"]}], "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["(89)", "Subsubsection",
 CellTags->"T1.6.70.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"g", "[", 
    RowBox[{"\[Mu]1_", ",", "\[Mu]2_", ",", " ", "\[Mu]3_"}], "]"}], ":=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[Mu]1", " ", "\[Mu]2"}], 
     RowBox[{"d", "-", "4"}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", 
       RowBox[{"(", 
        RowBox[{"d", "-", "4"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          PP], ")"}], "2"]}], "-", 
      RowBox[{"4", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"7", "d"}], "-", "24"}], ")"}], 
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", "\[Mu]3"}], "-", 
         RowBox[{"2", 
          SubscriptBox["t", "1"]}]}], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["t", "1"], "2"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"23", "d"}], "-", "80"}], ")"}]}], "+", 
      RowBox[{
       SubscriptBox["t", "2"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"9", "d"}], "-", "32"}], ")"}]}], "-", 
      RowBox[{"12", 
       SuperscriptBox["\[Mu]3", "2"], 
       RowBox[{"(", 
        RowBox[{"d", "-", "4"}], ")"}]}], "+", 
      RowBox[{"12", "\[Mu]1", " ", "\[Mu]2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"7", "d"}], "-", "24"}], ")"}]}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", 
    RowBox[{"1", ",", "2", ",", "3"}]], "=", 
   RowBox[{"g", "[", 
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", 
    RowBox[{"1", ",", "3", ",", "2"}]], "=", 
   RowBox[{"g", "[", 
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", 
    RowBox[{"2", ",", "3", ",", "1"}]], "=", 
   RowBox[{"g", "[", 
    RowBox[{
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"], ",", 
     SubsuperscriptBox["m", "1", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 SubscriptBox["g", 
  RowBox[{"1", ",", "2", ",", "3"}]]], "Input"],

Cell[TextData[{
 "general  ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "87"], "=", 
   RowBox[{
    RowBox[{"3", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{"d", "-", "3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", "4"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "d"}], "-", "8"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "d"}], "-", "10"}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {1, 1, 1}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], "2"], 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], ")"}], "3"]}], "-", 
       RowBox[{"2", 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], ")"}], "2"], 
        SubscriptBox["t", "1"], 
        RowBox[{"(", 
         RowBox[{"d", "-", "4"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"6", "d"}], "-", "23"}], ")"}]}], "+", 
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"5", 
           SuperscriptBox[
            SubscriptBox["t", "1"], "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"15", 
              SuperscriptBox["d", "2"]}], "-", 
             RowBox[{"117", "d"}], "+", "224"}], ")"}]}], "-", 
          RowBox[{
           SubscriptBox["t", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"42", 
              SuperscriptBox["d", "2"]}], "-", 
             RowBox[{"331", "d"}], "+", "640"}], ")"}]}]}], ")"}]}], "-", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{"d", "-", "5"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["t", "3"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"27", "d"}], "-", "90"}], ")"}]}], "-", 
          RowBox[{
           SubscriptBox["t", "1"], 
           SubscriptBox["t", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", "d"}], "-", "2"}], ")"}]}], "-", 
          RowBox[{"2", 
           SuperscriptBox[
            SubscriptBox["t", "1"], "3"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", "d"}], "-", "26"}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["f", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["f", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["f", 
         RowBox[{"3", ",", "2", ",", "1"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["g", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["g", 
         RowBox[{"1", ",", "3", ",", "2"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["g", 
         RowBox[{"2", ",", "3", ",", "1"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TJI[-2 + d, {1, 1, 1}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "87"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "87"], "]"}]}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (87)", "Subsection",
 CellTags->"T1.6.71.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "87"], "=", 
  SubscriptBox["s", "87"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["f", "87"], "=", 
   RowBox[{"MakeFun", "[", 
    RowBox[{
     SubscriptBox["h", "87"], ",", "\"\<(87)\>\"", ",", 
     RowBox[{"IFF", "[", 
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{"MatchQ", "[", 
        RowBox[{"d", ",", 
         RowBox[{"_Symbol", "+", 
          RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (90)", "Subsection",
 CellTags->"T1.6.72.1"],

Cell[TextData[{
 "general  ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "90"], "=", 
   RowBox[{
    RowBox[{"3", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{"d", "-", "3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", "4"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "d"}], "-", "10"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {1, 1, 1}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SubsuperscriptBox["m", "1", "2"]}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], ")"}], "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "24"}], ")"}]}], "-", 
          RowBox[{"2", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            PP], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"4", "d"}], "-", "15"}], ")"}], 
              SubscriptBox["t", "1"]}], "-", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"d", "-", "5"}], ")"}], 
              SubsuperscriptBox["m", "1", "2"]}]}], ")"}]}], "+", 
          RowBox[{
           FractionBox["3", "2"], 
           RowBox[{"(", 
            RowBox[{"d", "-", "5"}], ")"}], 
           SuperscriptBox[
            SubscriptBox["t", "1"], "2"]}], "+", 
          RowBox[{
           FractionBox["5", "2"], 
           RowBox[{"(", 
            RowBox[{"d", "-", "3"}], ")"}], 
           SubscriptBox["t", "2"]}], "-", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", "d"}], "-", "17"}], ")"}], 
           SuperscriptBox[
            RowBox[{"(", 
             SubsuperscriptBox["m", "1", "2"], ")"}], "2"]}], "-", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"13", "d"}], "-", "45"}], ")"}], 
           SubsuperscriptBox["m", "2", "2"], 
           SubsuperscriptBox["m", "3", "2"]}]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "2", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "-", "3"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "+", 
             SubsuperscriptBox["m", "2", "2"], "-", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "3", "2"]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "25"}], ")"}], 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "3", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "-", "3"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "2", "2"]}], "+", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "25"}], ")"}], 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", "\n", "\t\t\t\t", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", "3"}], ")"}], 
        RowBox[{"(", 
         RowBox[{"d", "-", "4"}], ")"}], 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], ")"}], "2"]}], "+", 
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "30"}], ")"}], 
           RowBox[{"(", 
            RowBox[{"d", "-", "3"}], ")"}], 
           SubscriptBox["t", "1"]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "31"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", "d"}], "-", "10"}], ")"}], 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}]}], "+", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{"d", "-", "5"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"17", "d"}], "-", "66"}], ")"}], 
           SuperscriptBox[
            SubscriptBox["t", "1"], "2"]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", "d"}], "-", "14"}], ")"}], 
           SubscriptBox["t", "2"]}], "-", 
          RowBox[{"4", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", "d"}], "-", "10"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               SubsuperscriptBox["m", "1", "2"], ")"}], "2"], "+", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "2", "2"], 
              SubsuperscriptBox["m", "3", "2"]}]}], ")"}]}]}], ")"}]}], "+", 
       "\t", 
       RowBox[{
        FractionBox["2", 
         RowBox[{"d", "-", "4"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                InterpretationBox[
                 SuperscriptBox["p", "2"],
                 PP], "+", 
                SubsuperscriptBox["m", "2", "2"]}], ")"}], 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"7", "d"}], "-", "24"}], ")"}]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"d", "-", "4"}], ")"}], 
              SubsuperscriptBox["m", "1", "2"]}], "-", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"5", "d"}], "-", "18"}], ")"}], 
              SubsuperscriptBox["m", "3", "2"]}]}], ")"}], 
           SubsuperscriptBox["m", "1", "2"], 
           SubsuperscriptBox["m", "2", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True]}], "+", "\n", "\t\t\t\t\t\t", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                InterpretationBox[
                 SuperscriptBox["p", "2"],
                 PP], "+", 
                SubsuperscriptBox["m", "3", "2"]}], ")"}], 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"7", "d"}], "-", "24"}], ")"}]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"d", "-", "4"}], ")"}], 
              SubsuperscriptBox["m", "1", "2"]}], "-", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"5", "d"}], "-", "18"}], ")"}], 
              SubsuperscriptBox["m", "2", "2"]}]}], ")"}], 
           SubsuperscriptBox["m", "1", "2"], 
           SubsuperscriptBox["m", "3", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "-"],
             FontWeight->"Bold"],
            Schiebe[2, "-"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True]}], "-", "\n", "\t\t\t\t\t\t", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                InterpretationBox[
                 SuperscriptBox["p", "2"],
                 PP], "+", 
                SubsuperscriptBox["m", "2", "2"], "+", 
                SubsuperscriptBox["m", "3", "2"]}], ")"}], 
              RowBox[{"(", 
               RowBox[{"d", "-", "3"}], ")"}]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"d", "-", "4"}], ")"}], 
              SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
           SubsuperscriptBox["m", "2", "2"], 
           SubsuperscriptBox["m", "3", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True]}]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TJI[-2 + d, {1, 1, 1}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "90"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "90"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (90)", "Subsection",
 CellTags->"T1.6.73.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["h", "90"], "=", 
   SubscriptBox["s", "90"]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "90"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "90"], ",", "\"\<(90)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (91)", "Subsection",
 CellTags->"T1.6.74.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "91.1"], "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", "2"}], ")"}], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SubscriptBox["u", 
         RowBox[{"1", ",", "2", ",", "3"}]]}], "-", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["1", "-"],
         FontWeight->"Bold"],
        Schiebe[1, "-"],
        Editable->True], "+", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["2", "-"],
         FontWeight->"Bold"],
        Schiebe[2, "-"],
        Editable->True], "+", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["3", "-"],
         FontWeight->"Bold"],
        Schiebe[3, "-"],
        Editable->True]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TKI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "91.1"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "91.1"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "91.2"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", "2"}], ")"}], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "3"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["3", "+"],
       FontWeight->"Bold"],
      Schiebe[3, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{"(", 
        RowBox[{"d", "-", "2", "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "1"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TKI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "91.2"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "91.2"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "91.3"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", "2"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", 
         RowBox[{"1", ",", "2", ",", "3"}]], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"1", ",", "2", ",", "3"}]], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "-"],
           FontWeight->"Bold"],
          Schiebe[1, "-"],
          Editable->True]}], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"2", ",", "1", ",", "3"}]], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "-"],
           FontWeight->"Bold"],
          Schiebe[2, "-"],
          Editable->True]}], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"3", ",", "1", ",", "2"}]], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["3", "-"],
           FontWeight->"Bold"],
          Schiebe[3, "-"],
          Editable->True]}]}], ")"}]}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TKI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "91.3"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "91.3"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "91.4"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "2", ",", "3"}]], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"3", ",", "1", ",", "2"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "91.4"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "91.4"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (911)", "Subsection",
 CellTags->"T1.6.75.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "911"], "=", 
  RowBox[{
   SubscriptBox["s", "91.1"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "911"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "911"], ",", "\"\<(911)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (912)", "Subsection",
 CellTags->"T1.6.76.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "912"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["s", "91.2"], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "912"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "912"], ",", "\"\<(912)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (913)", "Subsection",
 CellTags->"T1.6.77.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "913"], "=", 
  SubscriptBox["s", "91.3"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "913"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "913"], ",", "\"\<(913)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"n2", "===", "1"}], " ", "||", " ", 
        RowBox[{"n3", "===", "1"}]}], ")"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}], ",", 
    RowBox[{"n1", ":>", "1"}]}], "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (914)", "Subsection",
 CellTags->"T1.6.78.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "914"], "=", 
  RowBox[{
   SubscriptBox["s", "91.4"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "914"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "914"], ",", "\"\<(914)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "3"}]], "=!=", "0"}], "&&", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], " ", ">", " ", "1"}], " ", ")"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", "_Symbol"}], "]"}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (92)", "Subsection",
 CellTags->"T1.6.79.1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "2", ",", "3"}]], TraditionalForm]]],
 " = Cayley[1,2,3][{m1,m2,m3}]"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"m1", ",", "m2", ",", "m3"}], "}"}], "]"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", " ", 
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], "==", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["m", "2"], "+", 
        SubscriptBox["m", "3"]}], ")"}], "2"]}], " ", ")"}], " ", 
   "\[DoubleLongRightArrow]", " ", 
   RowBox[{"(", " ", 
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "2", ",", "3"}]], "==", "0"}], " ", ")"}]}], 
  TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "92"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["m", "2"], 
     SubscriptBox["m", "3"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["m", "2"], "+", 
       SubscriptBox["m", "3"]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "2"]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "3"]}], "-", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["m", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "-", 
             SubscriptBox["\[Nu]", "1"], "-", 
             SubscriptBox["\[Nu]", "2"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "3"]}], "-", "1"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              InterpretationBox[
               StyleBox[
                SuperscriptBox["1", "+"],
                FontWeight->"Bold"],
               Schiebe[1, "+"],
               Editable->True], 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["2", "-"],
                FontWeight->"Bold"],
               Schiebe[2, "-"],
               Editable->True]}], "-", "1"}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["\[Nu]", "1"], "-", 
             SubscriptBox["\[Nu]", "2"], "+", "1"}], ")"}], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True]}]}], ")"}]}], "+", "\n", "\t\t\t", 
       RowBox[{
        SubscriptBox["m", "3"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "-", 
             SubscriptBox["\[Nu]", "1"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              InterpretationBox[
               StyleBox[
                SuperscriptBox["1", "+"],
                FontWeight->"Bold"],
               Schiebe[1, "+"],
               Editable->True], 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["3", "-"],
                FontWeight->"Bold"],
               Schiebe[3, "-"],
               Editable->True]}], "-", "1"}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["\[Nu]", "1"], "-", 
             SubscriptBox["\[Nu]", "3"], "+", "1"}], ")"}], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "-"],
             FontWeight->"Bold"],
            Schiebe[2, "-"],
            Editable->True]}]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "92"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "92"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (93)", "Subsection",
 CellTags->"T1.6.80.1"],

Cell[TextData[{
 "( ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["m", "1"], "==", "0"}], ")"}], " ", "&&", " ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["m", "2"], "==", 
        SubscriptBox["m", "3"]}], ")"}]}], " ", " ", ")"}], " ", 
    "\[DoubleLongRightArrow]", " ", 
    RowBox[{"(", " ", 
     RowBox[{
      SubscriptBox["\[CapitalDelta]", 
       RowBox[{"1", ",", "2", ",", "3"}]], "==", "0"}], " ", ")"}]}], 
   TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "93"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "2", "2"], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", "2"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "3"]}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "2"]}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "93"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "93"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (93)", "Subsection",
 CellTags->"T1.6.81.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "93"], "=", 
  RowBox[{
   SubscriptBox["s", "93"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "93"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "93"], ",", "\"\<(93)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["m", "2"], "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", ":>", "0"}], ",", 
      RowBox[{"m3", ":>", "m2"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (94)", "Subsection",
 CellTags->"T1.6.82.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "94"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "2", ",", "6"}]], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TBI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"6", ",", "1", ",", "2"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "-"],
             FontWeight->"Bold"],
            Schiebe[2, "-"],
            Editable->True]}], "-", "d", "+", 
          SubscriptBox["\[Nu]", "1"], "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True]}], "-", "d", "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "+", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TBI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "94"], "=", 
  RowBox[{"OperatorApplyB", "[", 
   SubscriptBox["e", "94"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (94)", "Subsection",
 CellTags->"T1.6.83.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "94"], "=", 
  RowBox[{
   SubscriptBox["s", "94"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "94"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "94"], ",", "\"\<(94)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9401)", "Subsection",
 CellTags->"T1.6.84.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "9401"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["s", "94"], "/.", 
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "2", ",", "6"}]], "->", "0"}]}], "/.", 
   RowBox[{
    SubscriptBox["u", 
     RowBox[{"6", ",", "1", ",", "2"}]], "->", "0"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "9401"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["n", "9401"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "2"}], "]"}], "]"}]}], "==", 
   RowBox[{
    SubscriptBox["n", "9401"], "[", 
    RowBox[{"[", 
     RowBox[{"2", ",", "1"}], "]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "9401"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "9401"], ",", "\"\<(9401)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "===", "0"}], "&&", 
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"6", ",", "1", ",", "2"}]], "===", "0"}], "&&", 
      RowBox[{"m2", "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9402)", "Subsection",
 CellTags->"T1.6.85.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "9402"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["s", "94"], "/.", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "->", "0"}]}], "/.", 
     RowBox[{
      SubscriptBox["u", 
       RowBox[{"6", ",", "1", ",", "2"}]], "->", "0"}]}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "9402"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["n", "9402"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "==", 
   RowBox[{
    SubscriptBox["n", "9402"], "[", 
    RowBox[{"[", 
     RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "9402"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "9402"], ",", "\"\<(9402)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "===", "0"}], "&&", 
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"6", ",", "1", ",", "2"}]], "===", "0"}], "&&", 
      RowBox[{"m1", "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (95)", "Subsection",
 CellTags->"T1.6.86.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "95"], "=", 
   RowBox[{
    RowBox[{"2", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TBI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"1", ",", "2", ",", "6"}]], "+", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["1", "-"],
         FontWeight->"Bold"],
        Schiebe[1, "-"],
        Editable->True], "-", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["2", "-"],
         FontWeight->"Bold"],
        Schiebe[2, "-"],
        Editable->True]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TBI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "95"], "=", 
  RowBox[{"OperatorApplyB", "[", 
   SubscriptBox["e", "95"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (95)", "Subsection",
 CellTags->"T1.6.87.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "95"], "=", 
  RowBox[{
   SubscriptBox["s", "95"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "95"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "95"], ",", "\"\<(95)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], " ", "&&", " ", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (96)", "Subsection",
 CellTags->"T1.6.88.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "96"], "=", 
   RowBox[{
    RowBox[{"2", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "2"], "-", "1"}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TBI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"1", ",", "2", ",", "6"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"2", ",", "1", ",", "6"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TBI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "96"], "=", 
  RowBox[{"OperatorApplyB", "[", 
   SubscriptBox["e", "96"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (96)", "Subsection",
 CellTags->"T1.6.89.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "96"], "=", 
  SubscriptBox["s", "96"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "96"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "96"], ",", "\"\<(96)\>\"", ",", 
    RowBox[{"IFF", "[", " ", 
     RowBox[{"MatchQ", "[", 
      RowBox[{"d", ",", 
       RowBox[{"_Symbol", "+", 
        RowBox[{"_Integer", " ", "?", "Positive"}]}]}], "]"}], "]"}], ",", 
    RowBox[{"n1", ":>", "1"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (97)", "Subsection",
 CellTags->"T1.6.90.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "97"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "1", "2"], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TAI[d, {
        Subscript[\[Nu], 1]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TAI[d, {
        Subscript[\[Nu], 1]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "97"], "=", 
  RowBox[{"OperatorApplyA", "[", 
   SubscriptBox["e", "97"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (97)", "Subsection",
 CellTags->"T1.6.91.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "97"], "=", 
  RowBox[{
   SubscriptBox["s", "97"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "97"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "97"], ",", "\"\<(97)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", " ", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", "_Symbol"}], " ", "]"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (98)", "Subsection",
 CellTags->"T1.6.92.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "98"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TAI[d, {
        Subscript[\[Nu], 1]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"-", "2"}], 
     SubsuperscriptBox["m", "1", "2"], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TAI[-2 + d, {
        Subscript[\[Nu], 1]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "98"], "=", 
  RowBox[{"OperatorApplyA", "[", 
   SubscriptBox["e", "98"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (98)", "Subsection",
 CellTags->"T1.6.93.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "98"], "=", 
  SubscriptBox["s", "98"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "98"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "98"], ",", "\"\<(98)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{"MatchQ", "[", 
      RowBox[{"d", ",", 
       RowBox[{"_Symbol", " ", "+", 
        RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}], "]"}]}], 
   "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (99)", "Subsection",
 CellTags->"T1.6.94.1"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["m", "1"], "=", "M"}], ",", " ", 
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SuperscriptBox["M", "2"]}], ",", 
   RowBox[{
    SubscriptBox["m", "2"], "=", 
    RowBox[{
     SubscriptBox["m", "3"], "=", "0"}]}]}], TraditionalForm]]]], "Text"],

Cell[TextData[{
 "Herleitung in three.nb ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SubscriptBox["\[Nu]", "1"]}], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}], "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], " ", 
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]]}], "==", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "d"}], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}], "+", "2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", "d"}], "+", 
       SubscriptBox["\[Nu]", "1"], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "3"]}], "+", "2"}], ")"}], " ", 
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]]}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["e", "99"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Nu]", "1"], " ", 
    RowBox[{"(", 
     RowBox[{"d", "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "2"]}], "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"d", "-", 
      SubscriptBox["\[Nu]", "2"], "-", 
      SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], " ", 
    InterpretationBox[
     StyleBox[
      SuperscriptBox["1", "+"],
      FontWeight->"Bold"],
     Schiebe[1, "+"],
     Editable->True], 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TJI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3]}],
     Editable->True]}], "==", 
   RowBox[{
    RowBox[{"-", 
     SubscriptBox["\[Nu]", "2"]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "d"}], "+", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "2"]}], "+", "2"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "d"}], "+", 
      SubscriptBox["\[Nu]", "1"], "+", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "2"]}], "+", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "3"]}], "+", "2"}], ")"}], 
    InterpretationBox[
     StyleBox[
      SuperscriptBox["2", "+"],
      FontWeight->"Bold"],
     Schiebe[2, "+"],
     Editable->True], 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TJI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3]}],
     Editable->True]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "99"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "99"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (99)", "Subsection",
 CellTags->"T1.6.95.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "99"], "=", 
  RowBox[{
   SubscriptBox["s", "99"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "99"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "99"], ",", "\"\<(99)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m1", "^", "2"}], "===", "PP"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m2", ":>", "0"}], ",", 
      RowBox[{"m3", ":>", "0"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9912)", "Subsection",
 CellTags->"T1.6.96.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "9912"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["s", "99"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "9912"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "9912"], ",", "\"\<(9912)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m2", "^", "2"}], "===", "PP"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", ":>", "0"}], ",", 
      RowBox[{"m3", ":>", "0"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9913)", "Subsection",
 CellTags->"T1.6.97.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "9913"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["s", "99"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "9913"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "9913"], ",", "\"\<(9913)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m3", "^", "2"}], "===", "PP"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", ":>", "0"}], ",", 
      RowBox[{"m2", ":>", "0"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (100)", "Subsection",
 CellTags->"T1.6.98.1"],

Cell["Herleitung in three.nb ", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SubsuperscriptBox["m", "2", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SubscriptBox["\[Nu]", "2"]}], ")"}], " ", 
    SubscriptBox["\[Nu]", "2"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TJI[d, {
       Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3]}],
     Editable->True]}], "==", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SubscriptBox["\[Nu]", "2"]}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "-", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "d"}], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "1"]}], "+", "2"}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        StyleBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         FontColor->RGBColor[0, 0, 1]], 
        StyleBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         FontColor->RGBColor[1, 0, 0]], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {1 + Subscript[\[Nu], 1], -1 + Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], TraditionalForm]], "Input"]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["m", "1"], "=", "0"}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["e", "100"], "=", 
  RowBox[{
   RowBox[{"2", " ", 
    SubsuperscriptBox["m", "2", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SubscriptBox["\[Nu]", "2"]}], ")"}], " ", 
    SubscriptBox["\[Nu]", "2"], " ", 
    InterpretationBox[
     StyleBox[
      SuperscriptBox["2", "+"],
      FontWeight->"Bold"],
     Schiebe[2, "+"],
     Editable->True], 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TJI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3]}],
     Editable->True]}], "==", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SubscriptBox["\[Nu]", "2"]}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "-", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "d"}], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "1"]}], "+", "2"}], ")"}], " ", 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "-"],
       FontWeight->"Bold"],
      Schiebe[2, "-"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "100"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "100"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (100)", "Subsection",
 CellTags->"T1.6.99.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "100"], "=", 
  SubscriptBox["s", "100"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "100"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "100"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], ",", "\"\<(100)\>\"", ",", 
    
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
      RowBox[{"m2", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m1", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "100"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1002)", "Subsection",
 CellTags->"T1.6.100.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1002"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1002"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1002"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], ",", "\"\<(1002)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
      RowBox[{"m2", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m3", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1002"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1003)", "Subsection",
 CellTags->"T1.6.101.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1003"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1003"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1003"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], ",", "\"\<(1003)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
      RowBox[{"m1", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m3", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1003"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1004)", "Subsection",
 CellTags->"T1.6.102.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1004"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1004"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1004"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], ",", "\"\<(1004)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
      RowBox[{"m1", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m2", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1004"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1005)", "Subsection",
 CellTags->"T1.6.103.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1005"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"2", "\[LeftRightArrow]", "3"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1005"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1005"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}], ",", "\"\<(1005)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "2"}], "&&", 
      RowBox[{"m3", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m1", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1005"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1006)", "Subsection",
 CellTags->"T1.6.104.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1006"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"2", "\[LeftRightArrow]", "3"}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1006"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1006"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}], ",", "\"\<(1006)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "2"}], "&&", 
      RowBox[{"m3", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m2", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1006"], "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Off\"\>", TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (150)", "Subsection",
 CellTags->"T1.6.105.1"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Nu]", "4"], "=", 
   RowBox[{"-", "1"}]}], TextForm]], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["q", "2"], "-", "p"}], " ", ")"}], "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}], "=", " ", 
   RowBox[{
    SubsuperscriptBox["q", "2", "2"], "-", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    RowBox[{"2", "p", " ", 
     SubscriptBox["q", "2"]}], "+", 
    SuperscriptBox["p", "2"], "+", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}]}], TextForm]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "150"], "=", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3", ",", "m3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "m4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "==", 
   RowBox[{
    RowBox[{"TFI", "[", 
     RowBox[{"d", ",", "PP", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2", "-", "1"}], ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "-", 
    RowBox[{"2", 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"PP", "+", 
       RowBox[{"m2", "^", "2"}], "-", 
       RowBox[{"m4", "^", "2"}]}], ")"}], 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (150)", "Subsection",
 CellTags->"T1.6.106.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "150"], "=", 
  SubscriptBox["s", "150"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "150"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "150"], ",", "\"\<(150)\>\""}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (151)", "Subsection",
 CellTags->"T1.6.107.1"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Nu]", "4"], "=", 
   RowBox[{"-", "1"}]}], TextForm]], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["q", "2"], "-", "p"}], " ", ")"}], "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}], "=", " ", 
   RowBox[{
    SubsuperscriptBox["q", "2", "2"], "-", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    RowBox[{"2", "p", " ", 
     SubscriptBox["q", "2"]}], "+", 
    SuperscriptBox["p", "2"], "+", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}]}], TextForm]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "151"], "=", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", "PP", ",", "DP", ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3", ",", "m3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "m4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "==", 
   RowBox[{
    RowBox[{"TFI", "[", 
     RowBox[{"d", ",", "PP", ",", "DP", ",", 
      RowBox[{"{", 
       RowBox[{"a", ",", "b"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2", "-", "1"}], ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "-", 
    RowBox[{"2", 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", "DP", ",", 
       RowBox[{"{", 
        RowBox[{"a", ",", "b"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"PP", "+", 
       RowBox[{"m2", "^", "2"}], "-", 
       RowBox[{"m4", "^", "2"}]}], ")"}], 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", "DP", ",", 
       RowBox[{"{", 
        RowBox[{"a", ",", "b"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (151)", "Subsection",
 CellTags->"T1.6.108.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "151"], "=", 
  SubscriptBox["s", "151"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "151"], "=", 
  RowBox[{"ReplacePart", "[", 
   RowBox[{
    RowBox[{"MakeFun", "[", 
     RowBox[{
      SubscriptBox["h", "151"], ",", "\"\<(151)\>\""}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "4"}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (152)", "Subsection",
 CellTags->"T1.6.109.1"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Nu]", "4"], "=", 
   RowBox[{"-", "1"}]}], TextForm]], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["q", "2"], "-", "p"}], " ", ")"}], "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}], "=", " ", 
   RowBox[{
    SubsuperscriptBox["q", "2", "2"], "-", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    RowBox[{"2", "p", " ", 
     SubscriptBox["q", "2"]}], "+", 
    SuperscriptBox["p", "2"], "+", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}]}], TextForm]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "152"], "=", 
  RowBox[{
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3", ",", "m3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "m4"}], "}"}]}], "}"}]}], "]"}], "==", 
   RowBox[{
    RowBox[{"TVI", "[", 
     RowBox[{"d", ",", "PP", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2", "-", "1"}], ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "-", 
    RowBox[{"2", 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}]}], "}"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"PP", "+", 
       RowBox[{"m2", "^", "2"}], "-", 
       RowBox[{"m4", "^", "2"}]}], ")"}], 
     RowBox[{"TVI", "[", 
      RowBox[{"d", ",", "PP", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (152)", "Subsection",
 CellTags->"T1.6.110.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "152"], "=", 
  SubscriptBox["s", "152"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "152"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "152"], ",", "\"\<(152)\>\""}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (153)", "Subsection",
 CellTags->"T1.6.111.1"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Nu]", "2"], "=", 
   RowBox[{"-", "1"}]}], TextForm]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "153"], "=", 
  RowBox[{
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3", ",", "m3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}], "==", "\n", 
   "\t\t", 
   RowBox[{
    RowBox[{"TFI", "[", 
     RowBox[{"d", ",", " ", "PP", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n4", ",", "m4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}]}], "}"}]}], "]"}], "+", 
    RowBox[{
     RowBox[{"m2", "^", "2"}], " ", 
     RowBox[{"TVI", "[", 
      RowBox[{"d", ",", " ", "PP", ",", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (153)", "Subsection",
 CellTags->"T1.6.112.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "153"], "=", 
  SubscriptBox["s", "153"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "153"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "153"], ",", "\"\<(153)\>\""}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (200)", "Subsection",
 CellTags->"T1.6.113.1"],

Cell["Derivation in TJRspecial.nb", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["m", "3"], "=", 
    RowBox[{
     SubscriptBox["m", "2"], "=", 
     SubscriptBox["m", "1"]}]}], ",", " ", 
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SubsuperscriptBox["m", "1", "2"]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["s", "200"], "=", 
   RowBox[{
    RowBox[{"16", " ", 
     SubsuperscriptBox["m", "1", "2"], " ", 
     SubscriptBox["\[Nu]", "1"], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        StyleBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         FontColor->RGBColor[0, 0, 1]], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {1 + Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"3", " ", "d"}], "-", 
         RowBox[{"3", " ", 
          SubscriptBox["\[Nu]", "1"]}], "-", 
         RowBox[{"4", " ", 
          SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], " ", 
      SubscriptBox["\[Nu]", "3"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          FontColor->RGBColor[1, 0, 0]], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          FontColor->RGBColor[0, 0, 1]]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {-1 + Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 1 + Subscript[\[Nu], 3]}],
       Editable->True]}], "-", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"3", " ", "d"}], "-", 
        RowBox[{"3", " ", 
         SubscriptBox["\[Nu]", "1"]}], "-", 
        RowBox[{"4", " ", 
         SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          FontColor->RGBColor[1, 0, 0]], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          FontColor->RGBColor[0, 0, 1]], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {-1 + Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3]}],
       Editable->True]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"2", " ", "d"}], "-", 
        SubscriptBox["\[Nu]", "1"], "-", 
        RowBox[{"2", " ", 
         SubscriptBox["\[Nu]", "2"]}]}], ")"}], " ", 
      SubscriptBox["\[Nu]", "3"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          FontColor->RGBColor[1, 0, 0]], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          FontColor->RGBColor[0, 0, 1]]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {
         Subscript[\[Nu], 1], -1 + Subscript[\[Nu], 2], 1 + 
         Subscript[\[Nu], 3]}],
       Editable->True]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "6"}], " ", 
         SuperscriptBox["d", "2"]}], "+", 
        RowBox[{"22", " ", "d", " ", 
         SubscriptBox["\[Nu]", "1"]}], "-", 
        RowBox[{"16", " ", 
         SubsuperscriptBox["\[Nu]", "1", "2"]}], "-", 
        SubscriptBox["\[Nu]", "2"], "+", 
        RowBox[{"6", " ", "d", " ", 
         SubscriptBox["\[Nu]", "2"]}], "-", 
        RowBox[{"13", " ", 
         SubscriptBox["\[Nu]", "1"], " ", 
         SubscriptBox["\[Nu]", "2"]}], "-", 
        SubscriptBox["\[Nu]", "3"], "+", 
        RowBox[{"6", " ", "d", " ", 
         SubscriptBox["\[Nu]", "3"]}], "-", 
        RowBox[{"13", " ", 
         SubscriptBox["\[Nu]", "1"], " ", 
         SubscriptBox["\[Nu]", "3"]}], "-", 
        RowBox[{"4", " ", 
         SubscriptBox["\[Nu]", "2"], " ", 
         SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3]}],
       Editable->True]}], "+", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"2", " ", "d"}], "-", 
        SubscriptBox["\[Nu]", "1"], "-", 
        RowBox[{"2", " ", 
         SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          FontColor->RGBColor[0, 0, 1]], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          FontColor->RGBColor[1, 0, 0]]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {
         Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], -1 + 
         Subscript[\[Nu], 3]}],
       Editable->True]}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"Due", " ", "to", " ", "sorting", "  ", 
    SubscriptBox["\[Nu]", "1"]}], "\[GreaterSlantEqual]", 
   SubscriptBox["\[Nu]", "2"], "\[GreaterSlantEqual]", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "  ", "by", " ", "TJI", " ", "only", " ", 
    "this", " ", "rule", " ", "is", " ", 
    RowBox[{"necessary", "."}]}]}], TextForm]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (200)", "Subsection",
 CellTags->"T1.6.114.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "200"], "=", 
  RowBox[{
   SubscriptBox["s", "200"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "200"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "200"], ",", "\"\<(200)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{"m1", "=!=", "0"}], "&&", 
      RowBox[{"PP", "===", 
       RowBox[{"m1", "^", "2"}]}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m2", ":>", "m1"}], ",", 
      RowBox[{"m3", ":>", "m1"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (201)", "Subsection",
 CellTags->"T1.6.115.1"],

Cell["\<\
Subscript[f, 201] = 
  HoldForm[TJR[d_, PP_, 
     {{(n1_)?PQ, 0}, 
      {(n2_)?PQ, 0}, 
      {(n3_)?PQ, m3_}}] := 
    (TComment[\"(201)\", 
        TJIC[d, 
         {{n1, 0}, {n2, 0}, 
          {n3, m3}}]]; 
       -(((d - n1 - n2)*
          (-2 - d + 
         2*n1 + 2*n2)*
          (3*d - 2*n1 - 
         2*n2 - 2*n3)*
          (-1 - d + n1 + 
          n2 + n3)*
          TJI[d, PP, 
          {{-1 + n1, 0}, 
          {n2, 0}, {n3, m3}}]
)/(2*m3^2*(d - 2*n1)*
          (-1 + n1)*
          (2*d - 2*n1 - 
          2*n2 - n3)*
          (1 + 2*d - 2*n1 - 
          2*n2 - n3)))) /; 
     n1 > 1 && m3 =!= 0 && 
      PP === m3^2]\
\>", "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (202)", "Subsection",
 CellTags->"T1.6.116.1"],

Cell["\<\
Subscript[f, 202] = 
  HoldForm[TJR[d_, PP_, 
     {{(n1_)?PQ, 0}, 
      {(n3_)?PQ, m3_}, 
      {(n2_)?PQ, 0}}] := 
    (TComment[\"(202)\", 
        TJIC[d, 
         {{n1, 0}, {n2, 0}, 
          {n3, m3}}]]; 
       -(((d - n1 - n2)*
          (-2 - d + 2*n1 + 
          2*n2)*
          (3*d - 2*n1 - 
          2*n2 - 2*n3)*
          (-1 - d + n1 + 
          n2 + n3)*
          TJI[d, m3^2, 
          {{-1 + n1, 0}, 
          {n2, 0}, {n3, m3}}
])/(2*m3^2*(d - 2*n1)*
          (-1 + n1)*
          (2*d - 2*n1 - 
          2*n2 - n3)*
          (1 + 2*d - 2*n1 - 
          2*n2 - n3)))) /; 
     n1 > 1 && m3 =!= 0 && 
      PP === m3^2]\
\>", "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (203)", "Subsection",
 CellTags->"T1.6.117.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Subscript", "[", 
   RowBox[{"f", ",", " ", "203"}], "]"}], " ", "=", 
  RowBox[{"HoldForm", "[", "\n", "\t\t ", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", " ", "0", ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n1_", ")"}], "?", "PQ"}], ",", " ", "0"}], "}"}], 
        ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n2_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}], 
        ",", " ", "\n", "    ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n3_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}]}],
        "}"}]}], "]"}], " ", ":=", 
    RowBox[{"(", "\n", "\t\t\t\t ", 
     RowBox[{
      RowBox[{"TComment", "[", 
       RowBox[{"\"\<(203)\>\"", ",", " ", "\n", "        ", 
        RowBox[{"TJIC", "[", 
         RowBox[{"d", ",", "0", ",", " ", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"n1", ",", " ", "0"}], "}"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], "]"}],
       ";", " ", "\n", "\t\t\t\t", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n2"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{
          "1", " ", "+", " ", "d", " ", "-", " ", "n1", " ", "-", " ", "n2", 
           " ", "-", " ", "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n3"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"TJR", "[", 
          RowBox[{"d", ",", " ", "0", ",", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "1"}], " ", "+", " ", "n1"}], ",", " ", "0"}], 
              "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", "\n", 
             "       ", 
             RowBox[{"{", 
              RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], 
        ")"}], "/", "\n", "   ", 
       RowBox[{"(", 
        RowBox[{"2", "*", 
         RowBox[{"m2", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"2", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}]}], ")"}]}]}], "\n", "\t\t\t", "\n", "\t\t\t", 
     ")"}]}], "]"}]}]], "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (204)", "Subsection",
 CellTags->"T1.6.118.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Subscript", "[", 
   RowBox[{"f", ",", " ", "204"}], "]"}], " ", "=", 
  RowBox[{"HoldForm", "[", "\n", "\t\t ", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", " ", "0", ",", " ", 
      RowBox[{"{", " ", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n2_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}], 
        ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n1_", ")"}], "?", "PQ"}], ",", " ", "0"}], "}"}], 
        ",", "\n", "    ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n3_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}]}],
        "}"}]}], "]"}], " ", ":=", 
    RowBox[{"(", "\n", "\t ", 
     RowBox[{
      RowBox[{"TComment", "[", 
       RowBox[{"\"\<(204)\>\"", ",", " ", "\n", "        ", 
        RowBox[{"TJIC", "[", 
         RowBox[{"d", ",", "0", ",", " ", 
          RowBox[{"{", " ", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{"n1", ",", " ", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], "]"}],
       ";", " ", "\n", "\t", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n2"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{
          "1", " ", "+", " ", "d", " ", "-", " ", "n1", " ", "-", " ", "n2", 
           " ", "-", " ", "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n3"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"TJR", "[", 
          RowBox[{"d", ",", " ", "0", ",", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "1"}], " ", "+", " ", "n1"}], ",", " ", "0"}], 
              "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", "\n", 
             "       ", 
             RowBox[{"{", 
              RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], 
        ")"}], "/", "\n", "   ", 
       RowBox[{"(", 
        RowBox[{"2", "*", 
         RowBox[{"m2", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"2", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}]}], ")"}]}]}], "\n", "\t\t\t", ")"}]}], 
   "]"}]}]], "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (205)", "Subsection",
 CellTags->"T1.6.119.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Subscript", "[", 
   RowBox[{"f", ",", " ", "205"}], "]"}], " ", "=", 
  RowBox[{"HoldForm", "[", "\n", "\t\t ", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", " ", "0", ",", " ", 
      RowBox[{"{", " ", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n2_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}], 
        ",", "\n", "    ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n3_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}], 
        ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n1_", ")"}], "?", "PQ"}], ",", " ", "0"}], "}"}]}], 
       "}"}]}], "]"}], " ", ":=", 
    RowBox[{"(", "\n", "\t ", 
     RowBox[{
      RowBox[{"TComment", "[", 
       RowBox[{"\"\<(205)\>\"", ",", " ", "\n", "        ", 
        RowBox[{"TJIC", "[", 
         RowBox[{"d", ",", "0", ",", " ", 
          RowBox[{"{", " ", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"n3", ",", " ", "m2"}], "}"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{"n1", ",", " ", "0"}], "}"}]}], "}"}]}], "]"}]}], "]"}], 
      ";", " ", "\n", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n2"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{
          "1", " ", "+", " ", "d", " ", "-", " ", "n1", " ", "-", " ", "n2", 
           " ", "-", " ", "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n3"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"TJR", "[", 
          RowBox[{"d", ",", " ", "0", ",", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "1"}], " ", "+", " ", "n1"}], ",", " ", "0"}], 
              "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", "\n", 
             "       ", 
             RowBox[{"{", 
              RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], 
        ")"}], "/", "\n", "   ", 
       RowBox[{"(", 
        RowBox[{"2", "*", 
         RowBox[{"m2", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"2", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}]}], ")"}]}]}], "\n", "\t\t\t", ")"}]}], 
   "]"}]}]], "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (1xxxx)", "Subsection",
 CellTags->"T1.6.120.1"],

Cell["\<\
If[MatchQ[$RankLimit, {_Integer?NonNegative, 
     _Integer?NonNegative}], 
   $TarasovTdeltaplimit = $RankLimit[[1]]; 
    ($TarasovTplimit = $RankLimit[[2]]; ), 
   $TarasovTdeltaplimit = 2; $TarasovTplimit = 2]; \
\>", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tarti", "=", 
  RowBox[{"Timing", "[", 
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"0", "<", 
        RowBox[{"a", "+", "b", "+", "r", "+", "es"}]}], ",", 
       RowBox[{
        RowBox[{"WriteString", "[", 
         RowBox[{"\"\<stdout\>\"", ",", 
          RowBox[{"{", 
           RowBox[{"a", ",", "b", ",", "r", ",", "es"}], "}"}]}], "]"}], ";", 
        
        RowBox[{"Set", "@@", 
         RowBox[{"{", 
          RowBox[{
           SubscriptBox["s", 
            RowBox[{
             SuperscriptBox["10", "4"], "+", 
             RowBox[{"1000", " ", "a"}], "+", 
             RowBox[{"100", " ", "b"}], "+", 
             RowBox[{"10", " ", "r"}], "+", "es"}]], ",", 
           RowBox[{
            RowBox[{
             RowBox[{
              StyleBox["TFI",
               FontColor->RGBColor[1, 0.6, 0]], "[", 
              RowBox[{"d", ",", 
               InterpretationBox[
                SuperscriptBox["p", "2"],
                PP], ",", "DP", ",", 
               RowBox[{"{", 
                RowBox[{"a", ",", "b"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0", ",", "r", ",", "es", ",", "0"}], "}"}],
                ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"n1", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"n2", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"n3", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"n4", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"n5", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "5"],
                    m5]}], "}"}]}], "}"}]}], "]"}], "==", 
             RowBox[{"ApplyTarasovT", "[", 
              RowBox[{
               RowBox[{"TarasovT", "[", 
                RowBox[{"a", ",", "b", ",", "r", ",", "es", ",", 
                 InterpretationBox[
                  SuperscriptBox["p", "2"],
                  PP], ",", "DP"}], "]"}], ",", 
               RowBox[{"STLI", "[", 
                RowBox[{"d", ",", 
                 InterpretationBox[
                  SuperscriptBox["p", "2"],
                  PP], ",", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"n1", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1]}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"n2", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2]}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"n3", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3]}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"n4", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"n5", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "5"],
                    m5]}], "}"}]}], "}"}]}], "]"}]}], "]"}]}], "/.", 
            RowBox[{"STLI", "\[Rule]", 
             StyleBox["TFI",
              FontColor->RGBColor[1, 0.6, 0]]}]}]}], "}"}]}]}]}], "]"}], ",", 
     
     RowBox[{"{", 
      RowBox[{"a", ",", "0", ",", "$TarasovTdeltaplimit"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", "0", ",", 
       RowBox[{"$TarasovTdeltaplimit", "-", "a"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "0", ",", "$TarasovTplimit"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"es", ",", "0", ",", 
       RowBox[{"$TarasovTplimit", "-", "r"}]}], "}"}]}], "]"}], 
   "]"}]}]], "Input"],

Cell["\<\
{0, 0, 0, 1}{0, 0, 0, 2}{0, 0, 0, 3}{0, 0, 0, 4}{0, 0, 0, 5}{0, 0, 1, 0}{0, \
0, 1, 1}{0, 0, 1, 2}{0, 0, 1, 3}{0, 0, 1, 4}{0, 0, 2, 0}{0, 0, 2, 1}{0, 0, 2, \
2}{0, 0, 2, 3}{0, 0, 3, 0}{0, 0, 3, 1}{0, 0, 3, 2}{0, 0, 4, 0}{0, 0, 4, 1}{0, \
0, 5, 0}{0, 1, 0, 0}{0, 1, 0, 1}{0, 1, 0, 2}{0, 1, 0, 3}{0, 1, 0, 4}{0, 1, 0, \
5}{0, 1, 1, 0}{0, 1, 1, 1}{0, 1, 1, 2}{0, 1, 1, 3}{0, 1, 1, 4}{0, 1, 2, 0}{0, \
1, 2, 1}{0, 1, 2, 2}{0, 1, 2, 3}{0, 1, 3, 0}{0, 1, 3, 1}{0, 1, 3, 2}{0, 1, 4, \
0}{0, 1, 4, 1}{0, 1, 5, 0}{0, 2, 0, 0}{0, 2, 0, 1}{0, 2, 0, 2}{0, 2, 0, 3}{0, \
2, 0, 4}{0, 2, 0, 5}{0, 2, 1, 0}{0, 2, 1, 1}{0, 2, 1, 2}{0, 2, 1, 3}{0, 2, 1, \
4}{0, 2, 2, 0}{0, 2, 2, 1}{0, 2, 2, 2}{0, 2, 2, 3}{0, 2, 3, 0}{0, 2, 3, 1}{0, \
2, 3, 2}{0, 2, 4, 0}{0, 2, 4, 1}{0, 2, 5, 0}{1, 0, 0, 0}{1, 0, 0, 1}{1, 0, 0, \
2}{1, 0, 0, 3}{1, 0, 0, 4}{1, 0, 0, 5}{1, 0, 1, 0}{1, 0, 1, 1}{1, 0, 1, 2}{1, \
0, 1, 3}{1, 0, 1, 4}{1, 0, 2, 0}{1, 0, 2, 1}{1, 0, 2, 2}{1, 0, 2, 3}{1, 0, 3, \
0}{1, 0, 3, 1}{1, 0, 3, 2}{1, 0, 4, 0}{1, 0, 4, 1}{1, 0, 5, 0}{1, 1, 0, 0}{1, \
1, 0, 1}{1, 1, 0, 2}{1, 1, 0, 3}{1, 1, 0, 4}{1, 1, 0, 5}{1, 1, 1, 0}{1, 1, 1, \
1}{1, 1, 1, 2}{1, 1, 1, 3}{1, 1, 1, 4}{1, 1, 2, 0}{1, 1, 2, 1}{1, 1, 2, 2}{1, \
1, 2, 3}{1, 1, 3, 0}{1, 1, 3, 1}{1, 1, 3, 2}{1, 1, 4, 0}{1, 1, 4, 1}{1, 1, 5, \
0}{2, 0, 0, 0}{2, 0, 0, 1}{2, 0, 0, 2}{2, 0, 0, 3}{2, 0, 0, 4}{2, 0, 0, 5}{2, \
0, 1, 0}{2, 0, 1, 1}{2, 0, 1, 2}{2, 0, 1, 3}{2, 0, 1, 4}{2, 0, 2, 0}{2, 0, 2, \
1}{2, 0, 2, 2}{2, 0, 2, 3}{2, 0, 3, 0}{2, 0, 3, 1}{2, 0, 3, 2}{2, 0, 4, 0}{2, \
0, 4, 1}{2, 0, 5, 0}\
\>", "Print"]
}, Open  ]],

Cell["Share[]", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"matarti", "=", 
  RowBox[{"Timing", "[", 
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"0", "<", 
        RowBox[{"ia", "+", "ib", "+", "ir", "+", "es"}]}], ",", 
       RowBox[{
        RowBox[{"WriteString", "[", 
         RowBox[{"\"\<stdout\>\"", ",", 
          RowBox[{"{", 
           RowBox[{"ia", ",", "ib", ",", "ir", ",", "es"}], "}"}]}], "]"}], 
        ";", 
        RowBox[{"Set", "[", 
         RowBox[{
          SubscriptBox["f", 
           RowBox[{
            SuperscriptBox["10", "4"], "+", 
            RowBox[{"1000", " ", "ia"}], "+", 
            RowBox[{"100", " ", "ib"}], "+", 
            RowBox[{"10", " ", "ir"}], "+", "es"}]], ",", 
          RowBox[{
           RowBox[{"MakeFun", "[", 
            RowBox[{
             SubscriptBox["s", 
              RowBox[{
               SuperscriptBox["10", "4"], "+", 
               RowBox[{"1000", " ", "ia"}], "+", 
               RowBox[{"100", " ", "ib"}], "+", 
               RowBox[{"10", " ", "ir"}], "+", "es"}]], ",", 
             RowBox[{"\"\<(1\>\"", "<>", 
              RowBox[{"ToString", "[", "ia", "]"}], "<>", 
              RowBox[{"ToString", "[", "ib", "]"}], "<>", 
              RowBox[{"ToString", "[", "ir", "]"}], "<>", 
              RowBox[{"ToString", "[", "es", "]"}], "<>", "\"\<)\>\""}]}], 
            "]"}], "/.", 
           RowBox[{"PQ", ":>", "PNQ"}]}]}], "]"}]}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"ia", ",", "0", ",", " ", "$TarasovTdeltaplimit"}], "}"}], ",", 
     
     RowBox[{"{", 
      RowBox[{"ib", ",", "0", ",", 
       RowBox[{"$TarasovTdeltaplimit", "-", "ia"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"ir", ",", "0", ",", "  ", "$TarasovTplimit"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"es", ",", "0", ",", 
       RowBox[{"$TarasovTplimit", "-", "ir"}]}], "}"}]}], "]"}], 
   "]"}]}]], "Input"],

Cell["\<\
{0, 0, 0, 1}{0, 0, 0, 2}{0, 0, 0, 3}{0, 0, 0, 4}{0, 0, 0, 5}{0, 0, 1, 0}{0, \
0, 1, 1}{0, 0, 1, 2}{0, 0, 1, 3}{0, 0, 1, 4}{0, 0, 2, 0}{0, 0, 2, 1}{0, 0, 2, \
2}{0, 0, 2, 3}{0, 0, 3, 0}{0, 0, 3, 1}{0, 0, 3, 2}{0, 0, 4, 0}{0, 0, 4, 1}{0, \
0, 5, 0}{0, 1, 0, 0}{0, 1, 0, 1}{0, 1, 0, 2}{0, 1, 0, 3}{0, 1, 0, 4}{0, 1, 0, \
5}{0, 1, 1, 0}{0, 1, 1, 1}{0, 1, 1, 2}{0, 1, 1, 3}{0, 1, 1, 4}{0, 1, 2, 0}{0, \
1, 2, 1}{0, 1, 2, 2}{0, 1, 2, 3}{0, 1, 3, 0}{0, 1, 3, 1}{0, 1, 3, 2}{0, 1, 4, \
0}{0, 1, 4, 1}{0, 1, 5, 0}{0, 2, 0, 0}{0, 2, 0, 1}{0, 2, 0, 2}{0, 2, 0, 3}{0, \
2, 0, 4}{0, 2, 0, 5}{0, 2, 1, 0}{0, 2, 1, 1}{0, 2, 1, 2}{0, 2, 1, 3}{0, 2, 1, \
4}{0, 2, 2, 0}{0, 2, 2, 1}{0, 2, 2, 2}{0, 2, 2, 3}{0, 2, 3, 0}{0, 2, 3, 1}{0, \
2, 3, 2}{0, 2, 4, 0}{0, 2, 4, 1}{0, 2, 5, 0}{1, 0, 0, 0}{1, 0, 0, 1}{1, 0, 0, \
2}{1, 0, 0, 3}{1, 0, 0, 4}{1, 0, 0, 5}{1, 0, 1, 0}{1, 0, 1, 1}{1, 0, 1, 2}{1, \
0, 1, 3}{1, 0, 1, 4}{1, 0, 2, 0}{1, 0, 2, 1}{1, 0, 2, 2}{1, 0, 2, 3}{1, 0, 3, \
0}{1, 0, 3, 1}{1, 0, 3, 2}{1, 0, 4, 0}{1, 0, 4, 1}{1, 0, 5, 0}{1, 1, 0, 0}{1, \
1, 0, 1}{1, 1, 0, 2}{1, 1, 0, 3}{1, 1, 0, 4}{1, 1, 0, 5}{1, 1, 1, 0}{1, 1, 1, \
1}{1, 1, 1, 2}{1, 1, 1, 3}{1, 1, 1, 4}{1, 1, 2, 0}{1, 1, 2, 1}{1, 1, 2, 2}{1, \
1, 2, 3}{1, 1, 3, 0}{1, 1, 3, 1}{1, 1, 3, 2}{1, 1, 4, 0}{1, 1, 4, 1}{1, 1, 5, \
0}{2, 0, 0, 0}{2, 0, 0, 1}{2, 0, 0, 2}{2, 0, 0, 3}{2, 0, 0, 4}{2, 0, 0, 5}{2, \
0, 1, 0}{2, 0, 1, 1}{2, 0, 1, 2}{2, 0, 1, 3}{2, 0, 1, 4}{2, 0, 2, 0}{2, 0, 2, \
1}{2, 0, 2, 2}{2, 0, 2, 3}{2, 0, 3, 0}{2, 0, 3, 1}{2, 0, 3, 2}{2, 0, 4, 0}{2, \
0, 4, 1}{2, 0, 5, 0}\
\>", "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Share", "[", "]"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["symmetry", "Subsection",
 CellTags->"Tsymmetry"],

Cell["Uncomment this ...", "Text"],

Cell["\<\
TVR[d_, pp_, {a_, b_, c_, z_}] := 
  TVR[d, pp, {c, b, a, z}] /;  !OrderedQ[{a, c}]\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["special TVR reductions", "Subsection",
 CellTags->"specialTVR"],

Cell["\<\
TVR[2 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {2, 0}, {1, m1_}}] := 
   ((-6*m1^2 + 2*d*m1^2 + 3*pp)*TAI[d, 0, {{1, m1}}]^2)/
      (12*(-3 + d)*(-1 + d)*m1^2*pp) + 
     ((4*m1^2 - pp)*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (4*(-3 + d)*(-1 + d)*m1^2) - 
     ((6*m1^2 - 2*d*m1^2 - 7*pp + 2*d*pp)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/(6*(-2 + d)*(-1 + d)*pp) + 
     ((-4*m1^2 + pp)*(-6*m1^2 + 2*d*m1^2 + 3*pp)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (6*(-3 + d)*(-2 + d)*(-1 + d)*pp) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[4 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {2, 0}, {1, m1_}}] := 
   -((-96*d*m1^6 + 352*d^2*m1^6 - 384*d^3*m1^6 + 128*d^4*m1^6 - 
           192*m1^4*pp + 1008*d*m1^4*pp - 1632*d^2*m1^4*pp + 
           912*d^3*m1^4*pp - 144*d^4*m1^4*pp + 288*m1^2*pp^2 - 
           1536*d*m1^2*pp^2 + 2316*d^2*m1^2*pp^2 - 1086*d^3*m1^2*pp^2 + 
           66*d^4*m1^2*pp^2 + 72*d*pp^3 - 166*d^2*pp^3 + 81*d^3*pp^3 + 
           4*d^4*pp^3)*TAI[d, 0, {{1, m1}}]^2)/
      (36*(-1 + d)^2*d^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) + 
     ((4*m1^2 - pp)^2*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (4*(-1 + d)^2*d*(1 + d)) - 
     ((288*m1^8 - 1152*d*m1^8 + 1504*d^2*m1^8 - 768*d^3*m1^8 + 
          128*d^4*m1^8 - 816*m1^6*pp + 2480*d*m1^6*pp - 
          2224*d^2*m1^6*pp + 736*d^3*m1^6*pp - 80*d^4*m1^6*pp + 
          864*m1^4*pp^2 - 2196*d*m1^4*pp^2 + 1446*d^2*m1^4*pp^2 - 
          252*d^3*m1^4*pp^2 - 6*d^4*m1^4*pp^2 - 432*m1^2*pp^3 + 
          1086*d*m1^2*pp^3 - 895*d^2*m1^2*pp^3 + 315*d^3*m1^2*pp^3 - 
          44*d^4*m1^2*pp^3 + 4*d*pp^4 - 2*d^2*pp^4 - 4*d^3*pp^4 + 
          2*d^4*pp^4)*TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (18*(-2 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) + 
     (m1^2*(4*m1^2 - pp)*(-96*m1^6 + 352*d*m1^6 - 384*d^2*m1^6 + 
          128*d^3*m1^6 + 240*m1^4*pp - 624*d*m1^4*pp + 384*d^2*m1^4*pp - 
          48*d^3*m1^4*pp - 192*m1^2*pp^2 + 420*d*m1^2*pp^2 - 
          162*d^2*m1^2*pp^2 - 18*d^3*m1^2*pp^2 + 72*pp^3 - 154*d*pp^3 + 
          81*d^2*pp^3 - 8*d^3*pp^3)*TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (18*(-2 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) /; \
(m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[4 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {3, 0}, {1, m1_}}] := 
   ((-72*d^2*m1^6 + 144*d^3*m1^6 - 88*d^4*m1^6 + 16*d^5*m1^6 - 
          240*d*m1^4*pp + 572*d^2*m1^4*pp - 398*d^3*m1^4*pp + 
          84*d^4*m1^4*pp - 2*d^5*m1^4*pp - 96*m1^2*pp^2 + 
          480*d*m1^2*pp^2 - 744*d^2*m1^2*pp^2 + 390*d^3*m1^2*pp^2 - 
          50*d^4*m1^2*pp^2 + 4*d^5*m1^2*pp^2 - 24*d*pp^3 + 54*d^2*pp^3 - 
          27*d^3*pp^3)*TAI[d, 0, {{1, m1}}]^2)/
      (24*(-3 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*m1^2*pp^2) + 
     ((4*m1^2 - pp)^2*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (8*(-3 + d)*(-1 + d)^2*(1 + d)*m1^2) + 
     ((-72*d*m1^6 + 144*d^2*m1^6 - 88*d^3*m1^6 + 16*d^4*m1^6 - 
          96*m1^4*pp + 260*d*m1^4*pp - 166*d^2*m1^4*pp + 
          12*d^3*m1^4*pp + 6*d^4*m1^4*pp + 160*m1^2*pp^2 - 
          300*d*m1^2*pp^2 + 50*d^2*m1^2*pp^2 + 90*d^3*m1^2*pp^2 - 
          24*d^4*m1^2*pp^2 - 48*pp^3 + 114*d*pp^3 - 67*d^2*pp^3 + 
          4*d^3*pp^3 + 2*d^4*pp^3)*TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (12*(-2 + d)*(-1 + d)^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) - 
     ((-4*m1^2 + pp)*(72*d*m1^6 - 144*d^2*m1^6 + 88*d^3*m1^6 - 
          16*d^4*m1^6 + 96*m1^4*pp - 236*d*m1^4*pp + 122*d^2*m1^4*pp + 
          12*d^3*m1^4*pp - 10*d^4*m1^4*pp - 96*m1^2*pp^2 + 
          180*d*m1^2*pp^2 - 30*d^2*m1^2*pp^2 - 46*d^3*m1^2*pp^2 + 
          8*d^4*m1^2*pp^2 + 24*pp^3 - 54*d*pp^3 + 27*d^2*pp^3)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
(12*(-3 + d)*(-2 + d)*(-1 + d)^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) /; \
(m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[6 + d_Symbol, pp_, 
    {{1, m1_}, {1, m1_}, {4, 0}, {1, m1_}}] := 
   ((2304*d*m1^10 - 4608*d^2*m1^10 - 3520*d^3*m1^10 + 6976*d^4*m1^10 + 
          1472*d^5*m1^10 - 2624*d^6*m1^10 - 256*d^7*m1^10 + 
          256*d^8*m1^10 + 4608*m1^8*pp - 9600*d*m1^8*pp - 
          7680*d^2*m1^8*pp + 9312*d^3*m1^8*pp + 11680*d^4*m1^8*pp - 
          6560*d^5*m1^8*pp - 2880*d^6*m1^8*pp + 1088*d^7*m1^8*pp + 
          32*d^8*m1^8*pp + 6912*m1^6*pp^2 - 35712*d*m1^6*pp^2 + 
          1056*d^2*m1^6*pp^2 + 85296*d^3*m1^6*pp^2 - 
          37032*d^4*m1^6*pp^2 - 20484*d^5*m1^6*pp^2 + 
          8700*d^6*m1^6*pp^2 - 12*d^7*m1^6*pp^2 - 84*d^8*m1^6*pp^2 - 
          20736*m1^4*pp^3 + 58752*d*m1^4*pp^3 + 40560*d^2*m1^4*pp^3 - 
          150536*d^3*m1^4*pp^3 + 18980*d^4*m1^4*pp^3 + 
          48946*d^5*m1^4*pp^3 - 6748*d^6*m1^4*pp^3 - 146*d^7*m1^4*pp^3 + 
          128*d^8*m1^4*pp^3 + 3456*m1^2*pp^4 - 13824*d*m1^2*pp^4 - 
          4176*d^2*m1^2*pp^4 + 35784*d^3*m1^2*pp^4 - 
          8332*d^4*m1^2*pp^4 - 10162*d^5*m1^2*pp^4 + 
          1122*d^6*m1^2*pp^4 - 80*d^7*m1^2*pp^4 - 8*d^8*m1^2*pp^4 + 
          864*d*pp^5 - 216*d^2*pp^5 - 2268*d^3*pp^5 + 486*d^4*pp^5 + 
          729*d^5*pp^5)*TAI[d, 0, {{1, m1}}]^2)/
      (432*(-3 + d)*(-1 + d)^2*d^2*(1 + d)*(2 + d)*(3 + d)*(-4 + 3*d)*
        (-2 + 3*d)*(2 + 3*d)*m1^2*pp^3) + 
     ((4*m1^2 - pp)^3*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (16*(-3 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*m1^2) - 
     ((-1152*m1^10 + 2880*d*m1^10 + 320*d^2*m1^10 - 3648*d^3*m1^10 + 
          1088*d^4*m1^10 + 768*d^5*m1^10 - 256*d^6*m1^10 - 192*m1^8*pp + 
          1312*d*m1^8*pp + 2080*d^2*m1^8*pp - 6880*d^3*m1^8*pp + 
          4032*d^4*m1^8*pp - 192*d^5*m1^8*pp - 160*d^6*m1^8*pp + 
          6336*m1^6*pp^2 - 6960*d*m1^6*pp^2 - 16960*d^2*m1^6*pp^2 + 
          17292*d^3*m1^6*pp^2 - 1828*d^4*m1^6*pp^2 - 828*d^5*m1^6*pp^2 + 
          68*d^6*m1^6*pp^2 - 9504*m1^4*pp^3 + 3624*d*m1^4*pp^3 + 
          26104*d^2*m1^4*pp^3 - 10938*d^3*m1^4*pp^3 - 
          4976*d^4*m1^4*pp^3 + 330*d^5*m1^4*pp^3 + 400*d^6*m1^4*pp^3 + 
          6336*m1^2*pp^4 - 3640*d*m1^2*pp^4 - 14584*d^2*m1^2*pp^4 + 
          9190*d^3*m1^2*pp^4 + 1374*d^4*m1^2*pp^4 - 600*d^5*m1^2*pp^4 - 
          56*d^6*m1^2*pp^4 - 864*pp^5 + 732*d*pp^5 + 1864*d^2*pp^5 - 
          1695*d^3*pp^5 + 148*d^4*pp^5 + 36*d^5*pp^5 + 4*d^6*pp^5)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (216*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) + ((-4*m1^2 + pp)*
        (1152*m1^10 - 2880*d*m1^10 - 320*d^2*m1^10 + 3648*d^3*m1^10 - 
          1088*d^4*m1^10 - 768*d^5*m1^10 + 256*d^6*m1^10 + 576*m1^8*pp - 
          2208*d*m1^8*pp - 2304*d^2*m1^8*pp + 8000*d^3*m1^8*pp - 
          4256*d^4*m1^8*pp - 32*d^5*m1^8*pp + 224*d^6*m1^8*pp - 
          6336*m1^6*pp^2 + 6288*d*m1^6*pp^2 + 16320*d^2*m1^6*pp^2 - 
          14868*d^3*m1^6*pp^2 + 684*d^4*m1^6*pp^2 + 804*d^5*m1^6*pp^2 - 
          12*d^6*m1^6*pp^2 + 6912*m1^4*pp^3 - 3192*d*m1^4*pp^3 - 
          18160*d^2*m1^4*pp^3 + 7950*d^3*m1^4*pp^3 + 3128*d^4*m1^4*pp^3 - 
          78*d^5*m1^4*pp^3 - 160*d^6*m1^4*pp^3 - 3456*m1^2*pp^4 + 
          1800*d*m1^2*pp^4 + 8136*d^2*m1^2*pp^4 - 4322*d^3*m1^2*pp^4 - 
          1042*d^4*m1^2*pp^4 + 128*d^5*m1^2*pp^4 + 16*d^6*m1^2*pp^4 + 
          432*pp^5 - 324*d*pp^5 - 972*d^2*pp^5 + 729*d^3*pp^5)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (216*(-3 + d)*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*
        (-2 + 3*d)*(2 + 3*d)*pp^3) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
 TVR[6 + d_Symbol, pp_, {{2, m1_}, {1, m1_}, {3, 0}, {1, m1_}}] := 
   -((-1536*d*m1^10 + 2560*d^2*m1^10 + 3200*d^3*m1^10 - 3584*d^4*m1^10 - 
           2176*d^5*m1^10 + 1024*d^6*m1^10 + 512*d^7*m1^10 - 
           3072*m1^8*pp + 6528*d*m1^8*pp + 5376*d^2*m1^8*pp - 
           7520*d^3*m1^8*pp - 5568*d^4*m1^8*pp + 3232*d^5*m1^8*pp + 
           1344*d^6*m1^8*pp - 320*d^7*m1^8*pp - 2304*m1^6*pp^2 + 
           13824*d*m1^6*pp^2 + 192*d^2*m1^6*pp^2 - 31584*d^3*m1^6*pp^2 + 
           11232*d^4*m1^6*pp^2 + 8088*d^5*m1^6*pp^2 - 
           2304*d^6*m1^6*pp^2 - 24*d^7*m1^6*pp^2 + 6912*m1^4*pp^3 - 
           19584*d*m1^4*pp^3 - 12688*d^2*m1^4*pp^3 + 
           50584*d^3*m1^4*pp^3 - 7516*d^4*m1^4*pp^3 - 
           16982*d^5*m1^4*pp^3 + 2564*d^6*m1^4*pp^3 + 310*d^7*m1^4*pp^3 - 
           1152*m1^2*pp^4 + 4608*d*m1^2*pp^4 + 1392*d^2*m1^2*pp^4 - 
           11896*d^3*m1^2*pp^4 + 2820*d^4*m1^2*pp^4 + 
           3374*d^5*m1^2*pp^4 - 414*d^6*m1^2*pp^4 + 8*d^7*m1^2*pp^4 - 
           288*d*pp^5 + 72*d^2*pp^5 + 756*d^3*pp^5 - 162*d^4*pp^5 - 
           243*d^5*pp^5)*TAI[d, 0, {{1, m1}}]^2)/
      (144*(-1 + d)^2*d^2*(1 + d)*(2 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*m1^2*pp^3) + 
     ((4*m1^2 - pp)^3*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (16*(-1 + d)^2*d*(1 + d)*(3 + d)*m1^2) - 
     ((2304*m1^10 - 5760*d*m1^10 - 640*d^2*m1^10 + 7296*d^3*m1^10 - 
          2176*d^4*m1^10 - 1536*d^5*m1^10 + 512*d^6*m1^10 - 
          1344*m1^8*pp + 1120*d*m1^8*pp - 1376*d^2*m1^8*pp + 
          5280*d^3*m1^8*pp - 4896*d^4*m1^8*pp + 1280*d^5*m1^8*pp - 
          64*d^6*m1^8*pp - 5184*m1^6*pp^2 + 7584*d*m1^6*pp^2 + 
          14800*d^2*m1^6*pp^2 - 21496*d^3*m1^6*pp^2 + 
          6120*d^4*m1^6*pp^2 + 280*d^5*m1^6*pp^2 - 184*d^6*m1^6*pp^2 + 
          9504*m1^4*pp^3 - 3720*d*m1^4*pp^3 - 27296*d^2*m1^4*pp^3 + 
          15378*d^3*m1^4*pp^3 + 4114*d^4*m1^4*pp^3 - 666*d^5*m1^4*pp^3 - 
          674*d^6*m1^4*pp^3 - 6336*m1^2*pp^4 + 5720*d*m1^2*pp^4 + 
          13712*d^2*m1^2*pp^4 - 13926*d^3*m1^2*pp^4 + 
          1284*d^4*m1^2*pp^4 + 946*d^5*m1^2*pp^4 - 80*d^6*m1^2*pp^4 + 
          864*pp^5 - 1020*d*pp^5 - 1636*d^2*pp^5 + 2311*d^3*pp^5 - 
          693*d^4*pp^5 + 20*d^5*pp^5 + 4*d^6*pp^5)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (72*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) + ((4*m1^2 - pp)*
        (-768*m1^10 + 1664*d*m1^10 + 768*d^2*m1^10 - 2176*d^3*m1^10 + 
          512*d^5*m1^10 + 192*m1^8*pp + 288*d*m1^8*pp + 704*d^2*m1^8*pp - 
          2272*d^3*m1^8*pp + 1024*d^4*m1^8*pp + 64*d^5*m1^8*pp + 
          1920*m1^6*pp^2 - 1824*d*m1^6*pp^2 - 5472*d^2*m1^6*pp^2 + 
          4584*d^3*m1^6*pp^2 - 168*d^5*m1^6*pp^2 - 2304*m1^4*pp^3 + 
          616*d*m1^4*pp^3 + 6192*d^2*m1^4*pp^3 - 1946*d^3*m1^4*pp^3 - 
          1128*d^4*m1^4*pp^3 - 230*d^5*m1^4*pp^3 + 1152*m1^2*pp^4 - 
          600*d*m1^2*pp^4 - 2744*d^2*m1^2*pp^4 + 1414*d^3*m1^2*pp^4 + 
          374*d^4*m1^2*pp^4 - 16*d^5*m1^2*pp^4 - 144*pp^5 + 108*d*pp^5 + 
          324*d^2*pp^5 - 243*d^3*pp^5)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (72*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["five -> four", "Subsection",
 CellTags->"T1.6.121.1"],

Cell["\<\
TFI[d_, PP_/;Head[PP]=!=List,{ {0, m1_}, {n2_, m2_}, 
                               {n3_, m3_}, {n4_, m4_}, 
                               {n5_, m5_} }] := 
TVI[d,PP, {{n5,m5}, {n2,m2}, {n3,m3}, {n4,m4}}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", " ", 
    RowBox[{"PP_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "PP", "]"}], "=!=", "List"}]}], ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"n3_", ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4_", ",", " ", "m4_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"n5_", ",", " ", "m5_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{"TVI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n5", ",", "m5"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\
TFI[d_,  PP_/;Head[PP]=!=List,{ {(n2_), m2_},{0, m1_},
                               {(n4_), m4_}, {(n3_), m3_}, 
                               {(n5_), m5_} }] := 
TVI[d,PP, {{n5,m5}, {n2,m2}, {n3,m3}, {n4,m4}}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", "  ", 
    RowBox[{"PP_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "PP", "]"}], "=!=", "List"}]}], ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n4_", ")"}], ",", " ", "m4_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n5_", ")"}], ",", " ", "m5_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{"TVI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n5", ",", "m5"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\
TFI[d_,   PP_/;Head[PP]=!=List,{ {(n3_), m3_}, {(n4_), m4_},
\t\t                           {0, m1_}, {(n2_), m2_}, 
                               {(n5_), m5_} }] := 
TVI[d,PP, {{n5,m5}, {n2,m2}, {n3,m3}, {n4,m4}}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", "   ", 
    RowBox[{"PP_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "PP", "]"}], "=!=", "List"}]}], ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n4_", ")"}], ",", " ", "m4_"}], "}"}], ",", "\n", 
      "\t\t                           ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n5_", ")"}], ",", " ", "m5_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{"TVI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n5", ",", "m5"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\
TFI[d_,   PP_/;Head[PP]=!=List,{ {(n4_), m4_},{(n3_), m3_},
\t\t                           {(n2_), m2_}, {0, m1_}, 
                               {(n5_), m5_} }] := 
TVI[d,PP, {{n5,m5}, {n2,m2}, {n3,m3}, {n4,m4}}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", "   ", 
    RowBox[{"PP_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "PP", "]"}], "=!=", "List"}]}], ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n4_", ")"}], ",", " ", "m4_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}], ",", "\n", 
      "\t\t                           ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n5_", ")"}], ",", " ", "m5_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{"TVI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n5", ",", "m5"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\

TFI[d_,  PP_/;Head[PP]=!=List,{{(n1_), m1_}, 

    {(n2_), m2_}, 

    {(n3_), m3_}, 

    {(n4_), m4_}, {0, m5_}}] := 

  TBI[d, PP, {{n1, m1}, {n3, m3}}]*

   TBI[d, PP, {{n2, m2}, {n4, m4}}]
\
\>", "Text"],

Cell["\<\
TFR[d_, PP_ /; Head[PP] =!= List, 
   {{n1_, m1_}, {n2_, m2_}, {n3_, m3_}, {n4_, m4_}, 
    {0, m5_}}] := TBI[d, PP, {{n1, m1}, {n3, m3}}]*
   TBI[d, PP, {{n2, m2}, {n4, m4}}]\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["four -> three", "Subsection",
 CellTags->"T1.6.122.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"n3_", ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4_", ",", " ", "m4_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], "}"}]}], "]"}], " ", 
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4_", ",", " ", "m4_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], " ", 
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]}]], "Input"],

Cell["\<\
TVI[d_, PP_, {{n1_, m1_}, {0, m2_}, {n3_, m3_}, {n4_, m4_}}] := 
  TJI[d, 0, {{n1, m1}, {n3, m3}, {n4, m4}}]\
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"n3_", ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m4_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{"TJI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}]}], " ", "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["three -> two", "Subsection",
 CellTags->"T1.6.123.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], "}"}]}], "]"}], "  ", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n1_", ")"}], ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m2_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], "  ", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n1_", ")"}], ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m3_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], "  ", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], "}"}]}], "]"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["two -> one", "Subsection",
 CellTags->"T1.6.124.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{"TAI", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"n2", ",", "m2"}], "}"}], "}"}]}], "]"}], " "}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n1_", ")"}], ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m2_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{"TAI", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], " "}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d_", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n_Integer", ",", "m_"}], "}"}], "}"}]}], "]"}], ":=", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n", ",", "m"}], "}"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d_", ",", 
     RowBox[{"{", 
      RowBox[{"n_Integer", ",", "m_"}], "}"}]}], "]"}], ":=", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n", ",", "m"}], "}"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d_", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"n_Integer", ",", "m_"}], "}"}]}], "]"}], ":=", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n", ",", "m"}], "}"}], "}"}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerRecurse and TFIRecurse", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.7.1"],

Cell[CellGroupData[{

Cell["TFIRecurse & TFISimplify", "Subsection",
 CellTags->"T1.7.1.1"],

Cell["This cancels scalar products.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$TLRComment", "=", "False"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLRComment", "[", "s_String", "]"}], ":=", 
   RowBox[{
    RowBox[{"WriteString", "[", 
     RowBox[{"\"\<stdout\>\"", ",", "s", ",", "\"\< \>\""}], "]"}], "/;", 
    RowBox[{"$TLRComment", "===", "True"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"applytlrules", "[", 
    RowBox[{"y_", ",", "fun_"}], "]"}], " ", ":=", 
   RowBox[{"fun", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"y", " ", "/.", 
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "->", "TLR"}]}], "/.", "TLRules"}], 
     "/.", 
     RowBox[{"TLR", "->", 
      StyleBox["TFI",
       FontColor->RGBColor[1, 0.6, 0]]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFIRecurse", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"f_", ":", "Identity"}]}], "]"}], " ", ":=", " ", 
   RowBox[{"FixedPoint", "[", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"applytlrules", "[", 
       RowBox[{"#", ",", "f"}], "]"}], "&"}], ",", "z", ",", "1000"}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"applytlrules2", "[", 
    RowBox[{"y_", ",", "fun_"}], "]"}], " ", ":=", 
   RowBox[{"fun", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"y", " ", "/.", 
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "->", "TLR"}]}], "/.", "TLRules2"}],
      "/.", 
     RowBox[{"TLR", "->", 
      StyleBox["TFI",
       FontColor->RGBColor[1, 0.6, 0]]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFISimplify", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"f_", ":", "Identity"}]}], "]"}], " ", ":=", " ", 
   RowBox[{"FixedPoint", "[", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"applytlrules2", "[", 
       RowBox[{"#", ",", "f"}], "]"}], "&"}], ",", "z", ",", "1000"}], 
    "]"}]}], ";"}]], "Input"],

Cell["\<\
TFIRecurse[z_]:=z/.TFI\[Rule]TLR//.TLRules/.TLR\[Rule]TFI;\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ExpandMaybe", "=", "Identity"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fN", "[", "li_List", "]"}], ":=", 
   RowBox[{
    RowBox[{"fN", "[", "li", "]"}], "=", 
    RowBox[{"li", "/.", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "_"}], "}"}], "\[RuleDelayed]", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}]}]}]}]}], ";"}]], "Input"],

Cell["\<\
RM: new Feb. 1999  
need TLR[ ...]:= rule;
(copy from TFI definition) (in order to get e.g. 
TarcerRecurse[TFI[D,pp,{1,1,1,1,1},{{1,0},{1,M},{0,0},{1,m},{1,M}}]]
to work)\
\>", "Text",
 CellTags->"TLRdef"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"depp__", ",", 
     RowBox[{"{", 
      RowBox[{"a___", ",", "b_Integer", ",", "c___"}], "}"}]}], "]"}], ":=", 
   RowBox[{"TLR", "[", 
    RowBox[{"depp", ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", 
       RowBox[{"{", 
        RowBox[{"b", ",", "0"}], "}"}], ",", "c"}], "}"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "1", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      "pr_"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"NoTLRComment", "[", "\"\<tlrule1\>\"", "]"}], ";", 
      RowBox[{
       StyleBox["TFI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"d", ",", "pp", ",", 
        RowBox[{"pr", "/.", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "_"}], "}"}], "\[RuleDelayed]", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}]}]}]}], "]"}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell["(* q1^2 *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "2", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"v_", "?", "PQ"}], ",", "w_", ",", "x_", ",", "y_", ",", 
        "z_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n1_", "?", "PQ"}], ",", "m1_"}], "}"}], ",", "nm2_", ",", 
        "nm3_", ",", "nm4_", ",", "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule2\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"v", "\[LessEqual]", "n1"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"m1", "===", "0"}], ",", 
           RowBox[{"TLR", "[", 
            RowBox[{"dpp", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "w", ",", "x", ",", "y", ",", "z"}], "}"}], 
             ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"n1", "-", "v"}], ",", "0"}], "}"}], ",", "nm2", ",",
                "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], ",", 
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "v"], 
            RowBox[{
             RowBox[{"Binomial", "[", 
              RowBox[{"v", ",", "i"}], "]"}], " ", 
             RowBox[{"TLR", "[", 
              RowBox[{"dpp", ",", " ", 
               RowBox[{"{", 
                RowBox[{"0", ",", "w", ",", "x", ",", "y", ",", "z"}], "}"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"fN", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"n1", "-", "i"}], ",", "m1"}], "}"}], "]"}], ",", 
                 "nm2", ",", "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], 
             " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox["m1", "2"], ")"}], 
              RowBox[{"v", "-", "i"}]]}]}]}], "]"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"m1", "===", "0"}], ",", 
           RowBox[{"TLR", "[", 
            RowBox[{"dpp", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"v", "-", "n1"}], ",", "w", ",", "x", ",", "y", ",", 
               "z"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", "nm3", ",", 
               "nm4", ",", "nm5"}], "}"}]}], "]"}], ",", 
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "n1"], 
            RowBox[{
             RowBox[{"Binomial", "[", 
              RowBox[{"n1", ",", "i"}], "]"}], " ", 
             RowBox[{"TLR", "[", 
              RowBox[{"dpp", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"v", "-", "n1"}], ",", "w", ",", "x", ",", "y", ",", 
                 "z"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"fN", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"n1", "-", "i"}], ",", "m1"}], "}"}], "]"}], ",", 
                 "nm2", ",", "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], 
             " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox["m1", "2"], ")"}], 
              RowBox[{"n1", "-", "i"}]]}]}]}], "]"}]}], "]"}], "]"}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  SubsuperscriptBox["q", "2", "2"], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "3", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", 
        RowBox[{"w_", "?", "PQ"}], ",", "x_", ",", "y_", ",", "z_"}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2_", "?", "PQ"}], ",", "m2_"}], "}"}], ",", "nm3_", ",", 
        "nm4_", ",", "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule3\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"w", "\[LessEqual]", "n2"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"m2", "===", "0"}], ",", 
           RowBox[{"TLR", "[", 
            RowBox[{"dpp", ",", 
             RowBox[{"{", 
              RowBox[{"v", ",", "0", ",", "x", ",", "y", ",", "z"}], "}"}], 
             ",", 
             RowBox[{"{", 
              RowBox[{"nm1", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"n2", "-", "w"}], ",", "0"}], "}"}], ",", "nm3", ",",
                "nm4", ",", "nm5"}], "}"}]}], "]"}], ",", 
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "w"], 
            RowBox[{
             RowBox[{"Binomial", "[", 
              RowBox[{"w", ",", "i"}], "]"}], " ", 
             RowBox[{"TLR", "[", 
              RowBox[{"dpp", ",", 
               RowBox[{"{", 
                RowBox[{"v", ",", "0", ",", "x", ",", "y", ",", "z"}], "}"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{"nm1", ",", 
                 RowBox[{"fN", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"n2", "-", "i"}], ",", "m2"}], "}"}], "]"}], ",", 
                 "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox["m2", "2"], ")"}], 
              RowBox[{"w", "-", "i"}]]}]}]}], "]"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"m2", "===", "0"}], ",", 
           RowBox[{"TLR", "[", 
            RowBox[{"dpp", ",", 
             RowBox[{"{", 
              RowBox[{"v", ",", 
               RowBox[{"w", "-", "n2"}], ",", "x", ",", "y", ",", "z"}], 
              "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"nm1", ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", "nm4", ",", 
               "nm5"}], "}"}]}], "]"}], ",", 
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "n2"], 
            RowBox[{
             RowBox[{"Binomial", "[", 
              RowBox[{"n2", ",", "i"}], "]"}], " ", 
             RowBox[{"TLR", "[", 
              RowBox[{"dpp", ",", 
               RowBox[{"{", 
                RowBox[{"v", ",", 
                 RowBox[{"w", "-", "n2"}], ",", "x", ",", "y", ",", "z"}], 
                "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"nm1", ",", 
                 RowBox[{"fN", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"n2", "-", "i"}], ",", "m2"}], "}"}], "]"}], ",", 
                 "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox["m2", "2"], ")"}], 
              RowBox[{"n2", "-", "i"}]]}]}]}], "]"}]}], "]"}], "]"}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"p", "\[CenterDot]", 
   SubscriptBox["q", "1"]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "4", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "w_", ",", 
        RowBox[{"x_", "?", "PQ"}], ",", "y_", ",", "z_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n1_", "?", "PQ"}], ",", "m1_"}], "}"}], ",", "nm2_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n3_", "?", "PQ"}], ",", "m3_"}], "}"}], ",", "nm4_", ",", 
        "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule4\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"v", "+", "1"}], ",", "w", ",", 
             RowBox[{"x", "-", "1"}], ",", "y", ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", "nm2", ",", 
             RowBox[{"{", 
              RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", "nm5"}], 
            "}"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", 
             RowBox[{"x", "-", "1"}], ",", "y", ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", "nm2", ",", 
             RowBox[{"fN", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"n3", "-", "1"}], ",", "m3"}], "}"}], "]"}], ",", 
             "nm4", ",", "nm5"}], "}"}]}], "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"{", "dpp", "}"}], "\[LeftDoubleBracket]", "2", 
            "\[RightDoubleBracket]"}], "-", 
           SuperscriptBox["m3", "2"]}], ")"}], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", 
             RowBox[{"x", "-", "1"}], ",", "y", ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", "nm2", ",", 
             RowBox[{"{", 
              RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", "nm5"}], 
            "}"}]}], "]"}]}]}], "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"p", "\[CenterDot]", 
   SubscriptBox["q", "2"]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "5", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "w_", ",", "x_", ",", 
        RowBox[{"y_", "?", "PQ"}], ",", "z_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1_", ",", "m1_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2_", "?", "PQ"}], ",", "m2_"}], "}"}], ",", "nm3_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n4_", "?", "PQ"}], ",", "m4_"}], "}"}], ",", "nm5_"}], 
       "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule5\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", 
             RowBox[{"w", "+", "1"}], ",", "x", ",", 
             RowBox[{"y", "-", "1"}], ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm5"}], "}"}]}], 
          "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", "x", ",", 
             RowBox[{"y", "-", "1"}], ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", 
             RowBox[{"fN", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"n4", "-", "1"}], ",", "m4"}], "}"}], "]"}], ",", 
             "nm5"}], "}"}]}], "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"{", "dpp", "}"}], "\[LeftDoubleBracket]", "2", 
            "\[RightDoubleBracket]"}], "-", 
           SuperscriptBox["m4", "2"]}], ")"}], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", "x", ",", 
             RowBox[{"y", "-", "1"}], ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm5"}], "}"}]}], 
          "]"}]}]}], "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["q", "1"], "\[CenterDot]", 
   SubscriptBox["q", "2"]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "6", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "w_", ",", "x_", ",", "y_", ",", 
        RowBox[{"z_", "?", "PQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1_", ",", "m1_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2_", ",", "m2_"}], "}"}], ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n5_", "?", "PQ"}], ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule6\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"v", "+", "1"}], ",", "w", ",", "x", ",", "y", ",", 
             RowBox[{"z", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", "x", ",", "y", ",", 
             RowBox[{"z", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", "nm4", ",", 
             RowBox[{"fN", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"n5", "-", "1"}], ",", "m5"}], "}"}], "]"}]}], 
            "}"}]}], "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", 
             RowBox[{"w", "+", "1"}], ",", "x", ",", "y", ",", 
             RowBox[{"z", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         SuperscriptBox["m5", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", "x", ",", "y", ",", 
             RowBox[{"z", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}], "]"}]}],
      ")"}]}]}], ";"}]], "Input"],

Cell["(* q1.q1  --> q1.p *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "7", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp___", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"v_", "?", "PQ"}], ",", "w_", ",", "x_", ",", "y_", ",", 
        "z_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm2_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n3_", "?", "PQ"}], ",", "m3_"}], "}"}], ",", "nm4_", ",", 
        "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule7\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"v", "-", "1"}], ",", "w", ",", "x", ",", "y", ",", "z"}],
            "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", 
            RowBox[{"fN", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"n3", "-", "1"}], ",", "m3"}], "}"}], "]"}], ",", 
            "nm4", ",", "nm5"}], "}"}]}], "]"}], "+", 
        RowBox[{"2", " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"v", "-", "1"}], ",", "w", ",", 
             RowBox[{"x", "+", "1"}], ",", "y", ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", 
             RowBox[{"{", 
              RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", "nm5"}], 
            "}"}]}], "]"}]}], "-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"pp", "-", 
           SuperscriptBox["m3", "2"]}], ")"}], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"v", "-", "1"}], ",", "w", ",", "x", ",", "y", ",", 
             "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", 
             RowBox[{"{", 
              RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", "nm5"}], 
            "}"}]}], "]"}]}]}], "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell["(* q2.q2  --> q2.p *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "8", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp___", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", 
        RowBox[{"w_", "?", "PQ"}], ",", "x_", ",", "y_", ",", "z_"}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm3_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n4_", "?", "PQ"}], ",", "m4_"}], "}"}], ",", "nm5_"}], 
       "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule8\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", 
            RowBox[{"w", "-", "1"}], ",", "x", ",", "y", ",", "z"}], "}"}], 
          ",", 
          RowBox[{"{", 
           RowBox[{"nm1", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", 
            RowBox[{"fN", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"n4", "-", "1"}], ",", "m4"}], "}"}], "]"}], ",", 
            "nm5"}], "}"}]}], "]"}], "+", 
        RowBox[{"2", " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", 
             RowBox[{"w", "-", "1"}], ",", "x", ",", 
             RowBox[{"y", "+", "1"}], ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm5"}], "}"}]}], 
          "]"}]}], "-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"pp", "-", 
           SuperscriptBox["m4", "2"]}], ")"}], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", 
             RowBox[{"w", "-", "1"}], ",", "x", ",", "y", ",", "z"}], "}"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm5"}], "}"}]}], 
          "]"}]}]}], "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell["(* eq. (3) - (5) *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "9", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "w_", ",", "x_", ",", "y_", ",", 
        RowBox[{"z_", "?", "PQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", "nm2_", ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule9\>\"", "]"}], ";", 
      RowBox[{"FunctionExpand", "[", 
       RowBox[{"ExpandMaybe", "[", 
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"k", "=", "0"}], 
          RowBox[{"Floor", "[", 
           FractionBox["z", "2"], "]"}]], 
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], "k"], 
          RowBox[{
           UnderoverscriptBox["\[Sum]", 
            RowBox[{"j", "=", "0"}], "k"], 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Gamma", "[", 
               RowBox[{"k", "+", 
                FractionBox["1", "2"]}], "]"}], " ", 
              RowBox[{"Gamma", "[", 
               RowBox[{
                FractionBox["d", "2"], "-", 
                FractionBox["1", "2"]}], "]"}], " ", 
              RowBox[{"Binomial", "[", 
               RowBox[{"z", ",", 
                RowBox[{"2", " ", "k"}]}], "]"}], " ", 
              RowBox[{"Binomial", "[", 
               RowBox[{"k", ",", "i"}], "]"}], " ", 
              RowBox[{"Binomial", "[", 
               RowBox[{"k", ",", "j"}], "]"}], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], 
               RowBox[{"i", "+", "j"}]], " ", 
              SuperscriptBox["pp", 
               RowBox[{"i", "+", "j", "-", "z"}]], " ", 
              RowBox[{"TLR", "[", 
               RowBox[{"d", ",", "pp", ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"v", "+", "i"}], ",", 
                  RowBox[{"w", "+", "j"}], ",", 
                  RowBox[{"x", "+", "z", "-", 
                   RowBox[{"2", " ", "i"}]}], ",", 
                  RowBox[{"y", "+", "z", "-", 
                   RowBox[{"2", " ", "j"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], ")"}], 
            "/", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Gamma", "[", 
               FractionBox["1", "2"], "]"}], " ", 
              RowBox[{"Gamma", "[", 
               RowBox[{"k", "+", 
                FractionBox["d", "2"], "-", 
                FractionBox["1", "2"]}], "]"}]}], ")"}]}]}]}]}], "]"}], 
       "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell[" OLD trule[10]:", "Text"],

Cell["\<\
tlrule[10] = 
(* (9) and (10) *)
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {{n1_?PQ,m1_},{n2_?PQ,m2_}, 
                                {0,_},{0,_},nm5_}] :>  
( TLRComment[\"tlrule10\"]; 0 /; OddQ[x+y] );\
\>", "Text"],

Cell["New tlrule[10]:", "Text"],

Cell["\<\
tlrule[10] = 
   TLR[d_, pp_, {v_, w_, (x_)?PNQ, (y_)?PNQ, z_}, 
     {{(n1_)?PQ, m1_}, {(n2_)?PQ, m2_}, {0, _}, {0, _}, nm5_}] :> \
(TLRComment[\"tlrule10\"]; 0 /; OddQ[x + y]); \
\>", "Input"],

Cell["\<\
Bug in  tlrule[11] and tlrule[12] fixed on Feb. 26th 1998 by R.Scharf 
with thanks to York Schroeder for locating the bug.\
\>", "Text"],

Cell["\<\

tlrule[11] = 
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {nm1_,nm2_,{0,_},{0,_},{(n5_)?PQ, m5_}}] \
:> 
(
TLRComment[\"tlrule11\"];
 ExpandMaybe[
 pp/(d+x+y-2)( (x-1) TLR[d, pp, {v+1,w,x-2,y,z}, 
                         {{n1,m1},{n2,m2},{0,0},{0,0},{n5,m5}}] +
                   y TLR[d, pp, {v,w,x-1,y-1,z+1}, 
                         {nm1,nm2,{0,0},{0,0},{n5,m5}}]
             )
           ] /; EvenQ[x+y] && x > 1
);\
\>", "Input"],

Cell["\<\

tlrule[12] = 
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {nm1_,nm2_,{0,_},{0,_},{(n5_)?PQ, m5_}}] \
:> 
(
TLRComment[\"tlrule12\"];
 ExpandMaybe[
 pp/(d+x+y-2)( (y-1) TLR[d, pp, {v,w+1,x,y-2,z}, 
                         {{n1,m1},{n2,m2},{0,0},{0,0},{n5,m5}}] +
                   x TLR[d, pp, {v,w,x-1,y-1,z+1}, 
                         {nm1,nm2,{0,0},{0,0},{n5,m5}}]
             )
           ] /; EvenQ[x+y] && y > 1
);\
\>", "Input"],

Cell["(*symm.   1 <-> 3  &&  2 <-> 4 *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "13", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0", ",", 
        RowBox[{"y_", "?", "PQ"}], ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", "nm3_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n4_", "?", "PQ"}], ",", "m4_"}], "}"}], ",", "nm5_"}], 
       "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule13\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "y"], 
       RowBox[{
        RowBox[{"Binomial", "[", 
         RowBox[{"y", ",", "i"}], "]"}], " ", 
        SuperscriptBox["pp", 
         RowBox[{"y", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "i", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", 
           RowBox[{"nm3", ",", 
            RowBox[{"{", 
             RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm1", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", "nm5"}], "}"}]}], 
         "]"}]}]}]}], ")"}]}]}], ";"}]], "Input"],

Cell["(*symm.   1 <-> 3  &&  2 <-> 4 *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "14", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", 
        RowBox[{"x_", "?", "PQ"}], ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", "nm2_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n3_", "?", "PQ"}], ",", "m3_"}], "}"}], ",", "nm4_", ",", 
        "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule14\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "x"], 
       RowBox[{
        RowBox[{"Binomial", "[", 
         RowBox[{"x", ",", "i"}], "]"}], " ", 
        SuperscriptBox["pp", 
         RowBox[{"x", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "i", ",", "0", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", "nm5"}], 
           "}"}]}], "]"}]}]}]}], ")"}]}]}], ";"}]], "Input"],

Cell["(* 0's *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "15", "]"}], "=", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"TLR", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"v_", ",", "w_", ",", "x_", ",", "y_", ",", "z_"}], "}"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1_", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2_", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule15\>\"", "]"}], ";", "0"}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "16", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", 
        RowBox[{"x_", "?", "PQ"}], ",", 
        RowBox[{"y_", "?", "PQ"}], ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1_", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2_", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule16\>\"", "]"}], ";", 
      RowBox[{"0", "/;", 
       RowBox[{
        RowBox[{"OddQ", "[", "x", "]"}], "||", 
        RowBox[{"OddQ", "[", "y", "]"}]}]}]}], ")"}]}]}], ";"}]], "Input"],

Cell["bug in tlrule[17] fixedApril 23rd 1999", "Text",
 CellTags->"tlrule17"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "17", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"a_", "?", "PQ"}], ",", "b_", ",", "0", ",", "de_", ",", 
        "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2_", ",", "m2_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n4_", ",", "m4_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5_", ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule17\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "a"], 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], "i"], 
        RowBox[{
         RowBox[{"Binomial", "[", 
          RowBox[{"a", ",", "i"}], "]"}], " ", 
         RowBox[{"Binomial", "[", 
          RowBox[{"i", ",", "j"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "j"]}], ")"}], " ", 
         SuperscriptBox["2", 
          RowBox[{"j", "-", "1"}]], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"a", "-", "i"}], ",", 
             RowBox[{"b", "+", "i", "-", "j"}], ",", "0", ",", "de", ",", 
             "j"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n5", ",", "m5"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}]}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "18", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"a_", "?", "PNQ"}], ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2_", ",", "m2_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3_", ",", "m3_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n4_", ",", "m4_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5_", ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule18\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "a"], 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], "b"], 
        RowBox[{
         RowBox[{"Binomial", "[", 
          RowBox[{"a", ",", "i"}], "]"}], " ", 
         RowBox[{"Binomial", "[", 
          RowBox[{"b", ",", "j"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"pp", "+", 
              SuperscriptBox["m3", "2"]}], ")"}], " ", 
            RowBox[{"TLR", "[", 
             RowBox[{"d", ",", "pp", ",", "dp", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"n3", ",", "m3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n4", ",", "m4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n2", ",", "m2"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "+", 
           RowBox[{"TLR", "[", 
            RowBox[{"d", ",", "pp", ",", "dp", ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", "j"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"n3", "-", "1"}], ",", "m3"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n4", ",", "m4"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n2", ",", "m2"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"TLR", "[", 
             RowBox[{"d", ",", "pp", ",", "dp", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0", ",", "1", ",", "0", ",", "0"}], "}"}], 
              ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"n3", ",", "m3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n4", ",", "m4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n2", ",", "m2"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}], 
          ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"i", "+", "j"}]], " ", 
         SuperscriptBox["dp", 
          RowBox[{"a", "+", "b", "-", "i", "-", "j"}]]}]}]}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "19", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"a_", "?", "PNQ"}], ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1_", ",", "m1_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3_", ",", "m3_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n4_", ",", "m4_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5_", ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule18\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "a"], 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], "b"], 
        RowBox[{
         RowBox[{"Binomial", "[", 
          RowBox[{"a", ",", "i"}], "]"}], " ", 
         RowBox[{"Binomial", "[", 
          RowBox[{"b", ",", "j"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"pp", "+", 
              SuperscriptBox["m4", "2"]}], ")"}], " ", 
            RowBox[{"TLR", "[", 
             RowBox[{"d", ",", "pp", ",", "dp", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"n3", ",", "m3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n4", ",", "m4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n1", ",", "m1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "+", 
           RowBox[{"TLR", "[", 
            RowBox[{"d", ",", "pp", ",", "dp", ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", "j"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"n3", ",", "m3"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"n4", "-", "1"}], ",", "m4"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n1", ",", "m1"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"TLR", "[", 
             RowBox[{"d", ",", "pp", ",", "dp", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], 
              ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"n3", ",", "m3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n4", ",", "m4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n1", ",", "m1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}], 
          ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"i", "+", "j"}]], " ", 
         SuperscriptBox["dp", 
          RowBox[{"a", "+", "b", "-", "i", "-", "j"}]]}]}]}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell["(* q1.q1  --> q1.p *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "20", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"v_", "?", "PQ"}], ",", "w_", ",", "x_", ",", "y_", ",", 
        "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm2_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n5_", "?", "PQ"}], ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule20\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], "x"], 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], "v"], 
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"r", "=", "0"}], "i"], 
         RowBox[{
          RowBox[{"Binomial", "[", 
           RowBox[{"i", ",", "r"}], "]"}], " ", 
          RowBox[{"Binomial", "[", 
           RowBox[{"v", ",", "i"}], "]"}], " ", 
          RowBox[{"Binomial", "[", 
           RowBox[{"x", ",", "j"}], "]"}], " ", 
          SuperscriptBox["2", "r"], " ", 
          RowBox[{"TLR", "[", 
           RowBox[{"d", ",", "pp", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"v", "-", "i"}], ",", 
              RowBox[{
               RowBox[{"-", "r"}], "+", "i", "+", "w"}], ",", "j", ",", 
              RowBox[{"x", "-", "j", "+", "y"}], ",", "r"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"n5", ",", "m5"}], "}"}], ",", "nm2", ",", "0", ",", 
              "nm4", ",", "0"}], "}"}]}], "]"}]}]}]}]}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "21", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", 
        RowBox[{"w_", "?", "PQ"}], ",", "x_", ",", "y_", ",", "0"}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm3_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm5_"}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule21\>\"", "]"}], ";", 
      RowBox[{"TLR", "[", 
       RowBox[{"d", ",", "pp", ",", 
        RowBox[{"{", 
         RowBox[{"w", ",", "v", ",", "y", ",", "x", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}], ",", "nm1", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", "nm5"}], "}"}]}], 
       "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "22", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"w_", "?", "PNQ"}], ",", "0", ",", 
        RowBox[{"y_", "?", "PNQ"}], ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", "nm2_", ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule22\>\"", "]"}], ";", 
      RowBox[{
       FractionBox["1", "pp"], 
       RowBox[{"(", 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "w", ",", "1", ",", 
            RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[TextData[{
 "Using  FeynCalc : \n\n",
 Cell[BoxData[
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", "p", "}"}]}], "]"}]], "Input"],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[NoBreak]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["k1",
             TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            FormBox["p",
             TraditionalForm]}],
           TraditionalForm], ")"}], "2"], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           FormBox[
            RowBox[{
             FormBox["k1",
              TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], ")"}], "2"]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]]}], TraditionalForm]], "Output"],
 "\n",
 Cell[BoxData[
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"Calc", "[", " ", 
     RowBox[{
      RowBox[{"FVD", "[", 
       RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"k2", ",", "\[Nu]"}], "]"}], "%"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k1", ",", "k2"}], "}"}]}], "]"}]], "Input"],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["k1",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "2"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]], "-", 
    FractionBox[
     RowBox[{"D", " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["k1",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "2"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       FormBox["k1",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}]}], 
     RowBox[{"1", "-", "D"}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       FormBox["k1",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]]}], TraditionalForm]], "Output"],
 "\n"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "23", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"w_", "?", "PNQ"}], ",", "0", ",", 
        RowBox[{"y_", "?", "PNQ"}], ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", "nm2_", ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule23\>\"", "]"}], ";", 
      RowBox[{
       RowBox[{
        FractionBox["1", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], "pp"}]], 
        RowBox[{"(", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"b", "+", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "w", ",", "2", ",", "y", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}], "-", 
       
       RowBox[{
        FractionBox[
         RowBox[{"d", "  ", "dp"}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], 
          SuperscriptBox["pp", "2"]}]], 
        RowBox[{"(", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "b"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "w", ",", "2", ",", 
             RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}], "-", 
       
       RowBox[{
        FractionBox["1", 
         RowBox[{"1", "-", "d"}]], 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"b", "+", "1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "w", ",", "0", ",", "y", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], "+", 
       RowBox[{
        FractionBox["dp", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], "pp"}]], 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "w", ",", "0", ",", 
            RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[TextData[{
 Cell[BoxData[
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", "p", "}"}]}], "]"}]], "Input"],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["k1",
         TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"D", "+", "2"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            FormBox[
             RowBox[{
              FormBox["k1",
               TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
              FormBox["p",
               TraditionalForm]}],
             TraditionalForm], ")"}], "2"]}], "-", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           FormBox["k1",
            TraditionalForm], "2"], " ", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"]}]}], ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", "D"}], ")"}], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "6"]}]]}], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], "\[NoBreak]", 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["k1",
             TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            FormBox["p",
             TraditionalForm]}],
           TraditionalForm], ")"}], "2"], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[NoBreak]", 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["k1",
             TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            FormBox["p",
             TraditionalForm]}],
           TraditionalForm], ")"}], "2"], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[NoBreak]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["k1",
             TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            FormBox["p",
             TraditionalForm]}],
           TraditionalForm], ")"}], "2"], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]]}], TraditionalForm]], "Output"],
 "\n",
 Cell[BoxData[
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"Calc", "[", " ", 
     RowBox[{
      RowBox[{"FVD", "[", 
       RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"OPEDelta", ",", "\[Nu]"}], "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"k2", ",", "\[Rho]"}], "]"}], "%"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k1", ",", "k2"}], "}"}]}], "]"}]], "Input"],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"2", " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["k1",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "3"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]], "-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "+", "2"}], ")"}], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["k1",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "3"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "6"]}]], "-", 
    FractionBox[
     RowBox[{"2", " ", 
      SuperscriptBox[
       FormBox["k1",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]], "+", 
    FractionBox[
     RowBox[{"3", " ", 
      SuperscriptBox[
       FormBox["k1",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "2"], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]]}], TraditionalForm]], "Output"]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "24", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"w_", "?", "PNQ"}], ",", "0", ",", 
        RowBox[{"y_", "?", "PNQ"}], ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", "nm2_", ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule24\>\"", "]"}], ";", 
      RowBox[{
       RowBox[{"2", "dp", 
        RowBox[{
         FractionBox["1", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", "-", "d"}], ")"}], 
           SuperscriptBox["pp", "2"]}]], 
         RowBox[{"(", 
          RowBox[{"TLR", "[", 
           RowBox[{"d", ",", "pp", ",", "dp", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", 
              RowBox[{"b", "+", "1"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "w", ",", "3", ",", "y", ",", "0"}], "}"}], 
            ",", 
            RowBox[{"{", 
             RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}]}], 
       "-", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{"d", "+", "2"}], ")"}], "  ", 
          SuperscriptBox["dp", "2"]}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], 
          SuperscriptBox["pp", "3"]}]], 
        RowBox[{"(", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "b"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "w", ",", "3", ",", 
             RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}], "-", 
       
       RowBox[{
        FractionBox[
         RowBox[{"2", "dp"}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], "pp"}]], 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"b", "+", "1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "w", ",", "1", ",", "y", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], "+", 
       RowBox[{
        FractionBox[
         RowBox[{"3", 
          SuperscriptBox["dp", "2"]}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], 
          SuperscriptBox["pp", "2"]}]], 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "w", ",", "1", ",", 
            RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell["Symmetry 1 <--> 2; added February 21 1999 by R.M.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tlrule", "[", "25", "]"}], "=", 
    RowBox[{
     RowBox[{"TLR", "[", 
      RowBox[{"dim_", ",", "pp_", ",", " ", 
       RowBox[{"{", 
        RowBox[{"a_", ",", " ", "b_", ",", " ", 
         RowBox[{
          RowBox[{"(", "c_", ")"}], "?", "PNQ"}], ",", " ", 
         RowBox[{
          RowBox[{"(", "d_", ")"}], "?", "PQ"}], ",", " ", "0"}], "}"}], ",", 
       " ", 
       RowBox[{"{", 
        RowBox[{"nm1_", ",", " ", "nm2_", ",", " ", "nm3_", ",", " ", 
         RowBox[{"{", 
          RowBox[{"0", ",", " ", "_"}], "}"}], ",", " ", "nm5_"}], "}"}]}], 
      "\n", "   ", "]"}], " ", ":>", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"TLRComment", "[", "\"\<tlrule25\>\"", "]"}], ";", " ", 
        RowBox[{"TLR", "[", 
         RowBox[{"dim", ",", " ", "pp", ",", " ", 
          RowBox[{"{", 
           RowBox[{
           "b", ",", "a", ",", " ", "d", ",", " ", "c", ",", " ", "0"}], 
           "}"}], ",", " ", 
          RowBox[{"{", 
           RowBox[{"nm2", ",", " ", "nm1", ",", " ", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", " ", "nm3", ",", " ", 
            "nm5"}], "}"}]}], "]"}]}], ")"}], " ", "/;", " ", 
      RowBox[{"d", " ", ">", " ", "c"}]}]}]}], ";"}], "\n"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TLRules", "=", 
   RowBox[{"Array", "[", 
    RowBox[{"tlrule", ",", "25"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TLRules2", "=", 
   RowBox[{"Drop", "[", 
    RowBox[{"TLRules", ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "9"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"seeetDelayed", ",", "HoldAll"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"seeetDelayed", "[", 
   RowBox[{
    RowBox[{"TLR", "[", "a__", "]"}], ",", "b_"}], "]"}], ":=", 
  RowBox[{"SetDelayed", "@@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"TLR", "[", "a", "]"}], ",", "b"}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"lis1", "=", 
   RowBox[{"Permutations", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}]}], "}"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"lis2", "=", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"TLR", "[", 
        RowBox[{"__", ",", 
         RowBox[{
         "lis1", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}]}], 
        "]"}], "\[RuleDelayed]", "0"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "lis1", "]"}]}], "}"}]}], "]"}], "/.", 
    RowBox[{"RuleDelayed", "\[Rule]", "seeetDelayed"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"lis1", ",", "lis2"}], "]"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["T*IS", "Subsection",
 CellTags->"T1.7.2.1"],

Cell["\<\
These functions below are never used.
They can serve as tests.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TJIS", "[", 
    RowBox[{"d_", ",", 
     RowBox[{"Power", "[", 
      RowBox[{"M_", ",", " ", "2"}], "]"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
   RowBox[{
    SuperscriptBox["SMu", "2"], 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", " ", 
          SuperscriptBox["M", "2"]}], ")"}], 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]]], 
      RowBox[{"Exp", "[", 
       RowBox[{
        RowBox[{"-", "I"}], " ", "Pi", " ", 
        RowBox[{"(", 
         RowBox[{"d", "-", "4"}], ")"}]}], "]"}], " ", 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], 
        "]"}], 
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{
          FractionBox["3", "2"], "d"}], "-", "\[Alpha]", "-", "\[Beta]", "-", 
         "\[Gamma]"}], "]"}]], 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{
         FractionBox["d", "2"], "-", "\[Alpha]"}], "]"}], 
       RowBox[{"Gamma", "[", "\[Alpha]", "]"}]], 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{
         FractionBox["d", "2"], "-", "\[Beta]"}], "]"}], 
       RowBox[{"Gamma", "[", "\[Beta]", "]"}]], 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{
         FractionBox["d", "2"], "-", "\[Gamma]"}], "]"}], 
       RowBox[{"Gamma", "[", "\[Gamma]", "]"}]]}], ")"}]}]}], "\n"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", 
    RowBox[{"Power", "[", 
     RowBox[{"M_", ",", " ", "2"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  "\t", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", 
    RowBox[{"M", "^", "2"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", 
    RowBox[{"Power", "[", 
     RowBox[{"M_", ",", " ", "2"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  "\t", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", 
    RowBox[{"M", "^", "2"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", 
    RowBox[{"Power", "[", 
     RowBox[{"M_", ",", " ", "2"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  RowBox[{
   FractionBox[
    SuperscriptBox["SMu", "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["M", "2"], ")"}], 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]]], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], 
   FractionBox[
    RowBox[{"Gamma", "[", 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], "]"}], 
    RowBox[{
     RowBox[{"Gamma", "[", "\[Alpha]", "]"}], 
     RowBox[{"Gamma", "[", "\[Beta]", "]"}], 
     RowBox[{"Gamma", "[", "\[Gamma]", "]"}]}]], 
   FractionBox[
    RowBox[{
     RowBox[{"Gamma", "[", 
      RowBox[{
       FractionBox["d", "2"], "-", "\[Gamma]"}], "]"}], 
     RowBox[{"Gamma", "[", 
      RowBox[{
       FractionBox["d", "2"], "-", "\[Beta]"}], "]"}]}], 
    RowBox[{"Gamma", "[", 
     RowBox[{"d", "-", "\[Beta]", "-", "\[Gamma]"}], "]"}]], 
   FractionBox[
    RowBox[{
     RowBox[{"Gamma", "[", 
      RowBox[{"\[Beta]", "+", "\[Gamma]", "-", 
       FractionBox["d", "2"]}], "]"}], 
     RowBox[{"Gamma", "[", 
      RowBox[{
       RowBox[{"2", " ", "d"}], "-", "\[Alpha]", "-", 
       RowBox[{"2", "\[Beta]"}], "-", 
       RowBox[{"2", "\[Gamma]"}]}], "]"}]}], 
    RowBox[{"Gamma", "[", 
     RowBox[{
      FractionBox[
       RowBox[{"3", "d"}], "2"], "-", "\[Alpha]", "-", "\[Beta]", "-", 
      "\[Gamma]"}], "]"}]]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  RowBox[{
   FractionBox[
    SuperscriptBox["SMu", "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["M", "2"], ")"}], 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]]], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], 
   FractionBox[
    RowBox[{"Gamma", "[", 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], "]"}], 
    RowBox[{
     RowBox[{"Gamma", "[", "\[Alpha]", "]"}], 
     RowBox[{"Gamma", "[", "\[Beta]", "]"}], 
     RowBox[{"Gamma", "[", "\[Gamma]", "]"}]}]], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Gamma", "[", 
       RowBox[{"\[Beta]", "+", "\[Gamma]", "-", 
        RowBox[{"d", "/", "2"}]}], "]"}], 
      RowBox[{"Gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "\[Beta]"}], "]"}], 
      RowBox[{"Gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "\[Gamma]"}], "]"}]}], ")"}], "/", 
    RowBox[{"Gamma", "[", 
     RowBox[{"d", "/", "2"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "M"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "M"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  
  RowBox[{
   FractionBox[
    SuperscriptBox["SMu", "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["M", "2"], ")"}], 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]]], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], 
   FractionBox[
    RowBox[{"Gamma", "[", 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], "]"}], 
    RowBox[{
     RowBox[{"Gamma", "[", "\[Beta]", "]"}], 
     RowBox[{"Gamma", "[", "\[Gamma]", "]"}]}]], 
   FractionBox[
    RowBox[{"Gamma", "[", 
     RowBox[{
      FractionBox["d", "2"], "-", "\[Alpha]"}], "]"}], 
    RowBox[{"Gamma", "[", 
     FractionBox["d", "2"], "]"}]], 
   FractionBox[
    RowBox[{
     RowBox[{"Gamma", "[", 
      RowBox[{"\[Alpha]", "+", "\[Gamma]", "-", 
       FractionBox["d", "2"]}], "]"}], 
     RowBox[{"Gamma", "[", 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "-", 
       FractionBox["d", "2"]}], "]"}]}], 
    RowBox[{"Gamma", "[", 
     RowBox[{
      RowBox[{"2", "\[Alpha]"}], "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], 
     "]"}]]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["SMu", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], " ", 
     RowBox[{"Gamma", "[", 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], "]"}], 
     " ", 
     RowBox[{"Gamma", "[", 
      RowBox[{
       FractionBox["d", "2"], "-", "\[Alpha]"}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Gamma", "[", 
        RowBox[{"\[Alpha]", "+", "\[Gamma]", "-", 
         FractionBox["d", "2"]}], "]"}], " ", 
       RowBox[{"Gamma", "[", 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "-", 
         FractionBox["d", "2"]}], "]"}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["M", "2"], ")"}], 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Gamma", "[", "\[Beta]", "]"}], " ", 
       RowBox[{"Gamma", "[", "\[Gamma]", "]"}]}], ")"}], " ", 
     RowBox[{"Gamma", "[", 
      FractionBox["d", "2"], "]"}], " ", 
     RowBox[{"Gamma", "[", 
      RowBox[{
       RowBox[{"2", " ", "\[Alpha]"}], "+", "\[Beta]", "+", "\[Gamma]", "-", 
       "d"}], "]"}]}], ")"}]}]}]], "Input"],

Cell["(2.65)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TAIS", "[", 
   RowBox[{"d_", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"1", ",", "M_"}], "}"}], "}"}]}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"-", "I"}], "  ", 
   SuperscriptBox["M", "2"], "  ", "SMu", " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"4", "-", "d"}], ")"}], "/", "2"}], "-", "1"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TBIS", "[", 
    RowBox[{"d_", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Beta]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TBIS", "[", 
    RowBox[{"d_", ",", 
     RowBox[{"Power", "[", 
      RowBox[{"M_", ",", " ", "2"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Beta]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
   RowBox[{"I", " ", "SMu", " ", "*", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"\[Alpha]", "+", "\[Beta]"}]], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", " ", 
          SuperscriptBox["M", "2"]}], ")"}], 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "2"}]]], 
      RowBox[{"Exp", "[", 
       RowBox[{
        RowBox[{"-", "I"}], " ", "Pi", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], "/", "2"}]}], "]"}], " ", 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "-", 
         RowBox[{"d", "/", "2"}]}], "]"}], 
       RowBox[{
        RowBox[{"Gamma", "[", "\[Alpha]", "]"}], 
        RowBox[{"Gamma", "[", "\[Beta]", "]"}]}]], 
      FractionBox[
       RowBox[{
        RowBox[{"Gamma", "[", 
         RowBox[{
          RowBox[{"d", "/", "2"}], "-", "\[Alpha]"}], "]"}], 
        RowBox[{"Gamma", "[", 
         RowBox[{
          RowBox[{"d", "/", "2"}], "-", "\[Beta]"}], "]"}]}], 
       RowBox[{"Gamma", "[", 
        RowBox[{"d", "-", "\[Alpha]", "-", "\[Beta]"}], "]"}]]}], ")"}]}]}], 
  "\n"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_Symbol", ",", 
    SuperscriptBox["M_", "2"], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   SuperscriptBox["M", "2"], " ", 
   SuperscriptBox["SMu", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["6", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "4"}], "+", "d"}], ")"}], "2"]]}], "+", 
     FractionBox["17", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "4"}], "+", "d"}], ")"}]}]], "+", 
     RowBox[{
      FractionBox["1", "8"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "59"}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["\[Pi]", "2"]}]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "4"}], "+", "d"}], ")"}], "3"], " ", "TJI111e"}], "+", 
     RowBox[{
      FractionBox["1", "1920"], 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "4"}], "+", "d"}], ")"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{"16755", "-", 
          RowBox[{"4750", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", 
          RowBox[{"14", " ", 
           SuperscriptBox["\[Pi]", "4"]}], "+", 
          RowBox[{"3840", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           RowBox[{"Log", "[", "2", "]"}]}], "-", 
          RowBox[{"12080", " ", 
           RowBox[{"Zeta", "[", "3", "]"}]}]}], ")"}]}], ")"}]}], "+", 
     RowBox[{
      FractionBox["1", "96"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "4"}], "+", "d"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"195", "+", 
        RowBox[{"98", " ", 
         SuperscriptBox["\[Pi]", "2"]}], "-", 
        RowBox[{"48", " ", 
         RowBox[{"Zeta", "[", "3", "]"}]}]}], ")"}]}]}], ")"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TComment", "Subsection",
 CellTags->"T1.7.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$CommentNotebook", "=", "True"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TComment", "[", 
   RowBox[{"s_String", ",", "b_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"If", "[", "\[InvisibleSpace]", 
       RowBox[{
        RowBox[{"!", 
         RowBox[{"(", 
          RowBox[{"ValueQ", "[", "$TarcerRecursed", "]"}], ")"}]}], ",", 
        RowBox[{"$TarcerRecursed", "=", 
         RowBox[{"{", "}"}]}]}], "]"}], ";", "\[InvisibleSpace]", 
      RowBox[{"!", 
       RowBox[{"(", 
        RowBox[{"MemberQ", "[", 
         RowBox[{"$TarcerRecursed", ",", 
          RowBox[{"{", 
           RowBox[{"s", ",", "b"}], "}"}]}], "]"}], ")"}]}]}], ",", 
     RowBox[{
      RowBox[{"AppendTo", "[", 
       RowBox[{"$TarcerRecursed", ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", "b"}], "}"}]}], "]"}], ";", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"$Notebooks", "&&", 
         RowBox[{"$CommentNotebook", "===", "True"}]}], ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"$commentnb", "===", "False"}], ",", 
          RowBox[{
           RowBox[{"$commentnb", "=", "True"}], ";", 
           RowBox[{"$cnb", "=", 
            RowBox[{"NotebookCreate", "[", 
             RowBox[{
              RowBox[{
              "StyleDefinitions", "\[Rule]", "\"\<NaturalColor.nb\>\""}], ",", 
              RowBox[{"Magnification", "\[Rule]", "1.5"}]}], "]"}]}], ";", 
           RowBox[{"SetOptions", "[", 
            RowBox[{"$cnb", ",", 
             RowBox[{"WindowMargins", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "Automatic"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"Automatic", ",", "0"}], "}"}]}], "}"}]}]}], "]"}], 
           ";", 
           RowBox[{"SetOptions", "[", 
            RowBox[{"$cnb", ",", 
             RowBox[{"WindowSize", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"500", ",", "300"}], "}"}]}]}], "]"}], ";", 
           RowBox[{"NotebookWrite", "[", 
            RowBox[{"$cnb", ",", 
             RowBox[{"Cell", "[", 
              RowBox[{
               RowBox[{"BoxData", "[", 
                RowBox[{"ToBoxes", "[", 
                 RowBox[{
                  RowBox[{"b", "/.", 
                   RowBox[{"TFIC", "\[Rule]", "TFI"}]}], ",", 
                  "StandardForm"}], "]"}], "]"}], ",", "\"\<Input\>\"", ",", 
               RowBox[{"CellDingbat", "\[Rule]", 
                RowBox[{"s", "<>", "\"\< \>\""}]}]}], "]"}]}], "]"}]}], ",", 
          RowBox[{"NotebookWrite", "[", 
           RowBox[{"$cnb", ",", 
            RowBox[{"Cell", "[", 
             RowBox[{
              RowBox[{"BoxData", "[", 
               RowBox[{"ToBoxes", "[", 
                RowBox[{
                 RowBox[{"b", "/.", 
                  RowBox[{"TFIC", "\[Rule]", "TFI"}]}], ",", "StandardForm"}],
                 "]"}], "]"}], ",", "\"\<Input\>\"", ",", 
              RowBox[{"CellDingbat", "\[Rule]", 
               RowBox[{"s", "<>", "\"\< \>\""}]}]}], "]"}]}], "]"}]}], "]"}], 
        ",", 
        RowBox[{"Print", "[", 
         RowBox[{"s", ",", 
          RowBox[{"b", "/.", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"TFIC", "\[RuleDelayed]", "TFI"}], ",", 
             RowBox[{"TVIC", "\[RuleDelayed]", "TVI"}], ",", 
             RowBox[{"TJIC", "\[RuleDelayed]", "TJI"}], ",", 
             RowBox[{"TBIC", "\[RuleDelayed]", "TBI"}], ",", 
             RowBox[{"TAIC", "\[RuleDelayed]", "TAI"}]}], "}"}]}]}], "]"}]}], 
       "]"}]}]}], "]"}], "/;", 
   RowBox[{"$Comment", "===", "True"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["$TTable", "Subsection",
 CellTags->"$TTable"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$TTable", "=", 
   RowBox[{"{", "}"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerRecurse", "Subsection",
 CellTags->"T1.7.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FEPrint", "[", "a__String", "]"}], ":=", 
   RowBox[{
    RowBox[{"Print", "[", "a", "]"}], " ", "/;", " ", 
    RowBox[{"$Notebooks", "===", "False"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FEPrint", "[", "a__String", "]"}], ":=", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Head", "[", "$cnb", "]"}], "===", "NotebookObject"}], ",", 
      "\n", "\t\t", 
      RowBox[{
       RowBox[{"NotebookWrite", "[", 
        RowBox[{"$cnb", ",", 
         RowBox[{"Cell", "[", 
          RowBox[{
           RowBox[{"StringJoin", "@@", 
            RowBox[{"{", "a", "}"}]}], ",", "\"\<SmallText\>\""}], "]"}]}], 
        "]"}], ";", "\n", "\t", 
       RowBox[{"SelectionMove", "[", 
        RowBox[{"$cnb", ",", "After", ",", "Cell"}], "]"}]}]}], "]"}], "/;", 
    RowBox[{"$Notebooks", "===", "True"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "TarcerRecurse", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Factor", "\[Rule]", "Factor"}], ",", 
     RowBox[{"TimeConstraint", "->", "Infinity"}], ",", 
     RowBox[{"Table", ":>", "$TTable"}], ",", 
     RowBox[{"Together", "->", "Together"}], ",", 
     RowBox[{"Replace", "->", 
      RowBox[{"{", "}"}]}]}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerRecurse", "[", 
    RowBox[{"z_", ",", "opts___"}], "]"}], ":=", 
   RowBox[{"Catch", "[", 
    RowBox[{
     StyleBox["Block",
      FontSize->16,
      FontWeight->"Bold",
      FontColor->RGBColor[1, 0.2, 0.2]], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "basisdone", ",", "factorfun", ",", "fertig", ",", "dummm1", ",", 
        "dummm2", ",", "getvars", ",", "te", ",", "time", ",", "cutoff", ",", 
        "tog", ",", "ttable", ",", "facfun", ",", "nexp", ",", "oldexpr", ",",
         "rec", ",", "special", ",", "sti", ",", "tl", ",", "tcheck", ",", 
        "tjrd", ",", "tvrd", ",", "together", ",", "vars", ",", 
        RowBox[{"$cc", "=", "0"}], ",", "$cv"}], "}"}], ",", "\n", "\t\t\t", 
      RowBox[{
       RowBox[{"time", " ", "=", " ", 
        RowBox[{"AbsoluteTime", "[", "]"}]}], ";", "\n", "\t\t\t", 
       RowBox[{"ttable", "=", 
        RowBox[{
         RowBox[{"Table", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", "\n", 
       "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"MatchQ", "[", 
           RowBox[{"ttable", ",", 
            RowBox[{"{", "__RuleDelayed", "}"}]}], "]"}]}], ",", 
         RowBox[{"ttable", "=", 
          RowBox[{"{", "}"}]}]}], "]"}], ";", "\n", "\t\t\t", 
       RowBox[{"special", "=", 
        RowBox[{
         RowBox[{"Replace", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", "\n", 
       "\t\t\t", 
       RowBox[{"cutoff", "=", 
        RowBox[{
         RowBox[{"TimeConstraint", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", "\n", 
       "\t\t\t", 
       RowBox[{"together", "=", 
        RowBox[{
         RowBox[{"Together", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", 
       RowBox[{"factorfun", "=", 
        RowBox[{
         RowBox[{"Factor", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", 
       RowBox[{"$commentnb", "=", "False"}], ";", 
       RowBox[{"tog", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"#1", ">", "1001"}], ",", 
                RowBox[{"If", "[", 
                 RowBox[{"$Comment", ",", 
                  RowBox[{"FEPrint", "[", 
                   RowBox[{
                    RowBox[{"\"\<coefficient # \>\"", "<>", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"ValueQ", "[", "$cv", "]"}], ",", 
                    RowBox[{"ToString", "[", "$cc", "]"}], ",", "\"\<\>\""}], 
                    "]"}], "<>", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"ValueQ", "[", "$cv", "]"}], ",", 
                    RowBox[{"\"\< (\>\"", "<>", 
                    RowBox[{"ToString", "[", "$cv", "]"}], "<>", 
                    "\"\<)\>\""}], ",", "\"\<\>\""}], "]"}], "<>", 
                    "\"\< simplified, \>\""}], ",", 
                    RowBox[{"ToString", "[", "#1", "]"}]}], "]"}]}], "]"}]}], 
               "]"}], ";", "#2"}], ")"}], "&"}], ")"}], "[", 
          RowBox[{
           RowBox[{"LeafCount", "[", "#1", "]"}], ",", 
           RowBox[{
            RowBox[{"$cc", "++"}], ";", 
            RowBox[{"together", "[", "#1", "]"}]}]}], "]"}], "&"}]}], ";", 
       RowBox[{"If", "[", "\[InvisibleSpace]", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"(", 
           RowBox[{"ValueQ", "[", "$TarcerRecursed", "]"}], ")"}]}], ",", 
         RowBox[{"$TarcerRecursed", "=", 
          RowBox[{"{", "}"}]}]}], "]"}], ";", 
       RowBox[{"$TarcerRecursed", "=", 
        RowBox[{"{", "}"}]}], ";", 
       RowBox[{"If", "[", "\[InvisibleSpace]", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"(", 
           RowBox[{"MemberQ", "[", 
            RowBox[{
            "$ContextPath", ",", "\"\<HighEnergyPhysics`fcloops`ToTFi`\>\""}],
             "]"}], ")"}]}], ",", 
         RowBox[{"t0", "=", "z"}], ",", 
         RowBox[{"t0", "=", 
          RowBox[{
           RowBox[{"HighEnergyPhysics`fcloops`ToTFi`ToTFi", "[", "z", "]"}], "/.", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"HighEnergyPhysics`fcloops`TFi`TFi", "\[RuleDelayed]", 
              StyleBox["TFI",
               FontColor->RGBColor[1, 0.6, 0]]}], ",", 
             RowBox[{
             "HighEnergyPhysics`fcloops`ToTFi`ToTFi", "\[RuleDelayed]", 
              "Identity"}]}], "}"}]}]}]}], "]"}], ";", "\n", "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"ttable", "=!=", 
          RowBox[{"{", "}"}]}], ",", 
         RowBox[{"t0", "=", 
          RowBox[{"t0", "/.", "ttable"}]}]}], "]"}], ";", "\n", "\t\t\t", 
       RowBox[{"getvars", "=", 
        RowBox[{
         RowBox[{"Select", "[", 
          RowBox[{
           RowBox[{"Variables", "[", "#1", "]"}], ",", 
           RowBox[{
            RowBox[{"MatchQ", "[", 
             RowBox[{"#1", ",", 
              RowBox[{
               RowBox[{
                StyleBox["TFI",
                 FontColor->RGBColor[1, 0.6, 0]], "[", "__", "]"}], "|", 
               RowBox[{
                StyleBox["TVI",
                 FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "__", "]"}], "|", 
               RowBox[{"TJI", "[", "__", "]"}], "|", 
               RowBox[{"TBI", "[", "__", "]"}], "|", 
               RowBox[{"TAI", "[", "__", "]"}]}]}], "]"}], "&"}]}], "]"}], 
         "&"}]}], ";", "\n", "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"FreeQ", "[", 
           RowBox[{"t0", ",", "TFI"}], "]"}]}], ",", 
         RowBox[{"t0", "=", 
          RowBox[{"Collect", "[", 
           RowBox[{
            RowBox[{"t0", "=", 
             RowBox[{
              RowBox[{"TFIRecurse", "[", "t0", "]"}], "/.", "ttable"}]}], ",", 
            RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}]}], 
        "]"}], ";", "\n", "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"FreeQ", "[", 
           RowBox[{"t0", ",", 
            RowBox[{"TVI", "[", 
             RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", "\n", 
         "\t\t\t", 
         RowBox[{
          RowBox[{"tvrd", "=", 
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{
              RowBox[{"DownValues", "[", "TVR", "]"}], ",", 
              RowBox[{
               RowBox[{"!", 
                RowBox[{"FreeQ", "[", 
                 RowBox[{"#", ",", 
                  RowBox[{"TVI", "[", 
                   RowBox[{
                    RowBox[{"_", "-", "2"}], ",", "__"}], "]"}]}], "]"}]}], 
               "&"}]}], "]"}], "/.", 
            RowBox[{"TVR", "->", "TVI"}]}]}], ";", "\n", "\t\t\t\t", 
          RowBox[{"t0", "=", 
           RowBox[{"Collect", "[", 
            RowBox[{
             RowBox[{"t0", "=", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"t0", "/.", "tvrd"}], "/.", "tvrd"}], "/.", "tvrd"}],
                "/.", "tvrd"}]}], ",", 
             RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}], ";", 
          "\n", "\t\t\t\t", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"FreeQ", "[", 
              RowBox[{"t0", ",", 
               RowBox[{"TVI", "[", 
                RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", "\t", 
            RowBox[{"t0", "=", 
             RowBox[{"Collect", "[", 
              RowBox[{
               RowBox[{"t0", "=", 
                RowBox[{"t0", "//.", "tvrd"}]}], ",", 
               RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}]}], 
           "]"}]}]}], "]"}], ";", "\n", "\t\t\t\t", "\n", "\t\t\t", 
       RowBox[{"tjrd", "=", 
        RowBox[{
         RowBox[{"Select", "[", 
          RowBox[{
           RowBox[{"DownValues", "[", "TJR", "]"}], ",", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"FreeQ", "[", 
              RowBox[{"#", ",", 
               RowBox[{"TJI", "[", 
                RowBox[{
                 RowBox[{"_", "-", "2"}], ",", "__"}], "]"}]}], "]"}]}], 
            "&"}]}], "]"}], "/.", 
         RowBox[{"TJR", "->", "TJI"}]}]}], ";", "\n", "\t\t\t", "\n", 
       "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"FreeQ", "[", 
           RowBox[{"t0", ",", 
            RowBox[{"TJI", "[", 
             RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", "\n", 
         "\t\t\t", 
         RowBox[{
          RowBox[{"tjrd", "=", 
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{
              RowBox[{"DownValues", "[", "TJR", "]"}], ",", 
              RowBox[{
               RowBox[{"!", 
                RowBox[{"FreeQ", "[", 
                 RowBox[{"#", ",", 
                  RowBox[{"TJI", "[", 
                   RowBox[{
                    RowBox[{"_", "-", "2"}], ",", "__"}], "]"}]}], "]"}]}], 
               "&"}]}], "]"}], "/.", 
            RowBox[{"TJR", "->", "TJI"}]}]}], ";", "\n", "\t\t\t\t", 
          RowBox[{"t0", "=", 
           RowBox[{"Collect", "[", 
            RowBox[{
             RowBox[{"t0", "=", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"t0", "/.", "tjrd"}], "/.", "tjrd"}], "/.", "tjrd"}],
                "/.", "tjrd"}]}], ",", 
             RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}], ";", 
          "\n", "\t\t\t\t", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"FreeQ", "[", 
              RowBox[{"t0", ",", 
               RowBox[{"TJI", "[", 
                RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", "\t", 
            RowBox[{"t0", "=", 
             RowBox[{"Collect", "[", 
              RowBox[{
               RowBox[{"t0", "=", 
                RowBox[{"t0", "//.", "tjrd"}]}], ",", 
               RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}]}], 
           "]"}]}]}], "]"}], ";", "\n", "\t\t\t", "\n", "\t\t\t", 
       RowBox[{
        RowBox[{"rec", "[", 
         RowBox[{"expr_", ",", 
          RowBox[{"{", 
           RowBox[{"ti_", ",", "tr_"}], "}"}]}], "]"}], ":=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"sti", "=", 
           RowBox[{"ToString", "[", "ti", "]"}]}], ";", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"FreeQ", "[", 
             RowBox[{"expr", ",", "ti"}], "]"}], ",", "expr", ",", 
            RowBox[{
             RowBox[{"nexp", "=", "expr"}], ";", 
             RowBox[{"If", "[", 
              RowBox[{"$Comment", ",", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"If", "[", 
                   RowBox[{
                    RowBox[{"#1", "===", "1"}], ",", 
                    RowBox[{"FEPrint", "[", 
                    RowBox[{
                    "sti", "<>", "\"\< level, there is 1 \>\"", "<>", "sti"}],
                     "]"}], ",", 
                    RowBox[{"FEPrint", "[", 
                    RowBox[{
                    RowBox[{"sti", "<>", "\"\< level, there are \>\""}], ",", 
                    
                    RowBox[{"ToString", "[", "#1", "]"}], ",", 
                    RowBox[{"\"\< \>\"", "<>", "sti", "<>", "\"\<'s \>\""}]}],
                     "]"}]}], "]"}], "&"}], ")"}], "[", 
                RowBox[{"Count", "[", 
                 RowBox[{"nexp", ",", 
                  RowBox[{"ti", "[", "__", "]"}], ",", 
                  RowBox[{"-", "1"}]}], "]"}], "]"}]}], "]"}], ";", "\n", 
             "\t\t\t\t", "\n", "\t\t\t\t\t\t", 
             RowBox[{"nexp", "=", 
              RowBox[{
               RowBox[{"nexp", "/.", 
                RowBox[{"ti", "\[Rule]", "tr"}]}], "/.", 
               RowBox[{"tr", "\[Rule]", "ti"}]}]}], ";", "\n", 
             "\t\t\t\t\t\t\t", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"!", 
                RowBox[{"FreeQ", "[", 
                 RowBox[{"nexp", ",", 
                  RowBox[{"TJI", "[", 
                   RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", 
               RowBox[{"nexp", "=", 
                RowBox[{"nexp", "//.", "tjrd"}]}]}], "]"}], ";", "\n", 
             "\t\t\t\t\t\t", "\n", "\t\t\t\t\t\t", 
             RowBox[{"tvars", "=", 
              RowBox[{"getvars", "[", "nexp", "]"}]}], ";", 
             RowBox[{"$cc", "=", "0"}], ";", 
             RowBox[{"$cv", "=", 
              RowBox[{"Length", "[", "tvars", "]"}]}], ";", 
             RowBox[{"nexp", "=", 
              RowBox[{"Collect", "[", 
               RowBox[{"nexp", ",", "tvars", ",", "tog"}], "]"}]}], ";", 
             "\[IndentingNewLine]", "\n", "\t\t\t\t\t\t", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"cutoff", "=!=", "Infinity"}], ",", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"AbsoluteTime", "[", "]"}], "-", "time"}], ">", 
                  "cutoff"}], ",", 
                 RowBox[{"Throw", "[", "$Failed", "]"}]}], "]"}]}], "]"}], 
             ";", "\n", "\t\t\t\t\t\t", "nexp"}]}], "]"}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"basisdone", "=", "0"}], ";", 
       RowBox[{"tl", "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            StyleBox["TFI",
             FontColor->RGBColor[1, 0.6, 0]], ",", 
            StyleBox["TFR",
             FontColor->RGBColor[0, 0.6, 0]]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            StyleBox["TVI",
             FontColor->RGBColor[0.2, 0.4, 0.8]], ",", 
            StyleBox["TVR",
             FontColor->RGBColor[0.2, 0.8, 1]]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"TJI", ",", "TJR"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"TBI", ",", "TBR"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"TAI", ",", "TAR"}], "}"}]}], "}"}]}], ";", 
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"t0", "=", " ", 
           RowBox[{"FixedPoint", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"rec", "[", 
               RowBox[{"#", ",", 
                RowBox[{"tl", "[", 
                 RowBox[{"[", "i", "]"}], "]"}]}], "]"}], "&"}], ",", "t0", 
             ",", "100"}], "]"}]}], ";", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Head", "[", "t0", "]"}], "===", "Plus"}], ",", 
            RowBox[{
             RowBox[{"fertig", "=", 
              RowBox[{"Select", "[", 
               RowBox[{"t0", ",", "\[InvisibleSpace]", 
                RowBox[{"!", 
                 RowBox[{"(", 
                  RowBox[{"FreeQ", "[", 
                   RowBox[{"#1", ",", 
                    RowBox[{"tl", "\[LeftDoubleBracket]", 
                    RowBox[{"i", ",", "1"}], "\[RightDoubleBracket]"}]}], 
                   "]"}], ")"}]}]}], "]"}]}], ";", 
             RowBox[{"basisdone", "=", 
              RowBox[{"basisdone", "+", "fertig"}]}], ";", 
             RowBox[{"t0", "=", 
              RowBox[{"t0", "-", "fertig"}]}]}]}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", 
           RowBox[{"Length", "[", "tl", "]"}]}], "}"}]}], "]"}], ";", 
       RowBox[{"t0", "=", 
        RowBox[{"t0", "+", "basisdone"}]}], ";", 
       RowBox[{"$cv", "=", "."}], ";", 
       RowBox[{"t0", "=", 
        RowBox[{"Collect", "[", 
         RowBox[{"t0", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"TAI", "[", "__", "]"}], ",", 
            RowBox[{
             RowBox[{"TAI", "[", "a1__", "]"}], " ", 
             RowBox[{"TAI", "[", "a2__", "]"}]}], ",", 
            RowBox[{"TAI", "[", "__", "]"}], ",", 
            RowBox[{"TBI", "[", "__", "]"}], ",", 
            RowBox[{"TJI", "[", "__", "]"}], ",", 
            RowBox[{"TVI", "[", "__", "]"}], ",", 
            RowBox[{"TFI", "[", "__", "]"}]}], "}"}], ",", "factorfun"}], 
         "]"}]}], ";", 
       RowBox[{"If", "[", 
        RowBox[{"$Comment", ",", 
         RowBox[{"FEPrint", "[", 
          RowBox[{"\"\<the result has \>\"", ",", 
           RowBox[{"ToString", "[", 
            RowBox[{
             RowBox[{"Length", "[", 
              RowBox[{"t0", "+", "dummm1", "+", "dummm2"}], "]"}], "-", "2"}],
             "]"}], ",", "\"\< terms\>\""}], "]"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{"$Notebooks", ",", 
         RowBox[{"SetSelectedNotebook", "[", 
          RowBox[{"EvaluationNotebook", "[", "]"}], "]"}]}], "]"}], ";", 
       "t0"}]}], "]"}], "]"}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerExpand", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.8.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "TarcerExpand", "]"}], "=", 
   RowBox[{"{", "\n", "\t\t", 
    RowBox[{
     RowBox[{"TarcerReduce", ":>", "$BasisIntegrals"}], ",", "\n", "\t", 
     RowBox[{"TarcerRecurse", "->", "False"}], ",", 
     RowBox[{"Zeta", "->", "True"}], ",", 
     RowBox[{"Replace", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Prefactor1", "\[Rule]", "Identity"}], ",", 
        RowBox[{"EulerGamma", " ", "\[Rule]", "0"}]}], "}"}]}]}], "}"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerExpand", "[", 
    RowBox[{"expr_", ",", 
     RowBox[{"d_Symbol", "->", 
      RowBox[{"fe_", "/;", 
       RowBox[{
        RowBox[{"Length", "[", 
         RowBox[{"Variables", "[", "fe", "]"}], "]"}], "===", "1"}]}]}], ",", 
     "ru___Rule"}], "]"}], ":=", 
   RowBox[{"TarcerExpand", "[", 
    RowBox[{"expr", ",", 
     RowBox[{"d", " ", "->", "fe"}], ",", "0", ",", "ru"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TarcerExpand", "[", 
   RowBox[{"expr_", ",", 
    RowBox[{"d_Symbol", "\[Rule]", "fe_"}], ",", 
    RowBox[{"order_Integer", "?", "NonNegative"}], ",", "opts___Rule"}], 
   "]"}], ":=", 
  RowBox[{
   RowBox[{
    StyleBox["Block",
     FontSize->16,
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0.2, 0.2]], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "t1", ",", "t2", ",", "t3", ",", "t4", ",", "numfa", ",", "z2rule", ",",
        "restfa", ",", "ta0", ",", "tarcNumericalFactor", ",", "tarcFactor1", 
       ",", "eeps"}], 
      RowBox[{"(*", 
       RowBox[{",", "extracoll"}], "*)"}], "}"}], ",", "\[IndentingNewLine]", 
     
     RowBox[{
      RowBox[{"eeps", "=", 
       RowBox[{
        RowBox[{"Variables", "[", "fe", "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"tarcNumericalFactor", "[", "x_", "]"}], ":=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"NumberQ", "[", "x", "]"}], ",", "x", ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Head", "[", "x", "]"}], "===", "Times"}], ",", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"NumberQ", "[", 
              RowBox[{"First", "[", "x", "]"}], "]"}], ",", 
             RowBox[{"First", "[", "x", "]"}], ",", "1"}], "]"}], ",", "1"}], 
          "]"}]}], "]"}]}], ";", "\n", 
      RowBox[{
       RowBox[{"tarcFactor1", "[", "x_", "]"}], ":=", 
       RowBox[{"Block", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "factor1t1", ",", "factor1t2", ",", "factor1t3", ",", "mt", ",", 
           "mi", ",", "m1", ",", "mp1", ",", 
           RowBox[{"nx", "=", "x"}], ",", "iIii"}], "}"}], ",", 
         RowBox[{
          RowBox[{"mt", "=", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"#1", "/.", "\[InvisibleSpace]", 
                 RowBox[{"Plus", "\[Rule]", "mi"}]}], "/.", 
                "\[InvisibleSpace]", 
                RowBox[{"mi", "\[Rule]", "Plus"}]}], "/.", 
               "\[InvisibleSpace]", 
               RowBox[{"m1", "\[Rule]", 
                RowBox[{"-", "1"}]}]}], "/.", "\[InvisibleSpace]", 
              RowBox[{"mp1", "\[Rule]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"(", 
                   RowBox[{"+", "##1"}], ")"}]}], "&"}], ")"}]}]}], "/.", 
             "\[InvisibleSpace]", 
             RowBox[{"iIii", "\[Rule]", "\[ImaginaryI]"}]}], "&"}]}], ";", 
          RowBox[{
           RowBox[{"mi", "[", 
            RowBox[{"y_", ",", "z__"}], "]"}], ":=", 
           RowBox[{
            RowBox[{"m1", " ", 
             RowBox[{"mp1", "[", 
              RowBox[{"y", ",", "z"}], "]"}]}], "/;", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Head", "[", "#1", "]"}], "===", "Complex"}], ",", 
                 "False", ",", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"#1", "<", "0"}], ",", "True", ",", "False"}], 
                  "]"}]}], "]"}], "&"}], ")"}], "[", 
             RowBox[{"tarcNumericalFactor", "[", "y", "]"}], "]"}]}]}], ";", 
          RowBox[{"nx", "=", 
           RowBox[{"x", "/.", "\[InvisibleSpace]", 
            RowBox[{
             RowBox[{"Complex", "[", 
              RowBox[{"0", ",", "b_"}], "]"}], "\[Rule]", 
             RowBox[{"b", " ", "iIii"}]}]}]}], ";", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Head", "[", "nx", "]"}], "=!=", "Plus"}], ",", 
            RowBox[{"mt", "[", 
             RowBox[{"nx", "/.", "\[InvisibleSpace]", 
              RowBox[{"Plus", "\[Rule]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Factor1", "[", 
                  RowBox[{"+", "##1"}], "]"}], "&"}], ")"}]}]}], "]"}], ",", 
            RowBox[{
             RowBox[{"factor1t1", "=", 
              RowBox[{"List", "@@", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"#1", "/.", "\[InvisibleSpace]", 
                   RowBox[{"Plus", "\[Rule]", "factor1t3"}]}], "&"}], ")"}], "/@",
                 "nx"}]}]}], ";", 
             RowBox[{"factor1t2", "=", 
              RowBox[{
               RowBox[{"PolynomialGCD", "@@", "factor1t1"}], "/.", 
               "\[InvisibleSpace]", 
               RowBox[{"factor1t3", "\[Rule]", "Plus"}]}]}], ";", 
             RowBox[{"mt", "[", 
              RowBox[{"factor1t2", " ", 
               RowBox[{"Plus", "@@", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   FractionBox[
                    RowBox[{"#1", "/.", "\[InvisibleSpace]", 
                    RowBox[{"factor1t3", "\[Rule]", "Plus"}]}], "factor1t2"], 
                   "&"}], ")"}], "/@", "factor1t1"}]}]}], "]"}]}]}], 
           "]"}]}]}], "]"}]}], ";", 
      RowBox[{"Catch", "[", 
       RowBox[{
        RowBox[{"e", "=", "eeps"}], ";", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"ints", ",", "recurse", ",", "zeta2replace"}], "}"}], "=", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"TarcerReduce", ",", "TarcerRecurse", ",", "Zeta"}], "}"}],
           "/.", 
          RowBox[{"Flatten", "[", 
           RowBox[{"{", 
            RowBox[{"opts", ",", 
             RowBox[{"Options", "[", "TarcerExpand", "]"}]}], "}"}], 
           "]"}]}]}], ";", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"zeta2replace", "=!=", "True"}], "&&", 
           RowBox[{"MemberQ", "[", 
            RowBox[{
            "$ContextPath", ",", "\"\<HighEnergyPhysics`general`Zeta2`\>\""}],
             "]"}]}], ",", 
          RowBox[{"z2rule", "=", 
           RowBox[{"{", "}"}]}], ",", 
          RowBox[{"z2rule", "=", 
           RowBox[{
            SuperscriptBox["\[Pi]", "2"], "\[RuleDelayed]", 
            RowBox[{"6", " ", 
             RowBox[{
             "ToExpression", "[", 
              "\"\<HighEnergyPhysics`general`Zeta2`Zeta2\>\"", "]"}]}]}]}]}], 
         "]"}], ";", 
        RowBox[{"ints", "=", 
         RowBox[{"Flatten", "[", "ints", "]"}]}], ";", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"recurse", "=!=", "True"}], ",", 
          RowBox[{"ta0", "=", "expr"}], ",", 
          RowBox[{"ta0", "=", 
           RowBox[{"TarcerRecurse", "[", "expr", "]"}]}]}], "]"}], ";", 
        RowBox[{"t1", "=", 
         RowBox[{
          RowBox[{"ta0", "/.", "ints"}], "/.", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"SEpsilon", "[", "de_", "]"}], "\[RuleDelayed]", 
             RowBox[{"SEpsilon", "[", 
              RowBox[{"ToString", "[", "d", "]"}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"Prefactor1", "[", "de_", "]"}], "\[RuleDelayed]", 
             RowBox[{"Prefactor1", "[", " ", 
              RowBox[{"ToString", "[", 
               RowBox[{"de", ",", "InputForm"}], "]"}], "]"}]}]}], 
           "\[IndentingNewLine]", "}"}]}]}], ";", 
        RowBox[{"vars", "=", 
         RowBox[{"Select", "[", 
          RowBox[{
           RowBox[{"Variables", "[", "t1", "]"}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "#1", "]"}], ">", "0"}], "&"}]}], "]"}]}],
         ";", 
        RowBox[{"If", "[", "\[InvisibleSpace]", 
         RowBox[{
          RowBox[{"!", 
           RowBox[{"(", 
            RowBox[{"FreeQ", "[", 
             RowBox[{"vars", ",", 
              RowBox[{
               StyleBox["TFI",
                FontColor->RGBColor[1, 0.6, 0]], "|", 
               StyleBox["TVI",
                FontColor->RGBColor[0.2, 0.4, 0.8]], "|", "TJI", "|", "TKI", 
               "|", "TBI", "|", "TAI"}]}], "]"}], ")"}]}], ",", 
          RowBox[{
           RowBox[{
           "Print", "[", "\"\<Returning the list of remaining integrals.\>\"",
             "]"}], ";", 
           RowBox[{"Throw", "[", 
            RowBox[{"Select", "[", 
             RowBox[{"vars", ",", "\[InvisibleSpace]", 
              RowBox[{
               RowBox[{"!", 
                RowBox[{"(", 
                 RowBox[{"FreeQ", "[", 
                  RowBox[{"#1", ",", 
                   RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "|", 
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "|", "TJI", "|", 
                    "TKI", "|", "TBI", "|", "TAI"}]}], "]"}], ")"}]}], 
               "&"}]}], "]"}], "]"}]}]}], "]"}], ";", 
        RowBox[{"t1", "=", 
         RowBox[{"t1", "/.", 
          RowBox[{"d", "\[Rule]", "fe"}]}]}], ";", 
        RowBox[{
         RowBox[{"polygafu", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "=", 
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          RowBox[{"Zeta", "[", "3", "]"}]}]}], ";", 
        RowBox[{
         RowBox[{"polygafu", "[", 
          RowBox[{"2", ",", "2"}], "]"}], "=", 
         RowBox[{"2", "-", 
          RowBox[{"2", " ", 
           RowBox[{"Zeta", "[", "3", "]"}]}]}]}], ";", 
        RowBox[{
         RowBox[{"polygafu", "[", 
          RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
         RowBox[{
          RowBox[{"polygafu", "[", 
           RowBox[{"x", ",", "y"}], "]"}], "=", 
          RowBox[{"FunctionExpand", "[", 
           RowBox[{"PolyGamma", "[", 
            RowBox[{"x", ",", "y"}], "]"}], "]"}]}]}], ";", 
        RowBox[{"t2", "=", 
         RowBox[{
          RowBox[{
           RowBox[{"Normal", "[", 
            RowBox[{"t1", "+", 
             SuperscriptBox[
              RowBox[{"O", "[", "e", "]"}], 
              RowBox[{"order", "+", "1"}]]}], "]"}], "/.", 
           RowBox[{"PolyGamma", "\[Rule]", "polygafu"}]}], "/.", 
          "\[IndentingNewLine]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Replace", "/.", 
             RowBox[{"{", "opts", "}"}]}], "/.", 
            RowBox[{"Options", "[", "TarcerExpand", "]"}]}], ")"}]}]}], ";", 
        RowBox[{"t3", "=", 
         RowBox[{"Collect", "[", 
          RowBox[{"t2", ",", "e", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Expand", "[", "#1", "]"}], "/.", "z2rule"}], ")"}], 
            "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"t3", "=", 
         RowBox[{"t2", "/.", 
          RowBox[{
           RowBox[{"ToString", "[", "d", "]"}], "\[RuleDelayed]", 
           RowBox[{"(", "fe", ")"}]}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Global`T3", "=", "t3"}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"t4", "=", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"tarcFactor1", "[", 
               RowBox[{"Expand", "[", "t3", "]"}], "]"}], "//.", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"pre", ":", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"Prefactor1", "|", "SEpsilon"}], ")"}], "[", "a_",
                    "]"}], " ", 
                  RowBox[{"Prefactor1", "[", "b_", "]"}]}]}], ")"}], 
               "\[RuleDelayed]", 
               RowBox[{"Prefactor1", "[", 
                RowBox[{"a", " ", "b"}], "]"}]}]}], "/.", 
             RowBox[{
              RowBox[{"Prefactor1", "[", "1", "]"}], "\[RuleDelayed]", 
              "1"}]}], "/.", 
            RowBox[{
             RowBox[{"Prefactor1", "[", "xy_", "]"}], "\[RuleDelayed]", 
             RowBox[{"Prefactor1", "[", 
              RowBox[{"xy", "/.", 
               RowBox[{"st_String", "\[RuleDelayed]", 
                RowBox[{"ToExpression", "[", "st", "]"}]}]}], "]"}]}]}], "/.", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Prefactor1", "[", 
               RowBox[{
                RowBox[{"m_", "^", "e"}], " ", "xx_"}], "]"}], 
              RowBox[{"m_", "^", "any_"}]}], " ", ")"}], "\[RuleDelayed]", 
            RowBox[{"Prefactor1", "[", 
             RowBox[{
              RowBox[{"m", "^", 
               RowBox[{"(", 
                RowBox[{"e", "+", "any"}], ")"}]}], "xx"}], "]"}]}]}], "/.", 
          "z2rule"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Global`t4", "=", "t4"}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Head", "[", "t4", "]"}], "===", "Times"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"numfa", "=", 
               RowBox[{"tarcNumericalFactor", "[", "t4", "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"restfa", " ", "=", " ", 
               RowBox[{"Select", "[", 
                RowBox[{"t4", ",", 
                 RowBox[{
                  RowBox[{"FreeQ", "[", 
                   RowBox[{"#", ",", 
                    RowBox[{"eeps", "|", "Log", "|", "Zeta"}]}], "]"}], 
                  "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"Global`RR", "=", "restfa"}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"Global`NN", "=", "numfa"}], ";"}], "*)"}], 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"extracoll", "[", 
                RowBox[{"z_", ",", "e"}], "]"}], ":=", 
               RowBox[{"Block", "[", 
                RowBox[{
                 RowBox[{"{", "}"}], ",", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"Head", "[", "z", "]"}], "===", "Plus"}], ",", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"extracoll", "[", 
                    RowBox[{"#", ",", "e"}], "]"}], "&"}], "/@", "z"}], ",", 
                   RowBox[{"z", "/.", 
                    RowBox[{"xy_Plus", "\[RuleDelayed]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Length", "[", 
                    RowBox[{"Variables", "[", "xy", "]"}], "]"}], "===", 
                    "0"}], ",", "xy", ",", 
                    RowBox[{"Collect", "[", 
                    RowBox[{"xy", ",", 
                    RowBox[{"Complement", "[", 
                    RowBox[{
                    RowBox[{"Variables", "[", "xy", "]"}], ",", 
                    RowBox[{"{", "e", "}"}]}], "]"}]}], "]"}]}], "]"}]}]}]}], 
                  "]"}]}], " ", "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Select", "[", 
                  RowBox[{"t4", ",", 
                   RowBox[{
                    RowBox[{"!", 
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "Prefactor1"}], "]"}]}], "&"}]}], "]"}], 
                 RowBox[{"(", 
                  FractionBox["restfa", "numfa"], ")"}]}], ")"}], ".", 
               RowBox[{"extracoll", "[", " ", 
                RowBox[{
                 RowBox[{"Collect", "[", 
                  RowBox[{
                   RowBox[{"Expand", "[", 
                    FractionBox[
                    RowBox[{"numfa", "  ", 
                    RowBox[{"Select", "[", " ", 
                    RowBox[{"t4", ",", 
                    RowBox[{
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "Prefactor1"}], "]"}], "&"}]}], "]"}]}], 
                    RowBox[{"restfa", " "}]], "]"}], ",", "e"}], "]"}], ",", 
                 "e"}], "]"}]}]}], ",", "t4"}], "]"}], "/.", 
           RowBox[{"s_String", "\[RuleDelayed]", 
            RowBox[{"ToExpression", "[", "s", "]"}]}]}], "/.", 
          RowBox[{"d", "\[Rule]", "fe"}]}], "/.", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Replace", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "TarcerExpand", "]"}]}], ")"}]}]}], 
       "\[IndentingNewLine]", "]"}]}]}], "]"}], "/;", 
   RowBox[{
    RowBox[{"Length", "[", 
     RowBox[{"Variables", "[", "fe", "]"}], "]"}], "===", "1"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Construct T's", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.9.1"],

Cell[CellGroupData[{

Cell["remember", "Subsection",
 CellTags->"T1.9.1.1"],

Cell["The function remember is not used by default.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tfch", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"TFR", ":>", 
      RowBox[{"Hold", "[", "TFR", "]"}]}], ",", 
     RowBox[{"TVR", ":>", 
      RowBox[{"Hold", "[", "TVR", "]"}]}], ",", 
     RowBox[{"TJR", ":>", 
      RowBox[{"Hold", "[", "TJR", "]"}]}], ",", 
     RowBox[{"TBR", ":>", 
      RowBox[{"Hold", "[", "TBR", "]"}]}], ",", 
     RowBox[{"TAR", ":>", 
      RowBox[{"Hold", "[", "TAR", "]"}]}], ",", 
     RowBox[{"TComment", ":>", 
      RowBox[{"Hold", "[", "TComment", "]"}]}]}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rtfch", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Reverse", ",", "tfch"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"condrest", ",", "HoldRest"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "remember", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"remember", "[", "zi_HoldForm", "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"z", "=", 
       RowBox[{"zi", "/.", "tfch"}]}], "}"}], ",", 
     RowBox[{"If", "[", "\[InvisibleSpace]", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"FreeQ", "[", 
          RowBox[{"z", ",", "Condition"}], "]"}], ")"}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Hold", "[", "HoldForm", "]"}], "[", 
            RowBox[{
             RowBox[{"Hold", "[", "SetDelayed", "]"}], "[", 
             RowBox[{
              RowBox[{"z", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{"ReplacePart", "[", 
               RowBox[{
                RowBox[{"z", "\[LeftDoubleBracket]", 
                 RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}], ",", 
                RowBox[{"Prepend", "[", 
                 RowBox[{
                  RowBox[{"Extract", "[", 
                   RowBox[{"z", ",", 
                    RowBox[{"{", 
                    RowBox[{"1", ",", "2", ",", "1"}], "}"}], ",", "Hold"}], 
                   "]"}], ",", 
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"z", "\[LeftDoubleBracket]", 
                    RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "/.", 
                    
                    RowBox[{"PatternTest", "\[Rule]", "pt"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pt", "[", 
                    RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
                    "a"}]}], "/.", 
                    RowBox[{"Pattern", "\[Rule]", "pat"}]}], "/.", 
                   RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
                    "a"}]}]}], "]"}], ",", "1"}], "]"}]}], "]"}], "]"}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", "HoldForm", "]"}], "\[Rule]", "HoldForm"}]}],
           "/.", "rtfch"}], "/.", 
         RowBox[{
          RowBox[{"Hold", "[", "SetDelayed", "]"}], "\[Rule]", 
          "SetDelayed"}]}], "/.", 
        RowBox[{"Hold", "\[Rule]", "Set"}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Hold", "[", "HoldForm", "]"}], "[", 
            RowBox[{
             RowBox[{"Hold", "[", "SetDelayed", "]"}], "[", 
             RowBox[{
              RowBox[{"z", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{"Prepend", "[", 
               RowBox[{
                RowBox[{"Extract", "[", 
                 RowBox[{"z", ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}], ",", 
                
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"z", "\[LeftDoubleBracket]", 
                    RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "/.", 
                    
                    RowBox[{"PatternTest", "\[Rule]", "pt"}]}], "/.", 
                   RowBox[{
                    RowBox[{"pt", "[", 
                    RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
                    "a"}]}], "/.", 
                  RowBox[{"Pattern", "\[Rule]", "pat"}]}], "/.", 
                 RowBox[{
                  RowBox[{"pat", "[", 
                   RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
                  "a"}]}]}], "]"}]}], "]"}], "]"}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", "HoldForm", "]"}], "\[Rule]", "HoldForm"}]}],
           "/.", "rtfch"}], "/.", 
         RowBox[{
          RowBox[{"Hold", "[", "SetDelayed", "]"}], "\[Rule]", 
          "SetDelayed"}]}], "/.", 
        RowBox[{"Hold", "\[Rule]", "Set"}]}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData["$TarasovTdeltaplimit"], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"aa2", " ", "=", " ", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", "#"], "&"}], ",", " ", 
     RowBox[{"10000", "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"$TarasovTdeltaplimit", "+", "1"}], ")"}], " ", "1000"}]}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"bb12", "=", 
   RowBox[{"Select", "[", 
    RowBox[{"aa2", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Head", "[", "#", "]"}], "=!=", "Subscript"}], "&"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "aa2", "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Print", "[", 
  RowBox[{"\"\<there are \>\"", ",", 
   RowBox[{"Length", "[", "bb12", "]"}], ",", 
   "\"\< recursion equations for $RankLimit = \>\"", ",", "$RankLimit"}], 
  "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"there are \"\>", "\[InvisibleSpace]", "201", 
    "\[InvisibleSpace]", "\<\" recursion equations for $RankLimit = \"\>", 
    "\[InvisibleSpace]", 
    RowBox[{"{", 
     RowBox[{"2", ",", "5"}], "}"}]}],
   SequenceForm[
   "there are ", 201, " recursion equations for $RankLimit = ", {2, 5}],
   Editable->False], TraditionalForm]], "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"Global`$Remember", " ", "===", " ", "True"}], ",", 
    RowBox[{
     RowBox[{"Global`RCHECK", "=", "True"}], ";", " ", 
     RowBox[{"nb", " ", "=", 
      RowBox[{"Map", "[", 
       RowBox[{"remember", ",", "bb12"}], "]"}]}]}], ",", 
    RowBox[{"nb", "=", "bb12"}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "bb12", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"Unset", "[", "$Post", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ReleaseHold", "[", "nb", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Comment", "=", "False"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"End", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EndPackage", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"!", 
     RowBox[{
     "ValueQ", "[", "HighEnergyPhysics`FeynCalc`$FeynCalcDirectory", "]"}]}], 
    ",", 
    RowBox[{
    "BeginPackage", "[", "\"\<HighEnergyPhysics`general`Zeta2`\>\"", "]"}]}], 
   "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Zeta2", "::", "usage"}], "=", "\n", 
   "\"\<Zeta2 denotes Zeta[2].\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], ";"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Zeta2", " ", "/:", " ", 
   RowBox[{"N", "[", "Zeta2", "]"}], " ", "=", " ", 
   RowBox[{"N", "[", 
    RowBox[{"Zeta", "[", "2", "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Zeta2", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"Zeta2", ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "@@", 
    RowBox[{"{", "  ", 
     RowBox[{
      RowBox[{"RowBox", "[", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\[Zeta]\>\"", ",", "\"\<(\>\"", ",", "2", ",", "\"\<)\>\""}], 
        "}"}], "]"}], ",", "Zeta2", ",", 
      RowBox[{"Editable", "->", "False"}]}], "}"}]}]}], ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"End", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"EndPackage", "[", "]"}], ";"}]}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["$BasisIntegrals", "Section",
 CellDingbat->"\[EmptySmallCircle]",
 CellTags->"T1.10.1"],

Cell["\<\
This is a (incomplete) set of basis integrals. You can add your favorite \
integrals here.\
\>", "Text"],

Cell["eta=Gamma[d/2-1]/Gamma[d-3]Gamma[3-d/2];", "Text"],

Cell[CellGroupData[{

Cell["\<\
$BasisIntegrals =
  {TAI[d_, 0, {{1, M_}}] -> 
    -I*E^(1/2*(4 - d)*EulerGamma)*M^2*
     Gamma[-1 + (4 - d)/2]*
     Prefactor1[(M^2)^(1/2*(-4 + d))*
       SEpsilon[d]], 
       
       TBI[d_, pp_, {{1, 0},{1, 0}}] -> 
   (I*E^(((4 - d)*EulerGamma)/2)*Gamma[2 - d/2]*
      Gamma[-1 + d/2]^2*
      Prefactor1[ (-pp)^((-4 + d)/2)*
        SEpsilon[d]])/Gamma[-2 + d],
        
        TJI[d_, pp_, {{1,0},{1,0},{1,0}}] ->-((pp*Gamma[3 - d]*Gamma[-1 + \
d/2]^3*
    E^((4 - d)*EulerGamma)*  Prefactor1[ 
        (-pp)^(-4 + d)*SEpsilon[d]^2])/
    (Gamma[-3 + (3*d)/2] )
),
    
          TJI[d_, M_^2, {{1,M_},{1,0},{1,0}}] ->(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]*
    Gamma[-1 + d/2]^2*Gamma[-5 + 2*d]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[-2 + d]*Gamma[-3 + (3*d)/2]),
  
  TJI[d_Symbol, (M_)^2, 
  {{1, M_}, {1, M_}, {1, M_}}]->M^2*Prefactor1[
    (M^2)^(-4 + d)*SEpsilon[d]^2]*
  (-6/(-4 + d)^2 + 17/(2*(-4 + d)) + 
    (-59 - 2*Pi^2)/8 + 
    (-4 + d)^3*TJI111e + 
    ((-4 + d)^2*(16755 - 4750*Pi^2 - 
         14*Pi^4 + 3840*Pi^2*Log[2] - 
         12080*Zeta[3]))/1920 + 
    ((-4 + d)*(195 + 98*Pi^2 - 
         48*Zeta[3]))/96),
         
         TJI[d_, 0, {{1, M_}, {1, M_}, {1, 0}}]->(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]^2*
    Gamma[-1 + d/2]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[4 - d]*Gamma[d/2]),
  
  TJI[d_, 0, {{1,M_},{1,0},{1,0}}] ->(M^2*Gamma[3 - d]*Gamma[2 - d/2]*
    Gamma[-1 + d/2]^2*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  Gamma[d/2]
        }\
\>", "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TAI[
       Pattern[$CellContext`d, 
        Blank[]], 0, {{1, 
         Pattern[$CellContext`M, 
          Blank[]]}}],
      Editable->True], "\[Rule]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        TagBox["\[DoubledGamma]",
         Function[{}, EulerGamma]], " ", 
        RowBox[{"(", 
         RowBox[{"4", "-", "d"}], ")"}]}]], " ", 
      SuperscriptBox["M", "2"], " ", 
      TemplateBox[{RowBox[{
          FractionBox[
           RowBox[{"4", "-", "d"}], "2"], "-", "1"}]},
       "Gamma"], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SuperscriptBox["M", "2"], ")"}], 
          FractionBox[
           RowBox[{"d", "-", "4"}], "2"]], " ", 
         InterpretationBox[
          StyleBox[
           SubscriptBox["S", "Global`d"],
           FontWeight->"Bold"],
          HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
          Editable->False]}],
        HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(
           Rational[1, 2] (-4 + $CellContext`d)) 
         HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]],
        Editable->False], ")"}]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TBI[
       Pattern[$CellContext`d, 
        Blank[]], 
       Pattern[$CellContext`pp, 
        Blank[]], {{1, 0}, {1, 0}}],
      Editable->True], "\[Rule]", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]], " ", 
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}]}]], " ", 
       TemplateBox[{RowBox[{"2", "-", 
           FractionBox["d", "2"]}]},
        "Gamma"], " ", 
       SuperscriptBox[
        TemplateBox[{RowBox[{
            FractionBox["d", "2"], "-", "1"}]},
         "Gamma"], "2"], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "pp"}], ")"}], 
           FractionBox[
            RowBox[{"d", "-", "4"}], "2"]], " ", 
          InterpretationBox[
           StyleBox[
            SubscriptBox["S", "Global`d"],
            FontWeight->"Bold"],
           HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
           Editable->False]}],
         HighEnergyPhysics`Tarcer`Prefactor1[(-$CellContext`pp)^(
            Rational[1, 2] (-4 + $CellContext`d)) 
          HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]],
         Editable->False], ")"}]}], 
      TemplateBox[{RowBox[{"d", "-", "2"}]},
       "Gamma"]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[
       Pattern[$CellContext`d, 
        Blank[]], 
       Pattern[$CellContext`pp, 
        Blank[]], {{1, 0}, {1, 0}, {1, 0}}],
      Editable->True], "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          TagBox["\[DoubledGamma]",
           Function[{}, EulerGamma]], " ", 
          RowBox[{"(", 
           RowBox[{"4", "-", "d"}], ")"}]}]], " ", "pp", " ", 
        TemplateBox[{RowBox[{"3", "-", "d"}]},
         "Gamma"], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{
             FractionBox["d", "2"], "-", "1"}]},
          "Gamma"], "3"], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "pp"}], ")"}], 
            RowBox[{"d", "-", "4"}]], " ", 
           InterpretationBox[
            StyleBox[
             SubsuperscriptBox["S", "Global`d", "2"],
             FontWeight->"Bold"],
            HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
            Editable->False]}],
          
          HighEnergyPhysics`Tarcer`Prefactor1[(-$CellContext`pp)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
          Editable->False], ")"}]}], 
       TemplateBox[{RowBox[{
           FractionBox[
            RowBox[{"3", " ", "d"}], "2"], "-", "3"}]},
        "Gamma"]]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[
       Pattern[$CellContext`d, 
        Blank[]], Pattern[$CellContext`M, 
         Blank[]]^2, {{1, 
         Pattern[$CellContext`M, 
          Blank[]]}, {1, 0}, {1, 0}}],
      Editable->True], "\[Rule]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]], " ", 
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}]}]], " ", 
       SuperscriptBox["M", "2"], " ", 
       TemplateBox[{RowBox[{"3", "-", "d"}]},
        "Gamma"], " ", 
       TemplateBox[{RowBox[{"2", "-", 
           FractionBox["d", "2"]}]},
        "Gamma"], " ", 
       SuperscriptBox[
        TemplateBox[{RowBox[{
            FractionBox["d", "2"], "-", "1"}]},
         "Gamma"], "2"], " ", 
       TemplateBox[{RowBox[{
           RowBox[{"2", " ", "d"}], "-", "5"}]},
        "Gamma"], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "Global`d", "2"],
            FontWeight->"Bold"],
           HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
           Editable->False]}],
         HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
         Editable->False], ")"}]}], 
      RowBox[{
       TemplateBox[{RowBox[{"d", "-", "2"}]},
        "Gamma"], " ", 
       TemplateBox[{RowBox[{
           FractionBox[
            RowBox[{"3", " ", "d"}], "2"], "-", "3"}]},
        "Gamma"]}]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_Symbol",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[
       Pattern[$CellContext`d, 
        Blank[Symbol]], Pattern[$CellContext`M, 
         Blank[]]^2, {{1, 
         Pattern[$CellContext`M, 
          Blank[]]}, {1, 
         Pattern[$CellContext`M, 
          Blank[]]}, {1, 
         Pattern[$CellContext`M, 
          Blank[]]}}],
      Editable->True], "\[Rule]", 
     RowBox[{
      SuperscriptBox["M", "2"], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SuperscriptBox["M", "2"], ")"}], 
          RowBox[{"d", "-", "4"}]], " ", 
         InterpretationBox[
          StyleBox[
           SubsuperscriptBox["S", "Global`d", "2"],
           FontWeight->"Bold"],
          HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
          Editable->False]}],
        HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], "3"], " ", "TJI111e"}], "+", 
        RowBox[{
         FractionBox["1", "96"], " ", 
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "48"}], " ", 
            TemplateBox[{"3"},
             "Zeta"]}], "+", "195", "+", 
           RowBox[{"98", " ", 
            SuperscriptBox["\[Pi]", "2"]}]}], ")"}]}], "+", 
        FractionBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"d", "-", "4"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "12080"}], " ", 
             TemplateBox[{"3"},
              "Zeta"]}], "+", "16755", "-", 
            RowBox[{"4750", " ", 
             SuperscriptBox["\[Pi]", "2"]}], "-", 
            RowBox[{"14", " ", 
             SuperscriptBox["\[Pi]", "4"]}], "+", 
            RowBox[{"3840", " ", 
             SuperscriptBox["\[Pi]", "2"], " ", 
             RowBox[{"log", "(", "2", ")"}]}]}], ")"}]}], "1920"], "+", 
        FractionBox["17", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}]}]], "-", 
        FractionBox["6", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], "2"]], "+", 
        RowBox[{
         FractionBox["1", "8"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "59"}], "-", 
           RowBox[{"2", " ", 
            SuperscriptBox["\[Pi]", "2"]}]}], ")"}]}]}], ")"}]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[
       Pattern[$CellContext`d, 
        Blank[]], 0, {{1, 
         Pattern[$CellContext`M, 
          Blank[]]}, {1, 
         Pattern[$CellContext`M, 
          Blank[]]}, {1, 0}}],
      Editable->True], "\[Rule]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]], " ", 
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}]}]], " ", 
       SuperscriptBox["M", "2"], " ", 
       TemplateBox[{RowBox[{"3", "-", "d"}]},
        "Gamma"], " ", 
       SuperscriptBox[
        TemplateBox[{RowBox[{"2", "-", 
            FractionBox["d", "2"]}]},
         "Gamma"], "2"], " ", 
       TemplateBox[{RowBox[{
           FractionBox["d", "2"], "-", "1"}]},
        "Gamma"], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "Global`d", "2"],
            FontWeight->"Bold"],
           HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
           Editable->False]}],
         HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
         Editable->False], ")"}]}], 
      RowBox[{
       TemplateBox[{RowBox[{"4", "-", "d"}]},
        "Gamma"], " ", 
       TemplateBox[{FractionBox["d", "2"]},
        "Gamma"]}]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[
       Pattern[$CellContext`d, 
        Blank[]], 0, {{1, 
         Pattern[$CellContext`M, 
          Blank[]]}, {1, 0}, {1, 0}}],
      Editable->True], "\[Rule]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]], " ", 
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}]}]], " ", 
       SuperscriptBox["M", "2"], " ", 
       TemplateBox[{RowBox[{"3", "-", "d"}]},
        "Gamma"], " ", 
       TemplateBox[{RowBox[{"2", "-", 
           FractionBox["d", "2"]}]},
        "Gamma"], " ", 
       SuperscriptBox[
        TemplateBox[{RowBox[{
            FractionBox["d", "2"], "-", "1"}]},
         "Gamma"], "2"], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "Global`d", "2"],
            FontWeight->"Bold"],
           HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
           Editable->False]}],
         HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
         Editable->False], ")"}]}], 
      TemplateBox[{FractionBox["d", "2"]},
       "Gamma"]]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
$BasisIntegrals = 
  {TAI[d_, 0, {{1, M_}}] :> 
    -I*E^(1/2*(4 - d)*EulerGamma)*M^2*
     Gamma[-1 + (4 - d)/2]*
     Prefactor1[(M^2)^(1/2*(-4 + d))*
       SEpsilon[d]], 
       
       TBI[d_, pp_, {{1, 0},{1, 0}}] :> 
   (I*E^(((4 - d)*EulerGamma)/2 - 
         I/2*(-4 + d)*Pi)*Gamma[2 - d/2]*
      Gamma[-1 + d/2]^2*
      Prefactor1[(pp)^((-4 + d)/2)*
        SEpsilon[d]])/Gamma[-2 + d],
        
        TJI[d_, pp_, {{1,0},{1,0},{1,0}}] :>-((pp*Gamma[3 - d]*Gamma[-1 + \
d/2]^3*
    E^((4 - d)*EulerGamma)*  Prefactor1[
        (pp)^(-4 + d)*SEpsilon[d]^2])/
    (E^(I*(-4 + d)*Pi)*Gamma[-3 + (3*d)/2])
),
    
          TJI[d_, M_^2, {{1,M_},{1,0},{1,0}}] :>(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]*
    Gamma[-1 + d/2]^2*Gamma[-5 + 2*d]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[-2 + d]*Gamma[-3 + (3*d)/2]),
  
  TJI[d_Symbol, (M_)^2, 
  {{1, M_}, {1, M_}, {1, M_}}]:>M^2*Prefactor1[
    (M^2)^(-4 + d)*SEpsilon[d]^2]*
  (-6/(-4 + d)^2 + 17/(2*(-4 + d)) + 
    (-59 - 2*Pi^2)/8 + 
    (-4 + d)^3*TJI111e + 
    ((-4 + d)^2*(16755 - 4750*Pi^2 - 
         14*Pi^4 + 3840*Pi^2*Log[2] - 
         12080*Zeta[3]))/1920 + 
    ((-4 + d)*(195 + 98*Pi^2 - 
         48*Zeta[3]))/96),
         
         TJI[d_, 0, {{1, M_}, {1, M_}, {1, 0}}]:>(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]^2*
    Gamma[-1 + d/2]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[4 - d]*Gamma[d/2]),
  
  TJI[d_, 0, {{1,M_},{1,0},{1,0}}] :>(M^2*Gamma[3 - d]*Gamma[2 - d/2]*
    Gamma[-1 + d/2]^2*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  Gamma[d/2]
        }\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TAI[
       Pattern[d, 
        Blank[]], 0, {{1, 
         Pattern[M, 
          Blank[]]}}],
      Editable->True], "\[RuleDelayed]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"4", "-", "d"}], ")"}], " ", 
        TagBox["\[DoubledGamma]",
         Function[{}, EulerGamma]]}]], " ", 
      SuperscriptBox["M", "2"], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{
        FractionBox[
         RowBox[{"4", "-", "d"}], "2"], "-", "1"}], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SuperscriptBox["M", "2"], ")"}], 
          FractionBox[
           RowBox[{"d", "-", "4"}], "2"]], " ", 
         InterpretationBox[
          StyleBox[
           SubscriptBox["S", "d"],
           FontWeight->"Bold"],
          SEpsilon[d],
          Editable->False]}],
        Prefactor1[(M^2)^(Rational[1, 2] (-4 + d)) SEpsilon[d]],
        Editable->False], ")"}]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TBI[
       Pattern[d, 
        Blank[]], 
       Pattern[pp, 
        Blank[]], {{1, 0}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"4", "-", "d"}], ")"}], " ", 
          TagBox["\[DoubledGamma]",
           Function[{}, EulerGamma]]}], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], " ", "\[Pi]"}]}]], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"2", "-", 
         FractionBox["d", "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{
          FractionBox["d", "2"], "-", "1"}], ")"}], "2"], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox["pp", 
           FractionBox[
            RowBox[{"d", "-", "4"}], "2"]], " ", 
          InterpretationBox[
           StyleBox[
            SubscriptBox["S", "d"],
            FontWeight->"Bold"],
           SEpsilon[d],
           Editable->False]}],
         Prefactor1[pp^(Rational[1, 2] (-4 + d)) SEpsilon[d]],
         Editable->False], ")"}]}], 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"d", "-", "2"}], ")"}]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[]], 
       Pattern[pp, 
        Blank[]], {{1, 0}, {1, 0}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"pp", " ", 
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{"3", "-", "d"}], ")"}], " ", 
        SuperscriptBox[
         RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{
           FractionBox["d", "2"], "-", "1"}], ")"}], "3"], " ", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"4", "-", "d"}], ")"}], " ", 
          TagBox["\[DoubledGamma]",
           Function[{}, EulerGamma]]}]], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{
           SuperscriptBox["pp", 
            RowBox[{"d", "-", "4"}]], " ", 
           InterpretationBox[
            StyleBox[
             SubsuperscriptBox["S", "d", "2"],
             FontWeight->"Bold"],
            SEpsilon[d],
            Editable->False]}],
          Prefactor1[pp^(-4 + d) SEpsilon[d]^2],
          Editable->False], ")"}]}], 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], " ", "\[Pi]"}]], " ", 
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{
          FractionBox[
           RowBox[{"3", " ", "d"}], "2"], "-", "3"}], ")"}]}]]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[]], Pattern[M, 
         Blank[]]^2, {{1, 
         Pattern[M, 
          Blank[]]}, {1, 0}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["M", "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"3", "-", "d"}], ")"}], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"2", "-", 
         FractionBox["d", "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{
          FractionBox["d", "2"], "-", "1"}], ")"}], "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{
         RowBox[{"2", " ", "d"}], "-", "5"}], ")"}], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}], " ", 
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]]}]], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "d", "2"],
            FontWeight->"Bold"],
           SEpsilon[d],
           Editable->False]}],
         Prefactor1[(M^2)^(-4 + d) SEpsilon[d]^2],
         Editable->False], ")"}]}], 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"d", "-", "2"}], ")"}], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{
         FractionBox[
          RowBox[{"3", " ", "d"}], "2"], "-", "3"}], ")"}]}]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_Symbol",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[Symbol]], Pattern[M, 
         Blank[]]^2, {{1, 
         Pattern[M, 
          Blank[]]}, {1, 
         Pattern[M, 
          Blank[]]}, {1, 
         Pattern[M, 
          Blank[]]}}],
      Editable->True], "\[RuleDelayed]", 
     RowBox[{
      SuperscriptBox["M", "2"], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SuperscriptBox["M", "2"], ")"}], 
          RowBox[{"d", "-", "4"}]], " ", 
         InterpretationBox[
          StyleBox[
           SubsuperscriptBox["S", "d", "2"],
           FontWeight->"Bold"],
          SEpsilon[d],
          Editable->False]}],
        Prefactor1[(M^2)^(-4 + d) SEpsilon[d]^2],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"TJI111e", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], "3"]}], "+", 
        RowBox[{
         FractionBox["1", "8"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "59"}], "-", 
           RowBox[{"2", " ", 
            SuperscriptBox["\[Pi]", "2"]}]}], ")"}]}], "+", 
        RowBox[{
         FractionBox["1", "96"], " ", 
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "48"}], " ", 
            TagBox[
             RowBox[{"\[Zeta]", "(", 
              TagBox["3",
               Zeta,
               Editable->True], ")"}],
             InterpretTemplate[
              Function[BoxForm`e$, 
               Zeta[BoxForm`e$]]]]}], "+", 
           RowBox[{"98", " ", 
            SuperscriptBox["\[Pi]", "2"]}], "+", "195"}], ")"}]}], "+", 
        FractionBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"d", "-", "4"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3840", " ", 
             SuperscriptBox["\[Pi]", "2"], " ", 
             RowBox[{"log", "(", "2", ")"}]}], "-", 
            RowBox[{"14", " ", 
             SuperscriptBox["\[Pi]", "4"]}], "-", 
            RowBox[{"4750", " ", 
             SuperscriptBox["\[Pi]", "2"]}], "-", 
            RowBox[{"12080", " ", 
             TagBox[
              RowBox[{"\[Zeta]", "(", 
               TagBox["3",
                Zeta,
                Editable->True], ")"}],
              InterpretTemplate[
               Function[BoxForm`e$, 
                Zeta[BoxForm`e$]]]]}], "+", "16755"}], ")"}]}], "1920"], "+", 
        
        FractionBox["17", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}]}]], "-", 
        FractionBox["6", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], "2"]]}], ")"}]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[]], 0, {{1, 
         Pattern[M, 
          Blank[]]}, {1, 
         Pattern[M, 
          Blank[]]}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["M", "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"3", "-", "d"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{"2", "-", 
          FractionBox["d", "2"]}], ")"}], "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{
         FractionBox["d", "2"], "-", "1"}], ")"}], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}], " ", 
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]]}]], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "d", "2"],
            FontWeight->"Bold"],
           SEpsilon[d],
           Editable->False]}],
         Prefactor1[(M^2)^(-4 + d) SEpsilon[d]^2],
         Editable->False], ")"}]}], 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"4", "-", "d"}], ")"}], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        FractionBox["d", "2"], ")"}]}]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[]], 0, {{1, 
         Pattern[M, 
          Blank[]]}, {1, 0}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["M", "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"3", "-", "d"}], ")"}], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"2", "-", 
         FractionBox["d", "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{
          FractionBox["d", "2"], "-", "1"}], ")"}], "2"], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}], " ", 
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]]}]], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "d", "2"],
            FontWeight->"Bold"],
           SEpsilon[d],
           Editable->False]}],
         Prefactor1[(M^2)^(-4 + d) SEpsilon[d]^2],
         Editable->False], ")"}]}], 
      RowBox[{"\[CapitalGamma]", "(", 
       FractionBox["d", "2"], ")"}]]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Save tarcer*.mx file", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.11.1"],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{
   RowBox[{"Share", "[", "]"}], ";"}], "*)"}]], "Input",
 CellChangeTimes->{{3.5008842719239197`*^9, 3.5008842740848093`*^9}}],

Cell["\<\
DumpSave[StringJoin[\"tarcer\", StringJoin @@ 
     ToString /@ $RankLimit, \".mx\"], 
   {$ContextPath, HighEnergyPhysics`Tarcer`Cayley, 
    HighEnergyPhysics`Tarcer`CayleyD, 
    HighEnergyPhysics`Tarcer`Cayleyu, 
    HighEnergyPhysics`Tarcer`Prefactor1, 
    HighEnergyPhysics`Tarcer`TarcerRecurse, 
    HighEnergyPhysics`Tarcer`TFIRecurse, 
    HighEnergyPhysics`Tarcer`TFISimplify, 
    HighEnergyPhysics`Tarcer`SEpsilon, 
    HighEnergyPhysics`Tarcer`TarcerExpand, 
    HighEnergyPhysics`Tarcer`TarcerReduce, 
    HighEnergyPhysics`Tarcer`TAI, 
    HighEnergyPhysics`Tarcer`TAR, 
    HighEnergyPhysics`Tarcer`TBI, 
    HighEnergyPhysics`Tarcer`TBR, 
    HighEnergyPhysics`Tarcer`TFI, 
    HighEnergyPhysics`Tarcer`TFR, 
    HighEnergyPhysics`Tarcer`TJI, 
    HighEnergyPhysics`Tarcer`TJR, 
    HighEnergyPhysics`Tarcer`TKI, 
    HighEnergyPhysics`Tarcer`TVI, 
    HighEnergyPhysics`Tarcer`TVR, 
    HighEnergyPhysics`Tarcer`$BasisIntegrals, 
    HighEnergyPhysics`Tarcer`$Comment, 
    HighEnergyPhysics`Tarcer`$CommentNotebook, 
    HighEnergyPhysics`Tarcer`$RankLimit, 
    HighEnergyPhysics`Tarcer`$RecursionStop, 
    HighEnergyPhysics`Tarcer`$TTable, 
    HighEnergyPhysics`Tarcer`Private`applytlrules, 
    HighEnergyPhysics`Tarcer`Private`applytlrules2,
    HighEnergyPhysics`Tarcer`Private`ExpandMaybe, 
    HighEnergyPhysics`Tarcer`Private`FactorC, 
    HighEnergyPhysics`Tarcer`Private`FEPrint, 
    HighEnergyPhysics`Tarcer`Private`fN, 
    HighEnergyPhysics`Tarcer`Private`mbt, 
    HighEnergyPhysics`Tarcer`Private`PNQ, 
    HighEnergyPhysics`Tarcer`Private`PQ, 
    HighEnergyPhysics`Tarcer`Private`ta, 
    HighEnergyPhysics`Tarcer`Private`TA, 
    HighEnergyPhysics`Tarcer`Private`TAIC, 
    HighEnergyPhysics`Tarcer`Private`TBIC, 
    HighEnergyPhysics`Tarcer`Private`TComment, 
    HighEnergyPhysics`Tarcer`Private`TLRComment, 
    HighEnergyPhysics`Tarcer`Private`TFIC, 
    HighEnergyPhysics`Tarcer`Private`TJIC, 
    HighEnergyPhysics`Tarcer`Private`TLR, 
    HighEnergyPhysics`Tarcer`Private`tlrule, 
    HighEnergyPhysics`Tarcer`Private`redblue, 
    HighEnergyPhysics`Tarcer`Private`TLRules, 
    HighEnergyPhysics`Tarcer`Private`TLRules2, 
    HighEnergyPhysics`Tarcer`Private`TVIC, 
    HighEnergyPhysics`Tarcer`Private`$TLRComment, 
    HighEnergyPhysics`general`Zeta2`Zeta2}]; \
\>", "Input",
 Editable->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"$HistoryLength", "=", "\[Infinity]"}], ";"}]], "Input"],

Cell["\<\
This gives the absolute time (in minutes) to run this notebook.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Round", "[", 
    RowBox[{
     RowBox[{"AbsoluteTime", "[", "]"}], "-", 
     "HighEnergyPhysics`Tarcer`Private`atime"}], "]"}], "/", "60."}], " ", 
  "min"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"1.8833333333333333`", " ", "min"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Display the size of the generated file:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"filename", " ", "=", " ", 
  RowBox[{"\"\<tarcer\>\"", "<>", 
   RowBox[{"(", 
    RowBox[{"Apply", "[", 
     RowBox[{"StringJoin", ",", 
      RowBox[{"ToString", "/@", "$RankLimit"}]}], "]"}], ")"}], "<>", 
   "\"\<.mx\>\""}]}]], "Input"],

Cell[BoxData[
 FormBox["\<\"tarcer25.mx\"\>", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FileByteCount", "[", "filename", "]"}], " ", "Byte"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"5170492", " ", "Byte"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Directory", "[", "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"C:\\\\Users\\\\rolfm\\\\HighEnergyPhysics\\\\Tarcer\"\>", 
  TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MaxMemoryUsed", "[", "]"}]], "Input"],

Cell[BoxData[
 FormBox["132148200", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemoryInUse", "[", "]"}]], "Input"],

Cell[BoxData[
 FormBox["71755552", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input"],

Cell[BoxData[
 FormBox["85.76800000000001`", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->"EditBar",
WindowSize->{1158, 776},
WindowMargins->{{Automatic, 14}, {-33, Automatic}},
CellLabelAutoDelete->True,
Magnification->1,
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (February 23, 2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "T1.1"->{
  Cell[579, 22, 72, 2, 83, "Title",
   CellTags->"T1.1"]},
 "T1.2"->{
  Cell[828, 34, 409, 14, 101, "Text",
   CellTags->"T1.2"]},
 "T1.1.1"->{
  Cell[29648, 1124, 91, 2, 71, "Section",
   CellTags->"T1.1.1"]},
 "T1.2.1.1"->{
  Cell[36416, 1362, 77, 1, 27, "Subsubsection",
   CellTags->"T1.2.1.1"]},
 "T1.2.1"->{
  Cell[37048, 1382, 84, 2, 71, "Section",
   CellTags->"T1.2.1"]},
 "T1.3.1"->{
  Cell[44924, 1626, 110, 2, 71, "Section",
   CellTags->"T1.3.1"]},
 "T1.3.1.1"->{
  Cell[47534, 1731, 62, 1, 36, "Subsection",
   CellTags->"T1.3.1.1"]},
 "T1.3.2.1"->{
  Cell[110607, 3604, 57, 1, 36, "Subsection",
   CellTags->"T1.3.2.1"]},
 "T1.3.3.1"->{
  Cell[125623, 3949, 58, 1, 36, "Subsection",
   CellTags->"T1.3.3.1"]},
 "T1.3.4.1"->{
  Cell[140474, 4368, 59, 1, 36, "Subsection",
   CellTags->"T1.3.4.1"]},
 "T1.3.5.1"->{
  Cell[142839, 4444, 53, 1, 36, "Subsection",
   CellTags->"T1.3.5.1"]},
 "T1.3.6.1"->{
  Cell[159734, 4898, 58, 1, 36, "Subsection",
   CellTags->"T1.3.6.1"]},
 "T1.3.7.1"->{
  Cell[161531, 4954, 49, 1, 36, "Subsection",
   CellTags->"T1.3.7.1"]},
 "T1.3.7.1.1"->{
  Cell[161810, 4966, 56, 1, 28, "Subsubsection",
   CellTags->"T1.3.7.1.1"]},
 "T1.3.7.2.1"->{
  Cell[166515, 5134, 52, 1, 27, "Subsubsection",
   CellTags->"T1.3.7.2.1"]},
 "T1.3.7.3.1"->{
  Cell[166760, 5147, 53, 1, 27, "Subsubsection",
   CellTags->"T1.3.7.3.1"]},
 "T1.3.7.4.1"->{
  Cell[167111, 5165, 57, 1, 27, "Subsubsection",
   CellTags->"T1.3.7.4.1"]},
 "T1.3.7.5.1"->{
  Cell[167385, 5178, 53, 1, 27, "Subsubsection",
   CellTags->"T1.3.7.5.1"]},
 "T1.3.7.6.1"->{
  Cell[171004, 5265, 53, 1, 27, "Subsubsection",
   CellTags->"T1.3.7.6.1"]},
 "T1.3.7.7.1"->{
  Cell[173317, 5347, 53, 1, 27, "Subsubsection",
   CellTags->"T1.3.7.7.1"]},
 "T1.3.7.8.1"->{
  Cell[175047, 5411, 53, 1, 27, "Subsubsection",
   CellTags->"T1.3.7.8.1"]},
 "T1.4.1"->{
  Cell[175447, 5431, 102, 2, 71, "Section",
   CellTags->"T1.4.1"]},
 "T1.4.1.1"->{
  Cell[180087, 5582, 108, 3, 37, "Subsection",
   CellTags->"T1.4.1.1"]},
 "T1.4.2.1"->{
  Cell[180590, 5604, 106, 3, 37, "Subsection",
   CellTags->"T1.4.2.1"]},
 "T1.4.3.1"->{
  Cell[182723, 5686, 196, 6, 37, "Subsection",
   CellTags->"T1.4.3.1"]},
 "T1.4.4.1"->{
  Cell[188958, 5889, 233, 7, 51, "Subsection",
   CellTags->"T1.4.4.1"]},
 "T1.5.1"->{
  Cell[201106, 6296, 95, 2, 71, "Section",
   CellTags->"T1.5.1"]},
 "T1.5.1.1"->{
  Cell[201226, 6302, 64, 1, 36, "Subsection",
   CellTags->"T1.5.1.1"]},
 "T1.5.1.1.1"->{
  Cell[275011, 8246, 61, 1, 27, "Subsubsection",
   CellTags->"T1.5.1.1.1"]},
 "T1.5.2.1"->{
  Cell[281544, 8460, 58, 1, 36, "Subsection",
   CellTags->"T1.5.2.1"]},
 "T1.5.2.1.1"->{
  Cell[285921, 8594, 52, 1, 27, "Subsubsection",
   CellTags->"T1.5.2.1.1"]},
 "T1.5.3.1"->{
  Cell[301331, 9079, 59, 1, 36, "Subsection",
   CellTags->"T1.5.3.1"]},
 "T1.5.3.1.1"->{
  Cell[301734, 9094, 67, 1, 27, "Subsubsection",
   CellTags->"T1.5.3.1.1"]},
 "T1.5.3.2.1"->{
  Cell[312013, 9385, 67, 1, 27, "Subsubsection",
   CellTags->"T1.5.3.2.1"]},
 "T1.6.1"->{
  Cell[320304, 9620, 138, 2, 71, "Section",
   CellTags->"T1.6.1"]},
 "T1.6.1.1"->{
  Cell[321564, 9665, 55, 1, 37, "Subsection",
   CellTags->"T1.6.1.1"]},
 "T1.6.2.1"->{
  Cell[328465, 9902, 57, 1, 36, "Subsection",
   CellTags->"T1.6.2.1"]},
 "T1.6.2.1.1"->{
  Cell[328805, 9918, 167, 7, 27, "Subsubsection",
   CellTags->"T1.6.2.1.1"]},
 "T1.6.2.2.1"->{
  Cell[330075, 9969, 167, 7, 27, "Subsubsection",
   CellTags->"T1.6.2.2.1"]},
 "T1.6.2.3.1"->{
  Cell[331514, 10025, 167, 7, 27, "Subsubsection",
   CellTags->"T1.6.2.3.1"]},
 "T1.6.2.4.1"->{
  Cell[332953, 10081, 167, 7, 27, "Subsubsection",
   CellTags->"T1.6.2.4.1"]},
 "T1.6.3.1"->{
  Cell[334404, 10138, 57, 1, 36, "Subsection",
   CellTags->"T1.6.3.1"]},
 "T1.6.4.1"->{
  Cell[338284, 10285, 246, 9, 41, "Subsection",
   CellTags->"T1.6.4.1"]},
 "T1.6.5.1"->{
  Cell[339211, 10323, 57, 1, 36, "Subsection",
   CellTags->"T1.6.5.1"]},
 "T1.6.6.1"->{
  Cell[343091, 10470, 246, 9, 41, "Subsection",
   CellTags->"T1.6.6.1"]},
 "T1.6.7.1"->{
  Cell[344018, 10508, 246, 9, 41, "Subsection",
   CellTags->"T1.6.7.1"]},
 "T1.6.8.1"->{
  Cell[345239, 10555, 246, 9, 41, "Subsection",
   CellTags->"T1.6.8.1"]},
 "T1.6.9.1"->{
  Cell[346460, 10602, 55, 1, 37, "Subsection",
   CellTags->"T1.6.9.1"]},
 "T1.6.10.1"->{
  Cell[352853, 10821, 58, 1, 36, "Subsection",
   CellTags->"T1.6.10.1"]},
 "T1.6.11.1"->{
  Cell[354245, 10876, 56, 1, 37, "Subsection",
   CellTags->"T1.6.11.1"]},
 "T1.6.12.1"->{
  Cell[358417, 11038, 56, 1, 37, "Subsection",
   CellTags->"T1.6.12.1"]},
 "T1.6.13.1"->{
  Cell[362589, 11200, 56, 1, 36, "Subsection",
   CellTags->"T1.6.13.1"]},
 "T1.6.14.1"->{
  Cell[369271, 11429, 67, 1, 36, "Subsection",
   CellTags->"T1.6.14.1"]},
 "T1.6.15.1"->{
  Cell[370369, 11470, 58, 1, 36, "Subsection",
   CellTags->"T1.6.15.1"]},
 "T1.6.16.1"->{
  Cell[393121, 12014, 56, 1, 36, "Subsection",
   CellTags->"T1.6.16.1"]},
 "T1.6.17.1"->{
  Cell[397188, 12166, 58, 1, 36, "Subsection",
   CellTags->"T1.6.17.1"]},
 "T1.6.18.1"->{
  Cell[398285, 12214, 56, 1, 36, "Subsection",
   CellTags->"T1.6.18.1"]},
 "T1.6.19.1"->{
  Cell[403859, 12417, 58, 1, 36, "Subsection",
   CellTags->"T1.6.19.1"]},
 "T1.6.20.1"->{
  Cell[404946, 12463, 58, 1, 36, "Subsection",
   CellTags->"T1.6.20.1"]},
 "T1.6.21.1"->{
  Cell[406099, 12511, 56, 1, 36, "Subsection",
   CellTags->"T1.6.21.1"]},
 "T1.6.22.1"->{
  Cell[409391, 12638, 58, 1, 36, "Subsection",
   CellTags->"T1.6.22.1"]},
 "T1.6.23.1"->{
  Cell[410363, 12680, 56, 1, 36, "Subsection",
   CellTags->"T1.6.23.1"]},
 "T1.6.24.1"->{
  Cell[414394, 12832, 58, 1, 36, "Subsection",
   CellTags->"T1.6.24.1"]},
 "T1.6.25.1"->{
  Cell[415440, 12876, 56, 1, 45, "Subsection",
   CellTags->"T1.6.25.1"]},
 "T1.6.26.1"->{
  Cell[419228, 13019, 58, 1, 36, "Subsection",
   CellTags->"T1.6.26.1"]},
 "T1.6.27.1"->{
  Cell[420276, 13063, 56, 1, 37, "Subsection",
   CellTags->"T1.6.27.1"]},
 "T1.6.28.1"->{
  Cell[426474, 13282, 58, 1, 36, "Subsection",
   CellTags->"T1.6.28.1"]},
 "T1.6.29.1"->{
  Cell[428590, 13346, 57, 1, 37, "Subsection",
   CellTags->"T1.6.29.1"]},
 "T1.6.30.1"->{
  Cell[431883, 13473, 58, 1, 36, "Subsection",
   CellTags->"T1.6.30.1"]},
 "T1.6.31.1"->{
  Cell[432874, 13515, 57, 1, 37, "Subsection",
   CellTags->"T1.6.31.1"]},
 "T1.6.32.1"->{
  Cell[437805, 13711, 58, 1, 37, "Subsection",
   CellTags->"T1.6.32.1"]},
 "T1.6.33.1"->{
  Cell[439888, 13799, 60, 1, 37, "Subsection",
   CellTags->"T1.6.33.1"]},
 "T1.6.34.1"->{
  Cell[442092, 13890, 57, 1, 36, "Subsection",
   CellTags->"T1.6.34.1"]},
 "T1.6.35.1"->{
  Cell[447833, 14100, 58, 1, 36, "Subsection",
   CellTags->"T1.6.35.1"]},
 "T1.6.36.1"->{
  Cell[449397, 14160, 57, 1, 36, "Subsection",
   CellTags->"T1.6.36.1"]},
 "T1.6.37.1"->{
  Cell[456094, 14388, 58, 1, 36, "Subsection",
   CellTags->"T1.6.37.1"]},
 "T1.6.38.1"->{
  Cell[458552, 14486, 60, 1, 36, "Subsection",
   CellTags->"T1.6.38.1"]},
 "T1.6.39.1"->{
  Cell[461643, 14602, 60, 1, 36, "Subsection",
   CellTags->"T1.6.39.1"]},
 "T1.6.40.1"->{
  Cell[463945, 14686, 60, 1, 36, "Subsection",
   CellTags->"T1.6.40.1"]},
 "T1.6.41.1"->{
  Cell[465606, 14750, 57, 1, 36, "Subsection",
   CellTags->"T1.6.41.1"]},
 "T1.6.42.1"->{
  Cell[470859, 14942, 58, 1, 36, "Subsection",
   CellTags->"T1.6.42.1"]},
 "T1.6.43.1"->{
  Cell[472474, 15005, 67, 1, 37, "Subsection",
   CellTags->"T1.6.43.1"]},
 "T1.6.44.1"->{
  Cell[478587, 15230, 56, 1, 37, "Subsection",
   CellTags->"T1.6.44.1"]},
 "T1.6.45.1"->{
  Cell[492560, 15737, 58, 1, 36, "Subsection",
   CellTags->"T1.6.45.1"]},
 "T1.6.46.1"->{
  Cell[493981, 15794, 60, 1, 36, "Subsection",
   CellTags->"T1.6.46.1"]},
 "T1.6.47.1"->{
  Cell[495522, 15854, 56, 1, 37, "Subsection",
   CellTags->"T1.6.47.1"]},
 "T1.6.48.1"->{
  Cell[501913, 16077, 58, 1, 36, "Subsection",
   CellTags->"T1.6.48.1"]},
 "T1.6.49.1"->{
  Cell[503375, 16135, 60, 1, 36, "Subsection",
   CellTags->"T1.6.49.1"]},
 "T1.6.50.1"->{
  Cell[504976, 16197, 56, 1, 36, "Subsection",
   CellTags->"T1.6.50.1"]},
 "T1.6.51.1"->{
  Cell[517150, 16617, 58, 1, 36, "Subsection",
   CellTags->"T1.6.51.1"]},
 "T1.6.52.1"->{
  Cell[518044, 16654, 56, 1, 36, "Subsection",
   CellTags->"T1.6.52.1"]},
 "T1.6.53.1"->{
  Cell[523803, 16863, 58, 1, 36, "Subsection",
   CellTags->"T1.6.53.1"]},
 "T1.6.54.1"->{
  Cell[524970, 16912, 60, 1, 36, "Subsection",
   CellTags->"T1.6.54.1"]},
 "T1.6.55.1"->{
  Cell[526217, 16963, 60, 1, 36, "Subsection",
   CellTags->"T1.6.55.1"]},
 "T1.6.56.1"->{
  Cell[527464, 17014, 56, 1, 37, "Subsection",
   CellTags->"T1.6.56.1"]},
 "T1.6.57.1"->{
  Cell[533927, 17244, 58, 1, 36, "Subsection",
   CellTags->"T1.6.57.1"]},
 "T1.6.58.1"->{
  Cell[534948, 17288, 56, 1, 37, "Subsection",
   CellTags->"T1.6.58.1"]},
 "T1.6.59.1"->{
  Cell[540485, 17479, 58, 1, 36, "Subsection",
   CellTags->"T1.6.59.1"]},
 "T1.6.60.1"->{
  Cell[541497, 17521, 56, 1, 45, "Subsection",
   CellTags->"T1.6.60.1"]},
 "T1.6.61.1"->{
  Cell[547296, 17729, 58, 1, 36, "Subsection",
   CellTags->"T1.6.61.1"]},
 "T1.6.62.1"->{
  Cell[548639, 17783, 60, 1, 36, "Subsection",
   CellTags->"T1.6.62.1"]},
 "T1.6.63.1"->{
  Cell[550064, 17839, 60, 1, 36, "Subsection",
   CellTags->"T1.6.63.1"]},
 "T1.6.64.1"->{
  Cell[551504, 17895, 56, 1, 45, "Subsection",
   CellTags->"T1.6.64.1"]},
 "T1.6.65.1"->{
  Cell[565020, 18348, 58, 1, 36, "Subsection",
   CellTags->"T1.6.65.1"]},
 "T1.6.66.1"->{
  Cell[566213, 18397, 60, 1, 36, "Subsection",
   CellTags->"T1.6.66.1"]},
 "T1.6.67.1"->{
  Cell[567484, 18448, 60, 1, 36, "Subsection",
   CellTags->"T1.6.67.1"]},
 "T1.6.68.1"->{
  Cell[568757, 18499, 56, 1, 45, "Subsection",
   CellTags->"T1.6.68.1"]},
 "T1.6.69.1"->{
  Cell[576885, 18775, 58, 1, 36, "Subsection",
   CellTags->"T1.6.69.1"]},
 "T1.6.70.1"->{
  Cell[578049, 18822, 56, 1, 45, "Subsection",
   CellTags->"T1.6.70.1"]},
 "T1.6.70.1.1"->{
  Cell[578130, 18827, 55, 1, 27, "Subsubsection",
   CellTags->"T1.6.70.1.1"]},
 "T1.6.70.2.1"->{
  Cell[582644, 18988, 55, 1, 27, "Subsubsection",
   CellTags->"T1.6.70.2.1"]},
 "T1.6.71.1"->{
  Cell[591410, 19310, 58, 1, 36, "Subsection",
   CellTags->"T1.6.71.1"]},
 "T1.6.72.1"->{
  Cell[592101, 19339, 56, 1, 37, "Subsection",
   CellTags->"T1.6.72.1"]},
 "T1.6.73.1"->{
  Cell[604104, 19729, 58, 1, 36, "Subsection",
   CellTags->"T1.6.73.1"]},
 "T1.6.74.1"->{
  Cell[604785, 19758, 56, 1, 36, "Subsection",
   CellTags->"T1.6.74.1"]},
 "T1.6.75.1"->{
  Cell[616204, 20188, 59, 1, 36, "Subsection",
   CellTags->"T1.6.75.1"]},
 "T1.6.76.1"->{
  Cell[617032, 20222, 59, 1, 36, "Subsection",
   CellTags->"T1.6.76.1"]},
 "T1.6.77.1"->{
  Cell[618066, 20263, 59, 1, 36, "Subsection",
   CellTags->"T1.6.77.1"]},
 "T1.6.78.1"->{
  Cell[618867, 20294, 59, 1, 36, "Subsection",
   CellTags->"T1.6.78.1"]},
 "T1.6.79.1"->{
  Cell[619782, 20329, 56, 1, 36, "Subsection",
   CellTags->"T1.6.79.1"]},
 "T1.6.80.1"->{
  Cell[625688, 20536, 56, 1, 36, "Subsection",
   CellTags->"T1.6.80.1"]},
 "T1.6.81.1"->{
  Cell[629071, 20664, 58, 1, 36, "Subsection",
   CellTags->"T1.6.81.1"]},
 "T1.6.82.1"->{
  Cell[629920, 20699, 56, 1, 36, "Subsection",
   CellTags->"T1.6.82.1"]},
 "T1.6.83.1"->{
  Cell[633051, 20817, 58, 1, 36, "Subsection",
   CellTags->"T1.6.83.1"]},
 "T1.6.84.1"->{
  Cell[633781, 20847, 60, 1, 36, "Subsection",
   CellTags->"T1.6.84.1"]},
 "T1.6.85.1"->{
  Cell[635043, 20896, 60, 1, 36, "Subsection",
   CellTags->"T1.6.85.1"]},
 "T1.6.86.1"->{
  Cell[636426, 20948, 56, 1, 36, "Subsection",
   CellTags->"T1.6.86.1"]},
 "T1.6.87.1"->{
  Cell[638617, 21039, 58, 1, 36, "Subsection",
   CellTags->"T1.6.87.1"]},
 "T1.6.88.1"->{
  Cell[639396, 21071, 56, 1, 36, "Subsection",
   CellTags->"T1.6.88.1"]},
 "T1.6.89.1"->{
  Cell[641753, 21165, 58, 1, 36, "Subsection",
   CellTags->"T1.6.89.1"]},
 "T1.6.90.1"->{
  Cell[642344, 21189, 56, 1, 36, "Subsection",
   CellTags->"T1.6.90.1"]},
 "T1.6.91.1"->{
  Cell[643875, 21255, 58, 1, 36, "Subsection",
   CellTags->"T1.6.91.1"]},
 "T1.6.92.1"->{
  Cell[644577, 21284, 56, 1, 36, "Subsection",
   CellTags->"T1.6.92.1"]},
 "T1.6.93.1"->{
  Cell[645982, 21345, 58, 1, 36, "Subsection",
   CellTags->"T1.6.93.1"]},
 "T1.6.94.1"->{
  Cell[646535, 21369, 56, 1, 36, "Subsection",
   CellTags->"T1.6.94.1"]},
 "T1.6.95.1"->{
  Cell[651562, 21560, 58, 1, 36, "Subsection",
   CellTags->"T1.6.95.1"]},
 "T1.6.96.1"->{
  Cell[652361, 21593, 60, 1, 36, "Subsection",
   CellTags->"T1.6.96.1"]},
 "T1.6.97.1"->{
  Cell[653285, 21629, 60, 1, 36, "Subsection",
   CellTags->"T1.6.97.1"]},
 "T1.6.98.1"->{
  Cell[654209, 21665, 57, 1, 36, "Subsection",
   CellTags->"T1.6.98.1"]},
 "T1.6.99.1"->{
  Cell[660310, 21896, 59, 1, 36, "Subsection",
   CellTags->"T1.6.99.1"]},
 "T1.6.100.1"->{
  Cell[661218, 21936, 61, 1, 36, "Subsection",
   CellTags->"T1.6.100.1"]},
 "T1.6.101.1"->{
  Cell[662233, 21978, 61, 1, 36, "Subsection",
   CellTags->"T1.6.101.1"]},
 "T1.6.102.1"->{
  Cell[663299, 22021, 61, 1, 36, "Subsection",
   CellTags->"T1.6.102.1"]},
 "T1.6.103.1"->{
  Cell[664314, 22063, 61, 1, 36, "Subsection",
   CellTags->"T1.6.103.1"]},
 "T1.6.104.1"->{
  Cell[665329, 22105, 61, 1, 36, "Subsection",
   CellTags->"T1.6.104.1"]},
 "T1.6.105.1"->{
  Cell[666395, 22148, 58, 1, 36, "Subsection",
   CellTags->"T1.6.105.1"]},
 "T1.6.106.1"->{
  Cell[669429, 22250, 60, 1, 36, "Subsection",
   CellTags->"T1.6.106.1"]},
 "T1.6.107.1"->{
  Cell[669798, 22268, 58, 1, 45, "Subsection",
   CellTags->"T1.6.107.1"]},
 "T1.6.108.1"->{
  Cell[673138, 22378, 60, 1, 36, "Subsection",
   CellTags->"T1.6.108.1"]},
 "T1.6.109.1"->{
  Cell[673699, 22402, 58, 1, 45, "Subsection",
   CellTags->"T1.6.109.1"]},
 "T1.6.110.1"->{
  Cell[676518, 22498, 60, 1, 36, "Subsection",
   CellTags->"T1.6.110.1"]},
 "T1.6.111.1"->{
  Cell[676887, 22516, 58, 1, 45, "Subsection",
   CellTags->"T1.6.111.1"]},
 "T1.6.112.1"->{
  Cell[678663, 22578, 60, 1, 36, "Subsection",
   CellTags->"T1.6.112.1"]},
 "T1.6.113.1"->{
  Cell[679032, 22596, 58, 1, 37, "Subsection",
   CellTags->"T1.6.113.1"]},
 "T1.6.114.1"->{
  Cell[686704, 22853, 60, 1, 36, "Subsection",
   CellTags->"T1.6.114.1"]},
 "T1.6.115.1"->{
  Cell[687661, 22891, 60, 1, 36, "Subsection",
   CellTags->"T1.6.115.1"]},
 "T1.6.116.1"->{
  Cell[688452, 22928, 60, 1, 36, "Subsection",
   CellTags->"T1.6.116.1"]},
 "T1.6.117.1"->{
  Cell[689247, 22965, 61, 1, 36, "Subsection",
   CellTags->"T1.6.117.1"]},
 "T1.6.118.1"->{
  Cell[692733, 23064, 61, 1, 36, "Subsection",
   CellTags->"T1.6.118.1"]},
 "T1.6.119.1"->{
  Cell[696189, 23163, 61, 1, 36, "Subsection",
   CellTags->"T1.6.119.1"]},
 "T1.6.120.1"->{
  Cell[699639, 23262, 60, 1, 36, "Subsection",
   CellTags->"T1.6.120.1"]},
 "Tsymmetry"->{
  Cell[709629, 23505, 54, 1, 36, "Subsection",
   CellTags->"Tsymmetry"]},
 "specialTVR"->{
  Cell[709867, 23518, 69, 1, 36, "Subsection",
   CellTags->"specialTVR"]},
 "T1.6.121.1"->{
  Cell[720655, 23706, 59, 1, 36, "Subsection",
   CellTags->"T1.6.121.1"]},
 "T1.6.122.1"->{
  Cell[727047, 23915, 60, 1, 36, "Subsection",
   CellTags->"T1.6.122.1"]},
 "T1.6.123.1"->{
  Cell[730033, 24015, 59, 1, 36, "Subsection",
   CellTags->"T1.6.123.1"]},
 "T1.6.124.1"->{
  Cell[732516, 24102, 57, 1, 36, "Subsection",
   CellTags->"T1.6.124.1"]},
 "T1.7.1"->{
  Cell[734700, 24185, 106, 2, 71, "Section",
   CellTags->"T1.7.1"]},
 "T1.7.1.1"->{
  Cell[734831, 24191, 69, 1, 36, "Subsection",
   CellTags->"T1.7.1.1"]},
 "TLRdef"->{
  Cell[737450, 24290, 215, 7, 101, "Text",
   CellTags->"TLRdef"]},
 "tlrule17"->{
  Cell[770577, 25255, 77, 1, 29, "Text",
   CellTags->"tlrule17"]},
 "T1.7.2.1"->{
  Cell[813612, 26636, 49, 1, 37, "Subsection",
   CellTags->"T1.7.2.1"]},
 "T1.7.3.1"->{
  Cell[831186, 27210, 53, 1, 36, "Subsection",
   CellTags->"T1.7.3.1"]},
 "$TTable"->{
  Cell[835042, 27314, 51, 1, 36, "Subsection",
   CellTags->"$TTable"]},
 "T1.7.4.1"->{
  Cell[835225, 27325, 58, 1, 36, "Subsection",
   CellTags->"T1.7.4.1"]},
 "T1.8.1"->{
  Cell[854079, 27812, 90, 2, 71, "Section",
   CellTags->"T1.8.1"]},
 "T1.9.1"->{
  Cell[872063, 28277, 91, 2, 72, "Section",
   CellTags->"T1.9.1"]},
 "T1.9.1.1"->{
  Cell[872179, 28283, 53, 1, 37, "Subsection",
   CellTags->"T1.9.1.1"]},
 "T1.10.1"->{
  Cell[880975, 28567, 93, 2, 72, "Section",
   CellTags->"T1.10.1"]},
 "T1.11.1"->{
  Cell[915416, 29679, 99, 2, 71, "Section",
   CellTags->"T1.11.1"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"T1.1", 920348, 29851},
 {"T1.2", 920418, 29854},
 {"T1.1.1", 920492, 29857},
 {"T1.2.1.1", 920574, 29860},
 {"T1.2.1", 920662, 29863},
 {"T1.3.1", 920742, 29866},
 {"T1.3.1.1", 920825, 29869},
 {"T1.3.2.1", 920912, 29872},
 {"T1.3.3.1", 921000, 29875},
 {"T1.3.4.1", 921088, 29878},
 {"T1.3.5.1", 921176, 29881},
 {"T1.3.6.1", 921264, 29884},
 {"T1.3.7.1", 921352, 29887},
 {"T1.3.7.1.1", 921442, 29890},
 {"T1.3.7.2.1", 921537, 29893},
 {"T1.3.7.3.1", 921632, 29896},
 {"T1.3.7.4.1", 921727, 29899},
 {"T1.3.7.5.1", 921822, 29902},
 {"T1.3.7.6.1", 921917, 29905},
 {"T1.3.7.7.1", 922012, 29908},
 {"T1.3.7.8.1", 922107, 29911},
 {"T1.4.1", 922198, 29914},
 {"T1.4.1.1", 922282, 29917},
 {"T1.4.2.1", 922371, 29920},
 {"T1.4.3.1", 922460, 29923},
 {"T1.4.4.1", 922549, 29926},
 {"T1.5.1", 922636, 29929},
 {"T1.5.1.1", 922719, 29932},
 {"T1.5.1.1.1", 922809, 29935},
 {"T1.5.2.1", 922902, 29938},
 {"T1.5.2.1.1", 922992, 29941},
 {"T1.5.3.1", 923085, 29944},
 {"T1.5.3.1.1", 923175, 29947},
 {"T1.5.3.2.1", 923270, 29950},
 {"T1.6.1", 923361, 29953},
 {"T1.6.1.1", 923445, 29956},
 {"T1.6.2.1", 923533, 29959},
 {"T1.6.2.1.1", 923623, 29962},
 {"T1.6.2.2.1", 923719, 29965},
 {"T1.6.2.3.1", 923815, 29968},
 {"T1.6.2.4.1", 923912, 29971},
 {"T1.6.3.1", 924007, 29974},
 {"T1.6.4.1", 924096, 29977},
 {"T1.6.5.1", 924186, 29980},
 {"T1.6.6.1", 924275, 29983},
 {"T1.6.7.1", 924365, 29986},
 {"T1.6.8.1", 924455, 29989},
 {"T1.6.9.1", 924545, 29992},
 {"T1.6.10.1", 924635, 29995},
 {"T1.6.11.1", 924726, 29998},
 {"T1.6.12.1", 924817, 30001},
 {"T1.6.13.1", 924908, 30004},
 {"T1.6.14.1", 924999, 30007},
 {"T1.6.15.1", 925090, 30010},
 {"T1.6.16.1", 925181, 30013},
 {"T1.6.17.1", 925272, 30016},
 {"T1.6.18.1", 925363, 30019},
 {"T1.6.19.1", 925454, 30022},
 {"T1.6.20.1", 925545, 30025},
 {"T1.6.21.1", 925636, 30028},
 {"T1.6.22.1", 925727, 30031},
 {"T1.6.23.1", 925818, 30034},
 {"T1.6.24.1", 925909, 30037},
 {"T1.6.25.1", 926000, 30040},
 {"T1.6.26.1", 926091, 30043},
 {"T1.6.27.1", 926182, 30046},
 {"T1.6.28.1", 926273, 30049},
 {"T1.6.29.1", 926364, 30052},
 {"T1.6.30.1", 926455, 30055},
 {"T1.6.31.1", 926546, 30058},
 {"T1.6.32.1", 926637, 30061},
 {"T1.6.33.1", 926728, 30064},
 {"T1.6.34.1", 926819, 30067},
 {"T1.6.35.1", 926910, 30070},
 {"T1.6.36.1", 927001, 30073},
 {"T1.6.37.1", 927092, 30076},
 {"T1.6.38.1", 927183, 30079},
 {"T1.6.39.1", 927274, 30082},
 {"T1.6.40.1", 927365, 30085},
 {"T1.6.41.1", 927456, 30088},
 {"T1.6.42.1", 927547, 30091},
 {"T1.6.43.1", 927638, 30094},
 {"T1.6.44.1", 927729, 30097},
 {"T1.6.45.1", 927820, 30100},
 {"T1.6.46.1", 927911, 30103},
 {"T1.6.47.1", 928002, 30106},
 {"T1.6.48.1", 928093, 30109},
 {"T1.6.49.1", 928184, 30112},
 {"T1.6.50.1", 928275, 30115},
 {"T1.6.51.1", 928366, 30118},
 {"T1.6.52.1", 928457, 30121},
 {"T1.6.53.1", 928548, 30124},
 {"T1.6.54.1", 928639, 30127},
 {"T1.6.55.1", 928730, 30130},
 {"T1.6.56.1", 928821, 30133},
 {"T1.6.57.1", 928912, 30136},
 {"T1.6.58.1", 929003, 30139},
 {"T1.6.59.1", 929094, 30142},
 {"T1.6.60.1", 929185, 30145},
 {"T1.6.61.1", 929276, 30148},
 {"T1.6.62.1", 929367, 30151},
 {"T1.6.63.1", 929458, 30154},
 {"T1.6.64.1", 929549, 30157},
 {"T1.6.65.1", 929640, 30160},
 {"T1.6.66.1", 929731, 30163},
 {"T1.6.67.1", 929822, 30166},
 {"T1.6.68.1", 929913, 30169},
 {"T1.6.69.1", 930004, 30172},
 {"T1.6.70.1", 930095, 30175},
 {"T1.6.70.1.1", 930188, 30178},
 {"T1.6.70.2.1", 930286, 30181},
 {"T1.6.71.1", 930382, 30184},
 {"T1.6.72.1", 930473, 30187},
 {"T1.6.73.1", 930564, 30190},
 {"T1.6.74.1", 930655, 30193},
 {"T1.6.75.1", 930746, 30196},
 {"T1.6.76.1", 930837, 30199},
 {"T1.6.77.1", 930928, 30202},
 {"T1.6.78.1", 931019, 30205},
 {"T1.6.79.1", 931110, 30208},
 {"T1.6.80.1", 931201, 30211},
 {"T1.6.81.1", 931292, 30214},
 {"T1.6.82.1", 931383, 30217},
 {"T1.6.83.1", 931474, 30220},
 {"T1.6.84.1", 931565, 30223},
 {"T1.6.85.1", 931656, 30226},
 {"T1.6.86.1", 931747, 30229},
 {"T1.6.87.1", 931838, 30232},
 {"T1.6.88.1", 931929, 30235},
 {"T1.6.89.1", 932020, 30238},
 {"T1.6.90.1", 932111, 30241},
 {"T1.6.91.1", 932202, 30244},
 {"T1.6.92.1", 932293, 30247},
 {"T1.6.93.1", 932384, 30250},
 {"T1.6.94.1", 932475, 30253},
 {"T1.6.95.1", 932566, 30256},
 {"T1.6.96.1", 932657, 30259},
 {"T1.6.97.1", 932748, 30262},
 {"T1.6.98.1", 932839, 30265},
 {"T1.6.99.1", 932930, 30268},
 {"T1.6.100.1", 933022, 30271},
 {"T1.6.101.1", 933115, 30274},
 {"T1.6.102.1", 933208, 30277},
 {"T1.6.103.1", 933301, 30280},
 {"T1.6.104.1", 933394, 30283},
 {"T1.6.105.1", 933487, 30286},
 {"T1.6.106.1", 933580, 30289},
 {"T1.6.107.1", 933673, 30292},
 {"T1.6.108.1", 933766, 30295},
 {"T1.6.109.1", 933859, 30298},
 {"T1.6.110.1", 933952, 30301},
 {"T1.6.111.1", 934045, 30304},
 {"T1.6.112.1", 934138, 30307},
 {"T1.6.113.1", 934231, 30310},
 {"T1.6.114.1", 934324, 30313},
 {"T1.6.115.1", 934417, 30316},
 {"T1.6.116.1", 934510, 30319},
 {"T1.6.117.1", 934603, 30322},
 {"T1.6.118.1", 934696, 30325},
 {"T1.6.119.1", 934789, 30328},
 {"T1.6.120.1", 934882, 30331},
 {"Tsymmetry", 934974, 30334},
 {"specialTVR", 935066, 30337},
 {"T1.6.121.1", 935159, 30340},
 {"T1.6.122.1", 935252, 30343},
 {"T1.6.123.1", 935345, 30346},
 {"T1.6.124.1", 935438, 30349},
 {"T1.7.1", 935527, 30352},
 {"T1.7.1.1", 935612, 30355},
 {"TLRdef", 935699, 30358},
 {"tlrule17", 935782, 30361},
 {"T1.7.2.1", 935865, 30364},
 {"T1.7.3.1", 935954, 30367},
 {"$TTable", 936042, 30370},
 {"T1.7.4.1", 936130, 30373},
 {"T1.8.1", 936217, 30376},
 {"T1.9.1", 936299, 30379},
 {"T1.9.1.1", 936383, 30382},
 {"T1.10.1", 936471, 30385},
 {"T1.11.1", 936555, 30388}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 72, 2, 83, "Title",
 CellTags->"T1.1"],
Cell[654, 26, 171, 6, 29, "Text"],
Cell[828, 34, 409, 14, 101, "Text",
 CellTags->"T1.2"],
Cell[1240, 50, 56, 1, 29, "Text"],
Cell[1299, 53, 161, 5, 29, "Text"],
Cell[1463, 60, 140, 3, 29, "Text"],
Cell[1606, 65, 185, 4, 47, "Text"],
Cell[1794, 71, 154, 4, 47, "Text"],
Cell[1951, 77, 95, 2, 29, "Text"],
Cell[2049, 81, 384, 8, 83, "Text"],
Cell[2436, 91, 155, 3, 29, "Text"],
Cell[2594, 96, 131, 3, 29, "Text"],
Cell[2728, 101, 137, 3, 29, "Text"],
Cell[2868, 106, 177, 6, 29, "Text"],
Cell[3048, 114, 149, 6, 29, "Text"],
Cell[3200, 122, 162, 6, 29, "Text"],
Cell[3365, 130, 153, 6, 29, "Text"],
Cell[3521, 138, 153, 6, 29, "Text"],
Cell[3677, 146, 153, 6, 29, "Text"],
Cell[3833, 154, 158, 6, 29, "Text"],
Cell[3994, 162, 156, 3, 29, "Text"],
Cell[4153, 167, 1081, 21, 245, "Text"],
Cell[5237, 190, 147, 3, 29, "Text"],
Cell[5387, 195, 497, 9, 101, "Text"],
Cell[5887, 206, 92, 2, 29, "Text"],
Cell[5982, 210, 174, 3, 29, "Text"],
Cell[6159, 215, 219, 6, 47, "Text"],
Cell[6381, 223, 78, 2, 31, "Input"],
Cell[6462, 227, 180, 4, 29, "Text"],
Cell[6645, 233, 212, 4, 29, "Text"],
Cell[6860, 239, 249, 8, 29, "Text"],
Cell[CellGroupData[{
Cell[7134, 251, 74, 1, 71, "Section"],
Cell[7211, 254, 98, 2, 29, "Text"],
Cell[7312, 258, 91, 2, 29, "Text"],
Cell[7406, 262, 116, 2, 29, "Text"],
Cell[7525, 266, 117, 2, 29, "Text"],
Cell[7645, 270, 104, 2, 29, "Text"],
Cell[7752, 274, 99, 2, 29, "Text"],
Cell[7854, 278, 100, 2, 29, "Text"],
Cell[7957, 282, 101, 2, 29, "Text"],
Cell[8061, 286, 95, 2, 29, "Text"],
Cell[8159, 290, 100, 2, 29, "Text"],
Cell[8262, 294, 91, 2, 29, "Text"],
Cell[8356, 298, 100, 2, 24, "SmallText"],
Cell[8459, 302, 96, 2, 24, "SmallText"],
Cell[8558, 306, 97, 2, 24, "SmallText"],
Cell[8658, 310, 101, 2, 24, "SmallText"],
Cell[8762, 314, 97, 2, 24, "SmallText"],
Cell[8862, 318, 97, 2, 24, "SmallText"],
Cell[8962, 322, 97, 2, 24, "SmallText"],
Cell[9062, 326, 97, 2, 24, "SmallText"],
Cell[9162, 330, 109, 2, 29, "Text"],
Cell[9274, 334, 138, 3, 29, "Text"],
Cell[9415, 339, 136, 3, 29, "Text"],
Cell[9554, 344, 457, 15, 35, "Text"],
Cell[10014, 361, 499, 16, 35, "Text"],
Cell[10516, 379, 102, 2, 29, "Text"],
Cell[10621, 383, 106, 2, 29, "Text"],
Cell[10730, 387, 105, 2, 24, "SmallText"],
Cell[10838, 391, 175, 7, 29, "Text"],
Cell[11016, 400, 96, 2, 24, "SmallText"],
Cell[11115, 404, 101, 2, 29, "Text"],
Cell[11219, 408, 111, 2, 24, "SmallText"],
Cell[11333, 412, 111, 2, 24, "SmallText"],
Cell[11447, 416, 147, 3, 29, "Text"],
Cell[11597, 421, 97, 2, 29, "Text"],
Cell[11697, 425, 99, 2, 29, "Text"],
Cell[11799, 429, 520, 19, 33, "SmallText"],
Cell[12322, 450, 520, 19, 33, "SmallText"],
Cell[12845, 471, 520, 19, 33, "SmallText"],
Cell[13368, 492, 520, 19, 33, "SmallText"],
Cell[13891, 513, 99, 2, 29, "Text"],
Cell[13993, 517, 606, 21, 36, "Text"],
Cell[14602, 540, 99, 2, 29, "Text"],
Cell[14704, 544, 606, 21, 36, "Text"],
Cell[15313, 567, 606, 21, 36, "Text"],
Cell[15922, 590, 606, 21, 36, "Text"],
Cell[16531, 613, 97, 2, 29, "Text"],
Cell[16631, 617, 100, 2, 29, "Text"],
Cell[16734, 621, 98, 2, 29, "Text"],
Cell[16835, 625, 98, 2, 29, "Text"],
Cell[16936, 629, 98, 2, 29, "Text"],
Cell[17037, 633, 109, 2, 29, "Text"],
Cell[17149, 637, 100, 2, 29, "Text"],
Cell[17252, 641, 98, 2, 29, "Text"],
Cell[17353, 645, 100, 2, 29, "Text"],
Cell[17456, 649, 98, 2, 29, "Text"],
Cell[17557, 653, 100, 2, 29, "Text"],
Cell[17660, 657, 100, 2, 29, "Text"],
Cell[17763, 661, 98, 2, 29, "Text"],
Cell[17864, 665, 100, 2, 29, "Text"],
Cell[17967, 669, 98, 2, 29, "Text"],
Cell[18068, 673, 100, 2, 29, "Text"],
Cell[18171, 677, 98, 2, 29, "Text"],
Cell[18272, 681, 100, 2, 29, "Text"],
Cell[18375, 685, 98, 2, 29, "Text"],
Cell[18476, 689, 100, 2, 29, "Text"],
Cell[18579, 693, 98, 2, 29, "Text"],
Cell[18680, 697, 100, 2, 29, "Text"],
Cell[18783, 701, 98, 2, 29, "Text"],
Cell[18884, 705, 100, 2, 29, "Text"],
Cell[18987, 709, 102, 2, 29, "Text"],
Cell[19092, 713, 98, 2, 29, "Text"],
Cell[19193, 717, 100, 2, 29, "Text"],
Cell[19296, 721, 98, 2, 29, "Text"],
Cell[19397, 725, 100, 2, 29, "Text"],
Cell[19500, 729, 102, 2, 29, "Text"],
Cell[19605, 733, 102, 2, 29, "Text"],
Cell[19710, 737, 102, 2, 29, "Text"],
Cell[19815, 741, 98, 2, 29, "Text"],
Cell[19916, 745, 100, 2, 29, "Text"],
Cell[20019, 749, 109, 2, 29, "Text"],
Cell[20131, 753, 98, 2, 29, "Text"],
Cell[20232, 757, 100, 2, 29, "Text"],
Cell[20335, 761, 102, 2, 29, "Text"],
Cell[20440, 765, 98, 2, 29, "Text"],
Cell[20541, 769, 100, 2, 29, "Text"],
Cell[20644, 773, 102, 2, 29, "Text"],
Cell[20749, 777, 98, 2, 29, "Text"],
Cell[20850, 781, 100, 2, 29, "Text"],
Cell[20953, 785, 98, 2, 29, "Text"],
Cell[21054, 789, 100, 2, 29, "Text"],
Cell[21157, 793, 102, 2, 29, "Text"],
Cell[21262, 797, 102, 2, 29, "Text"],
Cell[21367, 801, 98, 2, 29, "Text"],
Cell[21468, 805, 100, 2, 29, "Text"],
Cell[21571, 809, 98, 2, 29, "Text"],
Cell[21672, 813, 100, 2, 29, "Text"],
Cell[21775, 817, 98, 2, 29, "Text"],
Cell[21876, 821, 100, 2, 29, "Text"],
Cell[21979, 825, 102, 2, 29, "Text"],
Cell[22084, 829, 102, 2, 29, "Text"],
Cell[22189, 833, 98, 2, 29, "Text"],
Cell[22290, 837, 100, 2, 29, "Text"],
Cell[22393, 841, 102, 2, 29, "Text"],
Cell[22498, 845, 102, 2, 29, "Text"],
Cell[22603, 849, 98, 2, 29, "Text"],
Cell[22704, 853, 100, 2, 29, "Text"],
Cell[22807, 857, 98, 2, 29, "Text"],
Cell[22908, 861, 99, 2, 24, "SmallText"],
Cell[23010, 865, 99, 2, 24, "SmallText"],
Cell[23112, 869, 100, 2, 29, "Text"],
Cell[23215, 873, 98, 2, 29, "Text"],
Cell[23316, 877, 100, 2, 29, "Text"],
Cell[23419, 881, 98, 2, 29, "Text"],
Cell[23520, 885, 101, 2, 29, "Text"],
Cell[23624, 889, 101, 2, 29, "Text"],
Cell[23728, 893, 101, 2, 29, "Text"],
Cell[23832, 897, 101, 2, 29, "Text"],
Cell[23936, 901, 98, 2, 29, "Text"],
Cell[24037, 905, 98, 2, 29, "Text"],
Cell[24138, 909, 100, 2, 29, "Text"],
Cell[24241, 913, 98, 2, 29, "Text"],
Cell[24342, 917, 100, 2, 29, "Text"],
Cell[24445, 921, 102, 2, 29, "Text"],
Cell[24550, 925, 102, 2, 29, "Text"],
Cell[24655, 929, 98, 2, 29, "Text"],
Cell[24756, 933, 100, 2, 29, "Text"],
Cell[24859, 937, 98, 2, 29, "Text"],
Cell[24960, 941, 100, 2, 29, "Text"],
Cell[25063, 945, 98, 2, 29, "Text"],
Cell[25164, 949, 100, 2, 29, "Text"],
Cell[25267, 953, 98, 2, 29, "Text"],
Cell[25368, 957, 100, 2, 29, "Text"],
Cell[25471, 961, 98, 2, 29, "Text"],
Cell[25572, 965, 100, 2, 29, "Text"],
Cell[25675, 969, 102, 2, 29, "Text"],
Cell[25780, 973, 102, 2, 29, "Text"],
Cell[25885, 977, 99, 2, 29, "Text"],
Cell[25987, 981, 101, 2, 29, "Text"],
Cell[26091, 985, 103, 2, 29, "Text"],
Cell[26197, 989, 103, 2, 29, "Text"],
Cell[26303, 993, 103, 2, 29, "Text"],
Cell[26409, 997, 103, 2, 29, "Text"],
Cell[26515, 1001, 103, 2, 29, "Text"],
Cell[26621, 1005, 100, 2, 29, "Text"],
Cell[26724, 1009, 102, 2, 29, "Text"],
Cell[26829, 1013, 100, 2, 29, "Text"],
Cell[26932, 1017, 102, 2, 29, "Text"],
Cell[27037, 1021, 100, 2, 29, "Text"],
Cell[27140, 1025, 102, 2, 29, "Text"],
Cell[27245, 1029, 100, 2, 29, "Text"],
Cell[27348, 1033, 102, 2, 29, "Text"],
Cell[27453, 1037, 100, 2, 29, "Text"],
Cell[27556, 1041, 102, 2, 29, "Text"],
Cell[27661, 1045, 102, 2, 29, "Text"],
Cell[27766, 1049, 102, 2, 29, "Text"],
Cell[27871, 1053, 102, 2, 29, "Text"],
Cell[27976, 1057, 102, 2, 29, "Text"],
Cell[28081, 1061, 102, 2, 29, "Text"],
Cell[28186, 1065, 102, 2, 29, "Text"],
Cell[28291, 1069, 101, 2, 29, "Text"],
Cell[28395, 1073, 102, 2, 29, "Text"],
Cell[28500, 1077, 101, 2, 29, "Text"],
Cell[28604, 1081, 99, 2, 29, "Text"],
Cell[28706, 1085, 113, 2, 29, "Text"],
Cell[28822, 1089, 97, 2, 29, "Text"],
Cell[28922, 1093, 91, 2, 29, "Text"],
Cell[29016, 1097, 95, 2, 29, "Text"],
Cell[29114, 1101, 100, 2, 29, "Text"],
Cell[29217, 1105, 97, 2, 29, "Text"],
Cell[29317, 1109, 98, 2, 29, "Text"],
Cell[29418, 1113, 101, 2, 29, "Text"],
Cell[29522, 1117, 89, 2, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29648, 1124, 91, 2, 71, "Section",
 CellTags->"T1.1.1"],
Cell[29742, 1128, 332, 10, 54, "SmallText"],
Cell[30077, 1140, 118, 3, 29, "Text"],
Cell[30198, 1145, 3986, 130, 89, "Text"],
Cell[34187, 1277, 222, 5, 83, "Text"],
Cell[34412, 1284, 80, 2, 31, "Input"],
Cell[34495, 1288, 146, 5, 31, "Input"],
Cell[34644, 1295, 246, 8, 31, "Input"],
Cell[34893, 1305, 80, 2, 31, "Input"],
Cell[34976, 1309, 107, 3, 29, "Text"],
Cell[35086, 1314, 339, 9, 31, "Input"],
Cell[35428, 1325, 215, 6, 24, "SmallText"],
Cell[35646, 1333, 520, 15, 52, "Input"],
Cell[36169, 1350, 109, 3, 31, "Input"],
Cell[36281, 1355, 110, 3, 31, "Input"],
Cell[CellGroupData[{
Cell[36416, 1362, 77, 1, 27, "Subsubsection",
 CellTags->"T1.2.1.1"],
Cell[36496, 1365, 503, 11, 101, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[37048, 1382, 84, 2, 71, "Section",
 CellTags->"T1.2.1"],
Cell[37135, 1386, 26, 0, 31, "Input"],
Cell[37164, 1388, 118, 3, 31, "Input"],
Cell[37285, 1393, 307, 6, 72, "Input"],
Cell[37595, 1401, 681, 11, 132, "Input"],
Cell[38279, 1414, 209, 5, 31, "Input"],
Cell[38491, 1421, 173, 5, 31, "Input"],
Cell[38667, 1428, 187, 5, 31, "Input"],
Cell[38857, 1435, 187, 5, 31, "Input"],
Cell[39047, 1442, 199, 5, 31, "Input"],
Cell[39249, 1449, 176, 5, 31, "Input"],
Cell[39428, 1456, 580, 15, 235, "Input"],
Cell[40011, 1473, 263, 6, 72, "Input"],
Cell[40277, 1481, 279, 6, 72, "Input"],
Cell[40559, 1489, 226, 5, 31, "Input"],
Cell[40788, 1496, 222, 5, 31, "Input"],
Cell[41013, 1503, 247, 6, 52, "Input"],
Cell[41263, 1511, 227, 5, 31, "Input"],
Cell[41493, 1518, 218, 5, 31, "Input"],
Cell[41714, 1525, 201, 5, 31, "Input"],
Cell[41918, 1532, 201, 5, 31, "Input"],
Cell[42122, 1539, 201, 5, 31, "Input"],
Cell[42326, 1546, 201, 5, 31, "Input"],
Cell[42530, 1553, 201, 5, 31, "Input"],
Cell[42734, 1560, 501, 9, 92, "Input"],
Cell[43238, 1571, 242, 5, 31, "Input"],
Cell[43483, 1578, 232, 5, 31, "Input"],
Cell[43718, 1585, 196, 5, 31, "Input"],
Cell[43917, 1592, 300, 6, 72, "Input"],
Cell[44220, 1600, 257, 6, 52, "Input"],
Cell[44480, 1608, 229, 5, 31, "Input"],
Cell[44712, 1615, 92, 2, 31, "Input"],
Cell[44807, 1619, 80, 2, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44924, 1626, 110, 2, 71, "Section",
 CellTags->"T1.3.1"],
Cell[45037, 1630, 999, 36, 29, "Text"],
Cell[46039, 1668, 125, 4, 31, "Input"],
Cell[46167, 1674, 253, 8, 52, "Input"],
Cell[46423, 1684, 73, 0, 29, "Text"],
Cell[46499, 1686, 85, 2, 31, "Input"],
Cell[46587, 1690, 115, 3, 29, "Text"],
Cell[46705, 1695, 82, 2, 31, "Input"],
Cell[46790, 1699, 719, 28, 29, "Text"],
Cell[CellGroupData[{
Cell[47534, 1731, 62, 1, 36, "Subsection",
 CellTags->"T1.3.1.1"],
Cell[47599, 1734, 106, 3, 31, "Input"],
Cell[47708, 1739, 632, 16, 52, "Input"],
Cell[48343, 1757, 660, 18, 54, "Input"],
Cell[49006, 1777, 418, 11, 31, "Input"],
Cell[49427, 1790, 115, 3, 29, "Text"],
Cell[49545, 1795, 551, 15, 31, "Input"],
Cell[50099, 1812, 425, 12, 31, "Input"],
Cell[50527, 1826, 514, 15, 31, "Input"],
Cell[51044, 1843, 570, 16, 31, "Input"],
Cell[51617, 1861, 330, 10, 31, "Input"],
Cell[51950, 1873, 412, 12, 31, "Input"],
Cell[52365, 1887, 466, 13, 31, "Input"],
Cell[52834, 1902, 454, 13, 31, "Input"],
Cell[53291, 1917, 513, 13, 133, "Input"],
Cell[53807, 1932, 271, 8, 31, "Input"],
Cell[54081, 1942, 271, 8, 31, "Input"],
Cell[54355, 1952, 271, 8, 31, "Input"],
Cell[54629, 1962, 271, 8, 31, "Input"],
Cell[54903, 1972, 271, 8, 31, "Input"],
Cell[55177, 1982, 174, 5, 31, "Input"],
Cell[55354, 1989, 227, 7, 31, "Input"],
Cell[55584, 1998, 340, 10, 31, "Input"],
Cell[55927, 2010, 630, 17, 31, "Input"],
Cell[56560, 2029, 420, 12, 31, "Input"],
Cell[56983, 2043, 513, 15, 31, "Input"],
Cell[57499, 2060, 340, 10, 31, "Input"],
Cell[57842, 2072, 420, 12, 31, "Input"],
Cell[58265, 2086, 630, 17, 31, "Input"],
Cell[58898, 2105, 1573, 42, 112, "Input"],
Cell[60474, 2149, 1836, 47, 112, "Input"],
Cell[62313, 2198, 1695, 48, 112, "Input"],
Cell[64011, 2248, 1940, 53, 92, "Input"],
Cell[65954, 2303, 629, 19, 31, "Input"],
Cell[66586, 2324, 1876, 50, 112, "Input"],
Cell[68465, 2376, 1709, 47, 92, "Input"],
Cell[70177, 2425, 1130, 32, 92, "Input"],
Cell[71310, 2459, 1709, 47, 92, "Input"],
Cell[73022, 2508, 573, 19, 31, "Input"],
Cell[73598, 2529, 1130, 32, 92, "Input"],
Cell[74731, 2563, 1454, 39, 112, "Input"],
Cell[76188, 2604, 511, 18, 31, "Input"],
Cell[76702, 2624, 1443, 39, 112, "Input"],
Cell[78148, 2665, 403, 13, 31, "Input"],
Cell[78554, 2680, 107, 3, 47, "Text"],
Cell[78664, 2685, 1723, 45, 112, "Input"],
Cell[80390, 2732, 1723, 45, 112, "Input"],
Cell[82116, 2779, 1698, 48, 112, "Input"],
Cell[83817, 2829, 1945, 53, 92, "Input"],
Cell[85765, 2884, 1881, 50, 112, "Input"],
Cell[87649, 2936, 1714, 47, 92, "Input"],
Cell[89366, 2985, 1132, 32, 92, "Input"],
Cell[90501, 3019, 1457, 39, 112, "Input"],
Cell[91961, 3060, 1446, 39, 112, "Input"],
Cell[93410, 3101, 1720, 45, 112, "Input"],
Cell[95133, 3148, 1720, 45, 112, "Input"],
Cell[96856, 3195, 1695, 48, 112, "Input"],
Cell[98554, 3245, 1940, 53, 92, "Input"],
Cell[100497, 3300, 1876, 50, 112, "Input"],
Cell[102376, 3352, 1709, 47, 92, "Input"],
Cell[104088, 3401, 1130, 32, 92, "Input"],
Cell[105221, 3435, 1454, 39, 112, "Input"],
Cell[106678, 3476, 1443, 39, 112, "Input"],
Cell[108124, 3517, 234, 6, 31, "Input"],
Cell[108361, 3525, 234, 6, 31, "Input"],
Cell[108598, 3533, 234, 6, 31, "Input"],
Cell[108835, 3541, 234, 6, 31, "Input"],
Cell[109072, 3549, 234, 6, 31, "Input"],
Cell[109309, 3557, 314, 10, 33, "Input"],
Cell[109626, 3569, 266, 8, 31, "Input"],
Cell[109895, 3579, 415, 11, 31, "Input"],
Cell[110313, 3592, 257, 7, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[110607, 3604, 57, 1, 36, "Subsection",
 CellTags->"T1.3.2.1"],
Cell[110667, 3607, 14919, 337, 1294, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[125623, 3949, 58, 1, 36, "Subsection",
 CellTags->"T1.3.3.1"],
Cell[125684, 3952, 2105, 57, 160, "Input"],
Cell[127792, 4011, 2105, 57, 160, "Input"],
Cell[129900, 4070, 2105, 57, 160, "Input"],
Cell[132008, 4129, 2105, 57, 160, "Input"],
Cell[134116, 4188, 2105, 57, 160, "Input"],
Cell[136224, 4247, 2105, 57, 160, "Input"],
Cell[138332, 4306, 2105, 57, 160, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[140474, 4368, 59, 1, 36, "Subsection",
 CellTags->"T1.3.4.1"],
Cell[140536, 4371, 2266, 68, 112, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[142839, 4444, 53, 1, 36, "Subsection",
 CellTags->"T1.3.5.1"],
Cell[142895, 4447, 129, 3, 48, "Input"],
Cell[143027, 4452, 8160, 199, 452, "Input"],
Cell[151190, 4653, 512, 14, 52, "Input"],
Cell[151705, 4669, 662, 25, 29, "Text"],
Cell[152370, 4696, 7327, 197, 359, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[159734, 4898, 58, 1, 36, "Subsection",
 CellTags->"T1.3.6.1"],
Cell[159795, 4901, 1699, 48, 132, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[161531, 4954, 49, 1, 36, "Subsection",
 CellTags->"T1.3.7.1"],
Cell[161583, 4957, 202, 5, 31, "Input"],
Cell[CellGroupData[{
Cell[161810, 4966, 56, 1, 28, "Subsubsection",
 CellTags->"T1.3.7.1.1"],
Cell[161869, 4969, 738, 28, 33, "Text"],
Cell[162610, 4999, 454, 12, 31, "Input"],
Cell[163067, 5013, 342, 10, 31, "Input"],
Cell[163412, 5025, 344, 10, 31, "Input"],
Cell[163759, 5037, 348, 10, 31, "Input"],
Cell[164110, 5049, 338, 10, 31, "Input"],
Cell[164451, 5061, 346, 10, 31, "Input"],
Cell[164800, 5073, 336, 10, 31, "Input"],
Cell[165139, 5085, 336, 10, 31, "Input"],
Cell[165478, 5097, 344, 10, 31, "Input"],
Cell[165825, 5109, 340, 10, 31, "Input"],
Cell[166168, 5121, 310, 8, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[166515, 5134, 52, 1, 27, "Subsubsection",
 CellTags->"T1.3.7.2.1"],
Cell[166570, 5137, 153, 5, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[166760, 5147, 53, 1, 27, "Subsubsection",
 CellTags->"T1.3.7.3.1"],
Cell[166816, 5150, 101, 3, 31, "Input"],
Cell[166920, 5155, 154, 5, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[167111, 5165, 57, 1, 27, "Subsubsection",
 CellTags->"T1.3.7.4.1"],
Cell[167171, 5168, 177, 5, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[167385, 5178, 53, 1, 27, "Subsubsection",
 CellTags->"T1.3.7.5.1"],
Cell[167441, 5181, 3526, 79, 1340, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[171004, 5265, 53, 1, 27, "Subsubsection",
 CellTags->"T1.3.7.6.1"],
Cell[171060, 5268, 1562, 49, 72, "Input"],
Cell[172625, 5319, 311, 10, 31, "Input"],
Cell[172939, 5331, 341, 11, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[173317, 5347, 53, 1, 27, "Subsubsection",
 CellTags->"T1.3.7.7.1"],
Cell[173373, 5350, 1358, 45, 52, "Input"],
Cell[174734, 5397, 276, 9, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[175047, 5411, 53, 1, 27, "Subsubsection",
 CellTags->"T1.3.7.8.1"],
Cell[175103, 5414, 283, 10, 31, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[175447, 5431, 102, 2, 71, "Section",
 CellTags->"T1.4.1"],
Cell[175552, 5435, 810, 25, 87, "Input"],
Cell[176365, 5462, 2976, 89, 229, "Input"],
Cell[179344, 5553, 182, 6, 32, "Text"],
Cell[179529, 5561, 435, 13, 33, "Input"],
Cell[179967, 5576, 95, 2, 31, "Input"],
Cell[CellGroupData[{
Cell[180087, 5582, 108, 3, 37, "Subsection",
 CellTags->"T1.4.1.1"],
Cell[180198, 5587, 224, 7, 31, "Input"],
Cell[180425, 5596, 128, 3, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[180590, 5604, 106, 3, 37, "Subsection",
 CellTags->"T1.4.2.1"],
Cell[180699, 5609, 147, 7, 36, "Text"],
Cell[180849, 5618, 1837, 63, 54, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[182723, 5686, 196, 6, 37, "Subsection",
 CellTags->"T1.4.3.1"],
Cell[182922, 5694, 2244, 71, 55, "Input"],
Cell[185169, 5767, 179, 5, 31, "Input"],
Cell[185351, 5774, 2260, 68, 76, "Input"],
Cell[187614, 5844, 1307, 40, 55, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[188958, 5889, 233, 7, 51, "Subsection",
 CellTags->"T1.4.4.1"],
Cell[189194, 5898, 2949, 92, 79, "Input"],
Cell[192146, 5992, 2791, 91, 74, "Input"],
Cell[194940, 6085, 2791, 91, 74, "Input"],
Cell[197734, 6178, 2791, 91, 74, "Input"],
Cell[200528, 6271, 179, 5, 31, "Input"],
Cell[200710, 6278, 179, 5, 31, "Input"],
Cell[200892, 6285, 165, 5, 31, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[201106, 6296, 95, 2, 71, "Section",
 CellTags->"T1.5.1"],
Cell[CellGroupData[{
Cell[201226, 6302, 64, 1, 36, "Subsection",
 CellTags->"T1.5.1.1"],
Cell[201293, 6305, 131, 3, 47, "Text"],
Cell[201427, 6310, 67, 0, 29, "Text"],
Cell[201497, 6312, 226, 8, 32, "Text"],
Cell[201726, 6322, 3442, 89, 121, "Input"],
Cell[205171, 6413, 3420, 86, 98, "Input"],
Cell[208594, 6501, 275, 8, 31, "Input"],
Cell[208872, 6511, 249, 7, 31, "Input"],
Cell[209124, 6520, 310, 9, 31, "Input"],
Cell[209437, 6531, 283, 8, 31, "Input"],
Cell[209723, 6541, 249, 7, 31, "Input"],
Cell[209975, 6550, 325, 9, 31, "Input"],
Cell[210303, 6561, 182, 6, 40, "Input"],
Cell[210488, 6569, 59, 0, 31, "Input"],
Cell[210550, 6571, 233, 7, 31, "Input"],
Cell[210786, 6580, 233, 7, 31, "Input"],
Cell[211022, 6589, 234, 7, 31, "Input"],
Cell[211259, 6598, 261, 7, 31, "Input"],
Cell[211523, 6607, 235, 7, 31, "Input"],
Cell[211761, 6616, 235, 7, 31, "Input"],
Cell[211999, 6625, 2167, 65, 72, "Input"],
Cell[214169, 6692, 81, 2, 31, "Input"],
Cell[214253, 6696, 610, 18, 31, "Input"],
Cell[214866, 6716, 186, 5, 31, "Input"],
Cell[215055, 6723, 76, 0, 29, "Text"],
Cell[215134, 6725, 173, 6, 31, "Input"],
Cell[215310, 6733, 134, 4, 31, "Input"],
Cell[215447, 6739, 9844, 243, 322, "Input"],
Cell[225294, 6984, 9528, 235, 343, "Input"],
Cell[234825, 7221, 12850, 315, 363, "Input"],
Cell[247678, 7538, 11337, 282, 351, "Input"],
Cell[259018, 7822, 12420, 304, 405, "Input"],
Cell[271441, 8128, 119, 3, 31, "Input"],
Cell[271563, 8133, 1389, 43, 72, "Input"],
Cell[272955, 8178, 1014, 31, 52, "Input"],
Cell[273972, 8211, 1014, 31, 52, "Input"],
Cell[CellGroupData[{
Cell[275011, 8246, 61, 1, 27, "Subsubsection",
 CellTags->"T1.5.1.1.1"],
Cell[275075, 8249, 4767, 148, 138, "Input"],
Cell[279845, 8399, 370, 11, 31, "Input"],
Cell[280218, 8412, 355, 11, 31, "Input"],
Cell[280576, 8425, 420, 14, 31, "Input"],
Cell[280999, 8441, 496, 13, 31, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[281544, 8460, 58, 1, 36, "Subsection",
 CellTags->"T1.5.2.1"],
Cell[281605, 8463, 4291, 127, 258, "Input"],
Cell[CellGroupData[{
Cell[285921, 8594, 52, 1, 27, "Subsubsection",
 CellTags->"T1.5.2.1.1"],
Cell[285976, 8597, 367, 11, 31, "Input"],
Cell[286346, 8610, 393, 12, 31, "Input"],
Cell[286742, 8624, 330, 11, 31, "Input"],
Cell[287075, 8637, 239, 10, 36, "Text"],
Cell[287317, 8649, 20, 0, 29, "Text"],
Cell[287340, 8651, 3284, 95, 98, "Input"],
Cell[290627, 8748, 20, 0, 29, "Text"],
Cell[290650, 8750, 2974, 87, 98, "Input"],
Cell[293627, 8839, 20, 0, 29, "Text"],
Cell[293650, 8841, 1741, 53, 54, "Input"],
Cell[295394, 8896, 20, 0, 29, "Text"],
Cell[295417, 8898, 1475, 44, 67, "Input"],
Cell[296895, 8944, 20, 0, 29, "Text"],
Cell[296918, 8946, 2177, 62, 94, "Input"],
Cell[299098, 9010, 20, 0, 29, "Text"],
Cell[299121, 9012, 2161, 61, 98, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[301331, 9079, 59, 1, 36, "Subsection",
 CellTags->"T1.5.3.1"],
Cell[301393, 9082, 174, 3, 29, "Text"],
Cell[301570, 9087, 139, 3, 47, "Text"],
Cell[CellGroupData[{
Cell[301734, 9094, 67, 1, 27, "Subsubsection",
 CellTags->"T1.5.3.1.1"],
Cell[301804, 9097, 1210, 40, 76, "Input"],
Cell[303017, 9139, 2525, 76, 165, "Input"],
Cell[305545, 9217, 6431, 163, 409, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[312013, 9385, 67, 1, 27, "Subsubsection",
 CellTags->"T1.5.3.2.1"],
Cell[312083, 9388, 1018, 34, 76, "Input"],
Cell[313104, 9424, 854, 26, 52, "Input"],
Cell[313961, 9452, 92, 2, 31, "Input"],
Cell[314056, 9456, 6187, 157, 429, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[320304, 9620, 138, 2, 71, "Section",
 CellTags->"T1.6.1"],
Cell[320445, 9624, 793, 26, 31, "Input"],
Cell[321241, 9652, 298, 9, 47, "Text"],
Cell[CellGroupData[{
Cell[321564, 9665, 55, 1, 37, "Subsection",
 CellTags->"T1.6.1.1"],
Cell[321622, 9668, 92, 4, 29, "Text"],
Cell[321717, 9674, 6571, 217, 78, "Input"],
Cell[328291, 9893, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[328465, 9902, 57, 1, 36, "Subsection",
 CellTags->"T1.6.2.1"],
Cell[328525, 9905, 255, 9, 31, "Input"],
Cell[CellGroupData[{
Cell[328805, 9918, 167, 7, 27, "Subsubsection",
 CellTags->"T1.6.2.1.1"],
Cell[328975, 9927, 1063, 37, 44, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330075, 9969, 167, 7, 27, "Subsubsection",
 CellTags->"T1.6.2.2.1"],
Cell[330245, 9978, 1232, 42, 44, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[331514, 10025, 167, 7, 27, "Subsubsection",
 CellTags->"T1.6.2.3.1"],
Cell[331684, 10034, 1232, 42, 44, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[332953, 10081, 167, 7, 27, "Subsubsection",
 CellTags->"T1.6.2.4.1"],
Cell[333123, 10090, 1232, 42, 44, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[334404, 10138, 57, 1, 36, "Subsection",
 CellTags->"T1.6.3.1"],
Cell[334464, 10141, 162, 4, 31, "Input"],
Cell[334629, 10147, 377, 12, 31, "Text"],
Cell[335009, 10161, 3094, 113, 35, "Input"],
Cell[338106, 10276, 141, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[338284, 10285, 246, 9, 41, "Subsection",
 CellTags->"T1.6.4.1"],
Cell[338533, 10296, 99, 3, 31, "Input"],
Cell[338635, 10301, 539, 17, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[339211, 10323, 57, 1, 36, "Subsection",
 CellTags->"T1.6.5.1"],
Cell[339271, 10326, 162, 4, 31, "Input"],
Cell[339436, 10332, 377, 12, 31, "Text"],
Cell[339816, 10346, 3094, 113, 35, "Input"],
Cell[342913, 10461, 141, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[343091, 10470, 246, 9, 41, "Subsection",
 CellTags->"T1.6.6.1"],
Cell[343340, 10481, 99, 3, 31, "Input"],
Cell[343442, 10486, 539, 17, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[344018, 10508, 246, 9, 41, "Subsection",
 CellTags->"T1.6.7.1"],
Cell[344267, 10519, 130, 3, 31, "Input"],
Cell[344400, 10524, 260, 7, 31, "Input"],
Cell[344663, 10533, 539, 17, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[345239, 10555, 246, 9, 41, "Subsection",
 CellTags->"T1.6.8.1"],
Cell[345488, 10566, 130, 3, 31, "Input"],
Cell[345621, 10571, 260, 7, 31, "Input"],
Cell[345884, 10580, 539, 17, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[346460, 10602, 55, 1, 37, "Subsection",
 CellTags->"T1.6.9.1"],
Cell[346518, 10605, 92, 4, 29, "Text"],
Cell[346613, 10611, 6063, 199, 78, "Input"],
Cell[352679, 10812, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[352853, 10821, 58, 1, 36, "Subsection",
 CellTags->"T1.6.10.1"],
Cell[352914, 10824, 232, 8, 31, "Input"],
Cell[353149, 10834, 1059, 37, 44, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[354245, 10876, 56, 1, 37, "Subsection",
 CellTags->"T1.6.11.1"],
Cell[354304, 10879, 624, 26, 31, "Text"],
Cell[354931, 10907, 3309, 120, 35, "Input"],
Cell[358243, 11029, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[358417, 11038, 56, 1, 37, "Subsection",
 CellTags->"T1.6.12.1"],
Cell[358476, 11041, 624, 26, 31, "Text"],
Cell[359103, 11069, 3309, 120, 35, "Input"],
Cell[362415, 11191, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[362589, 11200, 56, 1, 36, "Subsection",
 CellTags->"T1.6.13.1"],
Cell[362648, 11203, 6446, 215, 58, "Input"],
Cell[369097, 11420, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[369271, 11429, 67, 1, 36, "Subsection",
 CellTags->"T1.6.14.1"],
Cell[369341, 11432, 662, 21, 31, "Input"],
Cell[370006, 11455, 326, 10, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[370369, 11470, 58, 1, 36, "Subsection",
 CellTags->"T1.6.15.1"],
Cell[370430, 11473, 22654, 536, 9092, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[393121, 12014, 56, 1, 36, "Subsection",
 CellTags->"T1.6.16.1"],
Cell[393180, 12017, 92, 4, 29, "Text"],
Cell[393275, 12023, 3736, 132, 35, "Input"],
Cell[397014, 12157, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[397188, 12166, 58, 1, 36, "Subsection",
 CellTags->"T1.6.17.1"],
Cell[397249, 12169, 49, 1, 31, "Input"],
Cell[397301, 12172, 232, 8, 31, "Input"],
Cell[397536, 12182, 514, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[398075, 12203, 93, 2, 31, "Input"],
Cell[398171, 12207, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[398285, 12214, 56, 1, 36, "Subsection",
 CellTags->"T1.6.18.1"],
Cell[398344, 12217, 92, 4, 29, "Text"],
Cell[398439, 12223, 5243, 183, 58, "Input"],
Cell[403685, 12408, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[403859, 12417, 58, 1, 36, "Subsection",
 CellTags->"T1.6.19.1"],
Cell[403920, 12420, 274, 9, 31, "Input"],
Cell[404197, 12431, 514, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[404736, 12452, 93, 2, 31, "Input"],
Cell[404832, 12456, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[404946, 12463, 58, 1, 36, "Subsection",
 CellTags->"T1.6.20.1"],
Cell[405007, 12466, 340, 11, 31, "Input"],
Cell[405350, 12479, 514, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[405889, 12500, 93, 2, 31, "Input"],
Cell[405985, 12504, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[406099, 12511, 56, 1, 36, "Subsection",
 CellTags->"T1.6.21.1"],
Cell[406158, 12514, 92, 4, 29, "Text"],
Cell[406253, 12520, 2961, 107, 58, "Input"],
Cell[409217, 12629, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[409391, 12638, 58, 1, 36, "Subsection",
 CellTags->"T1.6.22.1"],
Cell[409452, 12641, 232, 8, 31, "Input"],
Cell[409687, 12651, 441, 14, 31, "Input"],
Cell[CellGroupData[{
Cell[410153, 12669, 93, 2, 31, "Input"],
Cell[410249, 12673, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[410363, 12680, 56, 1, 36, "Subsection",
 CellTags->"T1.6.23.1"],
Cell[410422, 12683, 92, 4, 29, "Text"],
Cell[410517, 12689, 3700, 132, 58, "Input"],
Cell[414220, 12823, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[414394, 12832, 58, 1, 36, "Subsection",
 CellTags->"T1.6.24.1"],
Cell[414455, 12835, 232, 8, 31, "Input"],
Cell[414690, 12845, 515, 16, 31, "Input"],
Cell[CellGroupData[{
Cell[415230, 12865, 93, 2, 31, "Input"],
Cell[415326, 12869, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[415440, 12876, 56, 1, 45, "Subsection",
 CellTags->"T1.6.25.1"],
Cell[415499, 12879, 92, 4, 29, "Text"],
Cell[415594, 12885, 3457, 123, 58, "Input"],
Cell[419054, 13010, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[419228, 13019, 58, 1, 36, "Subsection",
 CellTags->"T1.6.26.1"],
Cell[419289, 13022, 232, 8, 31, "Input"],
Cell[419524, 13032, 517, 16, 31, "Input"],
Cell[CellGroupData[{
Cell[420066, 13052, 93, 2, 31, "Input"],
Cell[420162, 13056, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[420276, 13063, 56, 1, 37, "Subsection",
 CellTags->"T1.6.27.1"],
Cell[420335, 13066, 230, 9, 31, "Text"],
Cell[420568, 13077, 5729, 194, 101, "Input"],
Cell[426300, 13273, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[426474, 13282, 58, 1, 36, "Subsection",
 CellTags->"T1.6.28.1"],
Cell[426535, 13285, 95, 3, 31, "Input"],
Cell[426633, 13290, 1867, 49, 98, "Input"],
Cell[428503, 13341, 50, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[428590, 13346, 57, 1, 37, "Subsection",
 CellTags->"T1.6.29.1"],
Cell[428650, 13349, 92, 4, 29, "Text"],
Cell[428745, 13355, 2961, 107, 58, "Input"],
Cell[431709, 13464, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[431883, 13473, 58, 1, 36, "Subsection",
 CellTags->"T1.6.30.1"],
Cell[431944, 13476, 232, 8, 31, "Input"],
Cell[432179, 13486, 448, 14, 31, "Input"],
Cell[CellGroupData[{
Cell[432652, 13504, 105, 2, 31, "Input"],
Cell[432760, 13508, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[432874, 13515, 57, 1, 37, "Subsection",
 CellTags->"T1.6.31.1"],
Cell[432934, 13518, 382, 16, 31, "Text"],
Cell[433319, 13536, 2278, 86, 35, "Input"],
Cell[435600, 13624, 137, 4, 31, "Input"],
Cell[435740, 13630, 2028, 76, 58, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[437805, 13711, 58, 1, 37, "Subsection",
 CellTags->"T1.6.32.1"],
Cell[437866, 13714, 381, 16, 31, "Text"],
Cell[438250, 13732, 95, 3, 31, "Input"],
Cell[438348, 13737, 117, 3, 24, "SmallText"],
Cell[438468, 13742, 725, 26, 24, "SmallText"],
Cell[439196, 13770, 602, 22, 31, "Input"],
Cell[439801, 13794, 50, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[439888, 13799, 60, 1, 37, "Subsection",
 CellTags->"T1.6.33.1"],
Cell[439951, 13802, 381, 16, 31, "Text"],
Cell[440335, 13820, 202, 6, 31, "Input"],
Cell[440540, 13828, 117, 3, 24, "SmallText"],
Cell[440660, 13833, 731, 26, 24, "SmallText"],
Cell[441394, 13861, 608, 22, 31, "Input"],
Cell[442005, 13885, 50, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[442092, 13890, 57, 1, 36, "Subsection",
 CellTags->"T1.6.34.1"],
Cell[442152, 13893, 512, 21, 49, "Text"],
Cell[442667, 13916, 4989, 173, 102, "Input"],
Cell[447659, 14091, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447833, 14100, 58, 1, 36, "Subsection",
 CellTags->"T1.6.35.1"],
Cell[447894, 14103, 274, 9, 31, "Input"],
Cell[448171, 14114, 1136, 39, 33, "Input"],
Cell[449310, 14155, 50, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[449397, 14160, 57, 1, 36, "Subsection",
 CellTags->"T1.6.36.1"],
Cell[449457, 14163, 277, 10, 49, "Text"],
Cell[449737, 14175, 6180, 202, 80, "Input"],
Cell[455920, 14379, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[456094, 14388, 58, 1, 36, "Subsection",
 CellTags->"T1.6.37.1"],
Cell[456155, 14391, 246, 10, 31, "Text"],
Cell[456404, 14403, 136, 4, 31, "Input"],
Cell[456543, 14409, 782, 25, 31, "Input"],
Cell[CellGroupData[{
Cell[457350, 14438, 93, 2, 31, "Input"],
Cell[457446, 14442, 65, 1, 23, "Print"]
}, Open  ]],
Cell[457526, 14446, 880, 28, 31, "Input"],
Cell[458409, 14476, 106, 5, 83, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[458552, 14486, 60, 1, 36, "Subsection",
 CellTags->"T1.6.38.1"],
Cell[458615, 14489, 263, 8, 31, "Input"],
Cell[458881, 14499, 596, 21, 31, "Input"],
Cell[459480, 14522, 80, 2, 31, "Input"],
Cell[459563, 14526, 172, 5, 31, "Input"],
Cell[459738, 14533, 126, 4, 31, "Input"],
Cell[459867, 14539, 479, 17, 31, "Input"],
Cell[460349, 14558, 1204, 37, 33, "Input"],
Cell[461556, 14597, 50, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[461643, 14602, 60, 1, 36, "Subsection",
 CellTags->"T1.6.39.1"],
Cell[461706, 14605, 263, 8, 31, "Input"],
Cell[461972, 14615, 141, 4, 31, "Input"],
Cell[462116, 14621, 126, 4, 31, "Input"],
Cell[462245, 14627, 479, 17, 31, "Input"],
Cell[462727, 14646, 1128, 33, 33, "Input"],
Cell[463858, 14681, 50, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[463945, 14686, 60, 1, 36, "Subsection",
 CellTags->"T1.6.40.1"],
Cell[464008, 14689, 447, 15, 31, "Input"],
Cell[464458, 14706, 479, 17, 31, "Input"],
Cell[464940, 14725, 576, 18, 31, "Input"],
Cell[465519, 14745, 50, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[465606, 14750, 57, 1, 36, "Subsection",
 CellTags->"T1.6.41.1"],
Cell[465666, 14753, 92, 4, 29, "Text"],
Cell[465761, 14759, 4921, 172, 58, "Input"],
Cell[470685, 14933, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[470859, 14942, 58, 1, 36, "Subsection",
 CellTags->"T1.6.42.1"],
Cell[470920, 14945, 588, 20, 31, "Input"],
Cell[471511, 14967, 728, 23, 31, "Input"],
Cell[CellGroupData[{
Cell[472264, 14994, 93, 2, 31, "Input"],
Cell[472360, 14998, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[472474, 15005, 67, 1, 37, "Subsection",
 CellTags->"T1.6.43.1"],
Cell[472544, 15008, 111, 3, 29, "Text"],
Cell[472658, 15013, 92, 4, 29, "Text"],
Cell[472753, 15019, 5657, 200, 58, "Input"],
Cell[478413, 15221, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[478587, 15230, 56, 1, 37, "Subsection",
 CellTags->"T1.6.44.1"],
Cell[478646, 15233, 92, 4, 29, "Text"],
Cell[478741, 15239, 5038, 179, 58, "Input"],
Cell[483782, 15420, 8601, 306, 58, "Text"],
Cell[492386, 15728, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[492560, 15737, 58, 1, 36, "Subsection",
 CellTags->"T1.6.45.1"],
Cell[492621, 15740, 463, 16, 31, "Input"],
Cell[493087, 15758, 659, 21, 31, "Input"],
Cell[CellGroupData[{
Cell[493771, 15783, 93, 2, 31, "Input"],
Cell[493867, 15787, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[493981, 15794, 60, 1, 36, "Subsection",
 CellTags->"T1.6.46.1"],
Cell[494044, 15797, 573, 19, 31, "Input"],
Cell[494620, 15818, 665, 21, 31, "Input"],
Cell[CellGroupData[{
Cell[495310, 15843, 95, 2, 31, "Input"],
Cell[495408, 15847, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[495522, 15854, 56, 1, 37, "Subsection",
 CellTags->"T1.6.47.1"],
Cell[495581, 15857, 92, 4, 29, "Text"],
Cell[495676, 15863, 6060, 203, 80, "Input"],
Cell[501739, 16068, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[501913, 16077, 58, 1, 36, "Subsection",
 CellTags->"T1.6.48.1"],
Cell[501974, 16080, 463, 16, 31, "Input"],
Cell[502440, 16098, 700, 22, 31, "Input"],
Cell[CellGroupData[{
Cell[503165, 16124, 93, 2, 31, "Input"],
Cell[503261, 16128, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[503375, 16135, 60, 1, 36, "Subsection",
 CellTags->"T1.6.49.1"],
Cell[503438, 16138, 573, 19, 31, "Input"],
Cell[504014, 16159, 725, 23, 31, "Input"],
Cell[CellGroupData[{
Cell[504764, 16186, 95, 2, 31, "Input"],
Cell[504862, 16190, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[504976, 16197, 56, 1, 36, "Subsection",
 CellTags->"T1.6.50.1"],
Cell[505035, 16200, 92, 4, 29, "Text"],
Cell[505130, 16206, 11843, 400, 175, "Input"],
Cell[516976, 16608, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[517150, 16617, 58, 1, 36, "Subsection",
 CellTags->"T1.6.51.1"],
Cell[517211, 16620, 95, 3, 31, "Input"],
Cell[517309, 16625, 591, 19, 31, "Input"],
Cell[517903, 16646, 104, 3, 47, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[518044, 16654, 56, 1, 36, "Subsection",
 CellTags->"T1.6.52.1"],
Cell[518103, 16657, 258, 11, 29, "Text"],
Cell[518364, 16670, 5262, 182, 90, "Input"],
Cell[523629, 16854, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[523803, 16863, 58, 1, 36, "Subsection",
 CellTags->"T1.6.53.1"],
Cell[523864, 16866, 407, 14, 31, "Input"],
Cell[524274, 16882, 461, 15, 31, "Input"],
Cell[CellGroupData[{
Cell[524760, 16901, 93, 2, 31, "Input"],
Cell[524856, 16905, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[524970, 16912, 60, 1, 36, "Subsection",
 CellTags->"T1.6.54.1"],
Cell[525033, 16915, 477, 16, 31, "Input"],
Cell[525513, 16933, 467, 15, 31, "Input"],
Cell[CellGroupData[{
Cell[526005, 16952, 95, 2, 31, "Input"],
Cell[526103, 16956, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[526217, 16963, 60, 1, 36, "Subsection",
 CellTags->"T1.6.55.1"],
Cell[526280, 16966, 477, 16, 31, "Input"],
Cell[526760, 16984, 467, 15, 31, "Input"],
Cell[CellGroupData[{
Cell[527252, 17003, 95, 2, 31, "Input"],
Cell[527350, 17007, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[527464, 17014, 56, 1, 37, "Subsection",
 CellTags->"T1.6.56.1"],
Cell[527523, 17017, 204, 9, 29, "Text"],
Cell[527730, 17028, 6020, 205, 80, "Input"],
Cell[533753, 17235, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[533927, 17244, 58, 1, 36, "Subsection",
 CellTags->"T1.6.57.1"],
Cell[533988, 17247, 269, 9, 31, "Input"],
Cell[534260, 17258, 453, 15, 31, "Input"],
Cell[CellGroupData[{
Cell[534738, 17277, 93, 2, 31, "Input"],
Cell[534834, 17281, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[534948, 17288, 56, 1, 37, "Subsection",
 CellTags->"T1.6.58.1"],
Cell[535007, 17291, 216, 9, 29, "Text"],
Cell[535226, 17302, 5082, 166, 102, "Input"],
Cell[540311, 17470, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[540485, 17479, 58, 1, 36, "Subsection",
 CellTags->"T1.6.59.1"],
Cell[540546, 17482, 274, 9, 31, "Input"],
Cell[540823, 17493, 439, 13, 31, "Input"],
Cell[CellGroupData[{
Cell[541287, 17510, 93, 2, 31, "Input"],
Cell[541383, 17514, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[541497, 17521, 56, 1, 45, "Subsection",
 CellTags->"T1.6.60.1"],
Cell[541556, 17524, 257, 11, 29, "Text"],
Cell[541816, 17537, 5303, 181, 80, "Input"],
Cell[547122, 17720, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[547296, 17729, 58, 1, 36, "Subsection",
 CellTags->"T1.6.61.1"],
Cell[547357, 17732, 493, 17, 31, "Input"],
Cell[547853, 17751, 551, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[548429, 17772, 93, 2, 31, "Input"],
Cell[548525, 17776, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[548639, 17783, 60, 1, 36, "Subsection",
 CellTags->"T1.6.62.1"],
Cell[548702, 17786, 565, 19, 31, "Input"],
Cell[549270, 17807, 557, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[549852, 17828, 95, 2, 31, "Input"],
Cell[549950, 17832, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[550064, 17839, 60, 1, 36, "Subsection",
 CellTags->"T1.6.63.1"],
Cell[550127, 17842, 580, 19, 31, "Input"],
Cell[550710, 17863, 557, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[551292, 17884, 95, 2, 31, "Input"],
Cell[551390, 17888, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[551504, 17895, 56, 1, 45, "Subsection",
 CellTags->"T1.6.64.1"],
Cell[551563, 17898, 203, 9, 29, "Text"],
Cell[551769, 17909, 12433, 402, 173, "Input"],
Cell[564205, 18313, 137, 4, 31, "Input"],
Cell[564345, 18319, 124, 4, 31, "Input"],
Cell[564472, 18325, 511, 18, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[565020, 18348, 58, 1, 36, "Subsection",
 CellTags->"T1.6.65.1"],
Cell[565081, 18351, 350, 12, 31, "Input"],
Cell[565434, 18365, 544, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[566003, 18386, 93, 2, 31, "Input"],
Cell[566099, 18390, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[566213, 18397, 60, 1, 36, "Subsection",
 CellTags->"T1.6.66.1"],
Cell[566276, 18400, 420, 14, 31, "Input"],
Cell[566699, 18416, 548, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[567272, 18437, 95, 2, 31, "Input"],
Cell[567370, 18441, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[567484, 18448, 60, 1, 36, "Subsection",
 CellTags->"T1.6.67.1"],
Cell[567547, 18451, 420, 14, 31, "Input"],
Cell[567970, 18467, 550, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[568545, 18488, 95, 2, 31, "Input"],
Cell[568643, 18492, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[568757, 18499, 56, 1, 45, "Subsection",
 CellTags->"T1.6.68.1"],
Cell[568816, 18502, 216, 9, 29, "Text"],
Cell[569035, 18513, 7673, 251, 102, "Input"],
Cell[576711, 18766, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[576885, 18775, 58, 1, 36, "Subsection",
 CellTags->"T1.6.69.1"],
Cell[576946, 18778, 350, 12, 31, "Input"],
Cell[577299, 18792, 515, 15, 31, "Input"],
Cell[CellGroupData[{
Cell[577839, 18811, 93, 2, 31, "Input"],
Cell[577935, 18815, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[578049, 18822, 56, 1, 45, "Subsection",
 CellTags->"T1.6.70.1"],
Cell[CellGroupData[{
Cell[578130, 18827, 55, 1, 27, "Subsubsection",
 CellTags->"T1.6.70.1.1"],
Cell[578188, 18830, 226, 7, 31, "Input"],
Cell[578417, 18839, 688, 22, 33, "Input"],
Cell[579108, 18863, 957, 29, 33, "Input"],
Cell[580068, 18894, 1648, 56, 33, "Input"],
Cell[581719, 18952, 294, 9, 33, "Input"],
Cell[582016, 18963, 294, 9, 33, "Input"],
Cell[582313, 18974, 294, 9, 33, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[582644, 18988, 55, 1, 27, "Subsubsection",
 CellTags->"T1.6.70.2.1"],
Cell[582702, 18991, 1446, 50, 47, "Input"],
Cell[584151, 19043, 294, 9, 33, "Input"],
Cell[584448, 19054, 294, 9, 33, "Input"],
Cell[584745, 19065, 294, 9, 33, "Input"],
Cell[585042, 19076, 81, 2, 31, "Input"],
Cell[585126, 19080, 99, 5, 29, "Text"],
Cell[585228, 19087, 5993, 211, 102, "Input"],
Cell[591224, 19300, 137, 4, 31, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[591410, 19310, 58, 1, 36, "Subsection",
 CellTags->"T1.6.71.1"],
Cell[591471, 19313, 95, 3, 31, "Input"],
Cell[591569, 19318, 443, 14, 31, "Input"],
Cell[592015, 19334, 49, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[592101, 19339, 56, 1, 37, "Subsection",
 CellTags->"T1.6.72.1"],
Cell[592160, 19342, 99, 5, 29, "Text"],
Cell[592262, 19349, 11665, 369, 224, "Input"],
Cell[603930, 19720, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[604104, 19729, 58, 1, 36, "Subsection",
 CellTags->"T1.6.73.1"],
Cell[604165, 19732, 115, 4, 31, "Input"],
Cell[604283, 19738, 413, 13, 31, "Input"],
Cell[604699, 19753, 49, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[604785, 19758, 56, 1, 36, "Subsection",
 CellTags->"T1.6.74.1"],
Cell[604844, 19761, 92, 4, 29, "Text"],
Cell[604939, 19767, 2266, 85, 35, "Input"],
Cell[607208, 19854, 141, 4, 31, "Input"],
Cell[607352, 19860, 2281, 85, 35, "Input"],
Cell[609636, 19947, 141, 4, 31, "Input"],
Cell[609780, 19953, 2582, 92, 35, "Input"],
Cell[612365, 20047, 141, 4, 31, "Input"],
Cell[612509, 20053, 3514, 124, 35, "Input"],
Cell[616026, 20179, 141, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[616204, 20188, 59, 1, 36, "Subsection",
 CellTags->"T1.6.75.1"],
Cell[616266, 20191, 230, 8, 31, "Input"],
Cell[616499, 20201, 444, 14, 31, "Input"],
Cell[616946, 20217, 49, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[617032, 20222, 59, 1, 36, "Subsection",
 CellTags->"T1.6.76.1"],
Cell[617094, 20225, 367, 13, 31, "Input"],
Cell[617464, 20240, 513, 16, 31, "Input"],
Cell[617980, 20258, 49, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[618066, 20263, 59, 1, 36, "Subsection",
 CellTags->"T1.6.77.1"],
Cell[618128, 20266, 98, 3, 31, "Input"],
Cell[618229, 20271, 549, 16, 31, "Input"],
Cell[618781, 20289, 49, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[618867, 20294, 59, 1, 36, "Subsection",
 CellTags->"T1.6.78.1"],
Cell[618929, 20297, 230, 8, 31, "Input"],
Cell[619162, 20307, 531, 15, 31, "Input"],
Cell[619696, 20324, 49, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[619782, 20329, 56, 1, 36, "Subsection",
 CellTags->"T1.6.79.1"],
Cell[619841, 20332, 181, 6, 31, "Text"],
Cell[620025, 20340, 181, 5, 31, "Input"],
Cell[620209, 20347, 501, 16, 31, "Text"],
Cell[620713, 20365, 4798, 160, 58, "Input"],
Cell[625514, 20527, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[625688, 20536, 56, 1, 36, "Subsection",
 CellTags->"T1.6.80.1"],
Cell[625747, 20539, 551, 20, 31, "Text"],
Cell[626301, 20561, 2593, 92, 35, "Input"],
Cell[628897, 20655, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[629071, 20664, 58, 1, 36, "Subsection",
 CellTags->"T1.6.81.1"],
Cell[629132, 20667, 227, 8, 31, "Input"],
Cell[629362, 20677, 469, 15, 31, "Input"],
Cell[629834, 20694, 49, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[629920, 20699, 56, 1, 36, "Subsection",
 CellTags->"T1.6.82.1"],
Cell[629979, 20702, 92, 4, 29, "Text"],
Cell[630074, 20708, 2800, 98, 35, "Input"],
Cell[632877, 20808, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[633051, 20817, 58, 1, 36, "Subsection",
 CellTags->"T1.6.83.1"],
Cell[633112, 20820, 227, 8, 31, "Input"],
Cell[633342, 20830, 402, 12, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[633781, 20847, 60, 1, 36, "Subsection",
 CellTags->"T1.6.84.1"],
Cell[633844, 20850, 335, 11, 31, "Input"],
Cell[634182, 20863, 335, 12, 31, "Input"],
Cell[634520, 20877, 486, 14, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[635043, 20896, 60, 1, 36, "Subsection",
 CellTags->"T1.6.85.1"],
Cell[635106, 20899, 456, 14, 31, "Input"],
Cell[635565, 20915, 335, 12, 31, "Input"],
Cell[635903, 20929, 486, 14, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[636426, 20948, 56, 1, 36, "Subsection",
 CellTags->"T1.6.86.1"],
Cell[636485, 20951, 92, 4, 29, "Text"],
Cell[636580, 20957, 1860, 71, 35, "Input"],
Cell[638443, 21030, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[638617, 21039, 58, 1, 36, "Subsection",
 CellTags->"T1.6.87.1"],
Cell[638678, 21042, 227, 8, 31, "Input"],
Cell[638908, 21052, 451, 14, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[639396, 21071, 56, 1, 36, "Subsection",
 CellTags->"T1.6.88.1"],
Cell[639455, 21074, 92, 4, 29, "Text"],
Cell[639550, 21080, 2026, 74, 35, "Input"],
Cell[641579, 21156, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[641753, 21165, 58, 1, 36, "Subsection",
 CellTags->"T1.6.89.1"],
Cell[641814, 21168, 95, 3, 31, "Input"],
Cell[641912, 21173, 395, 11, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[642344, 21189, 56, 1, 36, "Subsection",
 CellTags->"T1.6.90.1"],
Cell[642403, 21192, 92, 4, 29, "Text"],
Cell[642498, 21198, 1200, 46, 35, "Input"],
Cell[643701, 21246, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[643875, 21255, 58, 1, 36, "Subsection",
 CellTags->"T1.6.91.1"],
Cell[643936, 21258, 227, 8, 31, "Input"],
Cell[644166, 21268, 374, 11, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[644577, 21284, 56, 1, 36, "Subsection",
 CellTags->"T1.6.92.1"],
Cell[644636, 21287, 92, 4, 29, "Text"],
Cell[644731, 21293, 1074, 41, 35, "Input"],
Cell[645808, 21336, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[645982, 21345, 58, 1, 36, "Subsection",
 CellTags->"T1.6.93.1"],
Cell[646043, 21348, 95, 3, 31, "Input"],
Cell[646141, 21353, 357, 11, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[646535, 21369, 56, 1, 36, "Subsection",
 CellTags->"T1.6.94.1"],
Cell[646594, 21372, 330, 11, 29, "Text"],
Cell[646927, 21385, 2031, 72, 34, "Text"],
Cell[648961, 21459, 2424, 90, 35, "Input"],
Cell[651388, 21551, 137, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[651562, 21560, 58, 1, 36, "Subsection",
 CellTags->"T1.6.95.1"],
Cell[651623, 21563, 227, 8, 31, "Input"],
Cell[651853, 21573, 471, 15, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[652361, 21593, 60, 1, 36, "Subsection",
 CellTags->"T1.6.96.1"],
Cell[652424, 21596, 344, 11, 31, "Input"],
Cell[652771, 21609, 477, 15, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[653285, 21629, 60, 1, 36, "Subsection",
 CellTags->"T1.6.97.1"],
Cell[653348, 21632, 344, 11, 31, "Input"],
Cell[653695, 21645, 477, 15, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[654209, 21665, 57, 1, 36, "Subsection",
 CellTags->"T1.6.98.1"],
Cell[654269, 21668, 39, 0, 29, "Text"],
Cell[654311, 21670, 2717, 96, 34, "Text"],
Cell[657031, 21768, 116, 3, 29, "Text"],
Cell[657150, 21773, 2981, 112, 35, "Input"],
Cell[660134, 21887, 139, 4, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[660310, 21896, 59, 1, 36, "Subsection",
 CellTags->"T1.6.99.1"],
Cell[660372, 21899, 97, 3, 31, "Input"],
Cell[660472, 21904, 510, 17, 31, "Input"],
Cell[CellGroupData[{
Cell[661007, 21925, 94, 2, 31, "Input"],
Cell[661104, 21929, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[661218, 21936, 61, 1, 36, "Subsection",
 CellTags->"T1.6.100.1"],
Cell[661282, 21939, 203, 6, 31, "Input"],
Cell[661488, 21947, 508, 16, 31, "Input"],
Cell[CellGroupData[{
Cell[662021, 21967, 95, 2, 31, "Input"],
Cell[662119, 21971, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[662233, 21978, 61, 1, 36, "Subsection",
 CellTags->"T1.6.101.1"],
Cell[662297, 21981, 254, 7, 31, "Input"],
Cell[662554, 21990, 508, 16, 31, "Input"],
Cell[CellGroupData[{
Cell[663087, 22010, 95, 2, 31, "Input"],
Cell[663185, 22014, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[663299, 22021, 61, 1, 36, "Subsection",
 CellTags->"T1.6.102.1"],
Cell[663363, 22024, 203, 6, 31, "Input"],
Cell[663569, 22032, 508, 16, 31, "Input"],
Cell[CellGroupData[{
Cell[664102, 22052, 95, 2, 31, "Input"],
Cell[664200, 22056, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[664314, 22063, 61, 1, 36, "Subsection",
 CellTags->"T1.6.103.1"],
Cell[664378, 22066, 203, 6, 31, "Input"],
Cell[664584, 22074, 508, 16, 31, "Input"],
Cell[CellGroupData[{
Cell[665117, 22094, 95, 2, 31, "Input"],
Cell[665215, 22098, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[665329, 22105, 61, 1, 36, "Subsection",
 CellTags->"T1.6.104.1"],
Cell[665393, 22108, 254, 7, 31, "Input"],
Cell[665650, 22117, 508, 16, 31, "Input"],
Cell[CellGroupData[{
Cell[666183, 22137, 95, 2, 31, "Input"],
Cell[666281, 22141, 65, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[666395, 22148, 58, 1, 36, "Subsection",
 CellTags->"T1.6.105.1"],
Cell[666456, 22151, 116, 4, 29, "Text"],
Cell[666575, 22157, 525, 16, 33, "Text"],
Cell[667103, 22175, 2289, 70, 72, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[669429, 22250, 60, 1, 36, "Subsection",
 CellTags->"T1.6.106.1"],
Cell[669492, 22253, 97, 3, 31, "Input"],
Cell[669592, 22258, 169, 5, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[669798, 22268, 58, 1, 45, "Subsection",
 CellTags->"T1.6.107.1"],
Cell[669859, 22271, 116, 4, 30, "Text"],
Cell[669978, 22277, 525, 16, 33, "Text"],
Cell[670506, 22295, 2595, 78, 92, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[673138, 22378, 60, 1, 36, "Subsection",
 CellTags->"T1.6.108.1"],
Cell[673201, 22381, 97, 3, 31, "Input"],
Cell[673301, 22386, 361, 11, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[673699, 22402, 58, 1, 45, "Subsection",
 CellTags->"T1.6.109.1"],
Cell[673760, 22405, 116, 4, 30, "Text"],
Cell[673879, 22411, 525, 16, 33, "Text"],
Cell[674407, 22429, 2074, 64, 72, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[676518, 22498, 60, 1, 36, "Subsection",
 CellTags->"T1.6.110.1"],
Cell[676581, 22501, 97, 3, 31, "Input"],
Cell[676681, 22506, 169, 5, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[676887, 22516, 58, 1, 45, "Subsection",
 CellTags->"T1.6.111.1"],
Cell[676948, 22519, 116, 4, 29, "Text"],
Cell[677067, 22525, 1559, 48, 52, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[678663, 22578, 60, 1, 36, "Subsection",
 CellTags->"T1.6.112.1"],
Cell[678726, 22581, 97, 3, 31, "Input"],
Cell[678826, 22586, 169, 5, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[679032, 22596, 58, 1, 37, "Subsection",
 CellTags->"T1.6.113.1"],
Cell[679093, 22599, 43, 0, 29, "Text"],
Cell[679139, 22601, 305, 10, 31, "Text"],
Cell[679447, 22613, 6831, 224, 58, "Input"],
Cell[686281, 22839, 386, 9, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[686704, 22853, 60, 1, 36, "Subsection",
 CellTags->"T1.6.114.1"],
Cell[686767, 22856, 229, 8, 31, "Input"],
Cell[686999, 22866, 625, 20, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[687661, 22891, 60, 1, 36, "Subsection",
 CellTags->"T1.6.115.1"],
Cell[687724, 22894, 691, 29, 473, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[688452, 22928, 60, 1, 36, "Subsection",
 CellTags->"T1.6.116.1"],
Cell[688515, 22931, 695, 29, 473, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[689247, 22965, 61, 1, 36, "Subsection",
 CellTags->"T1.6.117.1"],
Cell[689311, 22968, 3385, 91, 312, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[692733, 23064, 61, 1, 36, "Subsection",
 CellTags->"T1.6.118.1"],
Cell[692797, 23067, 3355, 91, 292, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[696189, 23163, 61, 1, 36, "Subsection",
 CellTags->"T1.6.119.1"],
Cell[696253, 23166, 3349, 91, 292, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[699639, 23262, 60, 1, 36, "Subsection",
 CellTags->"T1.6.120.1"],
Cell[699702, 23265, 240, 6, 99, "Input"],
Cell[CellGroupData[{
Cell[699967, 23275, 4436, 118, 144, "Input"],
Cell[704406, 23395, 1563, 21, 23, "Print"]
}, Open  ]],
Cell[705984, 23419, 23, 0, 29, "Text"],
Cell[CellGroupData[{
Cell[706032, 23423, 1926, 50, 122, "Input"],
Cell[707961, 23475, 1563, 21, 23, "Print"]
}, Open  ]],
Cell[709539, 23499, 53, 1, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[709629, 23505, 54, 1, 36, "Subsection",
 CellTags->"Tsymmetry"],
Cell[709686, 23508, 34, 0, 29, "Text"],
Cell[709723, 23510, 107, 3, 47, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[709867, 23518, 69, 1, 36, "Subsection",
 CellTags->"specialTVR"],
Cell[709939, 23521, 579, 11, 184, "Input"],
Cell[710521, 23534, 1683, 27, 439, "Input"],
Cell[712207, 23563, 1597, 26, 422, "Input"],
Cell[713807, 23591, 3547, 56, 949, "Input"],
Cell[717357, 23649, 3261, 52, 881, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[720655, 23706, 59, 1, 36, "Subsection",
 CellTags->"T1.6.121.1"],
Cell[720717, 23709, 230, 5, 83, "Text"],
Cell[720950, 23716, 1143, 33, 92, "Input"],
Cell[722096, 23751, 237, 5, 83, "Text"],
Cell[722336, 23758, 1251, 37, 92, "Input"],
Cell[723590, 23797, 239, 5, 83, "Text"],
Cell[723832, 23804, 1257, 37, 92, "Input"],
Cell[725092, 23843, 238, 5, 83, "Text"],
Cell[725333, 23850, 1252, 37, 92, "Input"],
Cell[726588, 23889, 219, 14, 245, "Text"],
Cell[726810, 23905, 200, 5, 82, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[727047, 23915, 60, 1, 36, "Subsection",
 CellTags->"T1.6.122.1"],
Cell[727110, 23918, 949, 29, 72, "Input"],
Cell[728062, 23949, 949, 29, 72, "Input"],
Cell[729014, 23980, 133, 3, 48, "Input"],
Cell[729150, 23985, 846, 25, 72, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[730033, 24015, 59, 1, 36, "Subsection",
 CellTags->"T1.6.123.1"],
Cell[730095, 24018, 793, 25, 52, "Input"],
Cell[730891, 24045, 793, 25, 52, "Input"],
Cell[731687, 24072, 792, 25, 52, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[732516, 24102, 57, 1, 36, "Subsection",
 CellTags->"T1.6.124.1"],
Cell[732576, 24105, 512, 16, 52, "Input"],
Cell[733091, 24123, 511, 16, 52, "Input"],
Cell[733605, 24141, 362, 12, 31, "Input"],
Cell[733970, 24155, 334, 11, 31, "Input"],
Cell[734307, 24168, 344, 11, 31, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[734700, 24185, 106, 2, 71, "Section",
 CellTags->"T1.7.1"],
Cell[CellGroupData[{
Cell[734831, 24191, 69, 1, 36, "Subsection",
 CellTags->"T1.7.1.1"],
Cell[734903, 24194, 45, 0, 29, "Text"],
Cell[734951, 24196, 81, 2, 31, "Input"],
Cell[735035, 24200, 275, 7, 31, "Input"],
Cell[735313, 24209, 451, 15, 31, "Input"],
Cell[735767, 24226, 354, 11, 31, "Input"],
Cell[736124, 24239, 453, 15, 31, "Input"],
Cell[736580, 24256, 356, 11, 31, "Input"],
Cell[736939, 24269, 82, 2, 29, "Text"],
Cell[737024, 24273, 84, 2, 31, "Input"],
Cell[737111, 24277, 336, 11, 31, "Input"],
Cell[737450, 24290, 215, 7, 101, "Text",
 CellTags->"TLRdef"],
Cell[737668, 24299, 388, 13, 31, "Input"],
Cell[738059, 24314, 755, 23, 31, "Input"],
Cell[738817, 24339, 26, 0, 29, "Text"],
Cell[738846, 24341, 3735, 100, 224, "Input"],
Cell[742584, 24443, 102, 2, 31, "Text"],
Cell[742689, 24447, 3693, 97, 224, "Input"],
Cell[746385, 24546, 127, 3, 29, "Text"],
Cell[746515, 24551, 2718, 74, 139, "Input"],
Cell[749236, 24627, 127, 3, 29, "Text"],
Cell[749366, 24632, 2980, 82, 139, "Input"],
Cell[752349, 24716, 150, 4, 29, "Text"],
Cell[752502, 24722, 3363, 91, 175, "Input"],
Cell[755868, 24815, 37, 0, 29, "Text"],
Cell[755908, 24817, 2534, 69, 97, "Input"],
Cell[758445, 24888, 37, 0, 29, "Text"],
Cell[758485, 24890, 2524, 69, 97, "Input"],
Cell[761012, 24961, 35, 0, 29, "Text"],
Cell[761050, 24963, 2979, 76, 271, "Input"],
Cell[764032, 25041, 31, 0, 29, "Text"],
Cell[764066, 25043, 224, 6, 101, "Text"],
Cell[764293, 25051, 31, 0, 29, "Text"],
Cell[764327, 25053, 204, 5, 65, "Input"],
Cell[764534, 25060, 146, 3, 47, "Text"],
Cell[764683, 25065, 443, 15, 235, "Input"],
Cell[765129, 25082, 443, 15, 235, "Input"],
Cell[765575, 25099, 49, 0, 29, "Text"],
Cell[765627, 25101, 1495, 43, 77, "Input"],
Cell[767125, 25146, 49, 0, 29, "Text"],
Cell[767177, 25148, 1498, 43, 77, "Input"],
Cell[768678, 25193, 25, 0, 29, "Text"],
Cell[768706, 25195, 876, 27, 31, "Input"],
Cell[769585, 25224, 989, 29, 31, "Input"],
Cell[770577, 25255, 77, 1, 29, "Text",
 CellTags->"tlrule17"],
Cell[770657, 25258, 2195, 64, 129, "Input"],
Cell[772855, 25324, 3938, 106, 197, "Input"],
Cell[776796, 25432, 3938, 106, 197, "Input"],
Cell[780737, 25540, 37, 0, 29, "Text"],
Cell[780777, 25542, 1908, 54, 129, "Input"],
Cell[782688, 25598, 1071, 31, 52, "Input"],
Cell[783762, 25631, 1289, 35, 69, "Input"],
Cell[785054, 25668, 5310, 191, 162, "Text"],
Cell[790367, 25861, 3323, 91, 183, "Input"],
Cell[793693, 25954, 9990, 351, 111, "Text"],
Cell[803686, 26307, 3684, 103, 200, "Input"],
Cell[807373, 26412, 65, 0, 29, "Text"],
Cell[807441, 26414, 1327, 35, 72, "Input"],
Cell[808771, 26451, 137, 4, 31, "Input"],
Cell[808911, 26457, 190, 6, 31, "Input"],
Cell[809104, 26465, 496, 17, 31, "Input"],
Cell[809603, 26484, 496, 17, 31, "Input"],
Cell[810102, 26503, 496, 17, 31, "Input"],
Cell[810601, 26522, 496, 17, 31, "Input"],
Cell[811100, 26541, 496, 17, 31, "Input"],
Cell[811599, 26560, 496, 17, 31, "Input"],
Cell[812098, 26579, 125, 3, 31, "Input"],
Cell[812226, 26584, 263, 8, 31, "Input"],
Cell[812492, 26594, 466, 15, 31, "Input"],
Cell[812961, 26611, 505, 15, 31, "Input"],
Cell[813469, 26628, 106, 3, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[813612, 26636, 49, 1, 37, "Subsection",
 CellTags->"T1.7.2.1"],
Cell[813664, 26639, 86, 3, 47, "Text"],
Cell[813753, 26644, 1925, 57, 91, "Input"],
Cell[815681, 26703, 788, 25, 52, "Input"],
Cell[816472, 26730, 788, 25, 52, "Input"],
Cell[817263, 26757, 1878, 56, 90, "Input"],
Cell[819144, 26815, 682, 21, 31, "Input"],
Cell[819829, 26838, 682, 21, 31, "Input"],
Cell[820514, 26861, 1459, 43, 76, "Input"],
Cell[821976, 26906, 683, 21, 31, "Input"],
Cell[822662, 26929, 683, 21, 31, "Input"],
Cell[823348, 26952, 1536, 47, 90, "Input"],
Cell[824887, 27001, 1690, 49, 83, "Input"],
Cell[826580, 27052, 22, 0, 29, "Text"],
Cell[826605, 27054, 435, 15, 31, "Input"],
Cell[827043, 27071, 321, 11, 31, "Input"],
Cell[827367, 27084, 1605, 49, 78, "Input"],
Cell[828975, 27135, 2174, 70, 109, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[831186, 27210, 53, 1, 36, "Subsection",
 CellTags->"T1.7.3.1"],
Cell[831242, 27213, 85, 2, 31, "Input"],
Cell[831330, 27217, 3675, 92, 172, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[835042, 27314, 51, 1, 36, "Subsection",
 CellTags->"$TTable"],
Cell[835096, 27317, 92, 3, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[835225, 27325, 58, 1, 36, "Subsection",
 CellTags->"T1.7.4.1"],
Cell[835286, 27328, 220, 6, 31, "Input"],
Cell[835509, 27336, 713, 20, 72, "Input"],
Cell[836225, 27358, 415, 11, 31, "Input"],
Cell[836643, 27371, 17387, 435, 1057, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[854079, 27812, 90, 2, 71, "Section",
 CellTags->"T1.8.1"],
Cell[854172, 27816, 532, 14, 72, "Input"],
Cell[854707, 27832, 463, 14, 31, "Input"],
Cell[855173, 27848, 16853, 424, 793, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[872063, 28277, 91, 2, 72, "Section",
 CellTags->"T1.9.1"],
Cell[CellGroupData[{
Cell[872179, 28283, 53, 1, 37, "Subsection",
 CellTags->"T1.9.1.1"],
Cell[872235, 28286, 61, 0, 29, "Text"],
Cell[872299, 28288, 565, 16, 31, "Input"],
Cell[872867, 28306, 136, 4, 31, "Input"],
Cell[873006, 28312, 122, 3, 31, "Input"],
Cell[873131, 28317, 83, 2, 31, "Input"],
Cell[873217, 28321, 4161, 103, 192, "Input"],
Cell[877381, 28426, 46, 0, 31, "Input"],
Cell[877430, 28428, 330, 11, 31, "Input"],
Cell[877763, 28441, 230, 8, 31, "Input"],
Cell[877996, 28451, 78, 2, 31, "Input"],
Cell[CellGroupData[{
Cell[878099, 28457, 216, 5, 31, "Input"],
Cell[878318, 28464, 404, 10, 23, "Print"]
}, Open  ]],
Cell[878737, 28477, 366, 10, 31, "Input"],
Cell[879106, 28489, 79, 2, 31, "Input"],
Cell[879188, 28493, 62, 1, 31, "Input"],
Cell[879253, 28496, 83, 2, 31, "Input"],
Cell[879339, 28500, 78, 2, 31, "Input"],
Cell[879420, 28504, 69, 2, 31, "Input"],
Cell[879492, 28508, 76, 2, 31, "Input"],
Cell[879571, 28512, 297, 10, 31, "Input"],
Cell[879871, 28524, 139, 4, 52, "Input"],
Cell[880013, 28530, 92, 2, 31, "Input"],
Cell[880108, 28534, 648, 18, 52, "Input"],
Cell[880759, 28554, 69, 2, 31, "Input"],
Cell[880831, 28558, 95, 3, 31, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[880975, 28567, 93, 2, 72, "Section",
 CellTags->"T1.10.1"],
Cell[881071, 28571, 114, 3, 29, "Text"],
Cell[881188, 28576, 56, 0, 29, "Text"],
Cell[CellGroupData[{
Cell[881269, 28580, 1634, 53, 847, "Input"],
Cell[882906, 28635, 15601, 486, 251, "Output"]
}, Open  ]],
Cell[898522, 29124, 1675, 54, 911, "Text"],
Cell[900200, 29180, 15179, 494, 251, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[915416, 29679, 99, 2, 71, "Section",
 CellTags->"T1.11.1"],
Cell[915518, 29683, 167, 4, 31, "Input"],
Cell[915688, 29689, 2354, 56, 932, "Input"],
Cell[918045, 29747, 90, 2, 31, "Input"],
Cell[918138, 29751, 87, 2, 29, "Text"],
Cell[CellGroupData[{
Cell[918250, 29757, 217, 7, 31, "Input"],
Cell[918470, 29766, 99, 2, 30, "Output"]
}, Open  ]],
Cell[918584, 29771, 55, 0, 29, "Text"],
Cell[CellGroupData[{
Cell[918664, 29775, 265, 7, 31, "Input"],
Cell[918932, 29784, 74, 1, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[919043, 29790, 99, 2, 31, "Input"],
Cell[919145, 29794, 88, 2, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[919270, 29801, 57, 1, 31, "Input"],
Cell[919330, 29804, 117, 2, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[919484, 29811, 61, 1, 31, "Input"],
Cell[919548, 29814, 64, 1, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[919649, 29820, 59, 1, 31, "Input"],
Cell[919711, 29823, 63, 1, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[919811, 29829, 56, 1, 31, "Input"],
Cell[919870, 29832, 73, 1, 30, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
