(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     84355,       2851]
NotebookOptionsPosition[     78841,       2664]
NotebookOutlinePosition[     80114,       2705]
CellTagsIndexPosition[     79902,       2697]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "Doing a difficult with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and FeynCalc tools"
}], "Section"],

Cell[BoxData[
 RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input"],

Cell["Loading FeynCalc from /home/rolfm/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.501051682708992*^9}],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "8.0.0.beta1",
 " For help, type ?FeynCalc, use the built-in ",
 ButtonBox["help system",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>(FrontEndExecute[
    FrontEnd`HelpBrowserLookup["AddOns", #]]& ),
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open the help browser"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.5010516834985323`*^9}],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.501051685471167*^9}],

Cell["FeynArts 3.4 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.501051685570508*^9}],

Cell["\<\
Performing multidimensional Feynman paramater integrals is sometimes a \
nontrivial task. This is augmented by the absence of algorithms for certain \
integrals with end point singularities.
The following integral shows up in research related to massless 4-partice \
phase space and was solved together with Prof. T. Gehrmann from the \
University of Z\[UDoubleDot]rich.
Integration from 0 to 1 is assumed for the parameters u,z and t.
What one is after is the expansion up to order Epsilon.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integral", "=", 
  RowBox[{"HoldForm", "[", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "1"], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["z", 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{
           RowBox[{"-", "2"}], " ", "Epsilon"}]], " ", 
         SuperscriptBox["u", 
          RowBox[{
           RowBox[{"-", "Epsilon"}], "-", "1"}]], " ", 
         SuperscriptBox["t", 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "t"}], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]]}], 
        RowBox[{"1", "+", 
         FractionBox[
          RowBox[{"u", " ", "z", " ", "t"}], 
          RowBox[{"1", "-", "t"}]]}]], 
       RowBox[{"\[DifferentialD]", "z"}], 
       RowBox[{"\[DifferentialD]", "u"}], 
       RowBox[{"\[DifferentialD]", "t"}]}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "1"], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["z", 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{
           RowBox[{"-", "2"}], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox["u", 
          RowBox[{
           RowBox[{"-", 
            TagBox["\[CurlyEpsilon]",
             TraditionalForm]}], "-", "1"}]], " ", 
         SuperscriptBox["t", 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "t"}], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]]}], 
        RowBox[{"1", "+", 
         FractionBox[
          RowBox[{"u", " ", "z", " ", "t"}], 
          RowBox[{"1", "-", "t"}]]}]], 
       RowBox[{"\[DifferentialD]", "z"}], 
       RowBox[{"\[DifferentialD]", "u"}], 
       RowBox[{"\[DifferentialD]", "t"}]}]}]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.4729012563772497`*^9, 3.501051687534317*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integrand", "=", 
  RowBox[{"integral", "[", 
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox["t", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox["u", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]]}], 
   RowBox[{
    FractionBox[
     RowBox[{"t", " ", "u", " ", "z"}], 
     RowBox[{"1", "-", "t"}]], "+", "1"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901259565872*^9, 3.5010516879258633`*^9}]
}, Open  ]],

Cell[TextData[{
 "We set various assumptions which will help ",
 StyleBox["Mathematica's ",
  FontSlant->"Italic"],
 "Integrate",
 StyleBox[" ",
  FontSlant->"Italic"],
 "function",
 StyleBox[".",
  FontSlant->"Italic"]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Integrate", ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Re", "[", "Epsilon", "]"}], "<", "0"}], "&&", " ", 
       RowBox[{"0", "\[LessEqual]", "u", "\[LessEqual]", "1"}], "&&", " ", 
       RowBox[{"0", "\[LessEqual]", "z", "\[LessEqual]", "1"}], "&&", 
       RowBox[{
        RowBox[{"Im", "[", "u", "]"}], "\[Equal]", "0"}], "&&", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"Re", "[", 
          RowBox[{"1", "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"u", " ", "z"}], "-", "1"}], ")"}]}], "]"}], "+", "1"}], 
        "<", "0"}]}], "}"}]}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tint", "=", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "1"], 
   RowBox[{"integrand", 
    RowBox[{"\[DifferentialD]", "t"}]}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[Pi]", " ", 
    SuperscriptBox["16", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]], " ", 
    TemplateBox[{RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", " ", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]}], "+", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"u", " ", "z"}], "-", "1"}], ")"}], " ", 
       TemplateBox[{"1",RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{"2", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],
         RowBox[{"1", "-", 
           RowBox[{"u", " ", "z"}]}]},
        "Hypergeometric2F1"]}], "+", "1"}], ")"}]}], 
   RowBox[{
    TemplateBox[{RowBox[{
        FractionBox["1", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{
        FractionBox["3", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901279188428*^9, 3.501051705364524*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tint2", "=", 
  RowBox[{"PowerSimplify", "[", "tint", "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[Pi]", " ", 
    SuperscriptBox["16", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]], " ", 
    TemplateBox[{RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    SuperscriptBox["u", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]}], "+", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"u", " ", "z"}], "-", "1"}], ")"}], " ", 
       TemplateBox[{"1",RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{"2", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],
         RowBox[{"1", "-", 
           RowBox[{"u", " ", "z"}]}]},
        "Hypergeometric2F1"]}], "+", "1"}], ")"}]}], 
   RowBox[{
    TemplateBox[{RowBox[{
        FractionBox["1", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{
        FractionBox["3", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901290744051*^9, 3.501051705819482*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "PowerSimplify"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"PowerSimplify[exp]  simplifies (-x)^a to (-1)^a x^a \
and\\n(y-x)^n to (-1)^n (x-y)^n; thus assuming that the exponent is\\nan \
integer (even if it is symbolic). Furthermore\\n(-1)^(a+n) and I^(a+n) are \
expanded and (I)^(2 m) -> (-1)^m and\\n(-1)^(n_Integer?EvenQ m) -> 1 \
and\\n(-1)^(n_Integer?OddQ m) -> (-1)^m and\\n(-1)^(-n) -> (-1)^n and Exp[I m \
Pi] -> (-1)^m.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.501051706188442*^9},
 CellTags->"Info3501055305-2742430"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uint", "=", 
  RowBox[{"PowerSimplify", "@", 
   RowBox[{"Collect2", "[", 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{"tint2", 
       RowBox[{"\[DifferentialD]", "u"}]}]}], " ", ",", " ", "z"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.47290137110581*^9, 3.472901407159111*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Pi]", " ", 
      SuperscriptBox["16", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{"-", "2"}], " ", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]], " ", 
      SuperscriptBox["z", 
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
          SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["1",
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            HypergeometricPFQ,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             RowBox[{
              TagBox[
               RowBox[{"2", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm], "+", "1"}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            HypergeometricPFQ,
            Editable->False,
            Selectable->False], ";", 
           TagBox["z",
            HypergeometricPFQ,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       HypergeometricPFQ]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], ")"}], " ", 
      TemplateBox[{RowBox[{"1", "-", 
          RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          FractionBox["1", "2"], "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          FractionBox["3", "2"], "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"]}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox["16", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]], " ", 
     TemplateBox[{"1",RowBox[{"-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{
         TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}],"z"},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox["16", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],RowBox[{"2", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],"z"},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["16", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]]}], 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox["16", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{
          RowBox[{"-", "2"}], " ", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}]}], "-", "1"}]]}], 
    RowBox[{
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.47290133017855*^9, {3.472901387426004*^9, 3.472901415333036*^9}, 
   3.5010517371577177`*^9}]
}, Open  ]],

Cell["Perform the z - integration:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rint", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"FunctionExpand", "@", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{"uint", 
       RowBox[{"\[DifferentialD]", "z"}]}]}]}], " ", ",", " ", 
    "HypergeometricPFQ"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.472901462939465*^9, {3.4729015170057487`*^9, 3.472901518302103*^9}, {
   3.472901625298542*^9, 3.4729016484911137`*^9}, {3.472901681866604*^9, 
   3.472901693902978*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{RowBox[{"-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
         SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["1",
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"3", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{
               TagBox["\[CurlyEpsilon]",
                TraditionalForm], "+", "1"}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", 
      RowBox[{"3", "/", "2"}]], " ", 
     SuperscriptBox["2", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
         SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TemplateBox[{RowBox[{"-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{RowBox[{
            RowBox[{"-", "3"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
         "Gamma"]}], "+", 
       RowBox[{"\[Pi]", " ", 
        RowBox[{"csc", "(", 
         RowBox[{"\[Pi]", " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], ")"}], " ", 
        TemplateBox[{RowBox[{"1", "-", 
            RowBox[{"3", " ", 
              TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
         "Gamma"]}]}], ")"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "3"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.47290144104503*^9, 3.472901488075706*^9, 
  3.4729015257429667`*^9, 3.472901634811721*^9, 3.4729016679408293`*^9, 
  3.472901706204863*^9, 3.501051749524122*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hy", "=", 
  RowBox[{"Cases2", "[", 
   RowBox[{"rint", ",", "HypergeometricPFQ"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.472901537407631*^9, 3.472901538088868*^9}, 
   3.47290163468815*^9, {3.472901689226424*^9, 3.472901690153994*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
        SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
        SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{{3.4729015326003847`*^9, 3.4729015390410137`*^9}, 
   3.47290170646313*^9, 3.501051749626875*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hn", "=", 
  RowBox[{"HypergeometricSE", "[", 
   RowBox[{"hy", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FractionBox[
     RowBox[{
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"4", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{
           RowBox[{
             RowBox[{"-", "2"}], " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
           "1"}]},
        "Gamma"], "3"]}], 
     RowBox[{
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
        "Gamma"], "3"], " ", 
      TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "4"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "2"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"]}]], ",", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"3", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
       "Gamma"], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{
           RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
           "1"}]},
        "Gamma"], "2"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "\[Nu]", "-", "1"}], ")"}], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
        "Gamma"], "2"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "3"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "\[Nu]", "+", 
          "1"}]},
       "Gamma"]}]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901567151018*^9, 3.472901720031994*^9, 
  3.501051750104823*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "HypergeometricSE"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"HypergeometricSE[exp, nu] expresses Hypergeometric functions \
by\\ntheir series expansion in terms of a sum (the Sum is omitted and\\nnu, \
running from 0 to Infinity is the summation index).\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.501051750361581*^9},
 CellTags->"Info3501055350-2742430"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FunctionExpand", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"\[Nu]", "=", "0"}], "\[Infinity]"], "#"}], "]"}], "&"}], "/@", 
  "hn"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
        SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
        SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901735719531*^9, 3.5010517691588697`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionExpand", "[", 
  RowBox[{"hn", "/.", 
   RowBox[{"\[Nu]", "\[Rule]", "0"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "1"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901745961268*^9, 3.5010517694774942`*^9}]
}, Open  ]],

Cell[TextData[{
 "Do a series expansion up to order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "4"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hnexp", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Apart", "[", 
     RowBox[{
      RowBox[{"FunctionExpand", "[", "#", "]"}], ",", "\[Nu]"}], "]"}], "&"}],
    "/@", 
   RowBox[{"Series2", "[", 
    RowBox[{"hn", ",", "Epsilon", ",", "2"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.472901771006505*^9, 3.47290180158803*^9}, 
   3.4729018850967712`*^9, {3.4729019526335897`*^9, 3.47290197896279*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{"1",RowBox[{"\[Nu]", "+", "1"}]},
         "PolyGamma2"]}], "+", 
       RowBox[{"8", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"6", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "+", "1"}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox[
      RowBox[{"28", " ", 
       SuperscriptBox[
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "2"]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "4"]], "-", 
     FractionBox[
      RowBox[{"6", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"6", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "3"]]}], ",", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{"1",RowBox[{"\[Nu]", "+", "1"}]},
         "PolyGamma2"]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"4", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "+", "1"}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox[
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "2"]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "4"]], "-", 
     FractionBox[
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "3"]]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{{3.4729017567747993`*^9, 3.472901818963334*^9}, 
   3.472901886939904*^9, {3.472901956171627*^9, 3.472901980284807*^9}, 
   3.501051770897649*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Series2"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"Series2 performs a series expansion around 0.\\nSeries2 is \
similar to Series, except that it applies Normal\\non the result and that \
some Series bugs are fixed.\\nSeries2[f, e, n] is equivalent to \\nSeries2[f, \
{e, 0, n}] is equivalent to Series[f, {e, 0, n}].\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.501051771864942*^9},
 CellTags->"Info3501055371-2742430"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hypexp", "=", 
  RowBox[{"1", "+", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Sum", "[", 
       RowBox[{"#", ",", 
        RowBox[{"{", 
         RowBox[{"\[Nu]", ",", "1", ",", "\[Infinity]"}], "}"}]}], "]"}], 
      "&"}], "/@", "hnexp"}], ")"}]}]}]], "Input"],

Cell[BoxData[
 FormBox["$Aborted", TraditionalForm]], "Output",
 CellChangeTimes->{3.472901878629587*^9, 3.472901948935339*^9, 
  3.501051878595078*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"hy", "-", "hypexp"}], ")"}], "/.", 
  RowBox[{"Epsilon", "\[Rule]", "0.01"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "2.5024271775047247`*^-8"}], ",", 
    RowBox[{"-", "1.1627923049411867`*^-9"}]}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"hy", "-", "hypexp"}], ")"}], "/.", 
  RowBox[{"Epsilon", "\[Rule]", "0.001"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "2.4058671518051877`*^-13"}], ",", 
    RowBox[{"-", "1.1197113505744332`*^-14"}]}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["hy"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", 
         FormBox["3",
          TraditionalForm]], 
        SubscriptBox["F", 
         FormBox["2",
          TraditionalForm]]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{
              RowBox[{"-", "2"}], " ", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True], ",", 
            TagBox[
             RowBox[{
              RowBox[{"-", "2"}], " ", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True], ",", 
            TagBox[
             RowBox[{
              RowBox[{"-", "2"}], " ", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", 
         FormBox["4",
          TraditionalForm]], 
        SubscriptBox["F", 
         FormBox["3",
          TraditionalForm]]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True], ",", 
            TagBox[
             RowBox[{"-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True], ",", 
            TagBox[
             RowBox[{"-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True], ",", 
            TagBox[
             RowBox[{"-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["hypexp"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"1", "-", 
     RowBox[{
      FractionBox["8", "15"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Pi]", "4"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "4"]}], "+", 
        RowBox[{"15", " ", 
         TagBox[
          RowBox[{"\[Zeta]", "(", 
           TagBox["3",
            Zeta,
            Editable->True], ")"}],
          InterpretTemplate[
           Function[BoxForm`e$, 
            Zeta[BoxForm`e$]]]], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "3"]}]}], ")"}]}]}], ",", 
    RowBox[{
     RowBox[{
      FractionBox["1", "30"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["\[Pi]", "4"]}], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "4"]}], "-", 
        RowBox[{"30", " ", 
         TagBox[
          RowBox[{"\[Zeta]", "(", 
           TagBox["3",
            Zeta,
            Editable->True], ")"}],
          InterpretTemplate[
           Function[BoxForm`e$, 
            Zeta[BoxForm`e$]]]], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "3"]}]}], ")"}]}], "+", "1"}]}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[InvisibleSpace]", 
  RowBox[{"Thread", "[", 
   RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", 
          FormBox["3",
           TraditionalForm]], 
         SubscriptBox["F", 
          FormBox["2",
           TraditionalForm]]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False],
      HypergeometricPFQ], "\[Rule]", 
     RowBox[{"1", "-", 
      RowBox[{
       FractionBox["8", "15"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[Pi]", "4"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "4"]}], "+", 
         RowBox[{"15", " ", 
          TagBox[
           RowBox[{"\[Zeta]", "(", 
            TagBox["3",
             Zeta,
             Editable->True], ")"}],
           InterpretTemplate[
            Function[BoxForm`e$, 
             Zeta[BoxForm`e$]]]], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "3"]}]}], ")"}]}]}]}], ",", 
    RowBox[{
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", 
          FormBox["4",
           TraditionalForm]], 
         SubscriptBox["F", 
          FormBox["3",
           TraditionalForm]]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["1",
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"3", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{
               TagBox["\[CurlyEpsilon]",
                TraditionalForm], "+", "1"}],
              HypergeometricPFQ,
              Editable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False],
      HypergeometricPFQ], "\[Rule]", 
     RowBox[{
      RowBox[{
       FractionBox["1", "30"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox["\[Pi]", "4"]}], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "4"]}], "-", 
         RowBox[{"30", " ", 
          TagBox[
           RowBox[{"\[Zeta]", "(", 
            TagBox["3",
             Zeta,
             Editable->True], ")"}],
           InterpretTemplate[
            Function[BoxForm`e$, 
             Zeta[BoxForm`e$]]]], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "3"]}]}], ")"}]}], "+", "1"}]}]}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["rint"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Pi]", " ", 
     RowBox[{"csc", "(", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], " ", "\[Pi]"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], ")"}], "2"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", 
          FormBox["3",
           TraditionalForm]], 
         SubscriptBox["F", 
          FormBox["2",
           TraditionalForm]]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False],
      HypergeometricPFQ]}], 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"1", "-", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]}], ")"}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["2", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "+", "1"}]], " ", 
     SqrtBox["\[Pi]"], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"1", "-", 
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], ")"}], "2"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", 
          FormBox["4",
           TraditionalForm]], 
         SubscriptBox["F", 
          FormBox["3",
           TraditionalForm]]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["1",
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"3", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True], ",", 
             TagBox[
              RowBox[{
               TagBox["\[CurlyEpsilon]",
                TraditionalForm], "+", "1"}],
              HypergeometricPFQ,
              Editable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"1", "-", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{
       FractionBox["1", "2"], "-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rint", "/.", "\[InvisibleSpace]", 
  RowBox[{"Thread", "[", 
   RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Pi]", " ", 
     RowBox[{"csc", "(", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], " ", "\[Pi]"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], ")"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{
        FractionBox["8", "15"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["\[Pi]", "4"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "4"]}], "+", 
          RowBox[{"15", " ", 
           TagBox[
            RowBox[{"\[Zeta]", "(", 
             TagBox["3",
              Zeta,
              Editable->True], ")"}],
            InterpretTemplate[
             Function[BoxForm`e$, 
              Zeta[BoxForm`e$]]]], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "3"]}]}], ")"}]}]}], ")"}]}], 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"1", "-", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]}], ")"}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["2", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "+", "1"}]], " ", 
     SqrtBox["\[Pi]"], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"1", "-", 
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], ")"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["1", "30"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox["\[Pi]", "4"]}], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "4"]}], "-", 
          RowBox[{"30", " ", 
           TagBox[
            RowBox[{"\[Zeta]", "(", 
             TagBox["3",
              Zeta,
              Editable->True], ")"}],
            InterpretTemplate[
             Function[BoxForm`e$, 
              Zeta[BoxForm`e$]]]], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "3"]}]}], ")"}]}], "+", "1"}], ")"}]}], 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"1", "-", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{
       FractionBox["1", "2"], "-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{"%", ",", "Epsilon", ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[Pi]", "4"], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}], "160"]}], "+", 
   RowBox[{"12", " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]]]}], "+", 
   FractionBox[
    RowBox[{"21", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"4", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Expand", "[", "%", "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[Pi]", "4"], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}], "160"]}], "+", 
   RowBox[{"12", " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]]]}], "+", 
   FractionBox[
    RowBox[{"21", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"4", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyPolyLog", "[", "%", "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[Pi]", "4"], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}], "160"]}], "+", 
   RowBox[{"12", " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]]]}], "+", 
   FractionBox[
    RowBox[{"63", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"2", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SimplifyPolyLog"}]], "Input"],

Cell[BoxData["\<\"SimplifyPolyLog[y] performs several simplifications \
assuming  that the variables  occuring in the Log and PolyLog functions are \
between 0 and 1. The simplifications will in general not  be valid if the \
arguments are complex or outside the range between 0 and 1.\"\>"], "Print",
 CellTags->"Info3277485428-7347271"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"total", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{"SimplifyPolyLog", "[", 
    RowBox[{"Series2", "[", 
     RowBox[{
      RowBox[{"rint", "/.", "\[InvisibleSpace]", 
       RowBox[{"Thread", "[", 
        RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}], ",", "Epsilon", ",", 
      "1"}], "]"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[Pi]", "4"], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}], "160"]}], "+", 
   RowBox[{"12", " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]]]}], "+", 
   FractionBox[
    RowBox[{"63", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"2", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"epsdat", "=", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"rint", "/.", 
        RowBox[{"Epsilon", " ", "\[Rule]", " ", "x"}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"1", "/", "1000"}], ",", 
       RowBox[{"1", "/", "100"}], ",", 
       RowBox[{"1", "/", "1000"}]}], "}"}]}], "]"}], "//", "N"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.001`", 
      RowBox[{"-", "1.749991349672133`*^9"}]},
     {"0.002`", 
      RowBox[{"-", "2.1874566762484592`*^8"}]},
     {"0.003`", 
      RowBox[{"-", "6.481192175787556`*^7"}]},
     {"0.004`", 
      RowBox[{"-", "2.7341576602742467`*^7"}]},
     {"0.005`", 
      RowBox[{"-", "1.3998258399084933`*^7"}]},
     {"0.006`", 
      RowBox[{"-", "8.100398115672059`*^6"}]},
     {"0.007`", 
      RowBox[{"-", "5.100792698303607`*^6"}]},
     {"0.008`", 
      RowBox[{"-", "3.4168748460705876`*^6"}]},
     {"0.009`", 
      RowBox[{"-", "2.399574737676905`*^6"}]},
     {"0.01`", 
      RowBox[{"-", "1.7491219971002743`*^6"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Decimal}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
       "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fit", "[", 
  RowBox[{"epsdat", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "1", ",", 
     RowBox[{"1", "/", "x"}], ",", 
     RowBox[{"1", "/", 
      RowBox[{"x", "^", "2"}]}], ",", 
     RowBox[{"1", "/", 
      RowBox[{"x", "^", "3"}]}], ",", 
     RowBox[{"1", "/", 
      RowBox[{"x", "^", "4"}]}]}], "}"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "2.344086992142816`"}], " ", "x"}], "+", 
   "14.443443394100994`", "\[InvisibleSpace]", "+", 
   FractionBox["8.635806938762258`", "x"], "+", 
   FractionBox["2.5110892342126944`*^-7", 
    SuperscriptBox["x", "2"]], "-", 
   FractionBox["1.7500000003062979`", 
    SuperscriptBox["x", "3"]], "+", 
   FractionBox["1.350181130143507`*^-13", 
    SuperscriptBox["x", "4"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "7"}], "/", "4."}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1.75`"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"21", " ", 
  RowBox[{
   RowBox[{"Zeta", "[", "2", "]"}], "/", "4."}]}]], "Input"],

Cell[BoxData[
 FormBox["8.635903850953188`", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["total"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[Pi]", "4"], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}], "160"]}], "+", 
   RowBox[{"12", " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]]]}], "+", 
   FractionBox[
    RowBox[{"63", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"2", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input"],

Cell[BoxData[
 FormBox["35.77`", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1313, 831},
WindowMargins->{{146, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"Magnification"->1,
"PaperOrientation"->"Portrait",
"PaperSize"->{597.562, 842.375},
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "rolfm"}, 
  "denksport3.ps", CharacterEncoding -> "iso8859-1"]},
ShowSelection->True,
Magnification->1.5,
FrontEndVersion->"8.0 for Linux x86 (64-bit) (November 7, 2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3501055305-2742430"->{
  Cell[11780, 403, 517, 8, 226, "Print",
   CellTags->"Info3501055305-2742430"]},
 "Info3501055350-2742430"->{
  Cell[39064, 1262, 340, 6, 116, "Print",
   CellTags->"Info3501055350-2742430"]},
 "Info3501055371-2742430"->{
  Cell[48548, 1584, 414, 7, 170, "Print",
   CellTags->"Info3501055371-2742430"]},
 "Info3277485428-7347271"->{
  Cell[73715, 2470, 338, 4, 88, "Print",
   CellTags->"Info3277485428-7347271"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3501055305-2742430", 79462, 2682},
 {"Info3501055350-2742430", 79573, 2685},
 {"Info3501055371-2742430", 79685, 2688},
 {"Info3277485428-7347271", 79797, 2691}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 130, 5, 106, "Section"],
Cell[700, 29, 71, 1, 43, "Input"],
Cell[774, 32, 205, 4, 22, "Text"],
Cell[982, 38, 678, 22, 24, "Text"],
Cell[1663, 62, 371, 12, 22, "Text"],
Cell[2037, 76, 196, 4, 22, "Text"],
Cell[2236, 82, 515, 9, 176, "Text"],
Cell[CellGroupData[{
Cell[2776, 95, 1247, 37, 88, "Input"],
Cell[4026, 134, 1589, 49, 94, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5652, 188, 141, 4, 43, "Input"],
Cell[5796, 194, 1055, 37, 94, "Output"]
}, Open  ]],
Cell[6866, 234, 231, 10, 42, "Text"],
Cell[7100, 246, 747, 20, 71, "Input"],
Cell[CellGroupData[{
Cell[7872, 270, 175, 5, 67, "Input"],
Cell[8050, 277, 1706, 53, 96, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9793, 335, 96, 2, 43, "Input"],
Cell[9892, 339, 1792, 56, 96, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11721, 400, 56, 1, 43, "Input"],
Cell[11780, 403, 517, 8, 226, "Print",
 CellTags->"Info3501055305-2742430"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12334, 416, 359, 10, 67, "Input"],
Cell[12696, 428, 9755, 299, 366, "Output"]
}, Open  ]],
Cell[22466, 730, 44, 0, 42, "Text"],
Cell[CellGroupData[{
Cell[22535, 734, 505, 13, 67, "Input"],
Cell[23043, 749, 7960, 247, 250, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31040, 1001, 263, 5, 43, "Input"],
Cell[31306, 1008, 4805, 156, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36148, 1169, 122, 3, 43, "Input"],
Cell[36273, 1174, 2692, 80, 85, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39002, 1259, 59, 1, 43, "Input"],
Cell[39064, 1262, 340, 6, 116, "Print",
 CellTags->"Info3501055350-2742430"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39441, 1273, 216, 7, 77, "Input"],
Cell[39660, 1282, 4754, 155, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44451, 1442, 126, 3, 43, "Input"],
Cell[44580, 1447, 169, 4, 43, "Output"]
}, Open  ]],
Cell[44764, 1454, 195, 8, 44, "Text"],
Cell[CellGroupData[{
Cell[44984, 1466, 431, 11, 43, "Input"],
Cell[45418, 1479, 3040, 97, 162, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48495, 1581, 50, 1, 43, "Input"],
Cell[48548, 1584, 414, 7, 170, "Print",
 CellTags->"Info3501055371-2742430"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48999, 1596, 304, 10, 43, "Input"],
Cell[49306, 1608, 152, 3, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49495, 1616, 141, 4, 43, "Input"],
Cell[49639, 1622, 190, 6, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49866, 1633, 142, 4, 43, "Input"],
Cell[50011, 1639, 192, 6, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50240, 1650, 28, 0, 43, "Input"],
Cell[50271, 1652, 4242, 142, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54550, 1799, 32, 0, 43, "Input"],
Cell[54585, 1801, 1412, 49, 75, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[56034, 1855, 129, 3, 43, "Input"],
Cell[56166, 1860, 5797, 190, 138, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[62000, 2055, 30, 0, 43, "Input"],
Cell[62033, 2057, 5822, 189, 174, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67892, 2251, 143, 3, 43, "Input"],
Cell[68038, 2256, 2992, 97, 106, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71067, 2358, 99, 2, 43, "Input"],
Cell[71169, 2362, 747, 28, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71953, 2395, 59, 1, 43, "Input"],
Cell[72015, 2398, 747, 28, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72799, 2431, 68, 1, 43, "Input"],
Cell[72870, 2434, 747, 28, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[73654, 2467, 58, 1, 43, "Input"],
Cell[73715, 2470, 338, 4, 88, "Print",
 CellTags->"Info3277485428-7347271"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74090, 2479, 345, 9, 43, "Input"],
Cell[74438, 2490, 747, 28, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[75222, 2523, 437, 13, 43, "Input"],
Cell[75662, 2538, 895, 26, 317, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76594, 2569, 368, 11, 43, "Input"],
Cell[76965, 2582, 468, 12, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77470, 2599, 68, 2, 43, "Input"],
Cell[77541, 2603, 78, 2, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77656, 2610, 106, 3, 43, "Input"],
Cell[77765, 2615, 73, 1, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77875, 2621, 31, 0, 43, "Input"],
Cell[77909, 2623, 747, 28, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[78693, 2656, 56, 1, 43, "Input"],
Cell[78752, 2659, 61, 1, 43, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
