(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     99333,       3225]
NotebookOptionsPosition[     94226,       3051]
NotebookOutlinePosition[     95347,       3088]
CellTagsIndexPosition[     95177,       3081]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "Doing a difficult with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and FeynCalc tools"
}], "Section"],

Cell[CellGroupData[{

Cell[BoxData["$Version"], "Input",
 CellChangeTimes->{{3.501077397865684*^9, 3.501077398597706*^9}}],

Cell[BoxData[
 FormBox["\<\"7.0 for Linux x86 (64-bit) (February 18, 2009)\"\>", 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.501080469635028*^9, 3.5010806963666162`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input"],

Cell["Loading FeynCalc from /home/rolfm/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.50108069652215*^9}],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "8.0.0.beta1",
 " For help, type ?FeynCalc, use the built-in ",
 ButtonBox["help system",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>(FrontEndExecute[
    FrontEnd`HelpBrowserLookup["AddOns", #]]& ),
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open the help browser"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.5010806977368183`*^9}],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.50108069975399*^9}],

Cell["FeynArts 3.4 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.5010806999805737`*^9}],

Cell["\<\
Performing multidimensional Feynman paramater integrals is sometimes a \
nontrivial task. This is augmented by the absence of algorithms for certain \
integrals with end point singularities.
The following integral shows up in research related to massless 4-partice \
phase space and was solved together with Prof. T. Gehrmann from the \
University of Z\[UDoubleDot]rich.
Integration from 0 to 1 is assumed for the parameters u,z and t.
What one is after is the expansion up to order Epsilon.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integral", "=", 
  RowBox[{"HoldForm", "[", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "1"], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["z", 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{
           RowBox[{"-", "2"}], " ", "Epsilon"}]], " ", 
         SuperscriptBox["u", 
          RowBox[{
           RowBox[{"-", "Epsilon"}], "-", "1"}]], " ", 
         SuperscriptBox["t", 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "t"}], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]]}], 
        RowBox[{"1", "+", 
         FractionBox[
          RowBox[{"u", " ", "z", " ", "t"}], 
          RowBox[{"1", "-", "t"}]]}]], 
       RowBox[{"\[DifferentialD]", "z"}], 
       RowBox[{"\[DifferentialD]", "u"}], 
       RowBox[{"\[DifferentialD]", "t"}]}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "1"], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["z", 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{
           RowBox[{"-", "2"}], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox["u", 
          RowBox[{
           RowBox[{"-", 
            TagBox["\[CurlyEpsilon]",
             TraditionalForm]}], "-", "1"}]], " ", 
         SuperscriptBox["t", 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "t"}], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]]}], 
        RowBox[{"1", "+", 
         FractionBox[
          RowBox[{"u", " ", "z", " ", "t"}], 
          RowBox[{"1", "-", "t"}]]}]], 
       RowBox[{"\[DifferentialD]", "z"}], 
       RowBox[{"\[DifferentialD]", "u"}], 
       RowBox[{"\[DifferentialD]", "t"}]}]}]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080473730569*^9, 3.501080700571664*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integrand", "=", 
  RowBox[{"integral", "[", 
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox["t", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox["u", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]]}], 
   RowBox[{
    FractionBox[
     RowBox[{"t", " ", "u", " ", "z"}], 
     RowBox[{"1", "-", "t"}]], "+", "1"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080473821342*^9, 3.5010807006570997`*^9}]
}, Open  ]],

Cell[TextData[{
 "We set various assumptions which will help ",
 StyleBox["Mathematica's ",
  FontSlant->"Italic"],
 "Integrate",
 StyleBox[" ",
  FontSlant->"Italic"],
 "function",
 StyleBox[".",
  FontSlant->"Italic"]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Integrate", ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Re", "[", "Epsilon", "]"}], "<", "0"}], "&&", " ", 
       RowBox[{"0", "\[LessEqual]", "u", "\[LessEqual]", "1"}], "&&", " ", 
       RowBox[{"0", "\[LessEqual]", "z", "\[LessEqual]", "1"}], "&&", 
       RowBox[{
        RowBox[{"Im", "[", "u", "]"}], "\[Equal]", "0"}], "&&", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"Re", "[", 
          RowBox[{"1", "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"u", " ", "z"}], "-", "1"}], ")"}]}], "]"}], "+", "1"}], 
        "<", "0"}]}], "}"}]}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tint", "=", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "1"], 
   RowBox[{"integrand", 
    RowBox[{"\[DifferentialD]", "t"}]}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[Pi]", " ", 
    SuperscriptBox["16", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    TemplateBox[{RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", " ", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"u", " ", "z"}], "-", "1"}], ")"}], " ", 
       TemplateBox[{"1",RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{"2", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],
         RowBox[{"1", "-", 
           RowBox[{"u", " ", "z"}]}]},
        "Hypergeometric2F1"]}], "-", 
      RowBox[{"2", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "+", "1"}], ")"}]}], 
   RowBox[{
    TemplateBox[{RowBox[{
        FractionBox["1", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{
        FractionBox["3", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080490689913*^9, 3.501080716613492*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tint2", "=", 
  RowBox[{"PowerSimplify", "[", "tint", "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[Pi]", " ", 
    SuperscriptBox["16", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]], " ", 
    SuperscriptBox["u", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    TemplateBox[{RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"u", " ", "z"}], "-", "1"}], ")"}], " ", 
       TemplateBox[{"1",RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{"2", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],
         RowBox[{"1", "-", 
           RowBox[{"u", " ", "z"}]}]},
        "Hypergeometric2F1"]}], "-", 
      RowBox[{"2", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "+", "1"}], ")"}]}], 
   RowBox[{
    TemplateBox[{RowBox[{
        FractionBox["1", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{
        FractionBox["3", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080491131647*^9, 3.501080717176053*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "PowerSimplify"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"PowerSimplify[exp]  simplifies (-x)^a to (-1)^a x^a \
and\\n(y-x)^n to (-1)^n (x-y)^n; thus assuming that the exponent is\\nan \
integer (even if it is symbolic). Furthermore\\n(-1)^(a+n) and I^(a+n) are \
expanded and (I)^(2 m) -> (-1)^m and\\n(-1)^(n_Integer?EvenQ m) -> 1 \
and\\n(-1)^(n_Integer?OddQ m) -> (-1)^m and\\n(-1)^(-n) -> (-1)^n and Exp[I m \
Pi] -> (-1)^m.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5010807177021723`*^9},
 CellTags->"Info3501084317-3719129"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uint", "=", 
  RowBox[{"PowerSimplify", "@", 
   RowBox[{"Collect2", "[", 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{"tint2", 
       RowBox[{"\[DifferentialD]", "u"}]}]}], " ", ",", " ", "z"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.47290137110581*^9, 3.472901407159111*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["4", 
      RowBox[{"2", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
         SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["1",
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{
               TagBox["\[CurlyEpsilon]",
                TraditionalForm], "+", "1"}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["z",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["4", 
      RowBox[{"2", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{"1",RowBox[{"-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{
         TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}],"z"},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", 
      RowBox[{"3", "/", "2"}]], " ", 
     SuperscriptBox["4", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],RowBox[{"2", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],"z"},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["4", 
      RowBox[{"2", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}], 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox["2", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}]}], "-", "1"}]], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{
          RowBox[{"-", "2"}], " ", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"]}], 
    RowBox[{
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080525567533*^9, 3.501080752250188*^9}]
}, Open  ]],

Cell["Perform the z - integration:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rint", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"FunctionExpand", "@", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{"uint", 
       RowBox[{"\[DifferentialD]", "z"}]}]}]}], " ", ",", " ", 
    "HypergeometricPFQ"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.472901462939465*^9, {3.4729015170057487`*^9, 3.472901518302103*^9}, {
   3.472901625298542*^9, 3.4729016484911137`*^9}, {3.472901681866604*^9, 
   3.472901693902978*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"]}], "+", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"]}], ")"}], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
        "Gamma"], "2"], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
          SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["1",
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            HypergeometricPFQ,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             RowBox[{
              TagBox[
               RowBox[{"2", "-", 
                RowBox[{"3", " ", 
                 TagBox["\[CurlyEpsilon]",
                  TraditionalForm]}]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"2", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm], "+", "1"}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            HypergeometricPFQ,
            Editable->False,
            Selectable->False], ";", 
           TagBox["1",
            HypergeometricPFQ,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       HypergeometricPFQ]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", "1"}], ")"}], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"3", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"]}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", 
      RowBox[{"3", "/", "2"}]], " ", 
     SuperscriptBox["2", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
         SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"12", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "5"], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "+", 
       RowBox[{"40", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "-", 
       RowBox[{"51", " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "-", 
       SuperscriptBox[
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], "3"], "-", 
       RowBox[{"9", " ", 
        TemplateBox[{RowBox[{"1", "-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "2"]}], "-", 
       RowBox[{"31", " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "2"], " ", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"]}]}], ")"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", "1"}], ")"}], "2"], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080541654677*^9, 3.501080768506919*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hy", "=", 
  RowBox[{"Cases2", "[", 
   RowBox[{"rint", ",", "HypergeometricPFQ"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.472901537407631*^9, 3.472901538088868*^9}, 
   3.47290163468815*^9, {3.472901689226424*^9, 3.472901690153994*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
        SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
        SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080541718493*^9, 3.501080768729867*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hn", "=", 
  RowBox[{"HypergeometricSE", "[", 
   RowBox[{"hy", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FractionBox[
     RowBox[{
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"4", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{
           RowBox[{
             RowBox[{"-", "2"}], " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
           "1"}]},
        "Gamma"], "3"]}], 
     RowBox[{
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
        "Gamma"], "3"], " ", 
      TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "4"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "2"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"]}]], ",", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"3", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
       "Gamma"], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{
           RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
           "1"}]},
        "Gamma"], "2"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "\[Nu]", "-", "1"}], ")"}], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
        "Gamma"], "2"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "3"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "\[Nu]", "+", 
          "1"}]},
       "Gamma"]}]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5010805421762238`*^9, 3.501080769316103*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "HypergeometricSE"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"HypergeometricSE[exp, nu] expresses Hypergeometric functions \
by\\ntheir series expansion in terms of a sum (the Sum is omitted and\\nnu, \
running from 0 to Infinity is the summation index).\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.501080769565886*^9},
 CellTags->"Info3501084369-7793775"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FunctionExpand", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"\[Nu]", "=", "0"}], "\[Infinity]"], "#"}], "]"}], "&"}], "/@", 
  "hn"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
        SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
        SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080552840921*^9, 3.501080779877061*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionExpand", "[", 
  RowBox[{"hn", "/.", 
   RowBox[{"\[Nu]", "\[Rule]", "0"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "1"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501080553240078*^9, 3.501080780241243*^9}]
}, Open  ]],

Cell[TextData[{
 "Do a series expansion up to order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "4"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hnexp", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Apart", "[", 
     RowBox[{
      RowBox[{"FunctionExpand", "[", "#", "]"}], ",", "\[Nu]"}], "]"}], "&"}],
    "/@", 
   RowBox[{"Series2", "[", 
    RowBox[{"hn", ",", "Epsilon", ",", "2"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.472901771006505*^9, 3.47290180158803*^9}, 
   3.4729018850967712`*^9, {3.4729019526335897`*^9, 3.47290197896279*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{"1",RowBox[{"\[Nu]", "+", "1"}]},
         "PolyGamma2"]}], "+", 
       RowBox[{"8", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"6", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "+", "1"}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox[
      RowBox[{"28", " ", 
       SuperscriptBox[
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "2"]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "4"]], "-", 
     FractionBox[
      RowBox[{"6", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"6", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "3"]]}], ",", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{"1",RowBox[{"\[Nu]", "+", "1"}]},
         "PolyGamma2"]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"4", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "+", "1"}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox[
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "2"]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "4"]], "-", 
     FractionBox[
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "3"]]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50108055445688*^9, 3.501080781484612*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Series2"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"Series2 performs a series expansion around 0.\\nSeries2 is \
similar to Series, except that it applies Normal\\non the result and that \
some Series bugs are fixed.\\nSeries2[f, e, n] is equivalent to \\nSeries2[f, \
{e, 0, n}] is equivalent to Series[f, {e, 0, n}].\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.5010807820264273`*^9},
 CellTags->"Info3501084381-7541714"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"hypexp", "=", 
   RowBox[{"1", "+", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Sum", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "1", ",", "\[Infinity]"}], "}"}]}], "]"}], 
       "&"}], "/@", "hnexp"}], ")"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.501057323227414*^9, 3.501057325179188*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"451.35999999999996`", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        FractionBox["1", "30"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "1080"}], " ", 
           TemplateBox[{"3"},
            "Zeta"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"180", " ", 
           TemplateBox[{"3"},
            "Zeta"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", 
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"120", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
          RowBox[{"7", " ", 
           SuperscriptBox["\[Pi]", "4"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"60", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "-", 
          RowBox[{"30", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", 
          RowBox[{"5", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}], 
      ",", 
      RowBox[{
       RowBox[{
        FractionBox["1", "30"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "480"}], " ", 
           TemplateBox[{"3"},
            "Zeta"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"120", " ", 
           TemplateBox[{"3"},
            "Zeta"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", 
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"120", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
          RowBox[{"2", " ", 
           SuperscriptBox["\[Pi]", "4"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"35", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "-", 
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", 
          RowBox[{"5", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}]}],
      "}"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081151723864*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"hy", "-", "hypexp"}], ")"}], "/.", 
   RowBox[{"Epsilon", "\[Rule]", "0.01"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5010720182971907`*^9, 3.501072029003577*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "2.2873248535226764`*^-6"}], ",", 
    RowBox[{"-", "3.962859360218118`*^-6"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.501081152370819*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"hy", "-", "hypexp"}], ")"}], "/.", 
   RowBox[{"Epsilon", "\[Rule]", "0.001"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.501072040508606*^9, 3.501072043119749*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "2.303723878327446`*^-9"}], ",", 
    RowBox[{"-", "4.008294696156156`*^-9"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5010811526228647`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"hy", "-", "hypexp"}], ")"}], "/.", 
   RowBox[{"Epsilon", "\[Rule]", "0.000001"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.501072040508606*^9, 3.501072043119749*^9}, {
  3.501072333560631*^9, 3.501072345258131*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4.440892098500626`*^-16", ",", 
    RowBox[{"-", "1.1102230246251565`*^-16"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.501081152813365*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["hy"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
        SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
        SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081153119713*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["hypexp"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox["1", "30"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "1080"}], " ", 
         TemplateBox[{"3"},
          "Zeta"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "+", 
        RowBox[{"180", " ", 
         TemplateBox[{"3"},
          "Zeta"], " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "+", 
        RowBox[{"20", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
        RowBox[{"120", " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
        RowBox[{"7", " ", 
         SuperscriptBox["\[Pi]", "4"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "+", 
        RowBox[{"60", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "-", 
        RowBox[{"30", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "+", 
        RowBox[{"5", " ", 
         SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}], 
    ",", 
    RowBox[{
     RowBox[{
      FractionBox["1", "30"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "480"}], " ", 
         TemplateBox[{"3"},
          "Zeta"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "+", 
        RowBox[{"120", " ", 
         TemplateBox[{"3"},
          "Zeta"], " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "+", 
        RowBox[{"20", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
        RowBox[{"120", " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
        RowBox[{"2", " ", 
         SuperscriptBox["\[Pi]", "4"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "+", 
        RowBox[{"35", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "-", 
        RowBox[{"20", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "+", 
        RowBox[{"5", " ", 
         SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}]}], 
   "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081153329609*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[InvisibleSpace]", 
  RowBox[{"Thread", "[", 
   RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
         SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ], "\[Rule]", 
     RowBox[{
      RowBox[{
       FractionBox["1", "30"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "1080"}], " ", 
          TemplateBox[{"3"},
           "Zeta"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "+", 
         RowBox[{"180", " ", 
          TemplateBox[{"3"},
           "Zeta"], " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], "+", 
         RowBox[{"20", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
         RowBox[{"120", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
         RowBox[{"7", " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "+", 
         RowBox[{"60", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}]}], 
    ",", 
    RowBox[{
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
         SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["1",
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"3", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{
               TagBox["\[CurlyEpsilon]",
                TraditionalForm], "+", "1"}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ], "\[Rule]", 
     RowBox[{
      RowBox[{
       FractionBox["1", "30"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "480"}], " ", 
          TemplateBox[{"3"},
           "Zeta"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "+", 
         RowBox[{"120", " ", 
          TemplateBox[{"3"},
           "Zeta"], " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], "+", 
         RowBox[{"20", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
         RowBox[{"120", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "+", 
         RowBox[{"35", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         RowBox[{"20", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", 
      "1"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081153500649*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["rint"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"]}], "+", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"]}], ")"}], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
        "Gamma"], "2"], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
          SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["1",
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            HypergeometricPFQ,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             RowBox[{
              TagBox[
               RowBox[{"2", "-", 
                RowBox[{"3", " ", 
                 TagBox["\[CurlyEpsilon]",
                  TraditionalForm]}]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"2", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm], "+", "1"}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            HypergeometricPFQ,
            Editable->False,
            Selectable->False], ";", 
           TagBox["1",
            HypergeometricPFQ,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       HypergeometricPFQ]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", "1"}], ")"}], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"3", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"]}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", 
      RowBox[{"3", "/", "2"}]], " ", 
     SuperscriptBox["2", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
         SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"12", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "5"], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "+", 
       RowBox[{"40", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "-", 
       RowBox[{"51", " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "-", 
       SuperscriptBox[
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], "3"], "-", 
       RowBox[{"9", " ", 
        TemplateBox[{RowBox[{"1", "-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "2"]}], "-", 
       RowBox[{"31", " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "2"], " ", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"]}]}], ")"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", "1"}], ")"}], "2"], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081153629396*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rint", "/.", "\[InvisibleSpace]", 
  RowBox[{"Thread", "[", 
   RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"]}], "+", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"]}], ")"}], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
        "Gamma"], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["1", "30"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "480"}], " ", 
            TemplateBox[{"3"},
             "Zeta"], " ", 
            SuperscriptBox[
             TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "+", 
           RowBox[{"120", " ", 
            TemplateBox[{"3"},
             "Zeta"], " ", 
            TagBox["\[CurlyEpsilon]",
             TraditionalForm]}], "+", 
           RowBox[{"20", " ", 
            SuperscriptBox["\[Pi]", "2"], " ", 
            SuperscriptBox[
             TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
           RowBox[{"120", " ", 
            SuperscriptBox[
             TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
           RowBox[{"2", " ", 
            SuperscriptBox["\[Pi]", "4"], " ", 
            SuperscriptBox[
             TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "+", 
           RowBox[{"35", " ", 
            SuperscriptBox["\[Pi]", "2"], " ", 
            SuperscriptBox[
             TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "-", 
           RowBox[{"20", " ", 
            SuperscriptBox["\[Pi]", "2"], " ", 
            TagBox["\[CurlyEpsilon]",
             TraditionalForm]}], "+", 
           RowBox[{"5", " ", 
            SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}], 
       ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", "1"}], ")"}], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"3", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"]}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", 
      RowBox[{"3", "/", "2"}]], " ", 
     SuperscriptBox["2", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["1", "30"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "1080"}], " ", 
           TemplateBox[{"3"},
            "Zeta"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"180", " ", 
           TemplateBox[{"3"},
            "Zeta"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", 
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"120", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
          RowBox[{"7", " ", 
           SuperscriptBox["\[Pi]", "4"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"60", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "-", 
          RowBox[{"30", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", 
          RowBox[{"5", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}], 
      ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"12", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "5"], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "+", 
       RowBox[{"40", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "-", 
       RowBox[{"51", " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "3"]}], "-", 
       SuperscriptBox[
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], "3"], "-", 
       RowBox[{"9", " ", 
        TemplateBox[{RowBox[{"1", "-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "2"]}], "-", 
       RowBox[{"31", " ", 
        SuperscriptBox[
         TemplateBox[{RowBox[{"1", "-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
          "Gamma"], "2"], " ", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"]}]}], ")"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", "1"}], ")"}], "2"], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081153861774*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{"%", ",", "Epsilon", ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"12", " ", 
    TemplateBox[{"3"},
     "Zeta"]}], "+", 
   FractionBox[
    RowBox[{"21", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"4", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "160"], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081154340238*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"total", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{"Series2", "[", 
    RowBox[{
     RowBox[{"rint", "/.", "\[InvisibleSpace]", 
      RowBox[{"Thread", "[", 
       RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}], ",", "Epsilon", ",", 
     "1"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5010722039237213`*^9, 3.501072211868074*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"12", " ", 
    TemplateBox[{"3"},
     "Zeta"]}], "+", 
   FractionBox[
    RowBox[{"21", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"4", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "160"], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081154603355*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"epsdat", "=", 
   RowBox[{"N", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"rint", "/.", 
          RowBox[{"Epsilon", " ", "\[Rule]", " ", "x"}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"1", "/", "1000000"}], ",", 
         RowBox[{"1", "/", "100000"}], ",", 
         RowBox[{"1", "/", "1000000"}]}], "}"}]}], "]"}], ",", "32"}], 
    "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.501073106920843*^9, 3.50107310814318*^9}, 
   3.5010731551763887`*^9, {3.501073317639653*^9, 3.501073322318679*^9}, {
   3.501073369859943*^9, 3.501073375876134*^9}, {3.501073410201378*^9, 
   3.5010734241580887`*^9}, {3.501077299185699*^9, 3.501077300053425*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"52.05000000000007`", ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "1.74999999999136408172436458215894738244403751841569711292`32.*^\
18"}]},
       {"2.`32.*^-6", 
        RowBox[{
        "-", "2.1874999999568203364984178553772731103388916586115018538`32.*^\
17"}]},
       {"3.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "6.481481481193616577314940758497738463380864183410486249`32.*^\
16"}]},
       {"4.`32.*^-6", 
        RowBox[{
        "-", "2.734374999784100961258130105412440854335697298422639872`32.*^\
16"}]},
       {"5.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "1.399999999827280480512956982918804103837708113750744684`32.*^\
16"}]},
       {"6.`32.*^-6", 
        RowBox[{
        "-", "8.10185185041252011868047076039786034589760243795609683`32.*^\
15"}]},
       {"7.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "5.10204081509281563743015474275090482390961330143293169`32.*^\
15"}]},
       {"8.`32.*^-6", 
        RowBox[{
        "-", "3.41796874892049759395288769993677229213859780305105405`32.*^\
15"}]},
       {"9.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "2.40054869588543384428395160210297256220952556780734795`32.*^\
15"}]},
       {"1.`32.*^-5", 
        RowBox[{
        "-", "1.74999999913639519022793714809334358205548255454672764`32.*^\
15"}]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.501081197892313*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fit", "[", 
  RowBox[{"epsdat", ",", 
   RowBox[{"{", 
    RowBox[{"\[Epsilon]", ",", "1", ",", 
     RowBox[{"1", "/", "\[Epsilon]"}], ",", 
     RowBox[{"1", "/", 
      RowBox[{"\[Epsilon]", "^", "3"}]}]}], "}"}], ",", "\[Epsilon]"}], 
  "]"}]], "Input",
 CellChangeTimes->{
  3.501073205321957*^9, {3.501073245949606*^9, 3.50107324808694*^9}, {
   3.5010772767056932`*^9, 3.501077283881826*^9}, 3.5010804316385098`*^9}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     "1.7499999999999999999999999963334505722862840030787739308868`30.\
309746329822957", 
     SuperscriptBox["\[Epsilon]", "3"]]}], "-", 
   RowBox[{
   "0.60980681771855910538741392595271516818`7.183238865242681", " ", 
    "\[Epsilon]"}], "+", 
   FractionBox["8.63590385095318099046868924679982056484`19.024435657258707", 
    "\[Epsilon]"], "+", 
   "14.42468284299939111135634065086947452113`13.650135537374583"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50108119840725*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{"total", ",", "15"}], "]"}]], "Input",
 CellChangeTimes->{{3.50107318319795*^9, 3.501073184178829*^9}, {
  3.5010771721517153`*^9, 3.5010771772064447`*^9}, {3.501077313881576*^9, 
  3.5010773857635107`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"0.60880681896251523272775207930440694543`15.", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]}], "+", 
   FractionBox["8.63590385095318879148017962489163224345`15.", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "-", 
   FractionBox["1.75`15.", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]], "+", 
   "14.42468283791513141808737150099537144638`15."}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.501081198664321*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["total"], "Input",
 CellChangeTimes->{{3.501073186623065*^9, 3.50107318702059*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"12", " ", 
    TemplateBox[{"3"},
     "Zeta"]}], "+", 
   FractionBox[
    RowBox[{"21", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"4", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "160"], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501081198815867*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input"],

Cell[BoxData[
 FormBox["602.18`", TraditionalForm]], "Output",
 CellChangeTimes->{3.5010811989302197`*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1313, 831},
WindowMargins->{{37, Automatic}, {Automatic, 24}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"Magnification"->1,
"PaperOrientation"->"Portrait",
"PaperSize"->{597.562, 842.375},
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "rolfm"}, 
  "denksport3.ps", CharacterEncoding -> "iso8859-1"]},
ShowSelection->True,
Magnification->1.5,
FrontEndVersion->"7.0 for Linux x86 (64-bit) (February 25, 2009)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3501084317-3719129"->{
  Cell[12046, 412, 519, 8, 231, "Print",
   CellTags->"Info3501084317-3719129"]},
 "Info3501084369-7793775"->{
  Cell[38932, 1267, 340, 6, 119, "Print",
   CellTags->"Info3501084369-7793775"]},
 "Info3501084381-7541714"->{
  Cell[48309, 1587, 416, 7, 175, "Print",
   CellTags->"Info3501084381-7541714"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3501084317-3719129", 94848, 3069},
 {"Info3501084369-7793775", 94959, 3072},
 {"Info3501084381-7541714", 95071, 3075}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 130, 5, 108, "Section"],
Cell[CellGroupData[{
Cell[722, 31, 100, 1, 46, "Input"],
Cell[825, 34, 178, 3, 45, "Output"]
}, Open  ]],
Cell[1018, 40, 71, 1, 46, "Input"],
Cell[1092, 43, 204, 4, 24, "Text"],
Cell[1299, 49, 678, 22, 26, "Text"],
Cell[1980, 73, 370, 12, 24, "Text"],
Cell[2353, 87, 198, 4, 24, "Text"],
Cell[2554, 93, 515, 9, 182, "Text"],
Cell[CellGroupData[{
Cell[3094, 106, 1247, 37, 97, "Input"],
Cell[4344, 145, 1587, 49, 94, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5968, 199, 141, 4, 46, "Input"],
Cell[6112, 205, 1055, 37, 94, "Output"]
}, Open  ]],
Cell[7182, 245, 231, 10, 44, "Text"],
Cell[7416, 257, 747, 20, 77, "Input"],
Cell[CellGroupData[{
Cell[8188, 281, 175, 5, 66, "Input"],
Cell[8366, 288, 1681, 52, 96, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10084, 345, 96, 2, 46, "Input"],
Cell[10183, 349, 1767, 55, 96, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11987, 409, 56, 1, 46, "Input"],
Cell[12046, 412, 519, 8, 231, "Print",
 CellTags->"Info3501084317-3719129"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12602, 425, 359, 10, 66, "Input"],
Cell[12964, 437, 8535, 271, 276, "Output"]
}, Open  ]],
Cell[21514, 711, 44, 0, 44, "Text"],
Cell[CellGroupData[{
Cell[21583, 715, 505, 13, 66, "Input"],
Cell[22091, 730, 8856, 273, 250, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[30984, 1008, 263, 5, 46, "Input"],
Cell[31250, 1015, 4752, 155, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36039, 1175, 122, 3, 46, "Input"],
Cell[36164, 1180, 2669, 79, 85, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38870, 1264, 59, 1, 46, "Input"],
Cell[38932, 1267, 340, 6, 119, "Print",
 CellTags->"Info3501084369-7793775"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39309, 1278, 216, 7, 77, "Input"],
Cell[39528, 1287, 4752, 155, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44317, 1447, 126, 3, 46, "Input"],
Cell[44446, 1452, 167, 4, 45, "Output"]
}, Open  ]],
Cell[44628, 1459, 195, 8, 46, "Text"],
Cell[CellGroupData[{
Cell[44848, 1471, 431, 11, 46, "Input"],
Cell[45282, 1484, 2937, 95, 85, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48256, 1584, 50, 1, 46, "Input"],
Cell[48309, 1587, 416, 7, 175, "Print",
 CellTags->"Info3501084381-7541714"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48762, 1599, 412, 12, 46, "Input"],
Cell[49177, 1613, 3344, 96, 138, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52558, 1714, 240, 6, 46, "Input"],
Cell[52801, 1722, 231, 7, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53069, 1734, 239, 6, 46, "Input"],
Cell[53311, 1742, 232, 7, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53580, 1754, 291, 7, 46, "Input"],
Cell[53874, 1763, 213, 6, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54124, 1774, 28, 0, 46, "Input"],
Cell[54155, 1776, 4730, 155, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58922, 1936, 32, 0, 46, "Input"],
Cell[58957, 1938, 3095, 94, 138, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[62089, 2037, 129, 3, 46, "Input"],
Cell[62221, 2042, 7984, 247, 211, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[70242, 2294, 30, 0, 46, "Input"],
Cell[70275, 2296, 8834, 273, 250, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[79146, 2574, 143, 3, 46, "Input"],
Cell[79292, 2579, 7248, 214, 273, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86577, 2798, 99, 2, 46, "Input"],
Cell[86679, 2802, 618, 22, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87334, 2829, 364, 9, 46, "Input"],
Cell[87701, 2840, 618, 22, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88356, 2867, 824, 20, 46, "Input"],
Cell[89183, 2889, 2168, 57, 317, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[91388, 2951, 447, 11, 46, "Input"],
Cell[91838, 2964, 563, 15, 77, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[92438, 2984, 251, 5, 46, "Input"],
Cell[92692, 2991, 550, 16, 77, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[93279, 3012, 96, 1, 46, "Input"],
Cell[93378, 3015, 618, 22, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[94033, 3042, 56, 1, 46, "Input"],
Cell[94092, 3045, 106, 2, 45, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
